Package: legendry
Title: Extended Legends and Axes for 'ggplot2'
Version: 0.3.0
Authors@R: 
    person("Teun", "van den Brand", , "tahvdbrand@gmail.com", role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0002-9335-7468"))
Description: A 'ggplot2' extension that focusses on expanding the
    plotter's arsenal of guides. Guides in 'ggplot2' include axes and
    legends. 'legendry' offers new axes and annotation options, as well as
    new legends and colour displays.
License: MIT + file LICENSE
URL: https://teunbrand.github.io/legendry/,
        https://github.com/teunbrand/legendry
BugReports: https://github.com/teunbrand/legendry/issues
Depends: ggplot2 (>= 4.0.0), R (>= 4.1.0)
Imports: cli, grid (>= 4.1.0), gtable, lifecycle, rlang (>= 1.2.0), S7,
        scales (>= 1.1.1), vctrs (>= 0.6.0)
Suggests: knitr, ragg, rmarkdown, svglite, testthat (>= 3.0.0), vdiffr,
        withr
Config/testthat/edition: 3
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-12 19:00:54 UTC; Teun
Author: Teun van den Brand [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-9335-7468>)
Maintainer: Teun van den Brand <tahvdbrand@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-13 05:10:25 UTC
Built: R 4.5.2; ; 2026-05-13 09:56:42 UTC; unix

# Generated by roxygen2: do not edit by hand

export(Compose)
export(ComposeCrux)
export(ComposeOntop)
export(ComposeSandwich)
export(ComposeStack)
export(GizmoBarcap)
export(GizmoDensity)
export(GizmoGrob)
export(GizmoHistogram)
export(GizmoStepcap)
export(GuideAxisAnnotation)
export(GuideAxisPlot)
export(GuideAxisSymbols)
export(GuideCircles)
export(GuideColring)
export(GuideLegendBase)
export(GuideLegendGroup)
export(GuideLegendManual)
export(PrimitiveBox)
export(PrimitiveBracket)
export(PrimitiveFence)
export(PrimitiveLabels)
export(PrimitiveLine)
export(PrimitiveSpacer)
export(PrimitiveTicks)
export(PrimitiveTitle)
export(annotate_bottom)
export(annotate_left)
export(annotate_right)
export(annotate_top)
export(bracket_atan)
export(bracket_chevron)
export(bracket_curvy)
export(bracket_line)
export(bracket_round)
export(bracket_sigmoid)
export(bracket_square)
export(cap_arch)
export(cap_none)
export(cap_ogee)
export(cap_round)
export(cap_triangle)
export(compose_crux)
export(compose_ontop)
export(compose_sandwich)
export(compose_stack)
export(gizmo_barcap)
export(gizmo_density)
export(gizmo_grob)
export(gizmo_histogram)
export(gizmo_stepcap)
export(guide_axis_annotation)
export(guide_axis_base)
export(guide_axis_dendro)
export(guide_axis_nested)
export(guide_axis_plot)
export(guide_axis_symbols)
export(guide_axis_upset)
export(guide_circles)
export(guide_colbar)
export(guide_colring)
export(guide_colsteps)
export(guide_legend_base)
export(guide_legend_cross)
export(guide_legend_group)
export(guide_legend_manual)
export(key_auto)
export(key_bins)
export(key_dendro)
export(key_group_lut)
export(key_group_split)
export(key_log)
export(key_manual)
export(key_map)
export(key_minor)
export(key_none)
export(key_range_auto)
export(key_range_manual)
export(key_range_map)
export(key_range_rle)
export(key_segment_manual)
export(key_segment_map)
export(key_sequence)
export(key_symbols)
export(key_upset)
export(new_compose)
export(position_text)
export(primitive_box)
export(primitive_bracket)
export(primitive_fence)
export(primitive_labels)
export(primitive_line)
export(primitive_segments)
export(primitive_spacer)
export(primitive_ticks)
export(primitive_title)
export(scale_x_dendro)
export(scale_y_dendro)
export(theme_guide)
import(ggplot2)
import(grid)
import(gtable)
import(rlang)
import(scales)
import(vctrs)
importFrom(lifecycle,deprecated)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# legendry <a href="https://teunbrand.github.io/legendry/"><img src="man/figures/logo.png" align="right" height="138" alt="legendry website" /></a>

<!-- badges: start -->

[![R-CMD-check](https://github.com/teunbrand/legendry/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/teunbrand/legendry/actions/workflows/R-CMD-check.yaml)
[![CRAN
status](https://www.r-pkg.org/badges/version/legendry)](https://www.r-pkg.org/badges/version/legendry)
[![Codecov test
coverage](https://codecov.io/gh/teunbrand/legendry/graph/badge.svg)](https://app.codecov.io/gh/teunbrand/legendry)
<!-- badges: end -->

The goal of legendry is to provide additional guide functionality to the
ggplot2 ecosystem.

## Installation

You can install the development version of legendry from
[GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("teunbrand/legendry")
```

## Example

Let’s first set up a basic plot to experiment with:

``` r
library(legendry)
#> Loading required package: ggplot2

base <- ggplot(mpg, aes(displ, hwy, colour = cty)) +
  geom_point() +
  labs(
    x = "Engine displacement",
    y = "Highway miles per gallon",
    col = "City miles\nper gallon"
  ) +
  theme(axis.line = element_line())
```

The legendry package offers a selection of what it calls ‘complete
guides’. These complete guides can just be drop-in replacement of
regular guides, which you can specify using ggplot2’s `guides()`
function or using the `guide` argument in scales. In the example below,
we’re using two custom variants of vanilla guides, namely
`guide_axis_base()` and `guide_colbar()`. These custom variants have
additional options that allow a greater degree of customisation:

- The axis guide has an option for bidirectional ticks.
- The colourbar automatically recognises out-of-bounds values and
  displays this with a cap.

``` r
base + 
  scale_colour_viridis_c(
    limits = c(NA, 30),
    guide = "colbar"
  ) +
  guides(
    x = guide_axis_base(bidi = TRUE)
  )
```

<img src="man/figures/README-full_guides-1.svg" width="80%" style="display: block; margin: auto;" />

Besides complete guides, legendry also has incomplete guides that can be
composed. The `ggplot2::guide_axis_stack()` is an axis composition
function that can be used to display multiple guides. Here, we use a
‘primitive’ guide (incomplete building block) to display a range on the
axis. By stacking it with a regular axis the primitive guide is
completed.

``` r
# A partial guide to display a bracket
efficient_bracket <- primitive_bracket(
  # Keys determine what is displayed
  key = key_range_manual(start = 25, end = Inf, name = "Efficient"),
  bracket = "square",
  # We want vertical text
  theme = theme(
    legend.text = element_text(angle = 90, hjust = 0.5),
    axis.text.y.left = element_text(angle = 90, hjust = 0.5)
  )
)

base + guides(y = guide_axis_stack("axis", efficient_bracket))
```

<img src="man/figures/README-primitives-1.svg" width="80%" style="display: block; margin: auto;" />

The legendry package extends this guide composition concept beyond the
axes for other types of guides. In the example below we compose a
‘sandwich’: a central guide flanked by two others. Because our bracket
is a primitive, it does not matter what aesthetic it displays and we can
re-use it for the sandwich. I’ve yet to write the vignette on
composition.

``` r
base + 
  scale_colour_viridis_c(
    guide = compose_sandwich(
      middle = gizmo_density(), 
      text = "axis_base",
      opposite = efficient_bracket
    )
  )
```

<img src="man/figures/README-composition-1.svg" width="80%" style="display: block; margin: auto;" />

## Code of Conduct

Please note that the legendry project is released with a [Contributor
Code of
Conduct](https://teunbrand.github.io/legendry/CODE_OF_CONDUCT.html). By
contributing to this project, you agree to abide by its terms.

<!-- README.md is generated from README.Rmd. Please edit that file -->

# legendry <a href="https://teunbrand.github.io/legendry/"><img src="man/figures/logo.png" align="right" height="138" alt="legendry website" /></a>

<!-- badges: start -->

[![R-CMD-check](https://github.com/teunbrand/legendry/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/teunbrand/legendry/actions/workflows/R-CMD-check.yaml)
[![CRAN
status](https://www.r-pkg.org/badges/version/legendry)](https://www.r-pkg.org/badges/version/legendry)
[![Codecov test
coverage](https://codecov.io/gh/teunbrand/legendry/graph/badge.svg)](https://app.codecov.io/gh/teunbrand/legendry)
<!-- badges: end -->

The goal of legendry is to provide additional guide functionality to the
ggplot2 ecosystem.

## Installation

You can install the development version of legendry from
[GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("teunbrand/legendry")
```

## Example

Let’s first set up a basic plot to experiment with:

``` r
library(legendry)
#> Loading required package: ggplot2

base <- ggplot(mpg, aes(displ, hwy, colour = cty)) +
  geom_point() +
  labs(
    x = "Engine displacement",
    y = "Highway miles per gallon",
    col = "City miles\nper gallon"
  ) +
  theme(axis.line = element_line())
```

The legendry package offers a selection of what it calls ‘complete
guides’. These complete guides can just be drop-in replacement of
regular guides, which you can specify using ggplot2’s `guides()`
function or using the `guide` argument in scales. In the example below,
we’re using two custom variants of vanilla guides, namely
`guide_axis_base()` and `guide_colbar()`. These custom variants have
additional options that allow a greater degree of customisation:

- The axis guide has an option for bidirectional ticks.
- The colourbar automatically recognises out-of-bounds values and
  displays this with a cap.

``` r
base + 
  scale_colour_viridis_c(
    limits = c(NA, 30),
    guide = "colbar"
  ) +
  guides(
    x = guide_axis_base(bidi = TRUE)
  )
```

<img src="man/figures/README-full_guides-1.svg" alt="" width="80%" style="display: block; margin: auto;" />

Besides complete guides, legendry also has incomplete guides that can be
composed. The `ggplot2::guide_axis_stack()` is an axis composition
function that can be used to display multiple guides. Here, we use a
‘primitive’ guide (incomplete building block) to display a range on the
axis. By stacking it with a regular axis the primitive guide is
completed.

``` r
# A partial guide to display a bracket
efficient_bracket <- primitive_bracket(
  # Keys determine what is displayed
  key = key_range_manual(start = 25, end = Inf, name = "Efficient"),
  bracket = "square",
  # We want vertical text
  theme = theme(
    legend.text = element_text(angle = 90, hjust = 0.5),
    axis.text.y.left = element_text(angle = 90, hjust = 0.5)
  )
)

base + guides(y = guide_axis_stack("axis", efficient_bracket))
```

<img src="man/figures/README-primitives-1.svg" alt="" width="80%" style="display: block; margin: auto;" />

The legendry package extends this guide composition concept beyond the
axes for other types of guides. In the example below we compose a
‘sandwich’: a central guide flanked by two others. Because our bracket
is a primitive, it does not matter what aesthetic it displays and we can
re-use it for the sandwich. I’ve yet to write the vignette on
composition.

``` r
base + 
  scale_colour_viridis_c(
    guide = compose_sandwich(
      middle = gizmo_density(), 
      text = "axis_base",
      opposite = efficient_bracket
    )
  )
```

<img src="man/figures/README-composition-1.svg" alt="" width="80%" style="display: block; margin: auto;" />

## Code of Conduct

Please note that the legendry project is released with a [Contributor
Code of
Conduct](https://teunbrand.github.io/legendry/CODE_OF_CONDUCT.html). By
contributing to this project, you agree to abide by its terms.

# Constructor -------------------------------------------------------------

#' Guide primitive: spacer
#'
#' This function constructs a spacer [guide primitive][guide-primitives].
#' The spacer is intended for use in [guide composition][guide-composition].
#'
#' @param space A [`<unit[1]>`][grid::unit()]
#' @inheritParams common_parameters
#'
#' @return A `<PrimitiveSpacer>` primitive guide that can be used inside
#'   other guides.
#' @export
#' @family primitives
#'
#' @details
#' ## Styling options
#'
#' Below are the [theme][ggplot2::theme] options that determine the styling of
#' this guide. In context to many primitive guides, whether it is used in an
#' axis or legend has no bearing on the style.
#'
#' | **Theme setting** | **Context** | **Type** | **Description** |
#' | ----------------- | ----------- | -------- | --------------- |
#' | `legendry.guide.spacing` | Any | [`unit()`] | Fallback amount of spacing when the `space` argument is `NULL` |
#'
#' There are no other styling options.
#' The context-agnostic alternative to using `theme()` is to use
#' [`theme_guide()`]:
#'
#' ```r
#' primitive_spacer(theme = theme_guide(
#'   spacing = unit(5, "mm"),
#' ))
#' ```
#'
#' @examples
#' ggplot(mpg, aes(displ, hwy)) +
#'   geom_point() +
#'   guides(
#'     x = guide_axis_stack("axis", primitive_spacer(unit(1, "cm")), "axis")
#'   )
primitive_spacer <- function(space = NULL, title = waiver(),
                             theme = NULL, position = waiver()) {
  check_object(space, is.unit, "a {.cls unit}", allow_null = TRUE)
  new_guide(
    space = space, title = title, theme = theme, position = position,
    available_aes = c("any", "x", "y", "r", "theta"),
    super = PrimitiveSpacer
  )
}

# Class -------------------------------------------------------------------

#' @export
#' @rdname legendry_extensions
#' @format NULL
#' @usage NULL
PrimitiveSpacer <- ggproto(
  "Spacer", Guide,

  params = new_params(space = NULL),

  train = function(self, params = self$params, scale, aesthetic = NULL, ...) {
    params$aesthetic <- aesthetic %||% scale$aesthetics[1L]
    params$position  <- params$position %|W|% NULL
    params$hash <- hash(list(params$position, params$space))
    params
  },

  transform = function(self, params, coord, panel_params) {
    params
  },

  draw = function(self, theme, position = NULL, direction = NULL,
                  params = self$params) {
    space <- params$space %||%
      calc_element("legendry.guide.spacing", theme + params$theme)
    primitive_grob(
      zeroGrob(), sum(space), params$position %||% position, "spacer"
    )
  }
)

test_that("gizmo_stepcap trains correctly with even.steps = FALSE", {

  na_value <- "#FF0000" # red
  make_scale <- function(limits = NULL, train = numeric()) {
    scale <- scale_colour_viridis_c(limits = limits, na.value = na_value)
    scale$train(train)
    scale
  }

  guide <- gizmo_stepcap(key = key_bins(even.steps = FALSE, show.limits = TRUE))

  scale  <- make_scale(train = c(10.0, 30.0))
  params <- guide$train(guide$params, scale, "colour")

  # Test all key columns
  expect_identical(params$limits, c(10.0, 30.0))
  expect_identical(dim(params$key), c(5L, 5L))
  expect_identical(
    params$key$colour[c(1L, 4L, 5L)],
    c("#452F73", "#B0DA45", NA)
  )
  expect_identical(params$key$min, c(10.0, 15.0, 20.0, 25.0, NA))
  expect_identical(params$key$max, c(15.0, 20.0, 25.0, 30.0, NA))
  expect_identical(params$key$.label, c("10", "15", "20", "25", "30"))
  expect_identical(params$key$.value, c(10.0, 15.0, 20.0, 25.0, 30.0))

  # Upper out of bounds
  scale  <- make_scale(c(10.0, 30.0), c(10.0, 40.0))
  params <- guide$train(guide$params, scale, "colour")

  expect_identical(params$limits, c(10.0, 30.02))
  expect_identical(dim(params$key), c(5L, 5L)) # still 5 because last row was empty
  expect_identical(
    params$key$colour[c(1L, 4L, 5L)],
    c("#452F73", "#B0DA45", na_value)
  )
  expect_identical(params$key$max, c(15.0, 20.0, 25.0, 30.0, Inf))

  # Lower out of bounds
  scale  <- make_scale(c(10.0, 30.0), c(0.0, 30.0))
  params <- guide$train(guide$params, scale, "colour")

  expect_identical(params$limits, c(9.98, 30.0))
  expect_identical(dim(params$key), c(6L, 5L)) # row insertion
  expect_identical(
    params$key$colour[c(1L, 2L, 5L, 6L)],
    c(na_value, "#452F73", "#B0DA45", NA)
  )
  expect_equal(params$key$min, c(-Inf, 10.0, 15.0, 20.0, 25.0, NA))

  # Both out of bounds
  scale <- make_scale(c(10.0, 30.0), c(0.0, 40.0))
  params <- guide$train(guide$params, scale, "colour")

  expect_identical(params$limits, c(9.98, 30.02))
  expect_identical(dim(params$key), c(6L, 5L)) # lower row insertion
  expect_identical(
    params$key$colour[c(1L, 2L, 5L, 6L)],
    c(na_value, "#452F73", "#B0DA45", na_value)
  )
  expect_identical(params$key$min, c(-Inf, 10.0, 15.0, 20.0, 25.0, 30.0))
  expect_identical(params$key$max, c(10.0, 15.0, 20.0, 25.0, 30.0, Inf))

})

test_that("gizmo_stepcap trains correctly with even.steps = TRUE", {

  na_value <- "#FF0000" # red
  colours  <- c("#440154", "#21908C", "#FDE725")
  breaks   <- c(15.0, 20.0)

  make_scale <- function(limits = NULL, train = numeric()) {
    scale <- scale_colour_viridis_b(
      limits = limits, na.value = na_value, breaks = breaks,
      oob = oob_censor
    )
    scale$train(train)
    scale
  }

  guide <- gizmo_stepcap(key = key_bins(even.steps = TRUE))

  scale  <- make_scale(train = c(10.0, 40.0))
  params <- guide$train(guide$params, scale, "colour")

  # Test all key columns
  expect_identical(params$limits, c(10.0, 40.0))
  expect_identical(dim(params$key), c(4L, 5L))
  expect_identical(params$key$colour, c(colours, NA))
  expect_identical(params$key$min, c(10.0, 20.0, 30.0, NA))
  expect_identical(params$key$max, c(20.0, 30.0, 40.0, NA))
  expect_identical(params$key$.label, c(NA, "15", "20", NA))
  expect_identical(params$key$.value, c(NA, 20.0, 30.0, NA))

  # Upper out of bounds
  scale  <- make_scale(limits = c(10.0, 40.0), c(10.0, 50.0))
  params <- guide$train(guide$params, scale, "colour")

  expect_identical(params$limits, c(10.0, 40.03))
  expect_identical(dim(params$key), c(4L, 5L)) # still 4 because last row was empty
  expect_identical(params$key$colour, c(colours, na_value))
  expect_identical(params$key$max, c(20.0, 30.0, 40.0, Inf))

  # Lower out of bounds
  scale <- make_scale(limits = c(10.0, 40.0), train = c(0.0, 40.0))
  params <- guide$train(guide$params, scale, "colour")

  expect_identical(params$limits, c(9.97, 40.0))
  expect_identical(dim(params$key), c(5L, 5L)) # row insertion
  expect_identical(params$key$colour, c(na_value, colours, NA))
  expect_identical(params$key$min, c(-Inf, 10.0, 20.0, 30.0, NA))

  # Both out of bounds
  scale <- make_scale(limits = c(10.0, 40.0), train = c(0.0, 50.0))
  params <- guide$train(guide$params, scale, "colour")

  expect_identical(params$limits, c(9.97, 40.03))
  expect_identical(dim(params$key), c(5L, 5L)) # lower row insertion
  expect_identical(params$key$colour, c(na_value, colours, na_value))
  expect_identical(params$key$min, c(-Inf, 10.0, 20.0, 30.0, 40.0))
  expect_identical(params$key$max, c(10.0, 20.0, 30.0, 40.0, Inf))

})

test_that("gizmo_stepcap can use show.limits correctly", {


  make_scale <- function(limits = NULL, breaks = numeric()) {
    scale <- scale_colour_viridis_b(
      limits = limits, breaks = breaks, oob = oob_censor
    )
    scale$train(limits)
    scale
  }
  scale <- make_scale(c(10.0, 30.0), breaks = c(15.0, 20.0))

  # For even steps = FALSE
  guide <-
    gizmo_stepcap(key = key_bins(even.steps = FALSE, show.limits = FALSE))
  params <- guide$train(guide$params, scale, "colour")

  expect_identical(params$key$.label, c(NA, "15", "20", NA))
  expect_identical(params$key$.value, c(NA, 15.0, 20.0, NA))

  guide <-
    gizmo_stepcap(key = key_bins(even.steps = FALSE, show.limits = TRUE))
  params <- guide$train(guide$params, scale, "colour")

  expect_identical(params$key$.label, c("10", "15", "20", "30"))
  expect_identical(params$key$.value, c(10.0, 15.0, 20.0, 30.0))

  scale <- make_scale(c(10.0, 40.0), breaks = c(15.0, 20.0))

  # For even steps = TRUE
  guide <-
    gizmo_stepcap(key = key_bins(even.steps = TRUE, show.limits = FALSE))
  params <- guide$train(guide$params, scale, "colour")

  expect_identical(params$key$.value, c(NA, 20.0, 30.0, NA))
  expect_identical(params$key$.label, c(NA, "15", "20", NA))

  guide <-
    gizmo_stepcap(key = key_bins(even.steps = TRUE, show.limits = TRUE))
  params <- guide$train(guide$params, scale, "colour")

  expect_identical(params$key$.value, c(10.0, 20.0, 30.0, 40.0))
  expect_identical(params$key$.label, c("10", "15", "20", "40"))
})

test_that("compose_ontop works as axis line", {

  base <- ggplot(mpg, aes(displ, hwy)) +
    geom_blank() +
    theme_test() +
    theme(
      panel.background = element_rect(fill = NA, colour = "grey80"),
      panel.grid.major = element_line(colour = "grey90"),
      panel.border = element_blank(),
      axis.ticks = element_line(colour = "dodgerblue"),
      axis.ticks.length = unit(0.5, "cm"),
      axis.line = element_line(colour = "dodgerblue")
    )

  top <- compose_stack(
    primitive_spacer(unit(0.25, "cm")),
    guide_axis_base(
      key_manual(c(2.5, 3.5, 4.5, 5.5, 6.5, 15.0, 25.0, 35.0)),
      theme = theme(
        axis.ticks = element_line(colour = "tomato"),
        axis.ticks.length = unit(0.5, "cm"),
        axis.line = element_line(colour = "tomato")
      )
    ),
    theme = theme(legendry.guide.spacing = unit(0.0, "cm"))
  )

  ontop <- compose_ontop(
    guide_axis_base(), top
  )

  p <- base +
    guides(
      x = ontop, x.sec = ontop,
      y = ontop, y.sec = ontop
    )

  vdiffr::expect_doppelganger("compose_ontop cartesian", p)

  p <- base +
    coord_radial(start = 0.25 * pi, end = 1.75 * pi, inner.radius = 0.5) +
    guides(
      theta = ontop, theta.sec = ontop,
      r = ontop, r.sec = ontop
    )

  vdiffr::expect_doppelganger("compose_ontop radial", p)

})

test_that("primitive_spacer works in guide_axis_stack()", {

  base <- ggplot(mpg, aes(displ, hwy)) +
    geom_blank() +
    theme_test() +
    theme(
      panel.background = element_rect(fill = NA, colour = "grey80"),
      panel.grid.major = element_line(colour = "grey90"),
      panel.border = element_blank(),
      axis.line = element_line()
    )

  spacer <- primitive_spacer(unit(0.5, "cm"))
  stack  <- guide_axis_stack("axis", spacer, "axis")

  p <- base + guides(
    x = stack, x.sec = stack,
    y = stack, y.sec = stack
  )

  vdiffr::expect_doppelganger("primitive_spacer cartesian", p)

  theta <- guide_axis_stack("axis_theta", spacer, "axis_theta")

  p <- base +
    coord_radial(start = 0.25 * pi, end = 1.75 * pi, inner.radius = 0.5) +
    guides(theta = theta, theta.sec = theta, r = stack, r.sec = stack)

  vdiffr::expect_doppelganger("primitive_spacer radial", p)
})

test_that("primitive_spacer works as legend", {

  p <- ggplot(mtcars) +
    aes(
      x = disp, y = mpg,
      colour = hp,
      fill = hp
    ) +
    geom_point() +
    guides(
      colour = primitive_spacer(space = unit(2.0, "cm")),
      fill = primitive_spacer(position = "bottom", space = unit(2.0, "cm"))
    )

  vdiffr::expect_doppelganger("primitive_spacer legend", p)

})

# Constructor -------------------------------------------------------------

#' Guide gizmo: capped colour bar
#'
#' This guide displays a colour bar with optional caps at either ends of
#' the bar.
#'
#' @param key A [sequence key][key_sequence] specification. Defaults to
#'   [`key_sequence(n = 15)`][key_sequence]. Changing the argument to
#'   `key_sequence()` is fine, but changing the key type is not advised.
#' @param shape A [cap][cap_options] specification by providing one of the
#'   following:
#'   * A cap `<function>`, such as `cap_triangle()`.
#'   * A `<character[1]>` naming a cap function without the '`cap_`'-prefix,
#'   e.g. `"round"`.
#'   * A two column `<matrix[n, 2]>` giving coordinates for a cap, like those
#'   created by cap functions such as `cap_arch()`.
#' @param size A [`<unit>`][grid::unit] setting the size of the cap. When
#'   `NULL` (default), cap size will be proportional to the `shape` coordinates
#'   and the `legend.key.size` theme setting.
#'
#' @param show A `<logical>` to control how caps are displayed at the ends
#'   of the bar. When `TRUE`, caps are always displayed. When `FALSE`, caps
#'   are never displayed. When `NA` (default), caps are displayed when the
#'   data range exceed the limits. When given as `<logical[2]>`, `show[1]`
#'   controls the display at the lower end and `show[2]` at the upper end.
#' @param alpha A `<numeric[1]>` between 0 and 1 setting the colour transparency
#'   of the bar. Use `NA` to preserve the alpha encoded in the colour itself.
#' @param oob An out-of-bounds handling function that affects the cap colour.
#'   Can be one of the following:
#'   * A `<function>` like [`oob_squish`][scales::oob_squish].
#'   * A `<character[1]>` naming such a function without the '`oob`'-prefix,
#'   such as `"keep"`.
#' @inheritParams common_parameters
#'
#' @details
#' ## Styling options
#'
#' Below are the [theme][ggplot2::theme] options that determine the styling of
#' this guide. Note that the width or height (depending on the `direction`
#' argument) *includes* the cap.
#'
#' | **Theme setting** | **Type** | **Description** |
#' | ----------------- | -------- | --------------- |
#' | `legend.frame` | [`element_rect()`] | Frame drawn around the bar and caps. The `fill` setting is ignored. |
#' | `legend.key.width` | [`unit()`] | Width of the bar |
#' | `legend.key.height` | [`unit()`] | Height of the bar |
#'
#' Please note that depending on the `direction` argument, the
#' `legend.key.width`/`legend.key.height` setting are expanded 5-fold if
#' originating from the global theme. To set these directly, you can use the
#' local `theme` argument in the guide.
#' These settings have shorthands in [`theme_guide()`]:
#'
#' ```r
#' gizmo_barcap(theme = theme_guide(
#'   frame = element_rect(),
#'   key.width = unit(5, "mm")
#'   key.height = unit(5, "cm")
#' ))
#' ```
#'
#' @return A `<GizmoBarcap>` object.
#' @family gizmos
#' @export
#'
#' @examples
#' # A standard plot
#' p <- ggplot(mpg, aes(displ, hwy, colour = cty)) +
#'   geom_point()
#'
#' # Just a bar
#' p + scale_colour_viridis_c(guide = gizmo_barcap())
#'
#' # Caps show up when there is data outside the limits
#' p + scale_colour_viridis_c(
#'   limits = c(10, 30),
#'   guide  = gizmo_barcap()
#' )
#'
#' # The scale's out-of-bounds handler determines cap colour
#' p + scale_colour_viridis_c(
#'   limits = c(10, 30), oob = scales::oob_squish,
#'   guide = gizmo_barcap()
#' )
#'
#' # Customising display of the guide
#' p +
#'   scale_colour_viridis_c(
#'     oob = scales::oob_squish,
#'     guide = gizmo_barcap(
#'       shape = "arch", show = c(FALSE, TRUE),
#'       size = unit(2, "cm"),
#'       theme = theme(legend.key.height = unit(4, "cm"))
#'     )
#'   ) +
#'   theme(
#'     legend.frame = element_rect(colour = "black"),
#'     legend.key.width = unit(0.5, "cm")
#'   )
gizmo_barcap <- function(key = "sequence", shape = "triangle", size = NULL,
                         show = NA, alpha = NA, oob = "keep", theme = NULL,
                         position = waiver(), direction = NULL) {
  check_number_decimal(
    alpha, min = 0.0, max = 1.0,
    allow_infinite = FALSE, allow_na = TRUE
  )
  check_logical(show)
  check_length(show, exact = 1L:2L)
  show <- rep_len(show, 2L)

  check_unit(size, allow_null = TRUE)
  shape <- resolve_cap_shape(shape)
  oob <- resolve_oob(oob)

  new_guide(
    key = key,
    shape = shape,
    size  = size,
    show  = show,
    alpha = alpha,
    oob = oob,
    theme = theme,
    position = position,
    direction = direction,
    available_aes = c("colour", "fill"),
    super = GizmoBarcap
  )
}

# Class -------------------------------------------------------------------

#' @export
#' @rdname legendry_extensions
#' @format NULL
#' @usage NULL
GizmoBarcap <- ggproto(
  "GizmoBarcap", Guide,

  elements = list(
    frame  = "legend.frame",
    width  = "legend.key.width",
    height = "legend.key.height"
  ),

  params = new_params(alpha = NA, shape = NULL, key = "sequence",
                      show = NA, size = NULL, oob = oob_keep),

  extract_key = function(scale, aesthetic, key, ...) {
    key <- resolve_key(key %||% "sequence")
    if (is.function(key)) {
      key <- key(scale, aesthetic)
    }
    key
  },

  extract_params = function(scale, params, ...) {
    params$position <- params$position %|W|% NULL
    key <- rename(params$key, params$aesthetic, "colour")
    limits <- scale$get_limits()
    range <- scale$range$range

    lower_oob <- range[1L] < limits[1L]
    upper_oob <- range[2L] > limits[2L]

    params$show[1L] <- !isFALSE(params$show[1L] %|NA|% lower_oob)
    params$show[2L] <- !isFALSE(params$show[2L] %|NA|% upper_oob)

    add <- abs(diff(limits)) / 1000.0
    if (params$show[1L]) {
      val <- params$oob(limits[1L] - add, limits)
      limits <- range(limits, val)
      key <- data_frame0(
        colour = c(scale$map(val), key$colour),
        .value = c(val, key$.value)
      )
    }
    if (params$show[2L]) {
      val <- params$oob(limits[2L] + add, limits)
      limits <- range(limits, val)
      key <- data_frame0(
        colour = c(key$colour, scale$map(val)),
        .value = c(key$.value, val)
      )
    }
    if ("colour" %in% names(key)) {
      key$colour <- alpha(key$colour, alpha = params$alpha)
    }
    params$limits <- limits
    params$key <- key
    params
  },

  setup_params = function(params) {
    key <- params$key
    key$.value <- guide_rescale(key$.value, params$limits)
    key$x <- switch(params$position, left = , right = 0.5, key$.value)
    key$y <- switch(params$position, left = , right = key$.value, 0.5)
    params$key <- key
    if (params$limits[1L] > params$limits[2L]) {
      params$show <- rev(params$show)
    }
    params
  },

  setup_elements = function(params, elements, theme) {
    theme$legend.frame <- theme$legend.frame %||% element_blank()
    if (params$direction == "horizontal") {
      theme$legend.key.width <- (theme$legend.key.width %||% rel(1.0)) * 5.0
    } else {
      theme$legend.key.height <- (theme$legend.key.height %||% rel(1.0)) * 5.0
    }
    Guide$setup_elements(params, elements, theme)
  },

  build_frame = function(key, elements, params) {

    short_side <-
      switch(params$direction, vertical = elements$width, elements$height)

    shape <- params$shape
    max   <- max(shape[, 2L])
    if (max != 0.0) {
      shape[, 2L] <- rescale_max(shape[, 2L], from = c(0.0, max))
    }

    size_lower <- size_upper <- params$size %||% (max * short_side)
    if (isFALSE(params$show[1L])) {
      lower <- cbind(c(0.0, 1.0), c(1.0, 1.0))
      size_lower <- unit(0.0, "cm")
    } else {
      lower <- cbind(shape[, 1L], 1.0 - shape[, 2L])
    }
    if (isFALSE(params$show[2L])) {
      upper <- cbind(c(1.0, 0.0), c(1.0, 1.0))
      size_upper <- unit(0.0, "cm")
    } else {
      upper <- cbind(rev(shape[, 1L]), 1.0 - rev(shape[, 2L]))
    }
    poly_args <- list(
      x = unit.c(
        unit(0.0, "npc") + lower[, 2L] * size_lower,
        unit(1.0, "npc") - upper[, 2L] * size_upper
      ),
      y = unit(c(lower[, 1L], upper[, 1L]), "npc")
    )
    if (params$direction == "vertical") {
      poly_args <- rename(poly_args, c("x", "y"), c("y", "x"))
    }
    frame <- element_grob(elements$frame)
    gp <- frame$gp %||% gpar(col = NA)
    frame <- polygonGrob(poly_args$x, poly_args$y, gp = gp)
    list(grob = frame, upper = size_upper, lower = size_lower)
  },

  fill_frame = function(key, elements, params, grobs = NULL) {

    check_device("gradients", call = expr(gizmo_barcap()))

    grad_args <- list(
      x1 = unit(0.0, "npc") + grobs$lower,
      x2 = unit(1.0, "npc") - grobs$upper,
      y1 = 0.5, y2 = 0.5, colours = key$colour, stops = key$.value
    )
    if (params$direction == "vertical") {
      grad_args <- flip_names(grad_args)
    }

    gradient   <- inject(linearGradient(!!!grad_args))
    grobs$grob <- editGrob(grobs$grob, gp = gpar(fill = gradient))
    grobs
  },

  assemble_drawing = function(grobs, layout, sizes, params, elements) {

    middle <- switch(layout, horizontal = elements$width, elements$height)
    if (unitType(middle) != "null") {
      middle <- max(middle - sizes$lower - sizes$upper, unit(0.0, "cm"))
    }
    sizes <- unit.c(sizes$lower, middle, sizes$upper)

    if (layout == "horizontal") {
      gt <- gtable(widths = sizes, heights = elements$height) |>
        gtable_add_grob(
          grobs, t = 1L, l = 1L, r = -1L,
          clip = "off", name = "barcap"
        )
      gt$align <- list(horizontal = c(2.0, -2.0))
    } else {
      gt <- gtable(widths = elements$width, heights = rev(sizes)) |>
        gtable_add_grob(
          grobs, t = 1L, b = -1L, l = 1L,
          clip = "off", name = "barcap"
        )
      gt$align <- list(vertical = c(2.0, -2.0))
    }
    gt
  },

  draw = function(self, theme, position = NULL, direction = NULL,
                  params = self$params) {

    params <- replace_null(params, position = position, direction = direction)
    params <- self$setup_params(params)
    elems  <- self$setup_elements(params, self$elements, theme)

    frame <- self$build_frame(params$key, elems, params)
    bar   <- self$fill_frame(params$key, elems, params, grobs = frame)

    self$assemble_drawing(
      grobs = bar$grob, layout = params$direction,
      sizes = list(lower = bar$lower, upper = bar$upper),
      elements = elems
    )
  }
)

# Helpers -----------------------------------------------------------------

resolve_oob <- function(x, call = caller_call(), arg = caller_arg(x)) {
  if (is.function(x)) {
    return(x)
  }
  if (is.character(x)) {
    check_string(x, arg = arg, call = call)
    x <- paste0("oob_", x)
    fun <- find_global(x, env = global_env(), mode = "function")
    if (is.function(fun)) {
      return(fun)
    }
    # Try the {scales} package
    fun <- find_global(x, env = asNamespace("scales"), mode = "function")
    if (is.function(fun)) {
      return(fun)
    }
  }
  cli::cli_abort("Unknown {.arg {arg}} handler: {x}.", call = call)
}

#' @name guide-primitives
#' @title Guide primitives
#'
#' @description
#' Guide primitives are the building blocks of more complex guides. On their
#' own, they are not very useful as a visual reflection of a scale.
#'
#' Their purpose is to be combined with one another to form a more complex,
#' complete guides that *do* reflect a scale in some way.
#'
#' @details
#' The guide primitives are simple, but flexible in that they are not tailored
#' for one particular aesthetic. That way they can be reused and combined
#' at will.
NULL

# Convenience function for automatically setting sensible gtable based on
# a single grob
primitive_grob <- function(grob, size, position, name) {

  if (!position %in% .trbl) {
    if (is.grob(grob)) {
      grob <- gList(grob)
    } else {
      grob <- inject(gList(!!!grob))
    }
    grob <- gTree(offset = sum(size), children = grob)
    return(grob)
  }

  origin   <- unit(as.numeric(position %in% c("left", "bottom")), "npc")
  opposite <- opposite_position(position)

  if (position %in% c("top", "bottom")) {
    width  <- unit(1.0, "npc")
    height <- size
    vp     <- viewport(y = origin, height = sum(size), just = opposite)
    gt <-
      gtable(widths = width, heights = height) |>
      gtable_add_grob(
        grob, t = seq_along(size), l = 1L,
        clip = "off", name = name
      )
  } else {
    height <- unit(1.0, "npc")
    width  <- size
    vp     <- viewport(x = origin, width = sum(size), just = opposite)
    gt <-
      gtable(widths = width, heights = height) |>
      gtable_add_grob(
        grob, t = 1L, l = seq_along(size),
        clip = "off", name = name
      )
  }

  absoluteGrob(gList(gt), width = sum(width), height = sum(height), vp = vp)
}

primitive_setup_elements <- function(params, elements, theme) {
  if (params$aesthetic %in% c("x", "y")) {
    elements <- suffix_position(elements$position, params$position)
  } else {
    elements <- elements$legend
  }
  theme <- theme + params$theme
  if (identical(elements$text, "legend.text")) {
    elements$text <- setup_legend_text(theme, direction = params$direction)
  }
  if (identical(elements$title, "legend.title")) {
    elements$title <- setup_legend_title(theme, direction = params$direction)
  }
  if (identical(elements$ticks_length, "legend.ticks.length")) {
    theme$legend.ticks.length <- theme$legend.ticks.length %||%
      (calc_element("legend.key.size", theme) * 0.2)
  }
  is_char <- map_lgl(elements, is.character)
  elements[is_char] <- lapply(elements[is_char], calc_element, theme = theme)
  elements$offset <- cm(params$stack_offset %||% 0.0)
  elements
}

primitive_extract_params <- function(scale, params, ...) {
  params$position <- params$position %|W|% NULL
  params$limits   <- scale$get_limits()
  params
}

primitive_setup_params <- function(params) {
  if (params$aesthetic %in% c("x", "y")) {
    return(params)
  }

  bottom_left <- params$position %in% c("left", "bottom")
  if (!is_empty(params$key)) {
    key   <- params$key
    value <- guide_rescale(key$.value, params$limits)
    oppo  <- key$oppo %||% as.numeric(bottom_left)
    key$x <- key$x %||% switch(params$position, left = , right = oppo, value)
    key$y <- key$y %||% switch(params$position, bottom = , top = oppo, value)
    params$key <- key
  }

  decor <- params$decor
  if (!is_empty(params$decor)) {
    decor <- params$decor
    oppo  <- decor$oppo %||% as.numeric(bottom_left)
    decor$x <- decor$x %||% switch(params$position, left = , right = oppo, 0.5)
    decor$y <- decor$y %||% switch(params$position, bottom = , top = oppo, 0.5)
    params$decor <- decor
  }
  params
}

test_that("new_params include default params", {
  # This test acts as a sentinel when ggplot2 changes default parameters
  expect_equal(new_params(), Guide$params, ignore_attr = "foo")
})

test_that("primitive grob makes grobs for theta positions", {

  grob <- pointsGrob()

  test <- primitive_grob(grob, unit(2.0, "cm"), "theta", "grob")
  expect_s3_class(test, "gTree")

  grob <- list(grob, grob)
  test <- primitive_grob(grob, unit(2.0, "cm"), "theta.sec", "grob")
  expect_s3_class(test, "gTree")

})

test_that("primitive_setup_elements sets up elements", {

  elements <- list(
    position = list(
      title = "axis.title",
      text = "axis.text",
      ticks_length = "axis.ticks.length"
    ),
    legend = list(
      title = "legend.title",
      text = "legend.text",
      ticks_length = "legend.ticks.length"
    )
  )

  theme <- theme_gray()

  params <- list(position = "right", aesthetic = "y", direction = "vertical")

  test <- primitive_setup_elements(params, elements, theme)
  if (inherits(element_text, "S7_class")) {
    expect_s7_class(test$title, element_text)
    expect_s7_class(test$text, element_text)
  } else {
    expect_s3_class(test$title, "element_text")
    expect_s3_class(test$text, "element_text")
  }
  expect_s3_class(test$ticks_length, "unit")

  params$aesthetic <- "colour"

  test <- primitive_setup_elements(params, elements, theme)
  if (inherits(element_text, "S7_class")) {
    expect_s7_class(test$title, element_text)
    expect_s7_class(test$text, element_text)
  } else {
    expect_s3_class(test$title, "element_text")
    expect_s3_class(test$text, "element_text")
  }
  expect_s3_class(test$ticks_length, "unit")
})

test_that("primitive_setup_params sets up non-position parameters", {

  params <- list(
    position = "right", direction = "vertical", aesthetic = "colour",
    limits = c(0.0, 10.0),
    key = data.frame(.value = c(2.0, 8.0)),
    decor = data.frame(y = 0.5)
  )

  test <- primitive_setup_params(params)
  expect_identical(test$key$x, c(0.0, 0.0))
  expect_identical(test$key$y, c(0.2, 0.8))
  expect_in(names(test$decor), c("x", "y"))

  params <- list(aesthetic = "x")
  test <- primitive_setup_params(params)
  expect_identical(params, test)
})

# Constructor -------------------------------------------------------------

#' Manual legend
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' This is a guide that displays user-defined keys independent of scales.
#' It should only be used as a last resort when struggling to format a
#' conventional legend.
#'
#' @param labels
#' Labels to display next to the keys. Can be a `<character>` or `<expression>`
#' vector to set labels, or `NULL` to draw no labels.
#' @param ...
#' Arguments interpreted as aesthetics. For example: `colour = "red"`.
#' The aesthetics must have the same size as the `labels` argument, or have
#' size 1. These aesthetics may be overruled by fixed aesthetics set in the
#' `layers` argument.
#' @param layers
#' A `<list>` of layers (`<LayerInstance>` objects) created by the `geom_*()` or
#' `stat_*()` family of functions. These layers are used for their
#' [`key_glyph`][ggplot2::draw_key] drawing functions, as well as to populate
#' default aesthetics. Any fixed aesthetics provided to these layers overrule
#' aesthetics passed to the `...` argument.
#' @inheritParams guide_legend_base
#'
#' @details
#' Because this guide is not tied to a scale, it can be given an arbitrary name
#' in `guides()`; as long as it doesn't clash with other aesthetics.
#'
#' ## Styling options
#'
#' Below are the [theme][ggplot2::theme] options that determine the styling of
#' this guide.
#'
#' | **Theme setting** | **Type** | **Description** |
#' | ----------------- | -------- | --------------- |
#' | `legend.background`| [`element_rect()`] | Background of the legend. |
#' | `legend.margin` | [`margin()`] | Padding around the legend. |
#' | `legend.text` | [`element_text()`] | Labels displayed next to keys. |
#' | `legend.text.position` | `<character[1]>` | One of `"top"`, `"right"`, `"bottom"` or `"left"`. |
#' | `legend.title` | [`element_text()`] | Title of the legend. |
#' | `legend.title.position` | `<character[1]>` | One of `"top"`, `"right"`, `"bottom"` or `"left"`. |
#' | `legend.key` | [`element_rect()`] | Background of the key areas. |
#' | `legend.key.height` | [`unit()`] | Height of keys. |
#' | `legend.key.width` | [`unit()`] | Width of keys. |
#' | `legend.key.justification` | `<numeric[2]>` | Justification for placing legend keys in excess space. |
#' | `legend.key.spacing.x` | [`unit()`] | Horizontal spacing between keys. |
#' | `legend.key.spacing.y` | [`unit()`] | Vertical spacing between keys. Taken literally. |
#' | `legend.byrow` | `<logical[1]>` | Row-order key filling (`TRUE`) or column-order (`FALSE`) |
#'
#' The context-agnostic alternative to using `theme()` is to use
#' [`theme_guide()`]:
#'
#' ```r
#' guide_legend_manual(legend_args = list(theme = theme_guide(
#'   text = element_text(),
#'   text.position = "right",
#'   title = element_text(),
#'   title.position = "top",
#'   key = element_rect(),
#'   key.height = unit(5, "mm"),
#'   key.width = unit(5, "mm"),
#'   key.justification = c(0.5, 0.5),
#'   key.spacing.x = unit(5, "mm"),
#'   key.spacing.y = unit(5, "mm"),
#'   margin = margin(5),
#'   background = element_rect(),
#' )))
#' ```
#'
#'
#' @returns A `<GuideCustom>` object.
#' @export
#' @family standalone guides
#' @family legend guides
#'
#' @examples
#' # A standard plot
#' p <- ggplot(mtcars, aes(disp, mpg)) +
#'   geom_point()
#'
#' # Typical usage: set `label` and some aesthetics
#' p + guides(
#'   some_name = guide_legend_manual(
#'     label  = c("foo", "bar"),
#'     colour = c(NA, "black"),
#'     fill   = c("grey40", NA),
#'     layers = geom_col()
#'   )
#' )
#'
#' # Alternative: use `layers` to set aesthetics
#' p + guides(
#'   some_name = guide_legend_manual(
#'     label = c("foo", "bar"),
#'     layers = geom_col(
#'       # Must match length of `label`
#'       colour = c(NA, "black"),
#'       fill = c("grey40", NA)
#'     )
#'   )
#' )
#'
#' # You can use >1 layer
#' p + guides(
#'   some_name = guide_legend_manual(
#'     label = c("foo", "bar"),
#'     colour = c("tomato", "dodgerblue"),
#'     fill = NA,
#'     layers = list(geom_col(), geom_point())
#'   )
#' )
guide_legend_manual <- function(
  labels,
  ...,
  layers = list(geom_point()),
  title = NULL,
  theme = NULL,
  design = NULL,
  nrow = NULL,
  ncol = NULL,
  reverse = FALSE,
  position = NULL,
  direction = NULL,
  order = 0L
) {
  labels <- label_as_vector(labels)
  key <- data_frame0(labels = labels, ..., .error_call = current_call()) |>
    rename_aes() |>
    rename("labels", ".label")
  if (is_empty(key)) {
    return(guide_none())
  }

  if (is_layer(layers)) {
    layers <- list(layers)
  }
  check_list_of(layers, "LayerInstance")

  legend <- guide_legend_base(
    title   = title,
    theme   = theme,
    design  = design,
    nrow    = nrow,
    ncol    = ncol,
    reverse = reverse,
    position = position,
    direction = direction
  )$params

  if (isTRUE(legend$reverse)) {
    key <- vec_slice(key, rev(vec_seq_along(key)))
    legend$reverse <- FALSE
  }
  legend$key <- key

  new_guide(
    legend = legend,
    layers = layers,
    title = legend$title,
    position = legend$position,
    direction = legend$direction,
    hash = hash(list(title, legend$key)),
    order = legend$order,
    super = GuideLegendManual
  )
}

# Class -------------------------------------------------------------------

#' @export
#' @rdname legendry_extensions
#' @format NULL
#' @usage NULL
GuideLegendManual <- ggproto(
  "GuideLegendManual", GuideCustom,

  params = new_params(legend = NULL, layers = NULL),

  elements = list(),

  hashables = exprs(title),

  get_layer_key = function(params, layers, data = NULL, theme = NULL, ...) {
    # This is a simplified version of GuideLegend$get_layer_key that doesn't
    # demand matching aesthetics

    if (nrow(params$key) < 1L) {
      return(params)
    }
    decor <- lapply(layers, function(layer) {

      key <- params$key
      key$.id <- seq_len(nrow(key))

      static_aes <- layer$aes_params
      static_aes <- static_aes[lengths(static_aes) %in% c(1L, nrow(key))]

      key <- layer$compute_geom_2(key, static_aes, theme)
      key <- modify_list(key, params$override.aes)

      # We're using *non*-computed geom/stat params here.
      # Computing these may demand `key` to have particular constraints.
      key_params <- c(layer$geom_params, layer$stat_params)

      list(
        draw_key = layer$geom$draw_key,
        data     = key,
        params   = key_params
      )
    })

    params$decor <- compact(decor)
    params
  },

  draw = function(self, theme, position = NULL, direction  = NULL,
                  params = self$params) {
    legend <- self$get_layer_key(
      params = params$legend,
      layers = params$layers,
      data   = NULL,
      theme  = theme
    )
    guide <- guide_legend_base()
    guide$draw(
      theme = theme,
      position = position,
      direction = direction,
      params = legend
    )
  }
)

test_that("guide_legend_manual() constructor works as expected", {
  # Empty key results in none guide
  g <- guide_legend_manual(NULL)
  expect_s3_class(g, "GuideNone")

  # Singular layers are expanded to list
  g <- guide_legend_manual("foo", layers = geom_point())
  expect_type(g$params$layers, "list")
  expect_length(g$params$layers, 1L)
  expect_s3_class(g$params$layers[[1L]], "LayerInstance")

  # The reverse legend setting is applied to the key
  g <- guide_legend_manual(c("foo", "bar"), reverse = TRUE)
  expect_identical(
    g$params$legend$key$.label,
    c("bar", "foo")
  )

  # Non-layers are rejected
  expect_snapshot(
    guide_legend_manual("foo", layers = list("invalid input")),
    error = TRUE
  )

  # Recycling rules are applied
  expect_snapshot(
    guide_legend_manual(c("foo", "bar"), colour = c("red", "green", "blue")),
    error = TRUE
  )
})

test_that("guide_legend_manual() can render a legend", {

  p <- ggplot() +
    guides(whatever = guide_legend_manual(
      title = "Foobar",
      label = c("foo", "bar"),
      colour = c("tomato", "dodgerblue"),
      fill = NA,
      layers = list(
        geom_point(shape = c(21L, 19L)),
        geom_col(colour = c("dodgerblue", "tomato"))
      ),
      ncol = 2L
    ))

  vdiffr::expect_doppelganger("manual legend", p)
})

# Constructor -------------------------------------------------------------

#' Guide primitive: brackets
#'
#' This function constructs a brackets [guide primitive][guide-primitives].
#'
#' @param key A [range key][key_range] specification. See more information
#'   in the linked topic.
#' @param bracket A [bracket][bracket_options] by providing one of the
#'  following:
#'  * A bracket `<function>`, such as `bracket_square`.
#'  * A `<character[1]>` naming a bracket function without the
#'  '`bracket_`'-prefix, e.g. `"square"`.
#'  * A two-column `<matrix[n, 2]>` giving line coordinates for a bracket,
#'  like those created by bracket functions, such as `bracket_round()`.
#' @param oob A method for dealing with out-of-bounds (oob) ranges. Can be one
#'  of `"squish"`, `"censor"` or `"none"`.
#' @param drop_zero A `<logical[1]>` whether to drop near-zero width ranges
#'   (`TRUE`, default) or preserve them (`FALSE`).
#' @param pad_discrete A `<numeric[1]>` giving the amount ranges should be
#'   extended when given as a discrete variable. This is applied after
#'   the `drop_zero` setting.
#' @param levels_brackets A list of `<element_line>` objects to customise how
#'   brackets appear at every level.
#' @param levels_text A list of `<element_text>` objects to customise how
#'   text appears at every level.
#' @inheritParams common_parameters
#'
#' @return A `<PrimitiveBracket>` primitive guide that can be used inside other
#'   guides.
#' @family primitives
#' @export
#'
#' @details
#' ## Styling options
#'
#' Below are the [theme][ggplot2::theme] options that determine the styling of
#' this guide, which may differ depending on whether the guide is used in
#' an axis or in a legend context.
#'
#' The possible `{position}` suffixes mentioned below are `x`, `x.top`,
#' `x.bottom`, `y`, `y.left`, `y.right`. The `theta` and `r` position suffixes
#' in \pkg{ggplot2} are *not* obeyed in \pkg{legendry}.
#'
#' | **Theme setting** | **Context** | **Type** | **Description** |
#' | ----------------- | ----------- | -------- | --------------- |
#' | `legendry.bracket` | Both | [`element_line()`] | The bracket lines themselves. |
#' | `legendry.bracket.size` | Both | [`unit()`] | The space (in the orthogonal direction) afforded to a bracket. |
#' | `axis.text.{position}` | Axis | [`element_text()`] | The text over brackets. |
#' | `legend.text` | Legend | [`element_text()`] | The text over brackets. |
#'
#' Styling options *per level* can be set in the `levels_brackets` and
#' `levels_text` arguments. These override theme settings.
#'
#' Styling options *per range* can be set in the [range key][key_range].
#' The `line` and `text` prefixed properties are prioritised for the brackets
#' and text respectively. These override theme settings and 'per level'
#' settings.
#'
#' The context-agnostic alternative to using `theme()` is to use
#' [`theme_guide()`]:
#'
#' ```r
#' primitive_bracket(theme = theme_guide(
#'   bracket = element_line(),
#'   bracket.size = unit(5, "mm")
#' ))
#' ```
#'
#' @examples
#' # A standard plot
#' p <- ggplot(mpg, aes(interaction(drv, year), displ)) +
#'  geom_point()
#'
#' key <- key_range_manual(c(2, 4), c(5, 6), c("A", "B"))
#'
#' # Adding as secondary guides
#' p + guides(
#'   x.sec = primitive_bracket(),
#'   y.sec = primitive_bracket(key = key)
#' )
primitive_bracket <- function(
  key = "range_auto",
  bracket = "line",
  angle = waiver(),
  oob = "squish",
  drop_zero = TRUE,
  pad_discrete = 0.4,
  levels_brackets = NULL,
  levels_text = NULL,
  theme = NULL,
  position = waiver()
) {

  key <- resolve_key(key)
  oob <- arg_match0(oob, c("squish", "censor", "none"))
  check_bool(drop_zero)
  check_number_decimal(pad_discrete, allow_infinite = FALSE)
  check_list_of(
    levels_brackets, element_classes("line", "blank"),
    allow_null = TRUE
  )
  check_list_of(
    levels_text, element_classes("text", "blank"),
    allow_null = TRUE
  )
  bracket <- resolve_bracket(bracket)

  new_guide(
    key = key,
    oob = oob,
    angle = angle,
    drop_zero = drop_zero,
    pad_discrete = pad_discrete,
    bracket = bracket,
    levels_brackets = levels_brackets,
    levels_text = levels_text,
    theme = theme,
    position = position,
    available_aes = c("any", "x", "y", "r", "theta"),
    super = PrimitiveBracket
  )
}

# Class -------------------------------------------------------------------

#' @export
#' @rdname legendry_extensions
#' @format NULL
#' @usage NULL
PrimitiveBracket <- ggproto(
  "PrimitiveBracket", Guide,

  params = new_params(
    key = NULL, oob = "squish", drop_zero = TRUE,
    pad_discrete = 0.4, angle = waiver(), bracket = cbind(c(0.0, 1.0), 0.5),
    levels_text = NULL, levels_brackets = NULL
  ),

  hashables = exprs(key, decor, bracket),

  elements = list(
    position = list(
      text = "axis.text", line = I("legendry.bracket"),
      size = I("legendry.bracket.size")
    ),
    legend = list(
      text = "legend.text", line = I("legendry.bracket"),
      size = I("legendry.bracket.size")
    )
  ),

  extract_key = range_extract_key,

  extract_params = extract_range_params,

  extract_decor = function(scale, aesthetic, position, key, bracket, ...) {
    key <- vec_slice(key, key$.draw)
    n_keys <- nrow(key)
    if (n_keys < 1) {
      return(NULL)
    }
    bracket  <- resolve_bracket(bracket)
    n_vertex <- nrow(bracket)
    decor <- vec_rep_each(key, n_vertex)
    brackets <- vec_rep(bracket, n_keys)

    decor[[aesthetic]] <-
      brackets[, 1L] * (decor$end - decor$start) + decor$start
    decor[["offset"]] <- brackets[, 2L]
    decor[["group"]] <- rep(seq_len(n_keys), each = n_vertex)
    decor[setdiff(names(decor), c("start", "end"))]
  },

  transform = function(self, params, coord, panel_params) {
    params$key <-
      transform_key(params$key, params$position, coord, panel_params)
    params$bbox  <- panel_params$bbox %||%
      list(x = c(0.0, 1.0), y = c(0.0, 1.0))
    params$decor <-
      transform_bracket(params$decor, params$position, coord, panel_params)
    params
  },

  setup_params = setup_range_params,

  setup_elements = primitive_setup_elements,

  override_elements = function(params, elements, theme) {
    elements$size <- cm(elements$size)
    elements
  },

  build_bracket = function(key, decor, elements, params) {
    levels   <- unique(c(key$.level, decor$.level))
    nlevels  <- length(levels)
    position <- params$position

    # Recycle custom elements per level to appropriate length
    bracket_levels <- rep0(params$levels_brackets, length.out = nlevels)
    text_levels    <- rep0(params$levels_text,     length.out = nlevels)

    # Justify labels along their ranges
    key <- justify_ranges(key, levels, elements$text, text_levels)

    if (is_theta(position)) {
      add <- if (position == "theta.sec") pi else 0.0
      key <- polar_xy(key, key$r, key$theta + add, params$bbox)
    }

    if (is_blank(elements$line) || is_empty(decor)) {
      decor <- vec_slice(decor, 0L)
    } else if (position %in% .trbl) {
      offset  <- decor$offset
      offset  <- if (position %in% .trbl[1L:2L]) 1.0 - offset else offset
      decor$x <- switch(position, left = , right = offset, decor$x)
      decor$y <- switch(position, top = , bottom = offset, decor$y)
      decor$offset <- 0.0
    }

    offset <- elements$offset
    angle  <- params$angle %|W|% NULL
    size   <- elements$size
    text   <- angle_labels(elements$text, angle, params$position)

    brackets <- vector("list", nlevels)
    labels   <- vector("list", nlevels)

    for (i in seq_len(nlevels)) {

      # Render bracket
      brackets[[i]] <- draw_bracket(
        decor   = vec_slice(decor, decor$.level == levels[[i]]),
        element = combine_elements(bracket_levels[[i]], elements$line),
        size = size, offset = offset, position = position
      )
      offset <- offset + get_size_attr(brackets[[i]])

      # Render text
      labels[[i]] <- draw_labels(
        key     = vec_slice(key, key$.level == levels[[i]]),
        element = combine_elements(text_levels[[i]], text),
        angle = angle, offset = offset, position = position
      )
      offset <- offset + get_size_attr(labels[[i]])
    }
    if (params$position %in% c("top", "left")) {
      brackets <- rev(brackets)
      labels   <- rev(labels)
    }

    list(brackets = brackets, labels = labels)
  },

  measure_grobs = function(grobs, params, elements) {
    labels <- switch(
      params$position,
      top  = , bottom = height_cm(grobs$labels),
      left = , right  =  width_cm(grobs$labels),
      map_dbl(grobs$labels, get_size_attr)
    )
    is_bracket <- as.numeric(!map_lgl(grobs$brackets, is_zero))
    bracket <- is_bracket * elements$size
    list(brackets = bracket, labels = labels)
  },

  draw = function(self, theme, position = NULL, direction = NULL,
                  params = self$params) {

    params <- replace_null(params, position = position, direction = direction)
    params <- self$setup_params(params)

    elems    <- self$setup_elements(params, self$elements, theme)
    elems    <- self$override_elements(params, elems, theme)
    brackets <- self$build_bracket(params$key, params$decor, elems, params)
    size     <- self$measure_grobs(brackets, params, elems)

    if (params$position %in% c("top", "left")) {
      grobs <- vec_interleave(brackets$labels, brackets$brackets)
      size  <- vec_interleave(size$labels, size$brackets)
    } else {
      grobs <- vec_interleave(brackets$brackets, brackets$labels)
      size  <- vec_interleave(size$brackets, size$labels)
    }

    primitive_grob(
      grob = grobs,
      size = unit(size, "cm"),
      position = params$position,
      name = "bracket"
    )
  }
)

# Helpers -----------------------------------------------------------------

draw_bracket <- function(decor, element, size, offset, position) {
  if (nrow(decor) < 2L) {
    return(zeroGrob())
  }
  x <- unit(decor$x, "npc")
  y <- unit(decor$y, "npc")

  if (is_theta(position)) {
    offset <- (1.0 - decor$offset) * size + offset
    x <- x + unit(sin(decor$theta) * offset, "cm")
    y <- y + unit(cos(decor$theta) * offset, "cm")
  }

  id <- new_rle(decor$group)
  props <- element_key_properties(vec_slice(decor, id$start), "line")

  grob <- inject(element_grob(
    element, x = x, y = y,
    id.lengths = id$times,
    !!!props
  ))
  if (!is_blank(element)) {
    attr(grob, "size") <- size
  }
  grob
}

test_that("compose_sandwich can compose a legend", {

  sandwich <- compose_sandwich(
    middle = guide_colourbar(
      theme = theme(text = element_text(colour = "limegreen"))
    ),
    text = guide_axis_base(
      theme = theme(text = element_text(colour = "tomato"))
    ),
    opposite = primitive_bracket(
      key = key_range_manual(
        c(10.0, 20.0),
        c(25.0, 30.0),
        c("A", "B")
      ),
      theme = theme(text = element_text(colour = "dodgerblue"))
    )
  )


  p <- ggplot(mpg, aes(displ, hwy)) +
    geom_point(aes(colour = cty)) +
    scale_colour_viridis_c(
      guide = sandwich
    )

  vdiffr::expect_doppelganger(
    "vertical sandwich", p + theme(
      legend.text.position = "right",
      legend.position = "right"
    )
  )

  vdiffr::expect_doppelganger(
    "vertical sandwich flipped", p + theme(
      legend.text.position = "left",
      legend.position = "right"
    )
  )

  vdiffr::expect_doppelganger(
    "horizontal sandwich", p + theme(
      legend.text.position = "bottom",
      legend.position = "bottom"
    )
  )

  vdiffr::expect_doppelganger(
    "horizontal sandwich flipped", p + theme(
      legend.text.position = "top",
      legend.position = "bottom"
    )
  )
})

# Constructor -------------------------------------------------------------

#' Guide primitives: segments
#'
#' This function constructs a [guide primitive][guide-primitives].
#'
#' @param key A [segment key][key_segments] specification. See more information
#'   in the linked topic. Alternatively, an object of class
#'   [`<hclust>`][stats::hclust] that automatically invokes `key_dendro()`.
#' @param space Either a [`<unit>`][grid::unit()] or [`<rel>`][ggplot2::rel()]
#'   object of length 1 determining the space allocated in the orthogonal
#'   direction. When the `space` argument is of class `<rel>` (default) the
#'   base size is taken from the tick length theme setting.
#' @param vanish Only relevant when the guide is used in the secondary theta
#'   position: a `<logical[1]>` on whether the continue to draw the segments
#'   until they meed in the center (`TRUE`) or strictly observe the `space`
#'   setting (`FALSE`).
#' @inheritParams common_parameters
#'
#' @return A `<PrimitiveSegments>` primitive guide that can be used inside other
#'   guides.
#' @family primitives
#' @export
#'
#' @details
#' ## Styling options
#'
#' Below are the [theme][ggplot2::theme] options that determine the styling of
#' this guide, which may differ depending on whether the guide is used in
#' an axis or in a legend context.
#'
#' The possible `{position}` suffixes mentioned below are `x`, `x.top`,
#' `x.bottom`, `y`, `y.left`, `y.right`. The `theta` and `r` position suffixes
#' in \pkg{ggplot2} are *not* obeyed in \pkg{legendry}.
#'
#' | **Theme setting** | **Context** | **Type** | **Description** |
#' | ----------------- | ----------- | -------- | --------------- |
#' | `axis.ticks.{position}` | Axis | [`element_line()`] | The line segments. |
#' | `axis.ticks.length.{position}` | Axis | [`unit()`] | Basis for the `space` argument |
#' | `legend.ticks` | Legend | [`element_line()`] | The line segments |
#' | `legend.ticks.length` | Legend | [`unit()`] | Basis for the `space` argument |
#'
#' Styling options *per segment* can be set in the [segment key][key_segments].
#' The `line` prefixed properties are prioritised for segments. These override
#' theme settings.
#'
#' The context-agnostic alternative to using `theme()` is to use
#' [`theme_guide()`]:
#'
#' ```r
#' primitive_segments(theme = theme_guide(
#'   ticks = element_line(),
#'   ticks.length = unit(5, "mm")
#' ))
#' ```
#'
#' @examples
#' # Building a key
#' key <- key_segment_manual(
#'   value     = c(1.6, 1.6, 3.4, 5.2),
#'   value_end = c(7.0, 7.0, 3.4, 5.2),
#'   oppo      = c(1.0, 2.0, 0.0, 0.0),
#'   oppo_end  = c(1.0, 2.0, 3.0, 3.0)
#' )
#'
#' # Using the primitive in a plot
#' ggplot(mpg, aes(displ, hwy)) +
#'   geom_point() +
#'   scale_x_continuous(
#'     guide = primitive_segments(key = key)
#'   )
primitive_segments <- function(key = NULL, space = rel(10.0), vanish = FALSE,
                               theme = NULL, position = waiver()) {
  check_unit(space, allow_rel = TRUE)

  if (inherits(key, "hclust")) {
    key <- key_dendro(key)
  }

  new_guide(
    key = key,
    space = space,
    vanish = vanish,
    theme = theme,
    position = position,
    available_aes = c("any", "x", "y", "r", "theta"),
    super = PrimitiveSegments
  )
}

# Class -------------------------------------------------------------------

PrimitiveSegments <- ggproto(
  "PrimitiveSegments", Guide,

  params = new_params(key = NULL, space = rel(10.0), vanish = FALSE),

  hashables = exprs(key),

  elements = list(
    position = list(line = "axis.ticks",   size = "axis.ticks.length"),
    legend   = list(line = "legend.ticks", size = "legend.ticks.length")
  ),

  extract_key = segment_extract_key,

  extract_params = primitive_extract_params,

  transform = function(self, params, coord, panel_params) {
    key <- params$key
    position <- params$position
    aesthetic <- params$aesthetic
    mult <- 10.0

    opposite <- setdiff(c("x", "y"), aesthetic)
    is_radius <- "theta.range" %in% names(panel_params) & !is_theta(position)
    if (is_radius) {
      range <- panel_params$r.range
      value <- squish_infinite(key[[aesthetic]], range)
      value <- rescale(value, panel_params$inner_radius, range)
      value <- rescale(value + 0.5, from = panel_params$bbox$x)
      key[[aesthetic]] <- value
      if (position == "left") {
        key[[opposite]] <- 1.0 - key[[opposite]]
      }
      if (aesthetic == "x") {
        key <- rename(key, c("x", "y"), c("y", "x"))
      }
      params$key <- key
      return(params)
    }

    range <- switch(
      position,
      top = , bottom = "y.range",
      left = , right = "x.range",
      theta = , theta.sec = "r.range"
    )
    range <- panel_params[[range]]

    margin_lower <- function(value) range[1L] - value * diff(range) / mult
    margin_upper <- function(value) range[2L] + value * diff(range) / mult

    key[[opposite]] <- switch(
      position,
      bottom = , theta.sec = , left = margin_lower(key[[opposite]]),
      top = , theta = , right = margin_upper(key[[opposite]]),
      key[[opposite]]
    )

    key <- coord_munch(coord, key, panel_params)

    key[[opposite]] <- switch(
      position,
      left = , bottom = key[[opposite]] * mult + 1.0,
      top = , right   = key[[opposite]] * mult - mult,
      key[[opposite]]
    )

    if (!is_theta(position)) {
      return(vec_assign(params, "key", list(key)))
    }

    radius <- panel_params$inner_radius
    if (position == "theta") {
      key$adjust <- (key$r - radius[2L]) * (2.0 * mult / radius[2L])
      key$r <- radius[2L]
    } else {
      key$adjust <- (key$r - radius[1L]) / diff(radius) * -mult
      key$r <- radius[1L]
    }
    bbox <- panel_params$bbox
    key$x <- rescale(key$r * sin(key$theta) + 0.5, from = bbox$x)
    key$y <- rescale(key$r * cos(key$theta) + 0.5, from = bbox$y)
    params$center <- c(rescale(0.5, from = bbox$x), rescale(0.5, from = bbox$y))

    params$key <- key
    params
  },

  setup_params = primitive_setup_params,
  setup_elements = primitive_setup_elements,
  override_elements = function(params, elements, theme) {
    size <- params$space
    if (is_rel(size)) {
      size <- unclass(size) * elements$size
    }
    elements$size <- convertUnit(size, "cm", valueOnly = TRUE)
    elements
  },

  build_ticks = function(key, elements, params, position = params$position) {
    if (is_empty(key)) {
      return(zeroGrob())
    }

    x <- unit(key$x, "npc")
    y <- unit(key$y, "npc")

    if (is_theta(position)) {
      vanish <- position == "theta.sec" && isTRUE(params$vanish)
      theta <- key$theta + as.numeric(position == "theta.sec") * pi
      offset <- elements$offset

      if (!vanish) {
        offset <- key$adjust * elements$size + offset
      }
      if (any(offset != 0.0)) {
        x <- x + unit(sin(theta) * offset, "cm")
        y <- y + unit(cos(theta) * offset, "cm")
      }
      if (vanish) {
        cx <- unit(params$center[1L] * key$adjust, "npc")
        cy <- unit(params$center[2L] * key$adjust, "npc")
        x <- (x * (1.0 - key$adjust)) + cx
        y <- (y * (1.0 - key$adjust)) + cy
      }
    }

    group <- new_rle(key$group)
    props <- element_key_properties(vec_slice(key, group$start), "line")

    inject(element_grob(
      elements$line, x = x, y = y,
      id.lengths = group$times,
      !!!props
    ))
  },

  draw = function(self, theme, position = NULL, direction = NULL,
                  params = self$params) {

    params <- replace_null(params, position = position, direction = direction)
    params <- self$setup_params(params)

    elems <- self$setup_elements(params, self$elements, theme)
    elems <- self$override_elements(params, elems, theme)
    segments <- self$build_ticks(params$key, elems, params)

    primitive_grob(
      grob = segments,
      size = unit(elems$size, "cm"),
      position = params$position,
      name = "segments"
    )
  }
)

test_that("primitive_segments works as axis", {

  base <- ggplot(mpg, aes(displ, hwy)) +
    geom_blank() +
    theme_test() +
    theme(
      panel.background = element_rect(fill = NA, colour = "grey80"),
      panel.grid.major = element_line(colour = "grey90"),
      panel.border = element_blank(),
      axis.line = element_line()
    )

  hkey <- key_segment_manual(
    value     = c(2.0, 4.0, 6.0, 1.6, 1.6),
    value_end = c(2.0, 4.0, 6.0, 7.0, 7.0),
    oppo      = c(0.0, 0.0, 0.0, 1.0, 2.0),
    oppo_end  = c(3.0, 3.0, 3.0, 1.0, 2.0)
  )

  vkey <- key_segment_manual(
    value     = c(20.0, 30.0, 30.0, 40.0),
    value_end = c(30.0, 40.0, 20.0, 30.0),
    oppo      = 0.0, oppo_end = 1.0
  )

  p <- base +
    guides(
      x     = primitive_segments(key = hkey),
      y     = primitive_segments(key = vkey),
      x.sec = primitive_segments(key = hkey),
      y.sec = primitive_segments(key = vkey)
    )

  vdiffr::expect_doppelganger("primitive_segments cartesian", p)

  p <- base +
    coord_radial(start = 0.25 * pi, end = 1.75 * pi, inner.radius = 0.5) +
    guides(
      theta     = primitive_segments(key = hkey),
      r         = primitive_segments(key = vkey),
      theta.sec = primitive_segments(key = hkey),
      r.sec     = primitive_segments(key = vkey)
    )

  vdiffr::expect_doppelganger("primitive_segments radial", p)

})

test_that("primitive_segments works as a legend", {

  key <- key_segment_manual(
    value     = c(100.0, 200.0, 200.0, 300.0),
    value_end = c(200.0, 300.0, 100.0, 200.0),
    oppo = 0.0, oppo_end = 1.0
  )

  p <- ggplot(mtcars) +
    aes(x = disp, y = mpg, colour = hp, fill = hp) +
    geom_point() +
    guides(
      colour = primitive_segments(key = key),
      fill = primitive_segments(key = key, position = "bottom")
    )

  vdiffr::expect_doppelganger("primitive_segments legend", p)
})

#' Compose guides as a sandwich
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' This guide composition has a middle guide flanked by two parallel guides.
#'
#' @param key A [standard key][key_standard] specification. The key is shared
#'   among all guides that have `NULL` keys themselves. See more information
#'   in the linked topic.
#' @param middle Guide to use as the middle guide. Each guide can be specified
#'   as one of the following:
#'   * A `<Guide>` class object.
#'   * A `<function>` that returns a `<Guide>` class object.
#'   * A `<character>` naming such a function, without the `guide_` or
#'   `primitive_` prefix.
#' @param text,opposite Guides to use at the `legend.text.position` location
#'   and on the opposite side of the `middle` guide respectively. Guide
#'   specification is the same as in the `middle` argument.
#' @param suppress_labels A `<character>` vector giving any of `"text"` and
#'   `"opposite"` for the parallel guides. The guide(s) listed here will not
#'   draw labels if they support a label suppression mechanism.
#' @inheritParams compose_crux
#'
#' @return A `<ComposeSandwich>` guide object.
#' @export
#' @family composition
#'
#' @details
#' The sandwich composition is effectively the same as a
#' [crux composition][compose_crux] lacking two opposing arms.
#'
#' ## Styling options
#'
#' Below are the [theme][ggplot2::theme] options that determine the styling of
#' this guide.
#'
#' | **Theme setting** | **Type** | **Description** |
#' | ----------------- | -------- | --------------- |
#' | `legend.title` | [`element_text()`] | The title of the legend. |
#' | `legend.title.position` | `<character[1]>` | One of `"top"`, `"right"`, `"bottom"` or `"left"`. |
#' | `legend.text.position` | `<character[1]>` | One of `"top"`, `"right"`, `"bottom"` or `"left"`. |
#' | `legend.margin` | [`margin()`] | Padding around the legend. |
#' | `legend.background` | [`element_rect()`] | Background of the legend. |
#'
#' There are no further styling options.
#'
#' The context-agnostic alternative to using `theme()` is to use
#' [`theme_guide()`]:
#'
#' ```r
#' compose_sandwich(theme = theme_guide(
#'   title = element_text(),
#'   title.position = "top",
#'   text.position = "right",
#'   margin = margin(5),
#'   background = element_rect()
#' ))
#' ```
#'
#' @examples
#' # A standard plot with a sandwich guide
#' ggplot(mpg, aes(displ, hwy)) +
#'   geom_point(aes(colour = cty)) +
#'   guides(colour = compose_sandwich(
#'     middle = "colourbar",
#'     text = "axis_base",
#'     opposite = primitive_bracket(key = key_range_manual(
#'       start = c(10, 20), end = c(25, 30), name = c("A", "B")
#'     ))
#'   ))
compose_sandwich <- function(
  key = key_auto(),
  middle = gizmo_barcap(),
  text = "none",
  opposite = "none",
  args = list(),
  suppress_labels = "opposite",
  complete = TRUE,
  theme = NULL,
  theme_defaults = list(),
  reverse = FALSE,
  order = 0L,
  title = waiver(),
  position = waiver(),
  available_aes = NULL
) {
  new_compose(
    guides = list(
      middle = middle,
      text = text,
      opposite = opposite
    ),
    title = title,
    key = key,
    args = args,
    complete = complete,
    reverse = reverse,
    suppress_labels = suppress_labels,
    available_aes = available_aes,
    order = order,
    theme = theme,
    theme_defaults = theme_defaults,
    position = position,
    name = "sandwich_composition",
    super = ComposeSandwich
  )
}

# Class -------------------------------------------------------------------

#' @export
#' @rdname legendry_extensions
#' @format NULL
#' @usage NULL
ComposeSandwich <- ggproto(
  "ComposeSandwich", Compose,

  params = c(
    Compose$params,
    list(
      complete = FALSE,
      theme_defaults = list(),
      reverse = FALSE,
      suppress_labels = "opposite"
    )
  ),

  draw = function(self, theme, position = NULL, direction = NULL,
                  params = self$params) {

    direction <- params$direction %||% direction
    text_position <- calc_element("legend.text.position", theme) %||%
      switch(direction, horizontal = "bottom", vertical = "right")

    valid_position <- switch(
      direction, horizontal = c("bottom", "top"), vertical = c("left", "right")
    )
    check_position(text_position, valid_position, arg = "legend.text.position")

    theme <- theme + params$theme
    theme <- apply_theme_defaults(theme, params$theme_defaults)

    opposite <- opposite_position(text_position)
    if ("opposite" %in% params$suppress_labels) {
      params$guide_params$opposite$draw_label <- FALSE
    }
    if ("text" %in% params$suppress_labels) {
      params$guide_params$text$draw_label <- FALSE
    }

    old <- c("middle", "text", "opposite")
    new <- c("centre", text_position, opposite)
    params$guides <- rename(params$guides, old, new)
    params$guide_params <- rename(params$guide_params, old, new)

    ComposeCrux$draw(theme, position, direction, params)
  }
)

# Constructor -------------------------------------------------------------

#' Custom colour bar guide
#'
#' Similar to [`guide_colourbar()`][ggplot2::guide_colourbar], this guide
#' displays continuous `colour` or `fill` aesthetics. It has additional options
#' to display caps at the end of the bar, depending on out-of-bounds values.
#'
#' @param first_guide,second_guide Guides to flank the colour bar. Each guide
#'   can be specified using one of the following:
#'   * A `<Guide>` class object.
#'   * A `<function>` that returns a `<Guide>` class object.
#'   * A `<character>` naming such a function, without the `guide_` or
#'   `primitive_` prefix.
#'
#' The `first_guide` will be placed at the location specified by the
#' `legend.text.position` theme setting. The `second_guide` will be placed
#' opposite that position. When `second_guide` has a label suppression
#' mechanism, no labels will be drawn for that guide.
#' @param nbin A positive `<integer[1]>` determining how many colours to use
#'   for the colour gradient.
#' @param vanilla A `<logical[1]>` whether to have the default style match
#'   the vanilla `guide_colourbar()` (`TRUE`) or take the theme
#'   verbatim (`FALSE`).
#' @param suppress_labels A `<character>` vector giving any of `"first"` and
#'   `"second"` for the parallel guides. The guide(s) listed here will not
#'   draw labels if they support a label suppression mechanism.
#' @inheritParams gizmo_barcap
#' @inheritParams compose_sandwich
#'
#' @details
#' As colours are always rendered as gradients, it is important to use a
#' graphics device that can render these. This can be checked by using
#' [`check_device("gradients")`][ggplot2::check_device].
#'
#' ## Styling options
#'
#' Because this guide is pure composite guide, the [theme][ggplot2::theme]
#' options that govern the styling are determined by its constituents. They are
#' linked below so you can find their 'Styling options' sections. Note that
#' `guide_axis_base()` is just a default that can be swapped out.
#'
#' | **Constituent** | **Description** |
#' | ------------- | --------------- |
#' | [`compose_sandwich`] | Combines the bar with two side-guides. |
#' | [`gizmo_barcap()`] | Makes up the colour bar. |
#' | [`guide_axis_base()`] | Makes up the tick marks and labels. |
#'
#' Styling options *per break* can be set in the [standard key][key_standard].
#' These override theme settings.
#'
#' The context-agnostic alternative to using `theme()` is to use
#' [`theme_guide()`]:
#'
#' ```r
#' guide_colbar(theme = theme_guide(
#'   # Composition settings
#'   title = element_text(),
#'   title.position = "top",
#'   text.position = "right",
#'   margin = margin(5),
#'   background = element_rect(),
#'
#'   # Bar settings
#'   frame = element_rect(),
#'   key.width = unit(5, "mm")
#'   key.height = unit(5, "cm")
#'
#'   # Common options for `guide_axis_base()`
#'   line = element_line(),
#'   text = element_text(),
#'   ticks = element_line(),
#'   ticks.length = unit(5, "mm"),
#' ))
#' ```
#'
#' @return A `<Guide>` object
#' @export
#' @family standalone guides
#'
#' @examples
#' # A standard plot
#' p <- ggplot(mpg, aes(displ, hwy)) +
#'   geom_point(aes(colour = cty))
#'
#' # The colourbar shows caps when values are out-of-bounds (oob)
#' p + scale_colour_viridis_c(
#'   limits = c(10, NA),
#'   guide = "colbar"
#' )
#'
#' # It also shows how oob values are handled
#' p + scale_colour_viridis_c(
#'   limits = c(10, NA), oob = scales::oob_squish,
#'   guide = "colbar"
#' )
#'
#' # Adjusting the type of cap
#' p + scale_colour_viridis_c(
#'   limits = c(10, 30), oob = scales::oob_squish,
#'   guide = guide_colbar(shape = "round")
#' )
#'
#' # One-sided ticks
#' p + scale_colour_viridis_c(
#'   guide = guide_colbar(second_guide = "none")
#' )
#'
#' # Colour bar with minor breaks
#' p + scale_colour_viridis_c(
#'   minor_breaks = scales::breaks_width(1),
#'   guide = guide_colbar(key = "minor")
#' )
#'
#' # Using log ticks on a colourbar
#' ggplot(msleep, aes(sleep_total, sleep_rem)) +
#'   geom_point(aes(colour = bodywt), na.rm = TRUE) +
#'   scale_colour_viridis_c(
#'     transform = "log10",
#'     guide = guide_colbar(key = "log")
#'   )
guide_colbar <- function(
  title = waiver(),
  key = "auto",
  first_guide  = "axis_base",
  second_guide = first_guide,
  shape = "triangle",
  size = NULL,
  show = NA,
  nbin = 15L,
  alpha = NA,
  reverse = FALSE,
  suppress_labels = "second",
  oob = scales::oob_keep,
  theme = NULL,
  vanilla = TRUE,
  position = waiver(),
  available_aes = c("colour", "fill")
) {

  bar <- gizmo_barcap(
    key = key_sequence(nbin),
    shape = shape, size = size, show = show, alpha = alpha,
    oob = oob, theme = theme
  )

  suppress_labels <- recode(
    suppress_labels,
    old = c("first", "second"),
    new = c("text", "opposite")
  )

  defaults <- if (isTRUE(vanilla)) vanilla_colourbar_theme() else NULL

  compose_sandwich(
    key = key,
    middle = bar,
    text = first_guide,
    opposite = second_guide,
    reverse = reverse,
    suppress_labels = suppress_labels,
    complete = TRUE,
    title = title,
    theme = theme,
    theme_defaults = defaults,
    position = position,
    available_aes = available_aes
  )
}

vanilla_colourbar_theme <- function(...) {
  theme(
    legend.axis.line = element_blank(),
    legend.ticks = element_line(colour = "white", linewidth = 0.5 / .pt),
    legend.ticks.length = rel(-1.0),
    legend.frame = element_blank(),
    ...
  )
}

# Cap types ---------------------------------------------------------------

#' Cap options
#'
#' These functions construct various sorts of caps. They construct a matrix
#' that can be supplied as the `shape` argument in [`gizmo_barcap()`].
#'
#' @param n An `<integer[n]>` number of points to use for the cap.
#'
#' @return A `<matrix[n, 2]>` with coordinates for points on the brackets.
#' @name cap_options
#'
#' @details
#' When designing custom cap shapes, the expectation is that the first point
#' starts at the `(0, 0)` coordinate and the last point ends at the `(0, 1)`
#' coordinate. The first column follows the orthogonal direction of the bar
#' whereas the second column follows the direction of the bar.
#'
#' @examples
#' plot(cap_arch(), type = 'l')
NULL

#' @export
#' @describeIn cap_options An equilateral triangle with `n = 3` points.
cap_triangle <- function() {
  cbind(
    c(0.0, 0.5, 1.0), # x
    c(0.0, sqrt(3.0) / 2.0, 0.0)# y
  )
}

#' @export
#' @describeIn cap_options A semicircle.
cap_round <- function(n = 100L) {
  t <- seq(1.0, 0.0, length.out = n) * pi
  cbind(
    cos(t) * 0.5 + 0.5,
    sin(t) * 0.5
  )
}

#' @export
#' @describeIn cap_options Two circular arcs forming an equilateral Gothic arch.
cap_arch <- function(n = 100L) {
  half_n <- round(n / 2L)
  t <- seq((1.0 * pi) / 3.0, 0.0, length.out = half_n)
  cbind(
    c(1.0 - rev(cos(t)), cos(t)),
    c(rev(sin(t)), sin(t))
  )
}

#' @export
#' @describeIn cap_options Four circular arcs forming an 'ogee' arch.
cap_ogee <- function(n = 100L) {
  quart_n <- round(n / 4L)
  t <- seq(0.0, 1.0 / 3.0, length.out = quart_n) * pi
  top <- sqrt(3.0)

  x <- c(0.0, cos(t + pi) + 1.0, rev(cos(t)), -cos(t) + 2.0, rev(cos(t)) + 1.0)
  y <- c(0.0, -sin(t + pi), top - rev(sin(t)), top - sin(t), rev(sin(t)))
  cbind(x / 2.0, y / 2.0)
}

#' @export
#' @describeIn cap_options No cap.
cap_none <- function() {
  cbind(c(0.0, 1.0), c(0.0, 0.0))
}

# Helpers -----------------------------------------------------------------

resolve_cap_shape <- function(x, arg = caller_arg(x), call = caller_env()) {
  if (is.character(x)) {
    x <- paste0("cap_", x)
    fun <- find_global(x, env = global_env(), mode = "function")
    if (!is.function(fun)) {
      cli::cli_abort("Cannot find function: {.fn {x}}", call = call)
    }
    x <- fun
  }
  if (is.function(x)) {
    msg <- "{.arg {arg}} must return a {type}, not {obj_type_friendly(x)}."
    x <- x()
  } else {
    msg <- "{.arg {arg}} must be a {type}, not {obj_type_friendly(x)}"
  }
  if (is.matrix(x) && ncol(x) == 2L && nrow(x) > 1L) {
    return(x)
  }
  type <- as_cli("a {.cls matrix}")
  if (ncol(x) != 2L) {
    type <- as_cli("a {.cls matrix} with 2 columns")
    msg <- c(msg, "The provided {.arg {arg}} has {ncol(x)} column{?s}.")
  }
  if (nrow(x) < 2L) {
    type <- as_cli("a {.cls matrix} with 2 or more rows")
    msg <- c(msg, "The provided {.arg {arg}} has {nrow(x)} row{?s}.")
  }
  cli::cli_abort(msg, call = call)
}

test_that("all cap shapes return matrices", {

  test <- cap_triangle()
  expect_true(is.matrix(test))
  expect_identical(dim(test), c(3L, 2L))

  test <- cap_round()
  expect_true(is.matrix(test))
  expect_identical(dim(test), c(100L, 2L))

  test <- cap_arch()
  expect_true(is.matrix(test))
  expect_identical(dim(test), c(100L, 2L))

  test <- cap_ogee()
  expect_true(is.matrix(test))
  expect_identical(dim(test), c(101L, 2L))

  test <- cap_none()
  expect_true(is.matrix(test))
  expect_identical(dim(test), c(2L, 2L))

})

test_that("resolve_cap_shape returns appropriate errors", {

  expect_error(
    resolve_cap_shape("foobar"),
    "Cannot find function"
  )

  expect_error(
    resolve_cap_shape(mtcars),
    "must be a"
  )

  expect_error(
    resolve_cap_shape(matrix(NA, 2L, 1L)),
    "2 columns"
  )

  expect_error(
    resolve_cap_shape(matrix(NA, 1L, 2L)),
    "2 or more rows"
  )
})

test_that("gizmo_barcap trains correctly", {

  scale <- scale_colour_viridis_c(na.value = "#FF0000")
  scale$train(c(10.0, 30.0))

  guide <- gizmo_barcap(key = key_sequence(11L))

  # No out-of-bounds
  params <- guide$train(guide$params, scale, "colour")
  expect_identical(params$limits, c(10.0, 30.0))
  expect_identical(params$key$.value, seq(10.0, 30.0, length.out = 11L))
  expect_identical(params$key$colour[c(1L, 11L)], c("#440154", "#FDE725"))

  # Upper out-of-bounds
  scale <- scale_colour_viridis_c(limits = c(10.0, 30.0), na.value = "#FF0000")
  scale$train(c(10.0, 40.0))

  params <- guide$train(guide$params, scale, "colour")
  expect_identical(params$limits, c(10.0, 30.0391), tolerance = 1e-3)
  expect_identical(nrow(params$key), 12L)
  expect_identical(params$key$colour[c(1L, 12L)], c("#440154", "#FF0000"))

  # Lower out-of-bounds
  scale <- scale_colour_viridis_c(limits = c(10.0, 30.0), na.value = "#FF0000")
  scale$train(c(0.0, 30.0))

  params <- guide$train(guide$params, scale, "colour")
  expect_identical(params$limits, c(9.98, 30.0), tolerance = 1e-3)
  expect_identical(nrow(params$key), 12L)
  expect_identical(params$key$colour[c(1L, 12L)], c("#FF0000", "#FDE725"))

  # Both out of bounds
  scale <- scale_colour_viridis_c(limits = c(10.0, 30.0), na.value = "#FF0000")
  scale$train(c(0.0, 40.0))

  params <- guide$train(guide$params, scale, "colour")
  expect_identical(params$limits, c(9.98, 30.02), tolerance = 1e-3)
  expect_identical(nrow(params$key), 13L)
  expect_identical(params$key$colour[c(1L, 13L)], c("#FF0000", "#FF0000"))

  # Can ignore out of bounds by setting upper/lower to FALSE
  params <- guide$params
  params$show <- c(FALSE, FALSE)
  params <- guide$train(params, scale, "colour")
  expect_identical(params$limits, c(10.0, 30.0))
  expect_identical(nrow(params$key), 11L)
  expect_identical(params$key$colour[c(1L, 11L)], c("#440154", "#FDE725"))

  # Can include caps even when all in bounds
  scale <- scale_colour_viridis_c(na.value = "#FF0000")
  scale$train(c(10.0, 30.0))

  params <- guide$params
  params$show <- c(TRUE, TRUE)
  params <- guide$train(params, scale, "colour")
  expect_identical(params$limits, c(9.98, 30.02), tolerance = 1e-3)
  expect_identical(nrow(params$key), 13L)
  expect_identical(params$key$colour[c(1L, 13L)], c("#FF0000", "#FF0000"))

  # Can also squish into limits
  params <- guide$params
  params$show <- c(TRUE, TRUE)
  params$oob <- oob_squish
  params <- guide$train(params, scale, "colour")
  expect_identical(params$limits, c(10.0, 30.0))
  expect_identical(nrow(params$key), 13L)
  expect_identical(params$key$colour[c(1L, 13L)], c("#440154", "#FDE725"))
})

test_that("gizmo_barcap shows caps correctly", {
  p <- ggplot(mpg, aes(displ, hwy, colour = cty)) + geom_point() +
    scale_colour_viridis_c(
      oob = oob_squish,
      guide = gizmo_barcap(show = TRUE)
    ) +
    theme(legend.frame = element_rect(colour = "black"),
          legend.key.size = unit(1.0, "cm"))
  b <- ggplot_build(p)

  theme <- b@plot@theme
  theme$legend.key.width  <- theme$legend.key.size
  theme$legend.key.height <- theme$legend.key.size

  guide <- b$plot$guides
  old_params <- guide$params[[1L]]
  guide <- guide$guides[[1L]]

  draw_guide <- function(guide, params, theme) {
    hor <-
      guide$draw(theme, position = "right", direction = "horizontal", params)
    ver <-
      guide$draw(theme, position = "right", direction = "vertical", params)
    gt <- gtable(width = unit(c(1.0, 1.0), "null"), heights = unit(1.0, "null"))
    gtable_add_grob(gt, list(hor, ver), t = 1L, l = 1L:2L)
  }

  params <- old_params
  gt <- draw_guide(guide, params, theme)
  vdiffr::expect_doppelganger("triangle-cap", gt)

  params$size <- unit(2.0, "cm")
  gt <- draw_guide(guide, params, theme)
  vdiffr::expect_doppelganger("triangle-cap-long", gt)

  params$size <- NULL
  params$shape <- cap_round(n = 30L)
  gt <- draw_guide(guide, params, theme)
  vdiffr::expect_doppelganger("round-cap", gt)

  params$size <- unit(2.0, "cm")
  params$shape <- cap_round(n = 30L)
  gt <- draw_guide(guide, params, theme)
  vdiffr::expect_doppelganger("round-cap-long", gt)

  params$size <- NULL
  params$shape <- cap_arch(n = 30L)
  gt <- draw_guide(guide, params, theme)
  vdiffr::expect_doppelganger("arch-cap", gt)

  params$size <- unit(2.0, "cm")
  params$shape <- cap_arch(n = 30L)
  gt <- draw_guide(guide, params, theme)
  vdiffr::expect_doppelganger("arch-cap-long", gt)

  params$size <- NULL
  params$shape <- cap_ogee(n = 30L)
  gt <- draw_guide(guide, params, theme)
  vdiffr::expect_doppelganger("ogee-cap", gt)

  params$size <- unit(2.0, "cm")
  params$shape <- cap_ogee(n = 30L)
  gt <- draw_guide(guide, params, theme)
  vdiffr::expect_doppelganger("ogee-cap-long", gt)

})

# Constructor -------------------------------------------------------------

#' Custom axis guide
#'
#' This axis guide is a visual representation of position scales and can
#' represent the `x`, `y`, `theta` and `r` aesthetics. It differs from
#' [`guide_axis()`][ggplot2::guide_axis] in that it can accept custom keys
#' and is can act as an axis for [`coord_radial()`][ggplot2::coord_radial] like
#' [`guide_axis_theta()`][ggplot2::guide_axis_theta].
#'
#' @param key
#' A [standard key][key_standard] specification. Defaults to
#' [`key_auto()`]. See more information in the linked topic and the 'Details'
#' section.
#' @param subtitle Passed on to [`primitive_title(title)`][primitive_title].
#'   Follow the linked topic for more details.
#' @inheritParams primitive_labels
#' @inheritParams primitive_line
#' @inheritParams primitive_ticks
#' @inheritParams compose_stack
#'
#' @details
#' Under the hood, this guide is a [stack composition][compose_stack] of a
#' [line][primitive_line], [ticks][primitive_ticks] and
#' [labels][primitive_labels] primitives.
#'
#' To set minor ticks, use `key = "minor"`, or use the `type` argument in
#' `key_manual()` or `key_map()`.
#'
#' To use this as a logarithmic axis, set `key = "log"`.
#'
#' ## Styling options
#'
#' Because this guide is pure composite guide, the [theme][ggplot2::theme]
#' options that govern the styling are determined by its constituents. They are
#' linked below so you can find their 'Styling options' sections.
#'
#' | **Primitive** | **Description** |
#' | ------------- | --------------- |
#' | [`compose_stack`] | Stacks the lines, tick marks and labels. |
#' | [`primitive_line()`] | Makes up the axis line. |
#' | [`primitive_ticks()`] | Makes up the tick marks. |
#' | [`primitive_labels()`] | Makes up the labels. |
#'
#' Styling options *per break* can be set in the [standard key][key_standard].
#' These override theme settings.
#'
#' The context-agnostic alternative to using `theme()` is to use
#' [`theme_guide()`]:
#'
#' ```r
#' guide_axis_base(theme = theme_guide(
#'   # Common options
#'   line = element_line(),
#'   text = element_text(),
#'   ticks = element_line(),
#'   ticks.length = unit(5, "mm"),
#'
#'   # Niche options below
#'   minor.ticks = element_line(),
#'   minor.ticks.length = unit(5, "mm"),
#'   mini.ticks = element_line(),
#'   mini.ticks.length = unit(5, "mm"),
#' ))
#' ```
#'
#' @return A `<Guide>` object.
#' @export
#' @family standalone guides
#'
#' @examples
#' # A standard plot with custom keys
#' p <- ggplot(mpg, aes(displ, hwy)) +
#'   geom_point() +
#'   scale_x_continuous(
#'     guide = guide_axis_base(key = key_minor())
#'   ) +
#'   scale_y_continuous(
#'     guide = guide_axis_base(key = key_manual(c(20, 25, 30, 40)))
#'   )
#' p
#'
#' # Is translated to theta axis without fuss
#' p + coord_radial()
#'
#' # To use as logarithmic axis:
#' ggplot(msleep, aes(bodywt, brainwt)) +
#'   geom_point(na.rm = TRUE) +
#'   scale_x_continuous(
#'     transform = "log10",
#'     guide = guide_axis_base("log")
#'   )
guide_axis_base <- function(
  key = NULL, title = waiver(), subtitle = NULL, theme = NULL,
  n.dodge = 1L, check.overlap = FALSE, angle = waiver(),
  cap = "none", bidi = FALSE, order = 0L, position = waiver()
) {
  theme <- theme %||% theme()
  theme$legendry.guide.spacing <-
    theme$legendry.guide.spacing %||% unit(0.0, "cm")

  guides <- list(
    primitive_line(cap = cap, position = position),
    primitive_ticks(bidi = bidi, position = position),
    primitive_labels(
      angle = angle, n.dodge = n.dodge, check.overlap = check.overlap
    )
  )

  if (!is.null(subtitle)) {
    guides <- c(guides, list(primitive_title(subtitle)))
  }

  compose_stack(
    !!!guides,
    key = key, side.titles = NULL, drop = 3L,
    title = title, theme = theme, order = order,
    available_aes = c("any", "x", "y", "r", "theta"),
    position = position
  )
}

test_that("guide_axis_base contains all primitive parameters", {
  fmls <- fn_fmls_names(guide_axis_base)
  expect_in(fn_fmls_names(primitive_line), fmls)
  expect_in(fn_fmls_names(primitive_labels), fmls)
  expect_in(fn_fmls_names(primitive_ticks), fmls)
})

test_that("guide_axis_base looks good as axis", {

  base <- ggplot(msleep, aes(bodywt, awake)) +
    geom_blank() +
    scale_x_continuous(trans = "log10") +
    theme_test() +
    theme(
      panel.background = element_rect(fill = NA, colour = "grey80"),
      panel.grid.major = element_line(colour = "grey90"),
      panel.border = element_blank(),
      axis.line = element_line(),
      axis.ticks.length = unit(5.5, "pt")
    )

  p <- base + guides(
    x = guide_axis_base("log", angle = 0.0),
    x.sec = guide_axis_base("minor", subtitle = "subtitle"),
    y = guide_axis_base(key = key_manual(c(5.0, 6.0, 7.0))),
    y.sec = guide_axis_base(key = key_manual(c(5.0, 20.0, 15.0),
                                             label = c("A", "B", "C")))
  )

  vdiffr::expect_doppelganger("guide_axis_base cartesian", p)

  p <- base +
    coord_radial(start = 0.25 * pi, end = 1.75 * pi, inner.radius = 0.5) +
    guides(
      theta = guide_axis_base("log", angle = 0.0),
      theta.sec = guide_axis_base("minor"),
      r = guide_axis_base(key = key_manual(c(5.0, 7.0, 9.0)), angle = 0.0),
      r.sec = guide_axis_base(
        key = key_manual(c(5.0, 20.0, 15.0), label = c("A", "B", "C"))
      )
    )

  vdiffr::expect_doppelganger("guide_axis_base radial", p)
})

# Constructor -------------------------------------------------------------

#' Guide gizmo: custom grob
#'
#' This guide displays a user-provided grob.
#'
#' @param grob A [`<grob>`][grid::grob] to display.
#' @param width,height A [`<unit[1]>`][grid::unit] setting the allocated
#'   width and height of the the grob respectively.
#' @param hjust,vjust A `<numeric[1]>` between 0 and 1 setting the horizontal
#'   and vertical justification of the grob when used as a guide for the `x`
#'   and `y` aesthetics.
#' @param position Where this guide should be drawn: one of `"top"`, `"bottom"`,
#'   `"left"`, or `"right"`.
#'
#' @details
#' ## Styling options
#'
#' There are no `theme()` styling options for `gizmo_grob()`.
#'
#'
#' @return A `<GizmoGrob>` object.
#' @export
#' @family gizmos
#'
#' @examples
#' circle <- grid::circleGrob()
#'
#' # A standard plot with grob gizmos
#' ggplot(mpg, aes(displ, hwy, colour = cty)) +
#'   geom_point() +
#'   guides(
#'     x.sec = gizmo_grob(
#'       circle, hjust = 0.75,
#'       width = unit(2, "cm"), height = unit(2, "cm")
#'     ),
#'     colour = gizmo_grob(
#'       circle, width = unit(1, "cm"), height = unit(1, "cm")
#'     )
#'   )
gizmo_grob <- function(
  grob, width = grobWidth(grob), height = grobHeight(grob),
  hjust = 0.5, vjust = 0.5,
  position = waiver()
) {

  check_grob(grob)
  check_unit(width)
  check_unit(height)
  check_length(width, exact = 1L)
  check_length(height, exact = 1L)
  check_number_decimal(hjust, min = 0.0, max = 1.0, allow_infinite = FALSE)
  check_number_decimal(vjust, min = 0.0, max = 1.0, allow_infinite = FALSE)

  new_guide(
    grob   = grob,
    width  = width,
    height = height,
    hjust = hjust,
    vjust = vjust,
    hash = hash(grob),
    position = position,
    available_aes = c("any", "x", "y", "r"),
    super = GizmoGrob
  )
}

# Class -------------------------------------------------------------------

#' @export
#' @rdname legendry_extensions
#' @format NULL
#' @usage NULL
GizmoGrob <- ggproto(
  "GizmoGrob", Guide,

  params = new_params(
    grob = zeroGrob(), width = unit(0.0, "cm"), height = unit(0.0, "cm"),
    hjust = 0.5, vjust = 0.5
  ),

  elements = list(),

  train = function(params, scale, aesthetic = NULL, ...) {
    params$aesthetic <- aesthetic %||% scale$aesthetics[1L]
    params$position  <- params$position %|W|% NULL
    params
  },

  transform = function(params, ...) {
    params
  },

  process_layers = function(params, ...) {
    params
  },

  draw = function(self, theme, position = NULL, direction = NULL,
                  params = self$params) {

    width  <- unit(width_cm(params$width),   "cm")
    height <- unit(height_cm(params$height), "cm")

    gt <-
      gtable(widths = width, heights = height) |>
      gtable_add_grob(
        params$grob, t = 1L, l = 1L,
        clip = "off", name = "gizmo_grob"
      )

    if (params$aesthetic %in% c("x", "y")) {
      padding <- margin(
        t = 1.0 - params$vjust,
        b = params$vjust,
        r = 1.0 - params$hjust,
        l = params$hjust,
        unit = "null"
      )
      gt <- gtable_add_padding(gt, padding)
    }
    gt
  }
)

test_that("compose_crux can compose a legend", {

  colours <- c("dodgerblue", "limegreen", "tomato", "goldenrod")

  themes <- lapply(colours, function(col) {
    theme(
      legend.text = element_text(colour = col),
      legend.axis.line = element_line(colour = col),
      legend.ticks = element_line(colour = col)
    )
  })

  guides <- lapply(colours, function(col) {
    theme <- theme(
      legend.text = element_text(colour = col),
      legend.axis.line = element_line(colour = col),
      legend.ticks = element_line(colour = col),
      legend.title = element_text(colour = col)
    )
    compose_stack(
      guide_axis_base,
      primitive_title(title = col),
      theme = theme
    )
  })

  crux <- compose_crux(
    key = key_auto(),
    centre = gizmo_grob(circleGrob(r = unit(2.0, "cm"))),
    top = guides[[1L]], bottom = guides[[2L]],
    left = guides[[3L]], right = guides[[4L]]
  )


  p <- ggplot(mpg, aes(displ, hwy, colour = cty)) +
    geom_point() +
    guides(colour = crux)

  vdiffr::expect_doppelganger("compose-crux legend", p)
})

test_that("gizmo_grob works for positions", {

  gizmo <- circleGrob(r = unit(1.0, "cm"))

  p <- ggplot(mpg, aes(displ, hwy)) +
    geom_blank() +
    theme_test() +
    theme(
      panel.background = element_rect(fill = NA, colour = "grey80"),
      panel.grid.major = element_line(colour = "grey90"),
      panel.border = element_blank(),
      axis.line = element_line()
    ) +
    guides(
      x = gizmo_grob(gizmo, hjust = 0.75),
      x.sec = gizmo_grob(gizmo, hjust = 0.25),
      y = gizmo_grob(gizmo, vjust = 0.25),
      y.sec = gizmo_grob(gizmo, vjust = 0.75)
    )

  vdiffr::expect_doppelganger("gizmo_grob cartesian", p)
})

test_that("gizmo_grob works for legends", {

  circle <- circleGrob(r = unit(1.0, "cm"))
  rect   <- rectGrob()

  p <- ggplot(mtcars, aes(disp, mpg, shape = factor(cyl), colour = cyl)) +
    geom_point() +
    theme_test() +
    theme(
      panel.background = element_rect(fill = NA, colour = "grey80"),
      panel.grid.major = element_line(colour = "grey90"),
      panel.border = element_blank(),
      axis.line = element_line()
    ) +
    guides(
      colour = gizmo_grob(
        rect, width = unit(1.0, "cm"), height = unit(2.0, "cm"),
        position = "bottom"
      ),
      shape = gizmo_grob(circle)
    )

  vdiffr::expect_doppelganger("gizmo_grob legends", p)
})

setup_legend_text <- function(theme, position = NULL, direction = "vertical") {

  position <- position %||%
    calc_element("legend.text.position", theme) %||%
    switch(direction, horizontal = "bottom", vertical = "right")

  gap    <- calc_element("legend.key.spacing", theme) %||% unit(0.0, "pt")

  margin <- calc_element("text", theme)$margin %||% margin()
  text <- justify_margins(position, margin, gap)
  calc_element("legend.text", theme + theme(text = text))
}

setup_side_title <- function(theme, position = NULL, type = "axis") {
  name <- switch(
    type,
    axis = "legendry.axis.subtitle",
    "legendry.legend.subtitle"
  )

  position <- position %||%
    calc_element(paste0(name, ".position"), theme) %||%
    "left"

  gap <- switch(
    type,
    axis = {
      position_name <- suffix_position("axis.ticks.length", position)[[1L]]
      (calc_element(position_name, theme) %||% unit(0.0, "pt")) * 2.0
    },
    calc_element("legend.key.spacing", theme) %||% unit(0.0, "pt")
  )

  margin <- calc_element("text", theme)$margin %||% margin()
  text <- justify_margins(position, margin, gap)
  calc_element(name, theme + theme(text = text))
}

setup_legend_title <- function(theme, position = NULL, direction = "vertical",
                               element = "legend.title") {
  position <- position %||%
    calc_element("legend.title.position", theme) %||%
    switch(direction, horizontal = "left", vertical = "top")
  gap <- calc_element("legend.key.spacing", theme) %||% unit(0.0, "pt")
  margin <- calc_element("text", theme)$margin %||% margin()
  margin <- position_margin(position, margin, gap)
  title <- theme(text = element_text(hjust = 0.0, vjust = 0.5, margin = margin))
  calc_element(element, theme + title)
}

justify_margins <- function(position, margin, gap) {
  margin <- position_margin(position, margin, gap)
  switch(
    position,
    top    = element_text(hjust = 0.5, vjust = 0.0, margin = margin),
    bottom = element_text(hjust = 0.5, vjust = 1.0, margin = margin),
    left   = element_text(hjust = 1.0, vjust = 0.5, margin = margin),
    right  = element_text(hjust = 0.0, vjust = 0.5, margin = margin),
    element_text(hjust = 0.5, vjust = 0.5, margin = margin)
  )
}

position_margin <- function(
  position, margin = margin(), gap = unit(0.0, "pt")
) {
  switch(
    position,
    top    = replace(margin, 3L, margin[3L] + gap),
    bottom = replace(margin, 1L, margin[1L] + gap),
    left   = replace(margin, 2L, margin[2L] + gap),
    right  = replace(margin, 4L, margin[4L] + gap),
    margin + gap
  )
}

# Utility for grabbing the justification of an element
get_just <- function(element) {
  element <- destructure_element(element)
  rotate_just(
    element$angle %||% 0.0,
    element$hjust %||% 0.5,
    element$vjust %||% 0.5
  )
}

validate_labels <- function(labels) {
  if (!is.list(labels)) {
    return(labels)
  }
  if (any(map_lgl(labels, is.language))) {
    do.call(expression, labels)
  } else {
    unlist(labels)
  }
}

angle_labels <- function(element, angle, position) {
  if (!is_theme_element(element, "text") ||
      is_waive(angle) ||
      is_null(angle)  ||
      !position %in% .trbl) {
    return(element)
  }

  position <- arg_match0(as.character(position), .trbl)
  radians <- deg2rad(angle)
  digits <- 3

  cosine <- sign(round(cos(radians), digits)) / 2 + 0.5
  sine   <- sign(round(sin(radians), digits)) / 2 + 0.5

  hjust <-
    switch(position, left = cosine, right = 1 - cosine, top = 1 - sine, sine)
  vjust <-
    switch(position, left = 1 - sine, right = sine, top = 1 - cosine, cosine)

  element$angle <- angle %||% element$angle
  element$hjust <- hjust %||% element$hjust
  element$vjust <- vjust %||% element$vjust

  element
}

# Standard keys -----------------------------------------------------------

#' Standard keys
#'
#' @description
#' These functions are helper functions for working with tick marks as keys
#' in guides. They all share the goal of creating a guide key, but have
#' different outcomes:
#'
#' * `key_auto()` is a function factory whose functions extract a typical
#'   key from major breaks in a scale.
#' * `key_manual()` uses user-provided vectors to make a key.
#' * `key_map()` makes mappings from a `<data.frame>` to make a key.
#' * `key_minor()` is a function factory whose functions also extract minor
#'   break positions for minor tick marks.
#' * `key_log()` is a function factory whose functions place ticks at intervals
#'   in log10 space.
#' * `key_none()` makes an empty key with no entries.
#'
#' @param aesthetic,value A vector of values for the guide to represent
#'   equivalent to the `breaks` argument in scales. The `aesthetic` will be
#'   mapped, whereas `value` will not. For most intents and purposes,
#'   `aesthetic` and `value` should be identical.
#' @param label A `<character>` or list of expressions to use as labels.
#' @param type A `<character>` vector representing the one of the break types:
#'   `"major"`, `"minor"` or `"mini"`. If `NULL` (default), all breaks are
#'   treated as major breaks.
#' @param data A `<data.frame>` or similar object coerced by
#'   [`fortify()`][ggplot2::fortify] to a `<data.frame>`, in which the `mapping`
#'   argument is evaluated.
#' @param ...
#' The `...` parameter has two purposes.
#' 1. In `key_map()` it is [`<data-masking>`][rlang::topic-data-mask]. A set of
#'   mappings similar to those provided to [`aes()`][ggplot2::aes], which will
#'   be evaluated in the `data` argument. These must contain `aesthetic`
#'   mapping.
#' 2. In other keys, `...` can be used to transfer graphical properties to the
#'   individual breaks of a guide. For example, using `colour = "blue"` will
#'   draw parts of the guides associated with breaks in blue. There is a shallow
#'   hierarchy in that `text_colour`, `line_colour`, `rect_colour` and
#'   `point_colour` are the specific properties for elements, but all inherit
#'   from the main `colour` setting. Likewise, `size`, `linewidth`, `linetype`
#'   and `fill` have specific variants for elements.
#' @param labeller A `<function>` that receives major breaks and returns
#'   formatted labels. For `key_log()`, `NULL` will default to
#'   [`scales::label_log()`] for strictly positive numbers and a custom labeller
#'   when negative numbers are included.
#' @param prescale_base A `<numeric[1]>` giving the base of logarithm to
#'   transform data manually. The default, `NULL`, will use the scale
#'   transformation to calculate positions. It is only advisable to set the
#'   `prescale_base` argument when the data have already been log-transformed.
#'   When using a log-transform in the scale or in
#'   [`coord_transform()`][ggplot2::coord_transform], the default `NULL` is
#'   recommended.
#' @param negative_small A `<numeric[1]>` setting the smallest absolute value
#'   that is marked with a tick in case the scale limits include 0 or negative
#'   numbers.
#' @param expanded A `<logical[1]>` determining whether the ticks should cover
#'   the entire range after scale expansion (`TRUE`, default), or be restricted
#'   to the scale limits (`FALSE`).
#' @param call A [call][rlang::topic-error-call] to display in messages.
#'
#' @name key_standard
#' @family keys
#' @return
#' For `key_auto()`, `key_minor()` and `key_log()` a function. For
#' `key_manual()` and `key_map()` a `<data.frame>` with the `<key_standard>`
#' class.
#'
#' @examples
#' # An example scale
#' template <- scale_x_continuous(limits = c(0, 10))
#'
#' # The auto, minor and log keys operate on scales
#' key_auto()(template)
#' key_minor()(template)
#'
#' # So does the log key
#' template <- scale_x_continuous(transform = "log10", limits = c(0.1, 10))
#' key_log()(template)
#'
#' # Providing custom values
#' key_manual(
#'   aesthetic = 1:5,
#'   label = c("one", "two", "three", "four", "five")
#' )
#'
#' # Values from a `<data.frame>`
#' key_map(ToothGrowth, aesthetic = unique(supp))
#'
#' # Empty key
#' key_none()
NULL

#' @rdname key_standard
#' @export
key_auto <- function(..., call = NULL) {
  call <- call %||% current_call()
  function(scale, aesthetic = NULL) {
    aesthetic <- aesthetic %||% scale$aesthetics[1L]
    df <- Guide$extract_key(scale, aesthetic)
    df <- data_frame0(df, !!!extra_args(...), .error_call = call)
    class(df) <- c("key_standard", "key_guide", class(df))
    df
  }
}

#' @rdname key_standard
#' @export
key_manual <- function(aesthetic, value = aesthetic,
                       label = as.character(value), type = NULL,
                       ..., call = NULL) {
  df <- data_frame0(
    aesthetic = aesthetic, value = value,
    label = label, type = type, !!!extra_args(...),
    .error_call = call %||% current_call()
  )
  check_columns(df, c("aesthetic", "value", "label"))
  df <- rename(df, c("value", "label", "type"), c(".value", ".label", ".type"))
  class(df) <- c("key_standard", "key_guide", class(df))
  df
}

#' @rdname key_standard
#' @export
key_map <- function(data, ..., call = NULL) {
  mapping <- aes(!!!enquos(...))

  df <- eval_aes(
    data, mapping,
    required = "aesthetic",
    optional = c("value", "label", .element_params),
    call = call %||% current_call(),
    arg_mapping = "mapping", arg_data = "data"
  )
  df$value <- df$value %||% df$aesthetic
  df$label <- df$label %||% as.character(df$aesthetic)
  check_columns(df, c("aesthetic", "value", "label"))

  df <- rename(
    df,
    c("value", "label", .label_params),
    c(".value", ".label", paste0(".", .label_params))
  )
  class(df) <- c("key_standard", "key_guide", class(df))
  df
}


#' @rdname key_standard
#' @export
key_minor <- function(..., call = NULL) {
  call <- call %||% current_call()
  dots <- extra_args(...)
  function(scale, aesthetic = NULL) {
    aesthetic <- aesthetic %||% scale$aesthetics[1L]
    df <- GuideAxis$extract_key(scale, aesthetic, minor.ticks = TRUE)
    df <- data_frame0(df, !!!dots, .error_call = call)
    class(df) <- c("key_standard", "key_guide", class(df))
    df
  }
}

#' @rdname key_standard
#' @export
key_log <- function(
  prescale_base = NULL, negative_small = 0.1, expanded = TRUE,
  labeller = NULL, ..., call = NULL
) {
  check_number_decimal(
    negative_small, min = 1e-100,
    allow_infinite = FALSE,
    allow_null = TRUE
  )
  check_bool(expanded)
  check_number_decimal(prescale_base, allow_infinite = FALSE, allow_null = TRUE)
  check_function(labeller, allow_null = TRUE)
  force(prescale_base)
  force(negative_small)
  force(expanded)
  dots <- extra_args(...)
  call <- call %||% current_call()
  function(scale, aesthetic = NULL) {
    log10_keys(
      scale = scale, aesthetic = aesthetic,
      prescale_base = prescale_base,
      negative_small = negative_small,
      expanded = expanded, extra_args = dots, call = call
    )
  }
}

#' @rdname key_standard
#' @export
key_none <- function() {
  new_data_frame(
    list(aesthetic = logical(), .value = logical(), .label = character()),
    n = 0L, class = c("key_standard", "key_guide")
  )
}

# Helpers -----------------------------------------------------------------

validate_key_types <- function(key, call = caller_env()) {
  types <- c("major", "minor", "mini")
  if (!all(key$.type %in% types)) {
    extra <- setdiff(unique(key$.type), types)
    cli::cli_warn(c(paste0(
      "Key types must be one of {.or {.val {types}}}, not ",
      "{.or {.val {extra}}}."
    ), i = "Unknown types are dropped."), call = call)
    key <- vec_slice(key, key$.type %in% types)
  }
  key
}

resolve_key <- function(x, allow_null = FALSE) {
  if (allow_null && is.null(x)) {
    return(NULL)
  }
  if (is.character(x)) {
    fun <- find_global(paste0("key_", x), env = global_env(),
                       mode = "function")
    if (is.function(fun)) {
      x <- fun()
    }
  }
  if (is.function(x)) {
    return(x)
  }
  if (inherits(x, "key_guide")) {
    return(x)
  }
  cli::cli_abort("Unknown key specification: {x}.")
}

is_key_string <- function(x) {
  if (!is_bare_string(x)) {
    return(FALSE)
  }
  fun <- find_global(paste0("key_", x), env = global_env(), mode = "function")
  is.function(fun)
}

log10_keys <- function(scale, aesthetic,
                       prescale_base = NULL,
                       negative_small = 0.1,
                       expanded = TRUE,
                       labeller = NULL,
                       extra_args = NULL,
                       call = caller_env()) {
  aesthetic <- aesthetic %||% scale$aesthetics[1L]
  if (scale$is_discrete()) {
    cli::cli_abort(
      "Cannot calculate logarithmic ticks for discrete scales.",
      call = call
    )
  }

  transform <- scale$get_transformation()
  if (!is.null(prescale_base)) {
    if (transform$name != "identity") {
      cli::cli_warn(paste0(
        "The {.arg prescale_base} argument will override the scale's ",
        "{.field {transform$name}} transformation in log-tick positioning."
      ), call = call)
    }
    transform <- transform_log(base = prescale_base)
  }

  limits <- transform$inverse(scale$get_limits())
  has_negatives <- any(limits <= 0.0)

  if (has_negatives) {
    negative_small <- negative_small %||% 0.1
    start <- floor(log10(abs(negative_small)))
    end   <- ceiling(log10(max(abs(limits)))) + 1L
  } else {
    start <- floor(log10(min(limits))) - 1L
    end   <- ceiling(log10(max(limits))) + 1L
  }

  tens  <- 10L^seq(start, end, by = 1L)
  fives <- tens * 5L
  ones  <- as.vector(outer(setdiff(2L:9L, 5L), tens))

  if (has_negatives) {
    tens  <- tens[tens >= negative_small]
    tens  <- c(tens, -tens, 0.0)
    fives <- fives[fives >= negative_small]
    fives <- c(fives, -fives)
    ones  <- ones[ones >= negative_small]
    ones  <- c(ones, -ones)
    labeller <- labeller %||% negative_log_label
  }
  labeller <- labeller %||% scales::label_log()
  labels <- labeller(tens)

  ticks  <- transform$transform(c(tens, fives, ones))
  nticks <- c(length(tens), length(fives), length(ones))

  if (is.expression(labels)) {
    labels <- as.list(labels)
    labels <- pad(labels, sum(nticks), fill = list(NULL))
  } else {
    labels <- pad(labels, sum(nticks))
  }

  key <- data_frame0(
    !!aesthetic := ticks,
    .value = ticks,
    .label = labels,
    .type = rep(c("major", "minor", "mini"), times = nticks),
    !!!extra_args
  )

  if (expanded) {
    range <- scale$continuous_range %||% scale$get_limits()
  } else {
    range <- scale$get_limits()
  }
  key <- vec_slice(key, !is_oob(ticks, range))
  class(key) <- c("key_standard", "key_guide", class(key))
  key
}

negative_log_label <- function(x) {
  if (length(x) == 0L) {
    return(expression)
  }
  sign <- as.character(sign(x))
  sign[sign == "-1"] <- "-"
  sign[sign == "1"] <- "+"
  sign[sign == "0"] <- ""

  abs <- abs(x)
  exponent <- format(log(abs, base = 10.0), digits = 3L)
  text <- paste0(sign, 10L, "^", exponent)
  text[x == 0.0] <- "0"

  out <- vector("expression", length(text))
  for (i in seq_along(text)) {
    expr <- parse(text = text[[i]])
    out[[i]] <- if (length(expr) == 0L) NA else expr[[1L]]
  }
  out[is.na(x)] <- NA
  out
}

transform_key <- function(key, position, coord, panel_params) {
  if (is_empty(key)) {
    return(key)
  }
  other <- switch(position, bottom = , left = , theta.sec = -Inf, Inf)
  key <- replace_null(key, x = other, y = other)
  transformed <- coord$transform(key, panel_params)

  if (is_theta(position)) {
    add <- if (position == "theta.sec") pi else 0.0
    transformed$theta <- transformed$theta + add
  } else if ("theta" %in% setdiff(names(transformed), names(key))) {
    # For radius axes, we want to keep any `theta` value it had before,
    # but discard these when the coord forces this.
    transformed[c("theta", "r")] <- NULL
  }

  ends <- c("xend", "yend") %in% names(key)
  if (!any(ends)) {
    return(transformed)
  }

  if (ends[1L]) {
    key <- rename(key, c("x", "xend"), rev)
  }
  if (ends[2L]) {
    key <- rename(key, c("y", "yend"), rev)
  }
  key <- coord$transform(key, panel_params)
  if (is_theta(position)) {
    transformed$thetaend <- key$theta + add
  } else {
    if (ends[1L]) {
      transformed$xend <- key$x
    }
    if (ends[2L]) {
      transformed$yend <- key$y
    }
  }

  transformed
}

test_that("key_auto works as intended", {

  fun <- key_auto(colour = "red")
  expect_type(fun, "closure")

  template <- scale_x_discrete(limits = LETTERS[1L:5L])
  test <- fun(template)
  expect_s3_class(test, "key_standard")

  expect_identical(test$x, as_mapped_discrete(1L:5L))
  expect_identical(test$.value, LETTERS[1L:5L], ignore_attr = TRUE)
  expect_identical(test$.label, LETTERS[1L:5L])
  expect_identical(test$.colour, rep("red", nrow(test)))

})

test_that("key_manual works as intended", {

  test <- key_manual(1L:5L, colour = "blue")
  expect_s3_class(test, "key_standard")

  expect_identical(test$aesthetic, 1L:5L)
  expect_identical(test$.value, 1L:5L)
  expect_identical(test$.label, as.character(1L:5L))
  expect_identical(test$.colour, rep("blue", nrow(test)))
})

test_that("key_map works as intended", {

  test <-
    key_map(iris, aesthetic = as.character(unique(Species)), colour = "green")
  expect_s3_class(test, "key_standard")

  expect_identical(test$aesthetic, levels(iris$Species))
  expect_identical(test$.value, levels(iris$Species))
  expect_identical(test$.label, levels(iris$Species))
  expect_identical(test$.colour, rep("green", nrow(test)))

  expect_error(expect_warning(expect_warning(
    key_map(iris, foo = Species),
    "No valid data"
  ), "Ignoring unknown"), "columns are required")
})

test_that("key_minor works as intended", {

  fun <- key_minor(colour = "purple")
  expect_type(fun, "closure")

  template <- scale_x_continuous(
    limits = c(0.0, 10.0),
    breaks = seq(0.0, 10.0, by = 2.0)
  )
  test <- fun(template)
  expect_s3_class(test, "key_standard")

  expect_equal(
    test$x, c(0.0, 2.0, 4.0, 6.0, 8.0, 10.0, 1.0, 3.0, 5.0, 7.0, 9.0),
    ignore_attr = TRUE
  )
  expect_equal(
    test$.value, c(0.0, 2.0, 4.0, 6.0, 8.0, 10.0, 1.0, 3.0, 5.0, 7.0, 9.0),
    ignore_attr = TRUE
  )
  expect_identical(
    test$.label,
    c("0", "2", "4", "6", "8", "10", rep(NA_character_, 5L))
  )
  expect_identical(test$.type, rep(c("major", "minor"), c(6L, 5L)))
  expect_identical(test$.colour, rep("purple", nrow(test)))

})

test_that("key_log works as intended", {

  fun <- key_log(colour = "pink")
  expect_type(fun, "closure")

  template <- scale_x_continuous(limits = c(0.1, 10.0), transform = "log10")
  test <- fun(template)
  expect_s3_class(test, "key_standard")
  expect_snapshot(test)
  expect_identical(test$.colour, rep("pink", nrow(test)))
})

test_that("validate_key_types throws appropriate warning", {

  expect_silent(
    test <- validate_key_types(data_frame0(.type = "major"))
  )
  expect_identical(dim(test), c(1L, 1L))
  expect_warning(
    test <- validate_key_types(data_frame0(.type = "foobar")),
    "Unknown types are dropped"
  )
  expect_identical(dim(test), c(0L, 1L))
})

test_that("resolve_key throws appropriate error", {

  expect_silent(resolve_key("auto"))
  expect_error(
    resolve_key(mtcars),
    "Unknown key specification"
  )

})

test_that("log10_keys returns sensible results", {

  scale <- scale_x_discrete()
  expect_error(
    log10_keys(scale, "x"),
    "Cannot calculate logarithmic ticks for discrete scales"
  )

  scale <- scale_x_log10()
  scale$train(c(0.0, 2.0))

  expect_warning(
    test <- log10_keys(scale, "x", 10L, negative_small = 0.1, expanded = FALSE),
    "argument will override"
  )
  expect_identical(unique(test$.type), c("major", "minor", "mini"))

  scale <- scale_x_continuous(transform = "asinh")
  scale$train(c(-5.0, 5.0))

  test <- log10_keys(scale, "x", NULL, 0.1, expanded = FALSE)


})

# Range keys --------------------------------------------------------------

#' Range keys
#'
#' @description
#' These functions are helper functions for working with ranged data as keys
#' in guides. They all share the goal creating of a guide key, but have
#' different methods:
#'
#' * `key_range_auto()` is a function factory whose functions make an attempt
#'   to infer ranges from the scale's labels.
#' * `key_range_manual()` uses user-provided vectors to set ranges.
#' * `key_range_map()` makes mappings from a `<data.frame>` to set ranges.
#' * `key_range_rle()` uses run-length encoding to determine the start and
#'   end of runs (blocks of repeated data values).
#'
#' @param sep A `<character[1]>` giving a [regular expression][base::regex] to
#'   use for splitting labels provided by the scale using
#'   [`strsplit()`][base::strsplit]. Defaults to splitting on any
#'   non-alphanumeric character.
#' @param reverse A `<logical[1]>` which if `FALSE` (default) treats the first
#'   labels as the inner labels and the last labels as the outer labels.
#'   If `TRUE`, thee first labels are treated as the outer labels and the last
#'   labels are treated as the inner labels.
#' @param start,end A vector that can be interpreted by the scale, giving the
#'   start and end positions of each range respectively.
#' @param name A `<character>` or list of expressions
#' @param level An `<integer>` giving the depth of each range to avoid overlaps
#'   between different ranges. When `level` is smaller than 1, no brackets are
#'   drawn.
#' @param data A `<data.frame>` or similar object coerced by
#'   [`fortify()`][ggplot2::fortify] to a `<data.frame>`, in which the `mapping`
#'   argument is evaluated.
#' @param ...
#' The `...` parameter has two purposes.
#' 1. In `key_range_map()` it is [`<data-masking>`][rlang::topic-data-mask].
#'   A set of mappings similar to those provided to [`aes()`][ggplot2::aes],
#'   which will be evaluated in the `data` argument. These *must* contain
#'   `start` and `end` mappings.
#' 2. In other keys, `...` can be used to transfer graphical properties to the
#'   individual ranges of a guide. For example, using `colour = "blue"` will
#'   draw parts of the guides associated with ranges in blue. There is a shallow
#'   hierarchy in that `text_colour`, `line_colour`, `rect_colour` and
#'   `point_colour` are the specific properties for elements, but all inherit
#'   from the main `colour` setting. Likewise, `size`, `linewidth`, `linetype`
#'   and `fill` have specific variants for elements.
#' @param x A `<vector[n]>` for which to determine run-starts and run-ends.
#' @param call A [call][rlang::topic-error-call] to display in messages.
#'
#' @details
#' The `level` variable is optional and when missing, the guides use an
#' algorithm similar to `IRanges::disjointBins()` to avoid overlaps.
#'
#' The `key_range_auto()` does *not* work with expression labels.
#'
#' @name key_range
#' @family keys
#' @return
#' For `key_range_auto()` a function. For `key_range_manual()` and
#' `key_range_map()` a `<data.frame>` with the `<key_range>` class.
#'
#' @examples
#' # Example scale
#' template <- scale_x_discrete(limits = c("A 1", "B 1", "C&1", "D&2", "E&2"))
#'
#' # By default, splits on all non-alphanumeric characters
#' auto <- key_range_auto()
#' auto(template)
#'
#' # Only split on a specific character
#' auto <- key_range_auto(sep = "&")
#' auto(template)
#'
#' # Treating the letters as outer labels and numbers as inner labels
#' auto <- key_range_auto(reverse = TRUE)
#' auto(template)
#'
#' # Providing custom values
#' key_range_manual(
#'   start = c(1, 5,  10),
#'   end   = c(4, 15, 11),
#'   level = c(0, 2, 1),
#'   name  = c("A", "B", "C")
#' )
#'
#' # Values from a <data.frame>
#' key_range_map(presidential, start = start, end = end, name = name)
#'
#' # Values from run length encoding
#' key_range_rle(c("AB", "AB", "C", "DEF", "DEF", "DEF"))
NULL

#' @rdname key_range
#' @export
key_range_auto <- function(sep = "[^[:alnum:]]+", reverse = FALSE, ...) {
  check_string(sep)
  check_bool(reverse)
  force(sep)
  force(reverse)
  dots <- extra_args(...)
  call <- current_call()
  fun <- function(scale, aesthetic = NULL) {
    range_from_label(
      scale = scale, aesthetic = aesthetic,
      sep = sep, reverse = reverse, extra_args = dots,
      call = call
    )
  }
  class(fun) <- union("key_range_auto_function", class(fun))
  fun
}

#' @rdname key_range
#' @export
key_range_manual <- function(start, end, name = NULL, level = NULL, ..., call = NULL) {
  df <- data_frame0(
    start = start, end = end, .label = name, .level = level,
    !!!extra_args(...),
    .error_call = call %||% current_call()
  )
  check_columns(df, c("start", "end"))
  class(df) <- c("key_range", "key_guide", class(df))
  df
}

#' @rdname key_range
#' @export
key_range_map <- function(data, ..., call = NULL) {
  mapping <- aes(!!!enquos(...))

  df <- eval_aes(
    data, mapping,
    required = c("start", "end"),
    optional = c("name", "level", .element_params),
    call = call %||% current_call(),
    arg_mapping = "mapping", arg_data = "data"
  )

  df <- rename(
    df, c("name", "level", .element_params),
    c(".label", ".level", paste0(".", .element_params))
  )
  class(df) <- c("key_range", "key_guide", class(df))
  df
}

#' @rdname key_range
#' @export
key_range_rle <- function(x, ..., call = NULL) {
  rle <- new_rle(x)
  key_range_manual(
    start = as_mapped_discrete(rle$start),
    end   = as_mapped_discrete(rle$end),
    name  = rle$key,
    level = 1L,
    ...,
    call = call %||% current_call()
  )
}

# Extractor ---------------------------------------------------------------

range_extract_key <- function(
  scale, aesthetic, key,
  drop_zero = TRUE, pad_discrete = 0.0,  oob = "squish",
  ...
) {
  if (is.function(key)) {
    key <- key(scale, aesthetic)
  }
  if (is.null(key$.level)) {
    key <- disjoin_ranges(key)
  }

  # Mark discrete variables separately for start and end
  pad_start <- -1.0 * is_discrete(key$start) * pad_discrete
  pad_end   <- +1.0 * is_discrete(key$end)   * pad_discrete

  map <- aesthetic %in% c("x", "y")
  key$start <- scale_transform(key$start, scale, map = map, "start")
  key$end   <- scale_transform(key$end,   scale, map = map, "end")

  # Backtransform AsIs variables
  limits <- scale$continuous_range %||% scale$get_limits()
  key$start <- descale(key$start, to = limits)
  key$end   <- descale(key$end,   to = limits)

  # Sort starts and ends
  key[c("start", "end")] <- list(
    start = pmin(key$start, key$end),
    end   = pmax(key$start, key$end)
  )

  # Mark ranges where no brackets should be drawn
  key$.draw <- TRUE
  if (!isFALSE(drop_zero)) {
    key$.draw <- abs(key$end - key$start) > sqrt(.Machine$double.eps)
  }
  key$.draw <- key$.draw & key$.level > 0L

  # Apply padding for discrete variables
  extend <- pad_discrete
  if (scale$is_discrete() && !is.null(extend)) {
    key$start <- key$start + pad_start
    key$end   <- key$end   + pad_end
  }

  # Apply out-of-bounds rules
  range_oob(key, oob, limits)
}


# Helpers -----------------------------------------------------------------

## Out-of-bounds utilities ------------------------------------------------

range_oob <- function(ranges, method, limits) {
  limits <- sort(limits)
  ranges <- switch(
    method,
    squish = range_squish(ranges, limits),
    censor = range_censor(ranges, limits),
    ranges
  )
  vec_slice(ranges, !is.na(ranges$.draw))
}

range_squish <- function(ranges, limits) {
  start  <- ranges$start
  end    <- ranges$end
  oob_start <- is_oob(start, limits)
  oob_end   <- is_oob(end,   limits)
  keep <- !oob_start | !oob_end | (start < limits[1L] & end > limits[2L])
  ranges$.draw[!keep] <- NA
  ranges$start <- pmin(pmax(ranges$start, limits[1L]), limits[2L])
  ranges$end   <- pmin(pmax(ranges$end,   limits[1L]), limits[2L])
  ranges
}

range_censor <- function(ranges, limits) {
  oob_start <- is_oob(ranges$start, limits)
  oob_end   <- is_oob(ranges$end, limits)
  keep      <- !oob_start & !oob_end
  ranges$.draw[!keep] <- NA
  ranges
}

## Other helpers ----------------------------------------------------------

range_from_label <- function(
  scale, aesthetic = NULL, sep =  "[^[:alnum:]]+", reverse = FALSE,
  extra_args = list(), call = caller_env()
) {
  # Extract a standard key from the scale
  aesthetic <- aesthetic %||% scale$aesthetics[1L]
  key <- Guide$extract_key(scale, aesthetic)

  # Reject expressions, as we cannot split these
  if (!is.character(key$.label)) {
    type <- obj_type_friendly(key$.label)
    cli::cli_abort(
      c("Cannot split the guide's {.field label}.",
        i = "It must be a {.cls character} vector, not {type}."),
      call = call
    )
  }

  # Split labels
  labels <- strsplit(key$.label, sep)

  # Pad label vectors if necessary
  lengths <- lengths(labels)
  max_len <- max(lengths)
  if (any(lengths != max_len)) {
    cli::cli_warn(c(
      paste0("Not all {.field labels} in {.fn key_range_auto} can be split ",
             "into equal lengths."),
      i = "Is \"{sep}\" the correct {.arg sep} argument?"
    ))
    labels <- lapply(labels, pad, length = max_len)
  }
  labels <- do.call(rbind, labels)

  if (isTRUE(reverse)) {
    labels <- labels[, rev(seq_len(ncol(labels)))]
  }

  key$.label <- labels[, 1L, drop = TRUE]
  labels <- labels[, -1L, drop = FALSE]

  # Set first series of unbracketed labels
  value  <- key[[1L]]
  key <- data_frame0(
    start = value, end = value, .label = key$.label, .level = 0L
  )
  if (is_empty(labels)) {
    return(data_frame0(key, !!!extra_args))
  }
  ranges <- apply(labels, 2L, function(labs) {
    rle   <- vec_unrep(labs)
    start <- cumsum(rle$times) - rle$times + 1L
    data_frame0(
      start = value[start],
      end   = value[cumsum(rle$times)],
      .label = labs[start]
    )
  })
  nrows  <- list_sizes(ranges)
  ranges <- vec_c(!!!ranges)
  ranges$.level <- rep.int(seq_along(nrows), nrows)
  range <- vec_slice(ranges, !is.na(ranges$.label))
  df <- vec_rbind(key, range)
  df <- data_frame0(df, !!!extra_args)
  class(df) <- c("key_range", "key_guide", class(df))
  df
}

justify_ranges <- function(key, levels, element, level_elements) {

  if (is_blank(element)) {
    return(key)
  }

  ends <- intersect(c("thetaend", "xend", "yend"), names(key))
  if (length(ends) < 1L) {
    return(key)
  }
  starts <- gsub("end$", "", ends[1L])

  just_name <- switch(ends[1L], yend = "vjust", "hjust")
  just <- element[[just_name]] %||% 0.5

  if (!is.null(level_elements)) {
    just <- map_dbl(level_elements, function(x) x[[just_name]] %||% just)
    just <- just[match(key$.level, levels)]
  }

  key[[starts]] <- switch(
    ends[1L],
    thetaend = justify_range(key$theta, key$thetaend, just, theta = TRUE),
    xend     = justify_range(key$x, key$xend, just),
    yend     = justify_range(key$y, key$yend, just)
  )

  key
}

justify_range <- function(start, end, just, theta = FALSE) {
  if (theta) {
    add <- end < start
    end[add] <- end[add] + 2.0 * pi
  }
  (end - start) * just + start
}

disjoin_ranges <- function(ranges) {

  n_ranges <- nrow(ranges)
  if (n_ranges < 2L) {
    ranges$.level <- rep(1L, nrow(ranges))
    return(ranges)
  }

  # Sort and extract
  ranges <- ranges[order(ranges$start, ranges$end), , drop = FALSE]
  ranges <- vec_slice(ranges, order(ranges$start, ranges$end))
  starts <- ranges$start
  ends   <- ranges$end

  # Initialise first range
  end_tracker <- ends[1L]
  bin <- rep(NA_integer_, nrow(ranges))
  bin[1L] <- 1L

  # Find bins
  for (range_id in seq_len(n_ranges)[-1L]) {
    candidate <- which(end_tracker < starts[range_id])
    if (length(candidate) > 0L) {
      # If there is room in this bin, update this bin
      ans <- candidate[1L]
      end_tracker[ans] <- ends[range_id]
    } else {
      # Register new bin
      end_tracker <- c(end_tracker, ends[range_id])
      ans <- length(end_tracker)
    }
    bin[range_id] <- ans
  }

  ranges$.level <- bin
  ranges
}

extract_range_params <- function(scale, params, ...) {
  params$position <- params$position %|W|% NULL
  params$limits   <- scale$get_limits()

  new_names <- c("start", "end")
  aesthetic <- params$aesthetic
  if (aesthetic %in% c("x", "y")) {
    new_names  <- paste0(aesthetic, c("", "end"))
  } else if (is_theta(params$position)) {
    new_names <- c("x", "xend")
  }
  params$key <- rename(params$key, c("start", "end"), new_names)
  params
}

setup_range_params <- function(params) {
  if (params$aesthetic %in% c("x", "y")) {
    # parameters are already transformed
    return(params)
  }

  limits   <- params$limits %||% c(0.0, 1.0)
  other    <- switch(params$position, bottom = , left = 1L, 0L)

  if (!is_empty(params$key)) {
    key <- params$key

    start <- guide_rescale(key$start, limits)
    end   <- guide_rescale(key$end, limits)

    key <- switch(
      params$direction,
      horizontal = replace_null(key, x = start, xend = end, y = other),
      vertical   = replace_null(key, y = start, yend = end, x = other),
      key
    )

    params$key <- key
  }

  if (!is_empty(params$decor)) {
    decor <- params$decor
    value <- guide_rescale(decor[[params$aesthetic]], limits)
    decor <- switch(
      params$direction,
      horizontal = replace_null(decor, x = value, y = other),
      vertical   = replace_null(decor, y = value, x = other),
      decor
    )

    params$decor <- decor
  }

  params
}

test_that("key_range_auto works as intended", {

  fun <- key_range_auto(colour = "red")
  expect_type(fun, "closure")

  template <- scale_x_discrete(limits = c("1 A", "2 A", "1 B", "2 B", "3 A"))
  test <- fun(template)
  expect_s3_class(test, "key_range")

  # First 5 should be singletons
  expect_identical(unclass(test$start), c(1.0:5.0, 1.0, 3.0, 5.0))
  expect_identical(unclass(test$end),   c(1.0:5.0, 2.0, 4.0, 5.0))
  expect_identical(test$.label, c(1L:2L, 1L:3L, "A", "B", "A"))
  expect_identical(test$.level, rep(c(0L, 1L), c(5L, 3L)))
  expect_identical(test$.colour, rep("red", nrow(test)))

  template$limits[5L] <- "3"
  expect_warning(fun(template), regexp = "can be split into equal lengths")

  # No nesting
  template$limits <- LETTERS[1L:5L]
  test <- fun(template)

  expect_identical(unclass(test$start), seq(1.0, 5.0, by = 1.0))
  expect_identical(unclass(test$end),   seq(1.0, 5.0, by = 1.0))
  expect_identical(test$.label, LETTERS[1L:5L])
  expect_identical(test$.level, rep(0L, 5L))
  expect_identical(test$.colour, rep("red", nrow(test)))

})

test_that("key_range_manual works as intended", {

  test <-
    key_range_manual(
      1L:5L, 4L:8L, LETTERS[1L:5L],
      c(1L, 1L, 2L, 2L, 1L), colour = "blue"
    )
  expect_s3_class(test, "key_range")

  expect_identical(test$start, 1L:5L)
  expect_identical(test$end,   4L:8L)
  expect_identical(test$.label, LETTERS[1L:5L])
  expect_identical(test$.level, c(1L, 1L, 2L, 2L, 1L))
  expect_identical(test$.colour, rep("blue", nrow(test)))

})

test_that("key_range_map works as intended", {

  test <- key_range_map(
    presidential,
    start = start, end = end,
    name = name, colour = "green"
  )
  expect_s3_class(test, "key_range")

  expect_identical(test$start,  presidential$start)
  expect_identical(test$end,    presidential$end)
  expect_identical(test$.label, presidential$name)
  expect_identical(test$.colour, rep("green", nrow(test)))

  expect_warning(expect_warning(
    key_range_map(presidential, foo = start),
    "No valid data"
  ), "Ignoring unknown")

  expect_error(
    key_range_map(presidential, start = start),
    "column is required"
  )
})

test_that("key_range_rle works as intended", {

  test <- key_range_rle(rep(LETTERS[1L:5L], 5L:1L), colour = "orange")
  expect_s3_class(test, "key_range")

  expect_identical(
    test$start,
    as_mapped_discrete(c(1.0, 6.0, 10.0, 13.0, 15.0))
  )
  expect_identical(test$end, as_mapped_discrete(c(5.0, 9.0, 12.0, 14.0, 15.0)))
  expect_identical(test$.label, LETTERS[1L:5L])
  expect_identical(test$.colour, rep("orange", nrow(test)))
})

test_that("range_extract_key can censor oob values", {

  scale <- scale_x_continuous(
    limits = c(0.0, 10.0)
  )

  key <- key_range_manual(
    start = c(-1.0, 2.0, 9.0),
    end   = c(1.0, 8.0, 11.0),
    name  = c("A", "B", "C")
  )

  test <- range_extract_key(scale, "x", key, oob = "censor")
  expect_identical(nrow(test), 1L)
  expect_identical(test$.label, "B")
})

test_that("range_extract_key backtransforms AsIs variables", {

  scale <- scale_x_continuous(limits = c(0.0, 10.0))

  key <- key_range_manual(
    start = I(c(0.1, 0.3, 0.5)),
    end   = c(1.0, 3.0, 5.0),
    name  = c("A", "B", "C")
  )

  test <- range_extract_key(scale, "x", key)
  expect_identical(test$start, test$end)
})

test_that("range_from_label can extract ranges", {

  values <- c("A 1", "B 1", "C 2")
  scale <- scale_x_discrete(limits = values)

  test <- range_from_label(scale, "x")
  expect_snapshot(test)

  exprs <- as.expression(values)
  scale <- scale_x_discrete(limits = values, labels = exprs)

  expect_error(
    range_from_label(scale, "x"),
    "Cannot split"
  )

})

test_that("setup_range_params sets up ranges correctly", {

  params <- list(
    aesthetics = "colour",
    position = "right",
    direction = "horizontal",
    limits = c(0.0, 10.0),
    key = data.frame(
      start = c(0.0, 4.0),
      end   = c(3.0, 9.0)
    ),
    decor = data.frame(colour = 5.0)
  )

  test <- setup_range_params(params)
  expect_equal(test$key$x, c(0.0, 0.4))
  expect_equal(test$key$y, c(0.0, 0.0))
  expect_equal(test$key$xend, c(0.3, 0.9))
  expect_equal(test$decor$x, 0.5)

  params$direction <- "vertical"
  test <- setup_range_params(params)
  expect_equal(test$key$y, c(0.0, 0.4))
  expect_equal(test$key$x, c(0.0, 0.0))
  expect_equal(test$key$yend, c(0.3, 0.9))
  expect_equal(test$decor$y, 0.5)
})

test_that("primitive_fence works as axis", {

  base <- ggplot(mpg, aes(displ, interaction(drv, year))) +
    geom_blank() +
    theme_test() +
    theme(
      panel.background = element_rect(fill = NA, colour = "grey80"),
      panel.grid.major = element_line(colour = "grey90"),
      panel.border = element_blank(),
      axis.line = element_line()
    )

  p <- base +
    guides(
      y = primitive_fence(rail = "both"),
      y.sec = primitive_fence(
        rail = "none",
        key = key_range_auto(reverse = TRUE),
        drop_zero = FALSE
      ),
      x = primitive_fence(
        rail = "inner",
        key = key_range_manual(
          start = c(2.0, 4.0),
          end = c(5.0, 7.0),
          name = c("A\nA", "B\nB")
        ),
        levels_post = list(NULL, element_line("red"))
      ),
      x.sec = primitive_fence(
        rail = "outer",
        key = key_range_manual(
          start = c(2.0, 4.0, 3.0),
          end = c(5.0, 7.0, 6.0),
          name = c("1", "2", "3"),
          level = c(1L, 2L, 3L),
          line_colour = c("tomato", "dodgerblue", "limegreen")
        )
      )
    )

  vdiffr::expect_doppelganger("primitive_fence cartesian", p)

  p <- base +
    coord_radial(start = 0.25 * pi, end = 1.75 * pi, inner.radius = 0.5) +
    guides(
      r = primitive_fence(rail = "both"),
      r.sec = primitive_fence(
        angle = 0.0,
        rail = "none",
        key = key_range_auto(reverse = TRUE),
        drop_zero = FALSE
      ),
      theta = primitive_fence(
        rail = "inner",
        key = key_range_manual(
          start = c(2.0, 4.0),
          end = c(5.0, 7.0),
          name = c("A\nA", "B\nB")
        ),
        levels_post = list(NULL, element_line("red"))
      ),
      theta.sec = primitive_fence(
        rail = "outer",
        key = key_range_manual(
          start = c(2.0, 4.0, 3.0),
          end = c(5.0, 7.0, 6.0),
          name = c("1", "2", "3"),
          level = c(1L, 2L, 3L),
          line_colour = c("tomato", "dodgerblue", "limegreen")
        )
      )
    )

  vdiffr::expect_doppelganger("primitive_fence radial", p)
})


test_that("primitive_bracket works as legend", {

  p <- ggplot(mtcars) +
    aes(
      x = disp, y = mpg,
      colour = hp,
      fill = hp
    ) +
    geom_point() +
    guides(
      colour = primitive_fence(key = key_range_manual(
        c(100.0, 200.0), c(250.0, 300.0), c("A", "B")
      ), rail = "outer"),
      fill = primitive_fence(key = key_range_manual(
        c(100.0, 150.0, 200.0), c(300.0, 300.0, 300.0), c("A", "B", "C")
      ), position = "bottom", rail = "inner")
    ) +
    theme(
      legend.box.just = "center"
    )

  vdiffr::expect_doppelganger("primitive_facet legend", p)

})

test_that("primitive_box works as axis", {

  base <- ggplot(mpg, aes(displ, interaction(drv, year))) +
    geom_blank() +
    theme_test() +
    theme(
      panel.background = element_rect(fill = NA, colour = "grey80"),
      panel.grid.major = element_line(colour = "grey90"),
      panel.border = element_blank(),
      axis.line = element_line()
    )

  p <- base +
    guides(
      y = primitive_box(),
      y.sec = primitive_box(
        min_size = 1L,
        key = key_range_auto(reverse = TRUE),
        drop_zero = FALSE
      ),
      x = primitive_box(
        key = key_range_manual(
          start = c(2.0, 4.0),
          end = c(5.0, 7.0),
          name = c("A\nA", "B\nB")
        )
      ),
      x.sec = primitive_box(
        key = key_range_manual(
          start = c(2.0, 4.0, 3.0),
          end = c(5.0, 7.0, 6.0),
          level = c(1L, 2L, 3L),
          rect_fill = c("tomato", "dodgerblue", "limegreen")
        )
      )
    )

  vdiffr::expect_doppelganger("primitive_box cartesian", p)

  p <- base +
    coord_radial(start = 0.25 * pi, end = 1.75 * pi, inner.radius = 0.5) +
    guides(
      r = primitive_box(),
      r.sec = primitive_box(
        key = key_range_auto(reverse = TRUE), drop_zero = FALSE
      ),
      theta = primitive_box(
        key = key_range_manual(
          start = c(2.0, 4.0),
          end = c(5.0, 7.0),
          name = c("A\nA", "B\nB"),
        )
      ),
      theta.sec = primitive_box(
        key = key_range_manual(
          start = c(2.0, 4.0, 3.0), end = c(5.0, 7.0, 6.0),
          level = c(1L, 2L, 3L),
          rect_fill = c("tomato", "dodgerblue", "limegreen")
        )
      )
    )

  vdiffr::expect_doppelganger("primitive_box radial", p)
})

test_that("primitive_box works as legend", {

  p <- ggplot(mtcars) +
    aes(
      x = disp, y = mpg,
      colour = hp,
      fill = hp
    ) +
    geom_point() +
    guides(
      colour = primitive_box(key = key_range_manual(
        c(100.0, 200.0), c(250.0, 300.0), c("A", "B")
      )),
      fill = primitive_box(key = key_range_manual(
        c(100.0, 150.0, 200.0), c(300.0, 300.0, 300.0), c("A", "B", "C")
      ), position = "bottom")
    )

  vdiffr::expect_doppelganger("primitive_box legend", p)

})

test_that("draw_box can draw theta boxes", {
  box <- draw_box(
    data.frame(x = c(0.0, 1.0), y = c(0.0, 1.0), group = 1L, theta = 0.0),
    element = theme_gray()$rect,
    position = "theta",
    size = 0.1, offset = 1.0
  )
  expect_s3_class(box, "polygon")
})

test_that("guide_axis_nested logic works", {

  g <- guide_axis_nested()
  expect_length(g$params$guides, 3L)
  expect_s3_class(g$params$guides[[1L]], "PrimitiveLine")
  expect_s3_class(g$params$guides[[2L]], "PrimitiveTicks")

  g <- guide_axis_nested(key = "range_auto")
  expect_length(g$params$guides, 3L)

  g <- guide_axis_nested(key = key_range_manual(1.0, 2.0, "A"))
  expect_length(g$params$guides, 4L)
  expect_s3_class(g$params$guides[[3L]], "PrimitiveLabels")


  g <- guide_axis_nested(type = "bracket")
  expect_s3_class(g$params$guides[[3L]], "PrimitiveBracket")

  g <- guide_axis_nested(type = "box")
  expect_s3_class(g$params$guides[[3L]], "PrimitiveBox")

  g <- guide_axis_nested(subtitle = "foobar")
  expect_length(g$params$guides, 4L)
  expect_s3_class(g$params$guides[[4L]], "PrimitiveTitle")
})

test_that("guide_axis_nested recognised `key_range_auto()`", {

  guide <- guide_axis_nested(key = "range_auto")
  expect_length(guide$params$guides, 3L)

  guide <- guide_axis_nested(key = key_range_auto(sep = "foobar"))
  expect_length(guide$params$guides, 3L)

  guide <- guide_axis_nested(key = key_range_manual(1.0, 2.0))
  expect_length(guide$params$guides, 4L)
  expect_s3_class(guide$params$guides[[3L]], "PrimitiveLabels")

})

# Visual test -------------------------------------------------------------

test_that("guide_axis_nested looks good as axis", {

  base <- ggplot(mpg, aes(interaction(cyl, drv), hwy)) +
    theme_test() +
    theme(
      panel.background = element_rect(fill = NA, colour = "grey80"),
      panel.grid.major = element_line(colour = "grey90"),
      panel.border = element_blank()
    )

  p <- base + guides(
    x = guide_axis_nested(),
    y = guide_axis_nested(
      key = key_range_manual(18.0, 32.0, "Foo"),
      type = "box"
    ),
    y.sec = guide_axis_nested(
      key = key_range_manual(28.0, 42.0, "Bar"),
      regular_key = key_manual(c(20.0, 30.0))
    ),
    x.sec = guide_axis_nested(type = "box", subtitle = "subtitle")
  )

  vdiffr::expect_doppelganger("guide_axis_base cartesian", p)

  p <- base +
    coord_radial(start = 0.25 * pi, end = 1.75 * pi, inner.radius = 0.5) +
    guides(
      theta = guide_axis_nested(),
      r = guide_axis_nested(
        key = key_range_manual(18.0, 32.0, "Foo"),
        type = "box"
      ),
      r.sec = guide_axis_nested(
        key = key_range_manual(28.0, 42.0, "Bar"),
        regular_key = key_manual(c(20.0, 30.0))
      ),
      theta.sec = guide_axis_nested(type = "box")
    )

  vdiffr::expect_doppelganger("guide_axis_base radial", p)
})

# Constructor -------------------------------------------------------------

#' Annotation axis guide
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' @description
#' This axis guide acts as annotation: it draws labels at specified places.
#' It also wraps an inner guide, making the behaviour look like one is 'adding'
#' the annotation on top of the regular guide.
#'
#' @param aesthetic
#' A vector of values for the guide to represent.
#' @param label
#' A `<character[n]>` or list of expressions to use as labels.
#' @param ...
#' Additional graphical properties parallel to `aesthetic`. Can be
#' `text_colour`, `size`, `face`, `hjust`, `vjust`, `angle` or `lineheight` for
#' the labels. Can be `line_colour`, `linewidth` and `linetype` for ticks.
#' Setting `colour` will also set `text_colour` and `line_colour`.
#' For `annotate_top()`, `annotate_right()`, `annotate_bottom()` and
#' `annotate_left()`, arguments are passed on to `guide_axis_annotation()`.
#' @param arrow
#' A [`grid::arrow()`] specification. Can be `NULL` (default) to draw no arrow.
#' @param inner
#' A guide that supports the aesthetic to draw the annotation across.
#' When `waiver()` (default), populates a `guide_axis_base()` except in the
#' `"top"`, `"right"` and `"theta.sec"` positions.
#' @param key
#' A [standard key][key_standard] overriding the `aesthetic`, `label` and `...`
#' arguments.
#' @param theme
#' A [`<theme>`][ggplot2::theme] object to style the *annotation* part of this
#' guide differently from the plot's theme settings. The `theme` argument
#' in this guide overrides and is combined with the plot's theme.
#' @inheritParams common_parameters
#'
#' @details
#' Under the hood, this guide is a hybrid composition guide. The `theme()`
#' options that govern the styling are partially determined by its consituents.
#' They are linked below so you can find their 'Styling options' sections.
#'
#' | **Constituent** | **Description** |
#' | `compose_ontop()` | Composes the annotation on top of the `inner` guide |
#' | `guide_axis_base()` | The default `inner` guide |
#' | `primitive_ticks()` | Display of the annotation ticks |
#' | `primitive_labels()` | Display of the annotation labels |
#'
#' Styling options *per annotation* can be set via `...` as described above.
#'
#' The context-agnostic alternative to using `theme()` is to use
#' [`theme_guide()`]:
#'
#' ```r
#' # Note that `theme` is used *only* for the annotation
#' guide_axis_annotation(theme = theme_guide(
#'   text = element_text(),
#'   ticks = element_line(),
#'   ticks.length = unit(5, "mm")
#' ))
#' ```
#'
#' @returns A `<Guide>` object
#' @export
#' @family standalone guides
#'
#' @examples
#' # Basic plot
#' p <- ggplot(mpg, aes(displ, hwy)) +
#'   geom_point()
#'
#' # Typical use
#' p +
#'   annotate_top(5, face = "bold") +
#'   annotate_bottom(c(2.5, 4.5), c("Bottom annotation", "Second label"))
#'
#' # If you want to combine them with secondary axes, you must
#' # set the `inner` argument manually.
#' p +
#'   scale_y_continuous(
#'     sec.axis = dup_axis(breaks = c(15, 25, 35))
#'   ) +
#'   annotate_right(30, inner = "axis")
#'
#' # Use in theta axis
#' p + coord_radial() +
#'   guides(theta = guide_axis_annotation(4.5, "Theta annotation"))
#'
#' # Specialised use as part of other guides
#' # Note that `guide_colbar` imposes white inward ticks
#' # We can overrule these impositions with a replacement theme
#' p + aes(colour = cty) +
#'   guides(colour = guide_colbar(
#'     second_guide = guide_axis_annotation(22, "This", theme = theme_gray())
#'   ))
guide_axis_annotation <- function(
  aesthetic,
  label = as.character(aesthetic),
  ...,
  key   = NULL,
  arrow = NULL,
  inner = waiver(),
  title = waiver(),
  theme = NULL,
  order = 0L,
  position = waiver(),
  call = NULL
) {
  call <- call %||% current_call()
  key <- key %||% key_manual(aesthetic, label = label, ..., call = call)

  if (!is.null(arrow)) {
    arrow <- theme_guide(ticks = element_line(arrow = arrow))
  }
  if (is_waiver(inner)) {
    inner <- guide_axis_base()
    inner$params$implicit <- TRUE
  }

  guides <- list(
    inner = inner %||% guide_none(),
    annotation = compose_stack(
      ticks  = primitive_ticks(theme = arrow),
      labels = primitive_labels(),
      key = key, title = NULL, theme = theme,
      position = position, side.titles = NULL
    )
  )

  new_compose(
    guides,
    key = "auto",
    title = title,
    theme = theme,
    order = order,
    available_aes = c("any", "x", "y", "r", "theta"),
    position = position,
    super = GuideAxisAnnotation,
    call = call
  )
}

#' @rdname guide_axis_annotation
#' @export
annotate_top <- function(..., position = "top") {
  guides(x.sec = guide_axis_annotation(
    ..., position = position, call = current_call()
  ))
}

#' @rdname guide_axis_annotation
#' @export
annotate_right <- function(..., position = "right") {
  guides(y.sec = guide_axis_annotation(
    ..., position = position, call = current_call()
  ))
}

#' @rdname guide_axis_annotation
#' @export
annotate_bottom <- function(..., position = "bottom") {
  guides(x = guide_axis_annotation(
    ..., position = position, call = current_call()
  ))
}

#' @rdname guide_axis_annotation
#' @export
annotate_left <- function(..., position = "left") {
  guides(y = guide_axis_annotation(
    ..., position = position, call = current_call()
  ))
}

# Class -------------------------------------------------------------------

#' @export
#' @rdname legendry_extensions
#' @format NULL
#' @usage NULL
GuideAxisAnnotation <- ggproto(
  "GuideAxisAnnotation",
  ComposeOntop,

  train = function(self, params, scale, aesthetic = NULL, ...) {
    aesthetic <- aesthetic %||% scale$aesthetics[1]
    if (isTRUE(aesthetic %in% c("x", "y")) &&
        isTRUE(params$position %in% c("top", "right", "theta.sec")) &&
        isTRUE(params$guide_params$inner$implicit)) {
      params$guides$inner <- guide_none()
      params$guide_params$inner <- params$guides$inner$params
    }
    ggproto_parent(ComposeOntop, self)$train(params, scale, aesthetic, ...)
  },

  draw = function(self, theme, position = NULL, direction = NULL,
                  params = self$params) {
    params$postion   <- position  <- params$position  %||% position
    params$direction <- direction <- params$direction %||% direction
    grobs <- build_annotate(params, theme, position, direction)
    if (is_theta(position)) {
      return(grobs)
    }
    self$assemble_drawing(grobs$children, params = params)
  }
)

build_annotate <- function(params, theme, position, direction) {
  stack_offset <- unit(cm(params$stack_offset %||% 0.0), "cm")
  offset <- stack_offset

  inner_guide  <- params$guides$inner
  if (inherits(inner_guide, "GuideNone")) {
    inner_grob <- zeroGrob()
  } else {
    inner_params <- params$guide_params$inner
    inner_params$draw_label <- params$draw_label
    inner_params$stack_offset <- stack_offset
    inner_grob <- inner_guide$draw(
      theme = theme, position = position, direction = direction,
      params = inner_params
    )
  }

  if (!is_zero(inner_grob)) {
    # We need to grab the size of the inner grob. The default `compose_ontop()`
    # only grabs this for theta axes.
    offset <- switch(
      position,
      theta = , theta.sec = inner_grob$offset,
      top = , bottom      = grobHeight(inner_grob),
      left = , right      = grobWidth(inner_grob),
      NULL
    ) %||% offset
  }

  anno_guide <- params$guides$annotation
  if (inherits(anno_guide, "GuideNone")) {
    anno_grob <- zeroGrob()
  } else {
    anno_params <- params$guide_params$annotation
    anno_params$stack_offset <- stack_offset
    if (!is.null(anno_params$guide_params$ticks)) {
      anno_params$guide_params$ticks$force_stretch <- offset
    }
    anno_grob <- anno_guide$draw(
      theme = theme, position = position, direction = direction,
      params = anno_params
    )
  }
  if (!is_zero(anno_grob) && !is.null(anno_grob$offset)) {
    offset <- unit(cm(max(anno_grob$offset, offset)), "cm")
  }
  gTree(
    offset = offset - stack_offset,
    children = gList(inner_grob, anno_grob)
  )
}

test_that("annotate_* family looks allright", {

  p <- ggplot(mpg, aes(drv, displ)) +
    theme_test() +
    theme(
      panel.background = element_rect(fill = NA, colour = "grey80"),
      panel.grid.major = element_line(colour = "grey90"),
      panel.border = element_blank()
    )

  p <- p +
    annotate_bottom(1.5, "Bottom", face = "bold") +
    annotate_top(
      "f", "forward",
      line_colour = "blue"
    ) +
    annotate_right(
      4, colour = "red"
    ) +
    annotate_left(
      c(3, 4, 5), c("three", "four", "five"),
      text_colour = c("forestgreen", "purple", "orange")
    )

  vdiffr::expect_doppelganger("annotate family", p)
})

test_that("gudie_axis_annotation fits in unusual places", {

  p <- ggplot(mtcars, aes(disp, mpg, colour = wt)) +
    geom_point(shape = NA, na.rm = TRUE) +
    coord_radial(start = 0.25 * pi, end = 1.75 * pi, inner.radius = 0.5) +
    theme_test() +
    theme(
      panel.background = element_rect(fill = NA, colour = "grey80"),
      panel.grid.major = element_line(colour = "grey90"),
      panel.border = element_blank()
    )

  p <- p + guides(
    theta = guide_axis_annotation(250, "Outside"),
    theta.sec = guide_axis_annotation(c(200, 400), c("Inside A", "Inside B")),
    r = guide_axis_annotation(15, face = "bold"),
    r.sec = guide_axis_annotation(25, text_colour = "red"),
    colour = guide_colbar(
      second_guide = guide_axis_annotation(3.5, theme = theme_gray(ink = "limegreen"))
    )
  )

  # Suppressing vdiffr's lack of support for gradients from the colbar
  suppressWarnings({
    vdiffr::expect_doppelganger("guide_axis_annotation", p)
  })
})

# Constructor -------------------------------------------------------------

#' Guide gizmo: capped colour steps
#'
#' This guide displays a binned variant of the colour bar with optional caps at
#' either ends of the bar.
#'
#' @param key A [bins key][key_bins] specificiation. Defaults to
#'   `key_bins(even.steps = FALSE, show.limits = NULL)`. Changing the
#'   arguments to `key_bins()` is fine, but changing the key type is not
#'   advised.
#' @inheritParams gizmo_barcap
#'
#' @details
#' ## Styling options
#'
#' Below are the [theme][ggplot2::theme] options that determine the styling of
#' this guide. Note that the width or height (depending on the `direction`
#' argument) *includes* the cap.
#'
#' | **Theme setting** | **Type** | **Description** |
#' | ----------------- | -------- | --------------- |
#' | `legend.frame` | [`element_rect()`] | Frame drawn around the bar and caps. The `fill` setting is ignored. |
#' | `legend.key.width` | [`unit()`] | Width of the bar |
#' | `legend.key.height` | [`unit()`] | Height of the bar |
#'
#' Please note that depending on the `direction` argument, the
#' `legend.key.width`/`legend.key.height` setting are expanded 5-fold if
#' originating from the global theme. To set these directly, you can use the
#' local `theme` argument in the guide.
#' These settings have shorthands in [`theme_guide()`]:
#'
#' ```r
#' gizmo_stepcap(theme = theme_guide(
#'   frame = element_rect(),
#'   key.width = unit(5, "mm")
#'   key.height = unit(5, "cm")
#' ))
#' ```
#'
#' @return A `GizmoStepcap` object.
#' @family gizmos
#' @export
#'
#' @examples
#' # A standard plot
#' p <- ggplot(mpg, aes(displ, hwy, colour = cty)) +
#'   geom_point()
#'
#' # Just some recangles
#' p + scale_colour_viridis_c(guide = gizmo_stepcap())
#'
#' # Caps show up when there is data outside the limits
#' p + scale_colour_viridis_c(
#'   limits = c(10, 30),
#'   guide = gizmo_stepcap()
#' )
#'
#' # The scale's out-of-bounds handler determines cap colour
#' p + scale_colour_viridis_c(
#'   limits = c(10, 30), oob = scales::oob_squish,
#'   guide = gizmo_stepcap()
#' )
#'
#' # Customising the display of the guide
#' p +
#'   scale_colour_viridis_c(
#'     oob = scales::oob_squish,
#'     guide = gizmo_stepcap(
#'       shape = "round", show = c(FALSE, TRUE),
#'       size = unit(1, "cm"),
#'       theme = theme(legend.key.height = unit(4, "cm"))
#'     )
#'   ) +
#'   theme(
#'     legend.frame = element_rect(colour = "black"),
#'     legend.key.width = unit(0.5, "cm")
#'   )
gizmo_stepcap <- function(key = "bins", shape = "triangle", size = NULL,
                          show = NA, alpha = NA, oob = "keep", theme = NULL,
                          position = waiver(), direction = NULL) {
  check_number_decimal(
    alpha, min = 0.0, max = 1.0,
    allow_infinite = FALSE, allow_na = TRUE
  )
  check_logical(show)
  check_length(show, exact = 1L:2L)
  show <- rep_len(show, 2L)

  check_unit(size, allow_null = TRUE)
  shape <- resolve_cap_shape(shape)
  oob <- resolve_oob(oob)

  new_guide(
    key   = key,
    shape = shape,
    size  = size,
    show  = show,
    alpha = alpha,
    oob = oob,
    theme = theme,
    position = position,
    direction = direction,
    available_aes = c("colour", "fill"),
    super = GizmoStepcap
  )
}

# Class -------------------------------------------------------------------

#' @export
#' @rdname legendry_extensions
#' @format NULL
#' @usage NULL
GizmoStepcap <- ggproto(
  "GizmoStepcap", GizmoBarcap,

  elements = list(
    frame  = "legend.frame",
    width  = "legend.key.width",
    height = "legend.key.height"
  ),

  params = new_params(alpha = NA, shape = NULL, show = NA, key = "steps",
                      size = NULL, oob = oob_keep),

  extract_params = function(scale, params, ...) {

    if (scale$is_discrete()) {
      cli::cli_abort(
        "This guide does not (yet) work with discrete scales. \\
        Please defer to {.fn guide_coloursteps}."
      )
    }

    params$position <- params$position %|W|% NULL
    limits <- scale$get_limits()
    range  <- scale$range$range
    aes <- params$aesthetic
    key <- params$key

    lower_oob <- range[1L] < limits[1L]
    upper_oob <- range[2L] > limits[2L]

    params$show[1L] <- !isFALSE(params$show[1L] %|NA|% lower_oob)
    params$show[2L] <- !isFALSE(params$show[2L] %|NA|% upper_oob)
    add <- diff(limits) / 1000.0

    if (params$show[1L]) {
      val <- params$oob(limits[1L] - add, limits)
      limits <- range(limits, val)
      key <- data_frame0(
        !!aes := c(scale$map(val), key[[aes]]),
        min    = c(-Inf, key$min),
        max    = c(key$min[1L], key$max),
        .label = c(NA, key$.label),
        .value = c(NA, key$.value)
      )
    }
    if (params$show[2L]) {
      val <- params$oob(limits[2L] + add, limits)
      limits <- range(limits, val)
      n <- max(which(!is.na(key[[aes]])))
      if (n == nrow(key)) {
        key <- data_frame0(
          !!aes := c(key[[aes]], scale$map(val)),
          min    = c(key$min, key$max[nrow(key)]),
          max    = c(key$max, Inf),
          .label = c(key$.label, NA),
          .value = c(key$.value, NA)
        )
      } else {
        n <- n + 1L
        key[[aes]][n] <- scale$map(val)
        key$min[n] <- key$max[n - 1L]
        key$max[n] <- Inf
      }
    }
    if (grepl("color|colour|fill", params$aesthetic)) {
      key[[aes]] <- alpha(key[[aes]], alpha = params$alpha)
    }
    params$limits <- limits
    params$key <- key
    params
  },

  setup_params = function(params) {
    key <- params$key
    key <- vec_slice(key, !is.na(key[[params$aesthetic]]))
    min <- guide_rescale(key$min, params$limits)
    max <- guide_rescale(key$max, params$limits)
    key$mid <- (max + min) / 2.0
    key$height <- abs(max - min)
    params$key <- key
    params
  },

  fill_frame = function(key, elements, params, grobs = NULL) {
    if (!any(c("colour", "fill") %in% names(key))) {
      return(grobs)
    }
    check_device("clippingPaths")

    lower <- grobs$lower
    upper <- grobs$upper

    min <- unit(0.0, "npc") + lower
    max <- unit(1.0, "npc") + upper
    delta <- unit(1.0, "npc") - (lower + upper)
    n <- nrow(key)

    args <- list(
      x = 0.5,
      y = unit.c(
        unit(0.0, "npc"), min + key$mid[-c(1L, n)] * delta, unit(1.0, "npc")
      ),
      height = unit.c(
        lower + key$height[1L] * delta,
        key$height[-c(1L, n)]  * delta,
        upper + key$height[n] * delta
      ),
      vjust = c(0.0, rep(0.5, nrow(key) - 2L), 1.0),
      gp = gpar(fill = key[[params$aesthetic]], col = NA)
    )

    if (params$direction == "horizontal") {
      args <- flip_names(args)
      args$height <- grobHeight(grobs$grob)
    } else {
      args$width <- grobWidth(grobs$grob)
    }
    args$vp <- viewport(clip = grobs$grob)
    grobs$grob <- editGrob(grobs$grob, gp = gpar(fill = NA))
    grobs$grob <- gTree(children = gList(inject(rectGrob(!!!args)), grobs$grob))
    grobs
  }
)

# Constructors ------------------------------------------------------------

#' Dendrogram scales
#'
#' These are speciality scales for use with hierarchically clustered data. The
#' scale automatically orders the limits according to the clustering result
#' and comes with a [dendrogram axis][guide_axis_dendro()].
#'
#' @param clust A data structure that can be coerced to an
#'   [`<hclust>`][stats::hclust] object through
#'   [`as.hclust()`][stats::as.hclust].
#' @inheritDotParams ggplot2::discrete_scale -scale_name -limits -minor_breaks
#' @inheritParams ggplot2::scale_x_discrete
#'
#' @details
#' The scale limits are determined by the order and labels in the `clust`
#' argument. While `limits` is not an argument in these scales, the `breaks`
#' argument can still be used to selectively omit some breaks and the `labels`
#' can be used for formatting purposes.
#'
#' @return A `<ScaleDiscretePosition>` object that can be added to a plot.
#' @seealso [guide_axis_dendro()]
#' @export
#'
#' @examples
#' # Hierarchically cluster data, separately for rows and columns
#' car_clust <- hclust(dist(scale(mtcars)), "ave")
#' var_clust <- hclust(dist(scale(t(mtcars))), "ave")
#'
#' long_mtcars <- data.frame(
#'   car = rownames(mtcars)[row(mtcars)],
#'   var = colnames(mtcars)[col(mtcars)],
#'   value = as.vector(scale(mtcars))
#' )
#'
#' # A standard heatmap adorned with dendrograms
#' p <- ggplot(long_mtcars, aes(var, car, fill = value)) +
#'   geom_tile() +
#'   scale_x_dendro(var_clust) +
#'   scale_y_dendro(car_clust)
#' p
#'
#' # Styling the dendrograms
#' p +
#'   guides(
#'     y = guide_axis_dendro(key_dendro(type = "triangle")),
#'     x = guide_axis_dendro(space = rel(5))
#'   ) +
#'   theme(
#'     axis.text.y.left = element_text(margin = margin(r = 3, l = 3)),
#'     axis.ticks.y = element_line("red"),
#'     axis.ticks.x = element_line(linetype = "dotted")
#'   )
#'
#' # In polar coordinates, plus some formatting
#' p +
#'   coord_radial(
#'     theta = "y", inner.radius = 0.5,
#'     start = 0.25 * pi, end = 1.75 * pi
#'   ) +
#'   guides(
#'     theta = primitive_labels(angle = 90),
#'     theta.sec = primitive_segments("dendro", vanish = TRUE),
#'     r = guide_axis_dendro(angle = 0)
#'   )
scale_x_dendro <- function(clust, ..., expand = waiver(), guide = "axis_dendro",
                           position = "bottom") {

  clust  <- validate_clust(clust)
  limits <- validate_clust_limits(clust)

  args <- list2(...)
  check_dendro_args(args)
  # Pre-validating guide here in case legendry is not loaded (#94)
  guide <- validate_guide(guide)

  sc <- inject(discrete_scale(
    aesthetics = c(
      "x", "xmin", "xmax", "xend", "xintercept",
      "xmin_final", "xmax_final", "xlower", "xmiddle", "xupper", "x0"
    ),
    palette = seq_len,
    scale_name = missing_arg(),
    limits = limits,
    !!!args,
    guide = guide,
    expand = expand,
    position = position,
    super = ScaleDiscretePosition
  ))
  sc$range_c <- ContinuousRange$new()
  sc$clust <- clust
  sc
}

#' @rdname scale_x_dendro
#' @export
scale_y_dendro <- function(clust, ..., expand = waiver(), guide = "axis_dendro",
                           position = "left") {
  clust  <- validate_clust(clust)
  limits <- validate_clust_limits(clust)

  args <- list2(...)
  check_dendro_args(args)
  # Pre-validating guide here in case legendry is not loaded (#94)
  guide <- validate_guide(guide)

  sc <- inject(discrete_scale(
    aesthetics = c(
      "y", "ymin", "ymax", "yend", "yintercept", "ymin_final", "ymax_final",
      "lower", "middle", "upper", "y0"
    ),
    palette = seq_len,
    scale_name = missing_arg(),
    limits = limits,
    !!!args,
    guide = guide,
    expand = expand,
    position = position,
    super = ScaleDiscretePosition
  ))
  sc$range_c <- ContinuousRange$new()
  sc$clust <- clust
  sc
}


# Helpers -----------------------------------------------------------------

validate_clust <- function(clust, .call = caller_env()) {
  try_fetch(
    stats::as.hclust(clust),
    error = function(cnd) {
      cli::cli_abort(
        "The {.arg clust} argument should be convertable to an {.cls hclust}
        object.", parent = cnd, call = .call
      )
    }
  )
}

validate_clust_limits <- function(clust, .call = caller_env()) {
  labels <- clust$labels %||% seq_along(clust$order)
  labels <- labels[clust$order]
  function(x) {
    union(labels, x)
  }
}

check_dendro_args <- function(args, .call = caller_env()) {
  if ("limits" %in% names(args)) {
    cli::cli_abort(
      "Function does not accept {.arg limits} argument: it is derived from the
      labels and order in the {.arg clust} argument.",
      call = .call
    )
  }
  if ("palette" %in% names(args)) {
    cli::cli_abort(
      "Function does not accept {.arg palette} argument: the scale requires
      fixed spacing between items.",
      call = .call
    )
  }
  invisible()
}

test_clust <- function(n) {
  m <- matrix(NA_real_, nrow = n, ncol = n)
  m[lower.tri(m)] <- seq_len(sum(lower.tri(m)))
  hclust(as.dist(m), method = "ave")
}

test_that("scale_xy_dendro throw appropriate error messages", {

  clust <- test_clust(3L)

  expect_error(
    scale_x_dendro(NULL),
    "argument should be convertable"
  )
  expect_error(
    scale_y_dendro(clust, limits = 1L:5L),
    "it is derived from the labels"
  )
  expect_error(
    scale_x_dendro(clust, palette = 1L:5L),
    "requires fixed spacing"
  )

})

test_that("scale_xy_dendro looks correct", {

  xclust <- test_clust(3L)
  yclust <- test_clust(4L)

  base <- ggplot() +
    geom_point(aes(x = "2", y = "3"), shape = NA, na.rm = TRUE) +
    scale_x_dendro(xclust) +
    scale_y_dendro(yclust) +
    theme(
      panel.background = element_rect(fill = NA, colour = "grey80"),
      panel.grid.major = element_line(colour = "grey90"),
      panel.border = element_blank(),
      axis.line = element_line()
    )

  p <- base +
    coord_cartesian(xlim = c(0.0, 4.0), ylim = c(1.0, 4.0)) +
    guides(
      x.sec = guide_axis_dendro(key = key_dendro(type = "triangle")),
      y.sec = guide_axis_dendro(
        space = rel(5.0),
        ticks = "ticks",
        axis_line = "line"
      )
    )

  vdiffr::expect_doppelganger("scale_dendro cartesian", p)

  p <- base +
    expand_limits(x = c(0.0, 4.0), y = c(1.0, 4.0)) +
    coord_radial(start = 0.25 * pi, end = 1.75 * pi, inner.radius = 0.5) +
    guides(
      r.sec = guide_axis_dendro(
        key = key_dendro(type = "triangle"),
        ticks = "ticks",
        axis_line = "line"
      ),
      theta.sec = guide_axis_dendro(ticks = "ticks", axis_line = "line")
    )

  vdiffr::expect_doppelganger("scale_dendro radial", p)

})

# Constructor -------------------------------------------------------------

#' Compose guides on top of one another
#'
#' @description
#' This guide can place other guides on top of one another.
#'
#' @inheritParams compose_stack
#'
#' @return A `<ComposeOntop>` composite guide object.
#' @export
#' @family composition
#'
#' @details
#' ## Styling options
#'
#' There are no styling options in `theme()` for this composition.
#'
#'
#' @examples
#' # Using the ontop composition to get two types of ticks with different
#' # lengths
#' ggplot(mpg, aes(displ, hwy)) +
#'   geom_point() +
#'   guides(x = compose_ontop(
#'     guide_axis_base(
#'       key_manual(c(2, 4, 6)),
#'       theme = theme(
#'         axis.ticks = element_line(colour = "limegreen"),
#'         axis.ticks.length = unit(11, "pt")
#'       )
#'     ),
#'     guide_axis_base(
#'       key_manual(c(3, 5, 7)),
#'       theme = theme(
#'         axis.ticks = element_line(colour = "tomato"),
#'         axis.ticks.length = unit(5.5, "pt")
#'       )
#'     )
#'   ))
compose_ontop <- function(
  ..., args = list(),
  key = NULL, title = waiver(),
  angle = waiver(), theme = NULL, order = 0L,
  position = waiver(), available_aes = NULL
) {
  new_compose(
    guides = list2(...),
    title = title,
    theme = theme,
    key = key,
    angle = angle,
    available_aes = available_aes,
    order = order,
    position = position,
    name = "ontop_composition",
    super = ComposeOntop
  )
}

# Class -------------------------------------------------------------------

#' @export
#' @rdname legendry_extensions
#' @format NULL
#' @usage NULL
ComposeOntop <- ggproto(
  "ComposeOntop", Compose,

  draw = function(self, theme, position = NULL, direction = NULL,
                  params = self$params) {
    theme <- theme + params$theme

    params$position  <- position  <- params$position  %||% position
    params$direction <- direction <- params$direction %||% direction

    n_guides <- length(params$guides)
    guide_index <- seq_len(n_guides)
    grobs <- vector("list", n_guides)

    if (is_theta(position)) {
      stack_offset <- unit(cm(params$stack_offset %||% 0.0), "cm")
      offset <- stack_offset

      for (i in guide_index) {
        pars <- params$guide_params[[i]]
        pars$stack_offset <- stack_offset
        grob <- params$guides[[i]]$draw(
          theme = theme, position = position, direction = direction,
          params = pars
        )
        if (!is.null(grob$offset) && !is_zero(grob)) {
          offset <- unit(cm(max(grob$offset, offset)), "cm")
        }
        grobs[[i]] <- grob
      }
      keep <- !map_lgl(grobs, is_zero)
      if (!any(keep)) {
        return(zeroGrob())
      }
      offset <- offset - stack_offset
      grobs <- gTree(offset = offset, children = inject(gList(!!!grobs[keep])))
      return(grobs)
    }

    draw_label <- params$draw_label %||% TRUE

    for (i in guide_index) {
      pars <- params$guide_params[[i]]
      pars$draw_label <- draw_label
      grobs[[i]] <- params$guides[[i]]$draw(
        theme = theme, position = position, direction = direction, params = pars
      )
    }

    self$assemble_drawing(grobs, params = params)
  },

  assemble_drawing = function(grobs, layout = NULL, sizes = NULL, params, elements = NULL) {
    keep <- !map_lgl(grobs, is_zero)
    grobs <- grobs[keep]
    if (length(grobs) == 0L) {
      return(zeroGrob())
    }

    position <- params$position
    anchor <- unit(as.numeric(position %in% c("left", "bottom")), "npc")
    just <- opposite_position(position)

    widths  <- width_cm(grobs)
    heights <- height_cm(grobs)

    index <- seq_along(grobs)
    names <- paste0("guide-ontop", index)

    if (position %in% c("bottom", "top")) {
      height <- unit(max(heights), "cm")
      gt <- gtable(widths = unit(1.0, "npc"), heights = height) |>
        gtable_add_grob(grobs, t = 1L, l = 1L, name = names, clip = "off")
      vp <- viewport(y = anchor, height = height, just = just)
    } else {
      width <- unit(max(widths), "cm")
      gt <- gtable(widths = width, heights = unit(1.0, "npc")) |>
        gtable_add_grob(grobs, t = 1L, l = 1L, name = names, clip = "off")
      vp <- viewport(x = anchor, width = width, just = just)
    }

    absoluteGrob(
      grob   = gList(gt), vp = vp,
      width  = gtable_width(gt),
      height = gtable_height(gt)
    )
  }
)

# Group keys --------------------------------------------------------------

#' Group keys
#'
#' @description
#' These functions are helper functions for working with grouped data as keys in
#' guides. They all share the goal of creating a guide key, but have different
#' methods.
#'
#' * `key_group_split()` is a function factory whose functions make an attempt
#'   to infer groups from the scale's labels.
#' * `key_group_lut()` is a function factory whose functions use a look up table
#'   to sort out group membership.
#'
#' @param sep A `<character[1]>` giving a [regular expression][base::regex] to
#'   use for splitting labels provided by the scale using the
#'   [`strsplit()`][base::strsplit] function. By defaults, labels are splitted
#'   on any non-alphanumeric character.
#' @param reverse A `<logical[1]>` which if `FALSE` (default) treats the first
#'   part of the split string as groups and later parts as members. If `TRUE`,
#'   treats the last part as groups.
#' @param members A vector including the scale's `breaks` values.
#' @param group A vector parallel to `members` giving  the group of each member.
#' @param ungrouped A `<character[1]>` giving a group label to assign to the
#'   scale's `breaks` that match no values in the `members` argument.
#'
#' @name key_group
#' @family keys
#' @return
#' A function to use as the `key` argument in a guide.
#'
#' @examples
#' # Example scale
#' values <- c("group A:value 1", "group A:value 2", "group B:value 1")
#' template <- scale_colour_hue(limits = values)
#'
#' # Treat the 'group X' part as groups
#' key <- key_group_split(sep = ":")
#' key(template)
#'
#' # Treat the 'value X' part as groups
#' key <- key_group_split(sep = ":", reverse = TRUE)
#' key(template)
#'
#' # Example scale
#' template <- scale_colour_hue(limits = msleep$name[c(1, 7, 9, 23, 24)])
#'
#' # A lookup table can have more entries than needed
#' key <- key_group_lut(msleep$name, msleep$order)
#' key(template)
#'
#' # Or less entries than needed
#' key <- key_group_lut(
#'   msleep$name[23:24], msleep$order[23:24],
#'   ungrouped = "Other animals"
#' )
#' key(template)
NULL

#' @rdname key_group
#' @export
key_group_split <- function(sep = "[^[:alnum:]]+", reverse = FALSE) {
  check_string(sep)
  check_bool(reverse)
  call <- current_call()
  function(scale, aesthetic = NULL) {
    group_from_split_label(
      scale = scale, aesthetic = aesthetic,
      sep = sep, reverse = reverse,
      call = call
    )
  }
}

#' @rdname key_group
#' @export
key_group_lut <- function(members, group, ungrouped = "Other") {
  check_string(ungrouped)
  check_unique(members)
  if (length(group) != length(members)) {
    cli::cli_abort(c(
      "{.arg group} must have the same length as {.arg members}.",
      i = "{.arg group} has length {length(group)}.",
      i = "{.arg members} has length {length(members)}."
    ))
  }
  lut <- vec_split(members, group)

  function(scale, aesthetic = NULL) {
    group_from_lut(
      scale = scale, aesthetic = aesthetic,
      lut = lut, ungrouped = ungrouped
    )
  }
}

# Helpers -----------------------------------------------------------------

group_from_split_label <- function(scale, aesthetic, sep = "[^[:alnum:]]+",
                                   reverse = FALSE, call = caller_env()) {

  # Extract a standard key from the scale
  aesthetic <- aesthetic %||% scale$aesthetics[1L]
  key <- Guide$extract_key(scale, aesthetic)

  # Reject expressions, as we cannot split these
  if (!is.character(key$.label)) {
    type <- obj_type_friendly(key$.label)
    cli::cli_abort(
      c("Cannot split the guide's {.field label}.",
        i = "It must be a {.cls character} vector, not {type}."),
      call = call
    )
  }

  # Split labels
  labels <- strsplit(key$.label, sep)
  if (isTRUE(reverse)) {
    i <- lengths(labels)
  } else {
    i <- rep(1L, length(labels))
  }

  groups <- vec_c(!!!Map(vec_slice, i = i, x = labels))
  labels <- lapply(Map(vec_slice, i = -i, x = labels), paste0, collapse = " ")
  labels <- vec_c(!!!labels)

  key$.label <- factor(labels, unique0(labels))
  key$.group <- factor(groups, unique0(groups))
  vec_slice(key, order(key$.group, key$.label))
}

group_from_lut <- function(scale, aesthetic, lut, ungrouped = "Other") {

  # Extract a standard key from the scale
  aesthetic <- aesthetic %||% scale$aesthetics[1L]
  key <- Guide$extract_key(scale, aesthetic)

  group <- lut$key[match_list(key$.value, lut$val)] %|NA|% ungrouped
  if (!is.factor(group)) {
    group <- factor(group, c(setdiff(unique(group), ungrouped), ungrouped))
  }
  key$.group <- group
  vec_slice(key, order(group))
}

test_that("key_group_split works correctly", {

  scale <- scale_colour_hue()
  scale$train(c("A:B", "C:D", "E:F"))

  # Standard case
  test <- key_group_split(sep = ":")(scale, "colour")
  expect_identical(
    test[c(".label", ".group")],
    data.frame(
      .label = factor(c("B", "D", "F")),
      .group = factor(c("A", "C", "E"))
    )
  )

  # Test reverse argument
  test <- key_group_split(sep = ":", reverse = TRUE)(scale, "colour")
  expect_identical(
    test[c(".label", ".group")],
    data.frame(
      .label = factor(c("A", "C", "E")),
      .group = factor(c("B", "D", "F"))
    )
  )

  # Missing label
  scale <- scale_colour_hue()
  scale$train(c("A", "C:D", "E:F"))

  test <- key_group_split(sep = ":")(scale, "colour")
  expect_identical(
    test[c(".label", ".group")],
    data.frame(
      .label = factor(c("", "D", "F")),
      .group = factor(c("A", "C", "E"))
    )
  )

  # Too many labels
  scale <- scale_colour_hue()
  scale$train(c("A:B", "C:D", "E:F:G"))

  test <- key_group_split(sep = ":")(scale, "colour")
  expect_identical(
    test[c(".label", ".group")],
    data.frame(
      .label = factor(c("B", "D", "F G")),
      .group = factor(c("A", "C", "E"))
    )
  )

  # Expression labels
  scale <- scale_colour_hue(labels = expression(A, B, C))
  scale$train(c("A", "B", "C"))
  expect_snapshot(
    key_group_split(sep = ":")(scale, "colour"),
    error = TRUE
  )
})

test_that("key_group_lut works as intended", {

  levels <- c("Coffee", "Tea", "Soda", "Water")
  groups <- rep(c("Hot drinks", "Cold drinks"), each = 2L)

  sc <- scale_colour_hue()
  sc$train(levels)
  sc$train("Car")

  key <- key_group_lut(levels,  groups)
  test <- key(sc, "colour")

  expect_identical(test$.label, c(levels, "Car"))
  expect_identical(
    test$.group,
    factor(c(groups, "Other"), unique(c(groups, "Other")))
  )

  # Mismatched lengths
  levels <- c("A", "B")
  groups <- c("X", "X", "Y")
  expect_error(
    key_group_lut(levels, groups),
    "must have the same length"
  )
})

# Constructor -------------------------------------------------------------

#' Side-plot axis
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' Displays an axis-sharing plot to the side of the panel.
#'
#' @param plot
#' A `<ggplot>` object, subject to limitations listed in the 'Details' section.
#' Alternatively, a `<function>` that takes the scale as argument and returns
#' a `<ggplot>` object.
#'
#' @param title
#' One of the following to indicate the title of the guide:
#' * A `<character[1]>` or `<expression[1]>` to set a custom title.
#' * `NULL` (default) to not display any title.
#' * [`waiver()`][ggplot2::waiver()] to take the name of the scale
#'   object or the name specified in [`labs()`][ggplot2::labs] as the title.
#' Please note that `plot` will still display a title unless instructed
#' otherwise. To avoid duplicated titles, the default is to have no title for
#' the guide.
#'
#' @param size
#' An absolute [`<unit>`][grid::unit] to set the size of the `plot` panel in
#' the orthogonal direction.
#'
#' @param reposition
#' A `<logical[1]>`. If `TRUE` (default) the `position` argument of this guide
#' will be propagated to the position scale bestowed upon the `plot` argument.
#' If `FALSE`, that position scale will retain its original `position` field.
#' Setting `reposition = TRUE` will generally tend to point axes outwards.
#'
#' @param theme
#' A [`<theme>`][ggplot2::theme] object to style the guide individually or
#' differently from the plot's theme settings. The order in which themes are
#' applies is as follows: (1) the main plot's theme (2) the `plot` argument's
#' theme and (3) this `theme` argument. The default `theme` argument suppresses
#' legends.
#'
#' @inheritParams common_parameters
#'
#' @details
#' This guide is subject to the following limitations:
#' * The x- or y-scale of the main plot override the corresponding scale in the
#'   `plot` argument. This ensures that the scales line up. The `plot` argument
#'   should not have the relevant scale.
#' * The `plot` argument cannot have custom facets. It must use the default
#'   `facet_null()`.
#' * This guide cannot be used in non-linear coordinate systems of the main plot
#'   and does not support non-linear coordinate systems in the `plot` argument.
#' * The `theme(panel.widths, panel.heights)` setting in the `plot` argument
#'   will be ignored in favour of the `size` argument.
#' * There is no mechanism to accommodate extra space needed by plot components
#'   outside the panel. This applies in the horizontal direction for x-axes and
#'   the vertical direction for y-axes. You may need to manually tweak the
#'   `theme(plot.margin)` setting of the main plot to accommodate these
#'   components.
#'
#' ## Styling options
#' This guide has no style options in `theme()` on itself. However, the
#' `plot` herein is subject to styling as described under the `theme` argument.
#'
#' @returns A `<Guide>` object.
#' @export
#' @family standalone guides
#'
#' @examples
#' # A standard plot
#' main_plot <- ggplot(mpg, aes(displ, hwy, colour = drv)) +
#'   geom_point()
#'
#' # Simple plot sharing the x-variable
#' x_plot <- ggplot(mpg, aes(displ, fill = drv)) +
#'   geom_density(alpha = 0.7)
#'
#' # Simple plot sharing the y-variable
#' y_plot <- ggplot(mpg, aes(drv, hwy, colour = drv)) +
#'   geom_boxplot()
#'
#' # Typical use
#' main_plot + guides(
#'   x = guide_axis_plot(x_plot),
#'   y = guide_axis_plot(y_plot)
#' )
#'
#' main_plot + guides(
#'   # Include `fill` legend by overriding theme
#'   x = guide_axis_plot(x_plot, theme = NULL),
#'   # Change the size of the side-plot
#'   y = guide_axis_plot(y_plot, size = unit(4, "cm"))
#' )
#'
#' # Components outside panels may need to be manually acommodated
#' main_plot +
#'   guides(y = guide_axis_plot(y_plot + labs(title = "Boxplot"))) +
#'   theme(plot.margin = margin(25, 5.5, 5.5, 5.5))
#'
#' # Recursive use of this guide
#' main_plot + guides(x = guide_axis_plot(
#'   main_plot + guides(x = guide_axis_plot(x_plot))
#' ))
guide_axis_plot <- function(
  plot, title = NULL, size = unit(2.0, "cm"), reposition = TRUE,
  theme = theme_sub_legend(position = "none"), position = waiver()
) {

  if (!is_function(plot)) {
    check_side_plot(plot)
  }
  check_bool(reposition)
  check_unit(size)

  new_guide(
    plot = plot,
    title = title,
    size = size,
    reposition = reposition,
    theme = theme,
    position = position,
    available_aes = c("x", "y"),
    super = GuideAxisPlot
  )
}

# Class -------------------------------------------------------------------

#' @export
#' @rdname legendry_extensions
#' @format NULL
#' @usage NULL
GuideAxisPlot <- ggproto(
  "GuideAxisPlot", Guide,

  params = new_params(plot = NULL, size = unit(0.2, "null"), reposition = TRUE),

  hashables = exprs(plot),

  extract_params = function(scale, params, ...) {

    scale_copy <- scale$make_fixed_copy()

    if (params$reposition) {
      # We can modify-in-place here because the scale is already a copy
      scale_copy$position <- params$position
    }

    if (is_function(params$plot)) {
      params$plot <- params$plot(scale_copy)
      check_side_plot(params$plot, arg = "plot")
    }

    params$plot <- params$plot + scale_copy
    params
  },

  transform = function(params, coord, ...) {
    if (!coord$is_linear()) {
      cli::cli_abort(
        "Non-linear coordinates do not support {.fn guide_axis_plot}."
      )
    }
    main_reverse <- coord$reverse %||% "none"
    side_reverse <- params$plot@coordinates$reverse %||% "none"
    new_reverse  <- "none"

    main <- params$aesthetic[1L]
    oppo <- setdiff(c("x", "y"), main)

    # Let's presume this is an x-axis,
    # we follow the following scheme
    #
    #         main plot -->
    #        |none|x   |y   |xy
    # g |----|----|----|----|----
    # u |none|none| x  |none| x
    # i |----|----|----|----|----
    # d | x  |none| x  |none| x
    # e |----|----|----|----|----
    # | | y  | y  | xy | y  | xy
    # V |----|----|----|----|----
    #   | xy | y  | xy | y  | xy
    #
    # When this is a y-axis,
    # swap the 'x' and 'y'

    if (main_reverse %in% c("none", oppo)) {
      if (side_reverse %in% c("none", main)) {
        new_reverse <- "none"
      } else {
        # guide_reveral %in% c("xy", oppo)
        new_reverse <- oppo
      }
    } else if (side_reverse %in% c("none", main)) {
      # main_reverse %in% c("xy", main)
      new_reverse <- main
    } else {
      # guide_reverse %in%
      new_reverse <- "xy"
    }

    if (new_reverse != side_reverse) {

      old_coords <- params$plot@coordinates
      coord_cls <- snake_class(old_coords)

      cli::cli_warn(c(
        "The {.code {coord_cls}(reverse = {.val {side_reverse}})} setting \\
        in side-plot is {.emph incompatible} with the \\
        main plot ({.val {main_reverse}}).",
        i = "Revising to {.code {coord_cls}(reverse = {.val {new_reverse}})}."
      ), call = expr(guide_axis_plot(plot)))

      params$plot@coordinates <- ggproto(
        NULL, old_coords,
        reverse = new_reverse
      )
    }

    params
  },

  draw = function(self, theme, position = NULL, direction = NULL,
                  params = self$params) {

    params <- replace_null(params, position = position, direction = direction)

    plot <- params$plot

    # Unify themes and apply
    theme <- theme + plot@theme + params$theme
    theme <- theme + switch(
      params$position,
      top = , bottom = theme(panel.heights = params$size, panel.widths  = NULL),
      left = , right = theme(panel.widths  = params$size, panel.heights = NULL),
      NULL
    )

    plot@theme <- theme
    plot <- ggplotGrob(plot)

    if (params$position %in% c("top", "bottom")) {

      panels <- panel_cols(plot)
      mid    <- plot[, (panels$l[1L]:panels$r[1L])]

      # Prepare plot parts to the left of the main panel
      left <- plot[, idx_before(ncol(plot), panels$l[1L])]
      left_vp <- viewport(
        just = "right", x = 1.0,
        width = unit(sum(width_cm(left$widths)), "cm")
      )
      left <- editGrob(left, vp = left_vp)

      # Perpare plot parts to the right of the main panel
      right <- plot[, idx_after(ncol(plot), panels$r[1L])]
      right_vp <- viewport(
        just = "left", x = 0.0,
        width = unit(sum(width_cm(right$widths)), "cm")
      )
      right <- editGrob(right, vp = right_vp)

      # Reassemble in gtable where only middle part has width
      height <- sum(plot$heights)
      gt <- gtable(widths = unit(c(0.0, 1.0, 0.0), "npc"), heights = height) |>
        gtable_add_grob(
          mid, t = 1L, l = 2L,
          clip = "off", name = "plot"
        ) |>
        gtable_add_grob(
          left, t = 1L, l = 1L,
          clip = "off", name = "left-decor"
        ) |>
        gtable_add_grob(
          right, t = 1L, l = 3L,
          clip = "off", name = "right-decor"
        ) |>
        gList() |>
        absoluteGrob(height = height)

    } else {

      panels <- panel_rows(plot)
      mid    <- plot[(panels$t[1L]:panels$b[1L])]

      # Prepare plot parts above the main panel
      top <- plot[idx_before(nrow(plot), panels$t[1L]), ]
      top_vp <- viewport(
        just = "bottom", y = 0.0,
        height = unit(sum(height_cm(top$heights)), "cm")
      )
      top <- editGrob(top, vp = top_vp)

      # Prepare plot parts below the main panel
      bottom <- plot[idx_after(nrow(plot), panels$b[1L]),  ]
      bottom_vp <- viewport(
        just = "top", y = 1.0,
        height = unit(sum(height_cm(bottom$heights)), "cm")
      )
      bottom <- editGrob(bottom, vp = bottom_vp)

      width <- sum(plot$widths)
      gt <- gtable(widths = width, heights = unit(c(0.0, 1.0, 0.0), "npc")) |>
        gtable_add_grob(
          mid, t = 2L, l = 1L,
          clip = "off", name = "plot"
        ) |>
        gtable_add_grob(
          top, t = 1L, l = 1L,
          clip = "off", name = "top-decor"
        ) |>
        gtable_add_grob(
          bottom, t = 3L, l = 1L,
          clip = "off", name = "bottom-decor"
        ) |>
        gList() |>
        absoluteGrob(width = width)
    }
    gt
  }

)

# Helpers -----------------------------------------------------------------

check_side_plot <- function(x, arg = caller_arg(x), call = caller_env()) {
  check_object(x, is_ggplot, "a {.cls ggplot} object", arg = arg, call = call)

  if (!inherits(x@facet, "FacetNull")) {
    facet_class <- class(x@facet)[1L]
    cli::cli_abort(c(
      "The {.arg {arg}} argument cannot have facets of class \\
      {.cls {facet_class}}.",
      i = "Only {.fn facet_null} is supported."
    ), call = call)
  }
  if (!x@coordinates$is_linear()) {
    cli::cli_abort(
      "The {.arg {arg}} argument cannot have a non-linear coordinate system.",
      call = call
    )
  }
  invisible()
}

test_that("guide_axis_plot() can be rendered", {

  x_plot <- ggplot(mtcars, aes(disp, factor(cyl), colour = factor(cyl))) +
    geom_boxplot() +
    guides(y.sec = "axis")

  y_plot <- ggplot(mtcars, aes(y = mpg, fill = factor(cyl))) +
    geom_histogram(bins = 10L, na.rm = TRUE) +
    guides(x.sec = "axis")

  p <- ggplot(mtcars, aes(disp, mpg, colour = factor(cyl))) +
    geom_point() +
    guides(
      x     = guide_axis_plot(x_plot),
      x.sec = guide_axis_plot(x_plot, reposition = FALSE),
      y     = guide_axis_plot(y_plot + labs(title = "Title")),
      y.sec = guide_axis_plot(y_plot + labs(caption = "Caption"))
    )

  vdiffr::expect_doppelganger("guide_axis_plot", p)
})

test_that("guide_axis_plot() rejects incompatible plots", {

  expect_snapshot(
    guide_axis_plot(plot = "foobar"),
    error = TRUE
  )

  plot <- ggplot(mtcars, aes(disp, mpg)) + geom_point()

  expect_snapshot(
    guide_axis_plot(plot + coord_radial()),
    error = TRUE
  )

  expect_snapshot(
    guide_axis_plot(plot + facet_wrap(~ cyl)),
    error = TRUE
  )

  wrong_plot <- plot +
    coord_transform(x = "sqrt") +
    scale_x_continuous(guide = guide_axis_plot(plot))

  expect_snapshot(
    ggplot_build(wrong_plot),
    error = TRUE
  )
})

is_blank <- function(x) is.null(x) || is_theme_element(x, "blank")

is_asis <- function(x) inherits(x, "AsIs")

.in2cm <- 2.54

match_self <- function(x) {
  match(x, sort(unique(x)))
}

eval_aes <- function(
  data, mapping,
  required = character(),
  optional = character(),
  call     = caller_env(),
  arg_mapping = caller_arg(mapping),
  arg_data    = caller_arg(data)
) {
  valid <- c(optional, required)
  call <- call %||% current_call()
  if (!is_mapping(mapping)) {
    cli::cli_abort(
      "{.arg {arg_mapping}} must be created by {.fn aes}.",
      call = call
    )
  }
  data <- fortify(data)

  values <- lapply(mapping, eval_tidy, data = data)
  sizes  <- list_sizes(values)
  values <- values[sizes > 0.0]

  if (!is.null(valid)) {
    extra_nms <- setdiff(names(values), valid)
    if (length(extra_nms) > 0L) {
      cli::cli_warn(
        "Ignoring unknown aesthetics: {.field {extra_nms}}.",
        call = call
      )
    }
    values <- values[intersect(names(values), valid)]
    sizes  <- list_sizes(values)
  }

  if (length(sizes) == 0L) {
    cli::cli_warn(
      "No valid data found with {.arg {arg_mapping}} in {.arg {arg_data}}.",
      call = call
    )
    return(data_frame0())
  }
  df <- data_frame0(
    !!!values, .size = max(sizes),
    .error_call = call
  )

  check_columns(df, required, arg = "key", call = call)

  df
}

`%|NA|%` <- function(x, y) {
  if (length(x) == 0L) {
    return(y)
  }
  if (length(y) == 1L) {
    x[is.na(x)] <- y
  } else {
    x[is.na(x)] <- y[is.na(x)]
  }
  x
}

get_attr <- function(x, which, default = NULL) {
  attr(x, which = which, exact = TRUE) %||% default
}

get_size_attr <- function(x, default = 0.0) {
  get_attr(x, "size", default = default)
}

get_width_attr <- function(x, default = 0.0) {
  get_attr(x, "width", default = default)
}

get_height_attr <- function(x, default = 0.0) {
  get_attr(x, "height", default = default)
}

pad <- function(x, length, fill = NA, where = "end") {
  padding <- rep(fill, length - length(x))
  switch(where, start = c(padding, x), c(x, padding))
}

rep0 <- function(x, ...) {
  if (is.null(x)) {
    return(NULL)
  }
  rep(x, ...)
}

recode <- function(x, old, new) {
  i <- match(x, old)
  skip <- is.na(i)
  x[!skip] <- new[i][!skip]
  x
}

rename <- function(df, old, new) {
  if (is.function(new)) {
    new <- new(old)
  }
  names(df) <- recode(names(df), old, new)
  df
}

.flip_names <-
  c(x = "y", y = "x", width = "height", height = "width", hjust = "vjust",
    vjust = "hjust", margin_x = "margin_y", margin_y = "margin_x",
    x1 = "y1", x2 = "y2", y1 = "x1", y2 = "x2")

flip_names <- function(x) {
  rename(x, .flip_names, names(.flip_names))
}

is_discrete <- function(x) {
  is.factor(x) || is.character(x) || is.logical(x) || is_mapped_discrete(x)
}

is_oob <- function(x, limits) {
  limits <- sort(limits)
  x < limits[1L] | x > limits[2L]
}

in_range <- function(x, range) {
  range <- sort(range)
  x >= range[1L] & x <= range[2L]
}

in_ranges <- function(x, start, end) {
  lower <- pmin(start, end)
  upper <- pmax(start, end)
  smaller <- outer(lower, x, FUN = "<")
  larger  <- outer(upper, x, FUN = ">")
  colSums(larger & smaller) > 0L
}

polar_xy <- function(data, r, theta, bbox) {
  data$x <- rescale(r * sin(theta) + 0.5, from = bbox$x)
  data$y <- rescale(r * cos(theta) + 0.5, from = bbox$y)
  data
}

scale_transform <- function(x, scale, map = FALSE, arg = caller_arg(x)) {
  if (is_discrete(x) && !scale$is_discrete()) {
    cli::cli_abort(
      "The key {.field {arg}} must be {.emph continuous}, not discrete."
    )
  }
  if (is_asis(x)) {
    return(x)
  }
  transform <- scale$get_transformation()
  if (is.null(transform)) {
    if (map) {
      x <- (scale$scale$map %||% scale$map)(x)
    }
    return(x)
  }
  x <- transform$transform(x)
  if (map) {
    x <- scale$map(x)
  }
  x
}

cm <- function(x) {
  if (!is.unit(x)) {
    return(x)
  }
  convertUnit(x, "cm", valueOnly = TRUE)
}

new_rle <- function(x) {
  rle <- vec_unrep(x)
  rle$end   <- cumsum(rle$times)
  rle$start <- rle$end - rle$times + 1L
  rle
}

suffix_position <- function(value, position) {

  aesthetic <- switch(position, left = , right = "y", "x")
  position  <- switch(position, theta = "bottom", theta.sec = "top", position)
  suffix <- paste0(".", aesthetic, ".", position)

  char <- map_lgl(value, is.character) & !map_lgl(value, is_asis)
  value[char] <- lapply(value[char], paste0, suffix)
  value
}

is_theta <- function(x) {
  if (is_missing(x) || !is.character(x)) {
    return(FALSE)
  }
  x %in% c("theta", "theta.sec")
}

# Based on example in ?vctrs::vec_chop
# It's faster than stats::ave
vec_ave <- function(x, group, fun, ...) {
  index   <- vec_group_loc(group)$loc
  chopped <- lapply(vec_chop(x, indices = index), fun, ...)
  list_unchop(chopped, indices = index)
}

by_group <- function(x, group, fun, ..., value = x[1L]) {
  index <- vec_group_loc(group)$loc
  vapply(vec_chop(x, indices = index), FUN = fun, FUN.VALUE = value, ...)
}

set_list_element <- function(x, i, value) {
  lapply(x, `[<-`, i = i, value = list(value))
}

guide_rescale <- function(
  value, from = range(value), oob = oob_squish_infinite
) {
  from <- from %||% c(0.0, 1.0)
  rescale(oob(value, from), to = c(0.0, 1.0), from)
}

map_lgl <- function(x, fun, ...) {
  vapply(x, FUN = fun, FUN.VALUE = logical(1L), ...)
}

map_dbl <- function(x, fun, ...) {
  vapply(x, FUN = fun, FUN.VALUE = numeric(1L), ...)
}

map_chr <- function(x, fun, ...) {
  vapply(x, FUN = fun, FUN.VALUE = character(1L), ...)
}

label_as_vector <- function(x) {
  if (obj_is_list(x)) {
    x[lengths(x) == 0L] <- ""
    x <- lapply(x, `[`, 1L)
  }
  if (is.expression(x)) {
    x <- as.list(x)
  }
  x
}

filter_finite <- function(x) {
  x[is.finite(x)]
}

match_list <- function(x, list) {
  findInterval(
    match(x, unlist(list, FALSE, FALSE)),
    cumsum(lengths(list)), left.open = TRUE
  ) + 1L
}

apply_theme_defaults <- function(theme, defaults = NULL) {
  if (is.null(defaults)) {
    return(theme)
  }
  theme    <- replace_null(theme, !!!defaults)
  relative <- names(defaults)[map_lgl(defaults, is_rel)]
  relative <- intersect(relative, names(theme))
  for (i in relative) {
    theme[[i]] <- theme[[i]] * unclass(defaults[[i]])
  }
  theme
}

insert_before <- function(x, i, value) {
  new <- vec_init(x, length(x) + length(i))
  i <- i + seq_along(i) - 1L
  new[i] <- value
  new[-i] <- x
  new
}

insert_after <- function(x, i, value) {
  new <- vec_init(x, length(x) + length(i))
  i <- i + seq_along(i)
  new[i] <- value
  new[-i] <- x
  new
}

extra_args <- function(..., .valid_args = c(.element_params, "theta"), call = caller_env()) {
  args <- list2(...)
  if (length(args) == 0L) {
    return(NULL)
  }
  args <- rename_aes(args, call = call)
  extra <- setdiff(names(args), .valid_args)
  if (length(extra) > 0L) {
    cli::cli_warn("Ignoring unknown parameters: {.and {extra}}.", call = call)
  }
  args <- args[lengths(args) > 0L]
  i <- names(args) != "theta"
  names(args)[i] <- paste0(".", names(args)[i])
  args
}

descale <- function(x, to = c(0.0, 1.0), from = c(0.0, 1.0)) {
  if (!is.numeric(x) || !is_asis(x)) {
    return(x)
  }
  rescale(as.numeric(x), to = to, from = from)
}

after <- function(x, i) {
  n <- length(x)
  if (i >= n) {
    return(vec_slice(x, 0L))
  }
  vec_slice(x, (i + 1L):n)
}

before <- function(x, i) {
  n <- length(x)
  if (i >= n) {
    return(x)
  }
  if (i <= 1L) {
    return(vec_slice(x, 0L))
  }
  vec_slice(x, 1L:(i - 1L))
}

idx_after  <- function(n, i) after(seq(n), i)
idx_before <- function(n, i) before(seq(n), i)

as_mapped_discrete <- function(x) {
  x <- as.numeric(x)
  class(x) <- c("mapped_discrete", class(x))
  x
}

test_that("eval_aes evaluates aesthetics", {

  test <- eval_aes(mtcars, aes(colour = cyl), optional = "colour")
  expect_identical(test$colour, mtcars$cyl)

  expect_warning(expect_warning(
    eval_aes(mtcars, aes(colour = cyl)),
    "Ignoring unknown"
  ), "No valid data")

  expect_error(
    eval_aes(mtcars, list("colour")),
    "must be created by"
  )
})

test_that("%|NA|% works as intended", {

  expect_equal(NULL %|NA|% 1.0, 1.0)
  expect_equal(1.0 %|NA|% 2.0, 1.0)
  expect_equal(c(1.0, NA, 2.0) %|NA|% 4.0, c(1.0, 4.0, 2.0))
  expect_equal(c(1.0, NA, 2.0) %|NA|% c(3.0, 4.0, 5.0), c(1.0, 4.0, 2.0))

})

test_that("pad pads", {
  expect_equal(pad(1.0, 2.0), c(1.0, NA))
  expect_equal(pad(1.0, 2.0, where = "start"), c(NA, 1.0))
  expect_equal(pad(1.0, 1.0), 1.0)
})

test_that("scale_transform throws appropriate error", {
  expect_error(
    scale_transform("A", scale_x_continuous()),
    "not discrete"
  )
})

test_that("by_group computes things by group", {
  test <- by_group(
    1L:4L, group = c("A", "A", "B", "B"),
    fun = mean, value = 1.0
  )
  expect_identical(test, c(1.5, 3.5))
})

test_that("match_list can find needle in list haystack", {
  needle <- c("D", "F")
  haystack <- list(c("A", "B", "C"), "D", c("E", "F", "G"))
  expect_identical(
    match_list(needle, haystack),
    c(2L, 3L)
  )
})

# Borrowed ggplot2 internals ----------------------------------------------

# The functions in this document are either literally copied, or copied with
# modification from ggplot2. The copyright of these functions belong to the
# ggplot2 authors. These functions are borrowed under the MIT licence that
# applies to the ggplot2 package and can be found at the link below:
# https://ggplot2.tidyverse.org/LICENSE.html

# nocov start

is_waive <- function(x) inherits(x, "waiver")

`%|W|%` <- function(x, y) if (is_waive(x)) y else x

opposite_position <- function(position) {
  switch(
    position,
    top    = "bottom",
    bottom = "top",
    left   = "right",
    right  = "left"
  )
}

absoluteGrob <- function(grob, width = NULL, height = NULL,
                         xmin = NULL, ymin = NULL, vp = NULL) {
  gTree(
    children = grob,
    width = width, height = height,
    widths = width, heights = height,
    xmin = xmin, ymin = ymin,
    vp = vp, cl = "absoluteGrob"
  )
}

unique0 <- function(x, ...) if (is.null(x)) x else vec_unique(x, ...)

is_empty <- function(df) {
  length(df) == 0L || nrow(df) == 0L || is_waive(df)
}

is_zero <- function(x) is.null(x) || inherits(x, "zeroGrob")

replace_null <- function(obj, ..., env = caller_env()) {
  dots <- enexprs(...)
  nms  <- names(dots)
  nms  <- nms[map_lgl(.subset(obj, nms), is.null)]
  obj[nms] <- inject(list(!!!dots[nms]), env = env)
  obj
}

.rad2deg <- 180.0 / pi
rad2deg <- function(rad) rad * .rad2deg

.deg2rad <- pi / 180.0
deg2rad <- function(deg) deg * .deg2rad

flip_text_angle <- function(angle) {
  angle <- angle %% 360.0
  flip  <- angle > 90.0 & angle < 270.0
  angle[flip] <- angle[flip] + 180.0
  angle
}

width_cm <- function(x) {
  if (is.grob(x)) x <- grobWidth(x)
  if (is.unit(x)) {
    convertWidth(x, "cm", valueOnly = TRUE)
  } else if (is.list(x)) {
    map_dbl(x, width_cm)
  } else {
    cli::cli_abort("Don't know how to get width of {.cls {class(x)}} object.")
  }
}

height_cm <- function(x) {
  if (is.grob(x)) x <- grobHeight(x)
  if (is.unit(x)) {
    convertHeight(x, "cm", TRUE)
  } else if (is.list(x)) {
    map_dbl(x, height_cm)
  } else {
    cli::cli_abort("Don't know how to get height of {.cls {class(x)}} object.")
  }
}

data_frame0 <- function(...) data_frame(..., .name_repair = "minimal")

find_global <- function(name, env, mode = "any") {
  if (exists(name, envir = env, mode = mode)) {
    return(get(name, envir = env, mode = mode))
  }
  nsenv <- asNamespace("legendry")
  if (exists(name, envir = nsenv, mode = mode)) {
    return(get(name, envir = nsenv, mode = mode))
  }
  nsenv <- asNamespace("ggplot2")
  if (exists(name, envir = nsenv, mode = mode)) {
    return(get(name, envir = nsenv, mode = mode))
  }
  NULL
}

is_mapped_discrete <- function(x) inherits(x, "mapped_discrete")

as_cli <- function(..., env = caller_env()) {
  cli::cli_fmt(cli::cli_text(..., .envir = env))
}

rotate_just <- function(
  angle = NULL, hjust = NULL, vjust = NULL, element = NULL
) {
  if (!is.null(element)) {
    element <- destructure_element(element)
    angle <- angle %||% element$angle
    hjust <- hjust %||% element$hjust
    vjust <- vjust %||% element$vjust
  }

  angle <- (angle %||% 0.0) %% 360.0

  if (is.character(hjust)) {
    hjust <- match(hjust, c("left", "right")) - 1.0
    hjust[is.na(hjust)] <- 0.5
  }
  if (is.character(vjust)) {
    vjust <- match(vjust, c("bottom", "top")) - 1.0
    vjust[is.na(vjust)] <- 0.5
  }

  # Apply recycle rules
  size  <- vec_size_common(angle, hjust, vjust)
  angle <- vec_recycle(angle, size)
  hjust <- vec_recycle(hjust, size)
  vjust <- vec_recycle(vjust, size)

  # Find quadrant on circle
  case <- findInterval(angle, c(0.0, 90.0, 180.0, 270.0, 360.0))

  hnew <- hjust
  vnew <- vjust

  is_case <- which(case == 2L) # 90 <= x < 180
  hnew[is_case] <- 1.0 - vjust[is_case]
  vnew[is_case] <- hjust[is_case]

  is_case <- which(case == 3L) # 180 <= x < 270
  hnew[is_case] <- 1.0 - hjust[is_case]
  vnew[is_case] <- 1.0 - vjust[is_case]

  is_case <- which(case == 4L) # 270 <= x < 360
  hnew[is_case] <- vjust[is_case]
  vnew[is_case] <- 1.0 - hjust[is_case]

  list(hjust = hnew, vjust = vnew)
}

combine_elements <- function(e1, e2) {
  if (inherits(e1, "S7_object") || inherits(e2, "S7_object")) {
    # TODO: this is a dirty hack that should be resolved at some point
    combine <-
      utils::getFromNamespace("combine_elements", asNamespace("ggplot2"))
    return(combine(e1, e2))
  }

  if (is.null(e2) || is_blank(e2)) {
    return(e1)
  }
  if (is.null(e1)) {
    return(e2)
  }
  if (is_rel(e1)) {
    if (is_rel(e2)) {
      return(rel(unclass(e1) * unclass(e2)))
    }
    if (is.numeric(e2) || is.unit(e2)) {
      return(unclass(e1) * e2)
    }
    return(e1)
  }
  if (!is_theme_element(e1) && !is_theme_element(e2)) {
    return(e1)
  }
  if (is_blank(e2)) {
    out <- if (e1$inherit.blank) e2 else e1
    return(out)
  }
  n <- names(e1)[map_lgl(e1, is.null)]
  e1[n] <- e2[n]

  if (is_rel(e1$size)) {
    e1$size <- e2$size * unclass(e1$size)
  }
  if (is_rel(e1$linewidth)) {
    e1$linewidth <- e2$linewidth * unclass(e1$linewidth)
  }
  if (is.subclass(e2, e1)) {
    new <- defaults(e1, e2)
    e2[names(new)] <- new
    return(e2)
  }
  e1
}

`%0%` <- function(e1, e2) if (length(e1) == 0L) e2 else e1

is_rel <- function(x) inherits(x, "rel")

defaults <- function(x, y) c(x, y[setdiff(names(y), names(x))])

is.subclass <- function(x, y) {{
  inheritance <- inherits(x, class(y), which = TRUE)
  !any(inheritance == 0L) && length(setdiff(class(x), class(y))) > 0L
}}

polar_bbox <- function(arc, margin = c(0.05, 0.05, 0.05, 0.05),
                       inner_radius = c(0.0, 0.4)) {
  if (abs(diff(arc) >= 2.0 * pi)) {
    return(list(x = c(0.0, 1.0), y = c(0.0, 1.0)))
  }
  xmax <- 0.5 * sin(arc) + 0.5
  ymax <- 0.5 * cos(arc) + 0.5
  xmin <- inner_radius[1L] * sin(arc) + 0.5
  ymin <- inner_radius[1L] * cos(arc) + 0.5
  margin <- rep_len(margin, 4L)
  margin <- c(
    max(ymin) + margin[1L],
    max(xmin) + margin[2L],
    min(ymin) - margin[3L],
    min(xmin) - margin[4L]
  )
  pos_theta <- c(0.0, 0.5, 1.0, 1.5) * pi
  in_sector <- in_arc(pos_theta, arc)
  bounds <- ifelse(
    in_sector,
    c(1.0, 1.0, 0.0, 0.0),
    c(max(ymax, margin[1L]), max(xmax, margin[2L]),
      min(ymax, margin[3L]), min(xmax, margin[4L]))
  )
  list(x = c(bounds[4L], bounds[2L]), y = c(bounds[3L], bounds[1L]))
}

rename_aes <- function(x, arg = caller_arg(x), call = caller_env()) {
  force(arg)
  names(x) <- standardise_aes_names(names(x))
  dups <- names(x)[duplicated(names(x))]
  if (length(dups) > 0L) {
    cli::cli_warn(
      "Duplicated aesthetics in {.arg {arg}} after name standardisation: \\
      {.field {unique(dups)}}.",
      call = call
    )
  }
  x
}

in_arc <- function(theta, arc) {
  if (abs(diff(arc)) > 2.0 * pi - sqrt(.Machine$double.eps)) {
    return(rep(TRUE, length(theta)))
  }
  arc <- arc %% (2.0 * pi)
  if (arc[1L] < arc[2L]) {
    in_range(theta, arc)
  } else {
    !(theta < arc[1L] & theta > arc[2L])
  }
}

snake_class <- function(x) snakeize(class(x)[1L])
snakeize <- function(x) {
  x <- gsub("([A-Za-z])([A-Z])([a-z])", "\\1_\\2\\3", x)
  x <- gsub(".", "_", x, fixed = TRUE)
  x <- gsub("([a-z])([A-Z])", "\\1_\\2", x)
  to_lower_ascii(x)
}
to_lower_ascii <- function(x) chartr(upper_ascii, lower_ascii, x)
upper_ascii <- paste(LETTERS, collapse = "")
lower_ascii <- paste(letters, collapse = "")

modify_list <- function(old, new) {
  for (i in names(new)) old[[i]] <- new[[i]]
  old
}

compact <- function(x) {
  null <- map_lgl(x, is.null)
  x[!null]
}

# nocov end

# Constructor -------------------------------------------------------------

#' Custom colour steps guide
#'
#' Similar to [`guide_coloursteps()`][ggplot2::guide_coloursteps], this guide
#' displays continuous `colour` or `fill` aesthetics. It has additional options
#' to display caps at the end of the bar, depending on out-of-bounds values.
#'
#' @param first_guide,second_guide Guides to flank the colour steps. Each guide
#'   can be specified using one of the following:
#'   * A `<Guide>` class object.
#'   * A `<function>` that returns a `<Guide>` class object.
#'   * A `<character>` naming such a function, without the `guide_` or
#'   `primitive_` prefix.
#'
#' The `first_guide` will be placed at the location specified by the
#' `legend.text.position` theme setting. The `second_guide` will be placed
#' opposite that position. When `second_guide` has a label suppression
#' mechanism, no labels will be drawn for that guide.
#'
#' @inheritParams gizmo_stepcap
#' @inheritParams compose_sandwich
#' @param vanilla A `<logical[1]>` whether to have the default style match
#'   the vanilla `guide_colourbar()` (`TRUE`) or take the theme
#'   verbatim (`FALSE`).
#'
#' @details
#' As steps are rendered as clipped rectangles, it is important to use a
#' graphics device that can render clipped paths. This can be checked by using
#' [`check_device("clippingPaths")`][ggplot2::check_device].
#'
#' ## Styling options
#'
#' Because this guide is pure composite guide, the [theme][ggplot2::theme]
#' options that govern the styling are determined by its constituents. They are
#' linked below so you can find their 'Styling options' sections. Note that
#' `guide_axis_base()` is just a default that can be swapped out.
#'
#' | **Constituent** | **Description** |
#' | ------------- | --------------- |
#' | [`compose_sandwich`] | Combines the bar with two side-guides. |
#' | [`gizmo_stepcap()`] | Makes up the colour bar. |
#' | [`guide_axis_base()`] | Makes up the tick marks and labels. |
#'
#' Styling options *per break* can be set in the [standard key][key_standard].
#' These override theme settings.
#'
#' The context-agnostic alternative to using `theme()` is to use
#' [`theme_guide()`]:
#'
#' ```r
#' guide_colsteps(theme = theme_guide(
#'   # Composition settings
#'   title = element_text(),
#'   title.position = "top",
#'   text.position = "right",
#'   margin = margin(5),
#'   background = element_rect(),
#'
#'   # Steps settings
#'   frame = element_rect(),
#'   key.width = unit(5, "mm")
#'   key.height = unit(5, "cm")
#'
#'   # Common options for `guide_axis_base()`
#'   line = element_line(),
#'   text = element_text(),
#'   ticks = element_line(),
#'   ticks.length = unit(5, "mm"),
#' ))
#' ```
#'
#' @return A `<Guide>` object
#' @export
#' @family standalone guides
#'
#' @examples
#' p <- ggplot(mpg, aes(displ, hwy)) +
#'   geom_point(aes(colour = cty))
#'
#' # The colour steps show caps when values are out-of-bounds
#' p + scale_colour_viridis_b(
#'   limits = c(10, NA),
#'   guide = "colsteps"
#' )
#'
#' # It also shows how oob values are handled
#' p + scale_colour_viridis_b(
#'   limits = c(10, 30), oob = scales::oob_censor,
#'   guide = "colsteps"
#' )
#'
#' # Adjusting the type of cap
#' p + scale_colour_viridis_b(
#'   limits = c(10, 30),
#'   guide = guide_colsteps(shape = "round")
#' )
#'
#' # The default is to use the breaks as-is
#' p + scale_colour_viridis_b(
#'   limits = c(10, 30), breaks = c(10, 20, 25),
#'   guide = "colsteps"
#' )
#'
#' # But the display can be set to use evenly spaced steps
#' p + scale_colour_viridis_b(
#'   limits = c(10, 30), breaks = c(10, 20, 25),
#'   guide = guide_colsteps(key = key_bins(even.steps = TRUE))
#' )
#'
#' # Using tick marks by swapping side guides
#' p + scale_colour_viridis_b(
#'   guide = guide_colsteps(
#'     first_guide  = "axis_base",
#'     second_guide = "axis_base"
#'   )
#' )
guide_colsteps <- function(
  title = waiver(),
  key = "bins",
  first_guide = "axis_base",
  second_guide = "axis_base",
  shape = "triangle",
  size = NULL,
  show = NA,
  alpha = NA,
  reverse = FALSE,
  suppress_labels = "second",
  oob = scales::oob_keep,
  theme = NULL,
  position = waiver(),
  vanilla = TRUE,
  available_aes = c("colour", "fill")
) {

  steps <- gizmo_stepcap(
    key = NULL, shape = shape, size = size, show = show, alpha = alpha,
    oob = oob
  )

  defaults <- if (isTRUE(vanilla)) vanilla_coloursteps_theme() else NULL

  suppress_labels <- recode(
    suppress_labels,
    old = c("first", "second"),
    new = c("text", "opposite")
  )

  compose_sandwich(
    key = key,
    middle = steps,
    text = first_guide,
    opposite = second_guide,
    suppress_labels = suppress_labels,
    complete = TRUE,
    title = title,
    theme = theme,
    theme_defaults = defaults,
    position = position,
    available_aes = available_aes
  )
}

vanilla_coloursteps_theme <- function(...) {
  theme(
    legend.axis.line = element_blank(),
    legend.ticks = element_blank(),
    legend.ticks.length = rel(-1.0),
    legend.frame = element_blank(),
    ...
  )
}

test_that("guide_colsteps works in all positions", {

  key <- key_bins(show.limits = TRUE)
  base <- ggplot(mtcars, aes(disp, mpg, colour = cyl)) +
    geom_point(shape = 21L) +
    scale_colour_viridis_c(
      oob = oob_squish,
      guide = compose_stack(
        guide_colsteps(show = c(FALSE, FALSE)),
        guide_colsteps(show = c(TRUE,  FALSE)),
        guide_colsteps(show = c(FALSE, TRUE)),
        guide_colsteps(show = c(TRUE,  TRUE))
      )
    ) +
    theme(
      legend.frame = element_rect(colour = "black"),
      legend.ticks = element_line(colour = "black"),
      legend.box.just = "center"
    )

  suppressWarnings({
    vdiffr::expect_doppelganger(
      "right position",
      base + theme(legend.position = "right")
    )
    vdiffr::expect_doppelganger(
      "left position",
      base + theme(legend.position = "left")
    )
    vdiffr::expect_doppelganger(
      "bottom position",
      base + theme(legend.position = "bottom")
    )
    vdiffr::expect_doppelganger(
      "top position",
      base + theme(legend.position = "top")
    )
  })
})

# Constructor -------------------------------------------------------------

#' Dendrogram guide
#'
#' This axis is a speciality axis for discrete data that has been
#' hierarchically clustered. Please be aware that the guide cannot affect the
#' scale limits, which should be set appropriately. This guide will give
#' misleading results when this step is skipped!
#'
#' @inheritParams primitive_segments
#' @inheritParams primitive_labels
#' @inheritParams common_parameters
#' @param labels,ticks,axis_line
#' Guides to use as labels, ticks or axis lines. Can be specified as one of
#' the following:
#' * A `<logical[1]>` which when `FALSE` will set the guide to `guide_none()`
#'   and if `TRUE`, will set guide to appropriate primitive.
#' * A `<Guide>` class object.
#' * A `<function>` that returns a `<Guide>` class object.
#' * A `<character[1]>` naming such a function, without the `guide_` or
#'   `primitive_` prefix.
#'
#' @details
#' ## Styling options
#'
#' Because this guide is pure composite guide, the [theme][ggplot2::theme]
#' options that govern the styling are determined by its constituents. They are
#' linked below so you can find their 'Styling options' sections.
#'
#' | **Primitive** | **Description** |
#' | ------------- | --------------- |
#' | [`compose_stack`] | Stacks the lines, tick marks and labels and dendrogram. |
#' | [`primitive_segments()`] | The dendrogram. |
#' | [`primitive_line()`] | Makes up the axis line. |
#' | [`primitive_ticks()`] | Makes up the tick marks. |
#' | [`primitive_labels()`] | Makes up the labels. |
#'
#' The context-agnostic alternative to using `theme()` is to use
#' [`theme_guide()`]:
#'
#' ```r
#' guide_axis_dendro(theme = theme_guide(
#'   line = element_line(),
#'   text = element_text(),
#'   ticks = element_line(),
#'   ticks.length = unit(5, "mm"),
#' ))
#' ```
#'
#' @return A `<Guide>` object.
#' @export
#' @family standalone guides
#'
#' @examples
#' # Hierarchically cluster data
#' clust <- hclust(dist(scale(mtcars)), "ave")
#'
#' # Using the guide along with appropriate limits
#' p <- ggplot(mtcars, aes(disp, rownames(mtcars))) +
#'   geom_col() +
#'   scale_y_discrete(limits = clust$labels[clust$order])
#'
#' # Standard usage
#' p + guides(y = guide_axis_dendro(clust))
#'
#' # Adding ticks and axis line
#' p +
#'   guides(y = guide_axis_dendro(clust, ticks = "ticks", axis_line = "line")) +
#'   theme(axis.line = element_line())
#'
#' # Controlling space allocated to dendrogram
#' p + guides(y = guide_axis_dendro(clust, space = unit(4, "cm"))) +
#'   theme(axis.ticks.y.left = element_line("red"))
#'
#' # If want just the dendrogram, use `labels = FALSE`
#' p + guides(y = guide_axis_dendro(clust, labels = FALSE), y.sec = "axis")
guide_axis_dendro <- function(
  key = "dendro", title = waiver(), theme = NULL,
  labels = TRUE, space = rel(10.0), vanish = TRUE,
  n.dodge = 1L, angle = waiver(), check.overlap = FALSE,
  ticks = "none", axis_line = "none",
  order = 0L, position = waiver()
) {

  theme <- replace_null(
    theme %||% theme(),
    legendry.guide.spacing = unit(0.0, "cm")
  )

  if (isTRUE(labels)) {
    labels <- primitive_labels(
      angle = angle,
      n.dodge = n.dodge,
      check.overlap = check.overlap
    )
  } else if (isFALSE(labels)) {
    labels <- "none"
  }
  if (isTRUE(ticks)) {
    ticks <- primitive_ticks()
  } else if (isFALSE(ticks)) {
    ticks <- "none"
  }
  if (isTRUE(axis_line)) {
    axis_line <- primitive_line()
  } else if (isFALSE(axis_line)) {
    axis_line <- "none"
  }

  dendro <- primitive_segments(
    key = key,
    space = space,
    vanish = vanish
  )

  compose_stack(
    axis_line, ticks, labels, dendro,
    drop = c(3L, 4L),
    title = title, theme = theme, order = order,
    available_aes = c("any", "x", "y", "r", "theta"),
    position = position
  )
}

test_that("we can easily turn on or off elements in `guide_axis_dendro()`", {

  g <- guide_axis_dendro(labels = TRUE, ticks = TRUE, axis_line = TRUE)
  inner_guides <- g$params$guides
  expect_s3_class(inner_guides[[1L]], "PrimitiveLine")
  expect_s3_class(inner_guides[[2L]], "PrimitiveTicks")
  expect_s3_class(inner_guides[[3L]], "PrimitiveLabels")
  expect_s3_class(inner_guides[[4L]], "PrimitiveSegments")

  g <- guide_axis_dendro(labels = FALSE, ticks = FALSE, axis_line = FALSE)
  inner_guides <- g$params$guides
  expect_s3_class(inner_guides[[1L]], "GuideNone")
  expect_s3_class(inner_guides[[2L]], "GuideNone")
  expect_s3_class(inner_guides[[3L]], "GuideNone")
  expect_s3_class(inner_guides[[4L]], "PrimitiveSegments")

})

# Constructor -------------------------------------------------------------

#' Guide composition
#'
#' @description
#' Guide composition is a meta-guide orchestrating an ensemble of other guides.
#' On their own, a 'composing' guide is not very useful as a visual reflection
#' of a scale.
#'
#' @param guides A `<list>` of guides wherein each element is one of the
#'   following:
#'   * A `<Guide>` class object.
#'   * A `<function>` that returns a `<Guide>` class object.
#'   * A `<character[1]>` naming such a function, without the `guide_` or
#'   `primitive_` prefix.
#' @param args A `<list>` of arguments to pass to guides that are given either
#'   as a function or as a string.
#' @param ... Additional parameters to pass on to
#'   [`new_guide()`][ggplot2::new_guide].
#' @param available_aes A `<character>` giving aesthetics that must match the
#'   the guides.
#' @param super A `<Compose>` class object giving a meta-guide for composition.
#' @param call A [call][rlang::topic-error-call] to display in messages.
#'
#' @details
#' ## Styling options
#'
#' Below are the [theme][ggplot2::theme] options that determine the styling of
#' this guide.
#'
#' | **Theme setting** | **Type** | **Description** |
#' | ----------------- | -------- | --------------- |
#' | `legendry.guide.spacing` | [`unit()`] | Spacing between guides. |
#'
#' There are no further styling options.
#'
#' @name guide-composition
#' @return A `<Compose>` (sub-)class guide that composes other guides.
#' @export
#' @family composition
#'
#' @examples
#' # The `new_compose()` function is not intended to be used directly
#' my_composition <- new_compose(list("axis", "axis"), super = ComposeStack)
#'
#' # Is the same as
#' my_composition <- compose_stack("axis", "axis")
new_compose <- function(guides, args = list(), ...,
                        available_aes = c("any", "x", "y", "r", "theta"),
                        call = caller_env(), super = Compose) {

  guides <- lapply(guides, validate_guide, args = args, call = call)
  if (length(guides) < 1L) {
    cli::cli_abort("There must be at least one guide to compose.", call = call)
  }

  available_aes <- compatible_aes(guides, available_aes)
  guide_params  <- lapply(guides, `[[`, name = "params")

  new_guide(
    guides = guides,
    guide_params = guide_params,
    available_aes = available_aes,
    super = super,
    ...
  )
}

# Class -------------------------------------------------------------------

#' @export
#' @rdname legendry_extensions
#' @format NULL
#' @usage NULL
Compose <- ggproto(
  "Compose", Guide,

  params = new_params(
    guides = list(), guide_params = list(),
    key = NULL, angle = waiver()
  ),

  elements = list(spacing = "legendry.guide.spacing"),

  train = function(self, params = self$params, scale, aesthetic = NULL,
                   title = waiver(), ...) {
    title <- scale$make_title(params$title, scale$name, title)
    position  <- params$position  <- params$position %|W|% NULL
    aesthetic <- params$aesthetic <- aesthetic %||% scale$aesthetics[1L]
    check_position(position, inside = TRUE, allow_null = TRUE)

    key <- resolve_key(params$key, allow_null = TRUE)
    if (is.function(key)) {
      key <- key(scale, aesthetic %||% scale$aesthetics[1L])
    }
    params$key <- NULL
    any_title <- FALSE

    guide_params <- params$guide_params
    for (i in seq_along(params$guides)) {
      if (inherits(params$guides[[i]], "PrimitiveTitle")) {
        guide_title <- title
        any_title   <- TRUE
      } else {
        guide_title <- waiver()
      }
      guide_params[[i]]$position <-
        (guide_params[[i]]$position %|W|% NULL) %||% position
      guide_params[[i]]$angle <- guide_params[[i]]$angle %|W|% params$angle
      guide_params[[i]]["key"] <- list(guide_params[[i]]$key %||% key)
      guide_params[[i]] <- params$guides[[i]]$train(
        params = guide_params[[i]], scale = scale, aesthetic = aesthetic,
        title = guide_title, ...
      )
      guide_params[[i]]$title <- guide_params[[i]]$title %|W|% NULL
    }
    if (any_title) {
      params$title <- NULL
    } else {
      params$title <- title
    }
    params$guide_params <- guide_params
    params$hash <- hash(lapply(guide_params, get_hash))
    params
  },

  transform = function(self, params, coord, panel_params) {
    params$guide_params <- loop_guides(
      params$guides, params$guide_params, "transform",
      coord = coord, panel_params = panel_params
    )
    params
  },

  get_layer_key = function(params, layers, data = NULL, ...) {
    params$guide_params <- loop_guides(
      params$guides, params$guide_params, "get_layer_key",
      layers = layers, data = data, ...
    )
    # Collect limits
    limits <- get_limits(params)
    params <- set_limits(params, limits)
    params
  },

  draw = function(...) {
    cli::cli_abort("Not implemented.")
  }
)

# Helpers -----------------------------------------------------------------

loop_guides <- function(guides, params, method, ...) {
  for (i in seq_along(guides)) {
    params[[i]] <- guides[[i]][[method]](params = params[[i]], ...)
  }
  params
}

compatible_aes <- function(guides, available_aes, call = caller_env()) {

  valid <- !map_lgl(guides, inherits, what = "GuideNone")
  available <- lapply(guides[valid], `[[`, name = "available_aes")
  common <- Reduce(any_intersect, available)

  if (length(common) < 1L) {
    cli::cli_abort(
      "The guides to combine have no shared {.field available aesthetics}.",
      call = call
    )
  }
  if (!is.null(available_aes)) {
    common <- any_intersect(available_aes, common)
    if (length(common) < 1L) {
      cli::cli_abort(c(
        "The guides have incompatible {.arg available_aes} settings.",
        "They must include {.or {.val {available_aes}}}."
      ), call  = call)
    }
  }
  common
}

any_intersect <- function(x, y) {
  if ("any" %in% x) {
    x <- union(x, setdiff(y, c("x", "y", "r", "theta")))
  }
  if ("any" %in% y) {
    y <- union(y, setdiff(x, c("x", "y", "r", "theta")))
  }
  intersect(x, y)
}

validate_guide <- function(guide, args = list(), env = global_env(),
                           call = caller_env()) {
  input <- guide
  if (is.character(guide)) {
    guide <- find_global(
      paste0("guide_", input),
      env = env, mode = "function"
    )
  }
  if (is.null(guide) && is.character(input)) {
    guide <- find_global(
      paste0("primitive_", input),
      env = env, mode = "function"
    )
  }
  if (is.function(guide)) {
    args  <- args[intersect(names(args), fn_fmls_names(guide))]
    guide <- inject(guide(!!!args))
  }
  if (is_guide(guide)) {
    return(guide)
  }
  cli::cli_abort("Unknown guide: {input}.", call = call)
}

accumulate_limits <- function(...) {
  args <- list2(...)
  args <- args[lengths(args) > 0L]
  if (length(args) == 0L) {
    return(NULL)
  }
  if (is.character(args[[1L]])) {
    unique(unlist(args))
  } else {
    inject(range(!!!args, na.rm = TRUE))
  }
}

get_limits <- function(params) {
  if ("guide_params" %in% names(params)) {
    limits <- lapply(params$guide_params, get_limits)
    accumulate_limits(!!!limits)
  } else {
    params$limits
  }
}

set_limits <- function(params, limits) {
  if ("guide_params" %in% names(params)) {
    params$guide_params <-
      lapply(params$guide_params, set_limits, limits = limits)
  }
  params$limits <- limits
  params
}

get_hash <- function(x) x$hash

test_that("new_compose throws appropriate messages", {

  expect_error(
    new_compose(list()),
    "must be at least one guide"
  )

  dummy1 <- new_guide(available_aes = "foo", super = Guide)
  dummy2 <- new_guide(available_aes = "bar", super = Guide)

  expect_error(
    new_compose(list(dummy1), available_aes = "bar"),
    "must include"
  )

  expect_error(
    new_compose(list(dummy1, dummy2)),
    "guides to combine have no shared"
  )

  expect_error(
    new_compose("foo"),
    "Unknown guide"
  )

})

# Constructor -------------------------------------------------------------

#' Guide primitive: line
#'
#' This function constructs a ticks [guide primitive][guide-primitives].
#'
#' @param bidi A `<logical[1]>`: whether ticks should be drawn bidirectionally
#'   (`TRUE`) or in a single direction (`FALSE`, default).
#' @inheritParams primitive_labels
#'
#' @return A `PrimitiveTicks` primitive guide that can be used inside other
#'   guides.
#' @export
#' @family primitives
#'
#' @details
#' ## Styling options
#'
#' Below are the [theme][ggplot2::theme] options that determine the styling of
#' this guide, which may differ depending on whether the guide is used in
#' an axis or in a legend context.
#'
#' The ticks can come in three variants: major, minor and minimal.
#' Which variants are drawn depends on the keys: [`key_minor()`] draws major
#' and minor ticks, whereas [`key_log()`] also has minimal ticks.
#' Each variant has a corresponding length setting.
#'
#' The possible `{position}` suffixes mentioned below are `x`, `x.top`,
#' `x.bottom`, `y`, `y.left`, `y.right`. The `theta` and `r` position suffixes
#' in \pkg{ggplot2} are *not* obeyed in \pkg{legendry}.
#'
#' | **Theme setting** | **Context** | **Type** | **Description** |
#' | ----------------- | ----------- | -------- | --------------- |
#' | `axis.ticks.{position}` | Axis | [`element_line()`] | Major tick lines |
#' | `axis.ticks.length.{position}` | Axis | [`unit()`] | Major tick length |
#' | `axis.minor.ticks.{position}` | Axis | [`element_line()`] | Minor tick lines |
#' | `axis.minor.ticks.length.{position}` | Axis | [`unit()`] | Minor tick length |
#' | `legendry.axis.mini.ticks` | Axis | [`element_line()`] | Minimal tick lines |
#' | `legendry.axis.mini.ticks.length` | Axis | [`unit()`] | Minimal tick length |
#' | `legend.ticks` | Legend | [`element_line()`] | Major tick lines |
#' | `legend.ticks.length` | Legend | [`unit()`] | Major ticks length |
#' | `legendry.legend.minor.ticks` | Legend | [`element_line()`] | Minor tick lines |
#' | `legendry.legend.minor.ticks.length` | Legend | [`unit()`] | Minor ticks length |
#' | `legendry.legend.mini.ticks` | Legend | [`element_line()`] | Minimal tick lines |
#' | `legendry.legend.mini.ticks.length` | Legend | [`unit()`] | Minimal tick length |
#'
#' Styling options *per break* can be set in the [key][key_standard].
#' The `line` and prefixed properties are prioritised for the tick lines.
#' These override theme settings.
#'
#' The context-agnostic alternative to using `theme()` is to use
#' [`theme_guide()`]:
#'
#' ```r
#' primitive_ticks(theme = theme_guide(
#'   ticks = element_line(),
#'   ticks.length = unit(5, "mm"),
#'   minor.ticks = element_line(),
#'   minor.ticks.length = unit(4, "mm"),
#'   mini.ticks = element_line(),
#'   mini.ticks.length = unit(3, "mm")
#' ))
#' ```
#'
#' @examples
#' # A standard plot
#' p <- ggplot(mpg, aes(displ, hwy)) +
#'   geom_point()
#'
#' # Adding as secondary guides
#' p + guides(x.sec = primitive_ticks(), y.sec = primitive_ticks())
primitive_ticks <- function(key = NULL, bidi = FALSE, theme = NULL,
                            position = waiver()) {
  check_bool(bidi)
  new_guide(
    key = key,
    bidi = bidi,
    theme = theme,
    position = position,
    available_aes = c("any", "x", "y", "r", "theta"),
    super = PrimitiveTicks
  )
}

# Class -------------------------------------------------------------------

#' @export
#' @rdname legendry_extensions
#' @format NULL
#' @usage NULL
PrimitiveTicks <- ggproto(
  "PrimitiveTicks", Guide,

  params = new_params(key = NULL, bidi = FALSE),

  hashables = exprs(key$.value),

  elements = list(
    position = list(
      ticks = "axis.ticks",       ticks_length = "axis.ticks.length",
      minor = "axis.minor.ticks", minor_length = "axis.minor.ticks.length"
    ),
    legend = list(
      ticks        = "legend.ticks",
      ticks_length = "legend.ticks.length",
      minor        = "legendry.legend.minor.ticks",
      minor_length = "legendry.legend.minor.ticks.length",
      # Mini ticks for legends don't have complicated inheritance
      mini         = "legendry.legend.mini.ticks",
      mini_length  = "legendry.legend.mini.ticks.length"
    )
  ),

  extract_key = standard_extract_key,

  extract_params = primitive_extract_params,

  transform = function(self, params, coord, panel_params) {
    params$key <-
      transform_key(params$key, params$position, coord, panel_params)
    params
  },

  setup_params = primitive_setup_params,

  setup_elements = primitive_setup_elements,

  override_elements = function(params, elements, theme) {

    # Count how many ticks of each type we need
    type <- params$key$.type %||% "major"
    n_major <- sum(type == "major")
    n_minor <- sum(type == "minor")
    n_mini  <- sum(type == "mini")

    # We need to setup mini ticks for axes, as the inheritance tree isn't
    # mirrored for every aesthetic/position combination.
    if (n_mini > 0L && params$aesthetic %in% c("x", "y")) {
      elements$mini <- combine_elements(
        theme$legendry.axis.mini.ticks,
        elements$minor
      )
      elements$mini_length <- combine_elements(
        theme$legendry.axis.mini.ticks.length,
        elements$minor_length
      )
    }

    # Set absent ticks to empty
    elements <- zap_tick(elements, "ticks", n_major)
    elements <- zap_tick(elements, "minor", n_minor)
    elements <- zap_tick(elements, "mini",  n_mini)

    if (is.unit(params$force_stretch)) {
      elements$ticks_length <- elements$ticks_length + cm(params$force_stretch)
    }

    elements
  },

  build_ticks = function(key, elements, params, position = params$position) {
    type <- key$.type %||% "major"
    offset <- elements$offset
    major <- draw_ticks(
      vec_slice(key, type == "major"),
      elements$ticks, params, position, elements$ticks_length, offset
    )
    minor <- draw_ticks(
      vec_slice(key, type == "minor"),
      elements$minor, params, position, elements$minor_length, offset
    )
    mini <- draw_ticks(
      vec_slice(key, type == "mini"),
      elements$mini, params, position, elements$mini_length, offset
    )
    # Discard zeroGrobs
    grob <- list(major, minor, mini)
    grob <- grob[!map_lgl(grob, is_zero)]
    if (length(grob) == 0L) {
      return(zeroGrob())
    }
    gTree(children = inject(gList(!!!grob)))
  },

  draw = function(self, theme, position = NULL, direction = NULL,
                  params = self$params) {

    params <- replace_null(params, position = position, direction = direction)
    params <- self$setup_params(params)

    elems <- self$setup_elements(params, self$elements, theme)
    elems <- self$override_elements(params, elems, theme)
    ticks <- self$build_ticks(params$key, elems, params)

    # If ticks have negative length, we want to preserve reasonable spacing
    # to text labels.
    ticks <- list(ticks, zeroGrob())
    size <- measure_tick_size(params$key, elems, params$position)

    grob <- primitive_grob(
      grob = ticks,
      size = size$size,
      position = params$position,
      name = "ticks"
    )
    attr(grob, "lengths") <- size$lengths
    grob
  }
)

# Helpers -----------------------------------------------------------------

draw_ticks <- function(key, element, params, position, length, offset = 0.0) {
  n_breaks <- nrow(key)
  if (n_breaks < 1L || is_blank(element) || all(length == 0L)) {
    return(zeroGrob())
  }
  props  <- element_key_properties(key, "line")
  length <- rep_len(length, n_breaks)
  bidi   <- c(1.0, -as.numeric(params$bidi %||% FALSE))

  # Every tick has two vertices
  length <- rep(length, each = 2L)
  key <- vec_rep_each(key, 2L)
  id <- rep(2L, n_breaks)

  # Set anchor positions
  switch(
    position,
    top    = {key$y <- 0.0},
    right  = {key$x <- 0.0},
    bottom = {key$y <- 1.0},
    left   = {key$x <- 1.0}
    # theta(.sec) already has appropriate x/y values
  )

  length <- rep(bidi, times = n_breaks) * length
  length <- unit(length + offset, "cm")
  theta  <- get_theta(key, position)

  args <- list(
    x = unit(key$x, "npc") + sin(theta) * length,
    y = unit(key$y, "npc") + cos(theta) * length
  )

  inject(element_grob(element, id.lengths = id, !!!args, !!!props))
}

zap_tick <- function(elements, name, n) {
  length <- paste0(name, "_length")
  # If there are no ticks, set the element to NULL
  if (n < 1L) {
    elements[[name]] <- NULL
  }
  # If there is no element, set the length to 0
  if (is_blank(elements[[name]])) {
    elements[[length]] <- 0.0
  }
  # Ensure tick lengths are in centimetres
  elements[[length]] <- cm(elements[[length]])
  elements
}

measure_tick_size <- function(key, elements, position) {
  lengths <- unname(unlist(elements[c("ticks_length", "minor_length", "mini_length")]))
  if (".type" %in% names(key)) {
    i <- match(key$.type, c("major", "minor", "mini"))
  } else {
    i <- rep(1L, nrow(key))
  }
  lengths <- lengths[i]
  theta <- get_theta(key, position)
  size <- switch(
    position,
    bottom = , top = range(cos(theta) * lengths, 0),
    left = , right = range(sin(theta) * lengths, 0),
    range(lengths, 0)
  )
  size <- switch(
    position,
    bottom = , left = c(-size[1], max(0, -diff(size))),
    c(size[2],  max(0, -diff(size)))
  )
  list(
    size  = unit(size, "cm"),
    lengths = lengths
  )
}

test_that("primitive_ticks works as axis", {

  base <- ggplot(mpg, aes(displ, hwy)) +
    geom_blank() +
    theme_test() +
    theme(
      panel.background = element_rect(fill = NA, colour = "grey80"),
      panel.grid.major = element_line(colour = "grey90"),
      panel.border = element_blank(),
      axis.line = element_line()
    )

  p <- base +
    guides(
      x     = primitive_ticks(),
      x.sec = primitive_ticks(
        theme = theme(axis.ticks.length = unit(-2.0, "mm"))
      ),
      y     = primitive_ticks(),
      y.sec = primitive_ticks(
        theme = theme(axis.ticks = element_line(colour = "red"))
      )
    )

  vdiffr::expect_doppelganger("primitive_ticks cartesian", p)

  p <- base +
    coord_radial(start = 0.25 * pi, end = 1.75 * pi, inner.radius = 0.5) +
    guides(
      theta     = primitive_ticks(),
      theta.sec = primitive_ticks(
        theme = theme(axis.ticks.length = unit(-2.0, "mm"))
      ),
      r         = primitive_ticks(),
      r.sec = primitive_ticks(
        theme = theme(axis.ticks = element_line(colour = "red"))
      )
    )

  vdiffr::expect_doppelganger("primitive_ticks radial", p)

})

test_that("primitive_ticks works as legend", {

  p <- ggplot(mtcars) +
    aes(
      x = disp, y = mpg,
      colour = hp,
      fill = hp
    ) +
    geom_point() +
    guides(
      colour = primitive_ticks(key = key_manual(c(100.0, 300.0))),
      fill = primitive_ticks(position = "bottom")
    )

  vdiffr::expect_doppelganger("primitive_ticks legend", p)

})

test_that("draw_ticks works with theta position", {

  ticks <- draw_ticks(
    key = data.frame(x = 1.0, y = 0.0, theta = 0.0, r = 0.0),
    theme_gray()$line,
    list(),
    "theta",
    1.0
  )
  expect_s3_class(ticks, "polyline")
})

# Constructor -------------------------------------------------------------

#' Guide primitive: title
#'
#' This function constructs a title [guide primitive][guide-primitives].
#'
#' @inheritParams common_parameters
#'
#' @return A `<PrimitiveTitle>` primitive guide that can be used inside other
#'   guides.
#' @export
#' @family primitives
#'
#' @details
#' ## Styling options
#'
#' Below are the [theme][ggplot2::theme] options that determine the styling of
#' this guide, which may differ depending on whether the guide is used in
#' an axis or in a legend context.
#'
#' | **Theme setting** | **Context** | **Type** | **Description** |
#' | ----------------- | ----------- | -------- | --------------- |
#' | `axis.title.{x/y}.{position}` | Axis | [`element_text()`] | The title itself. |
#' | `legend.title` | Legend| [`element_text()`] | The title itself. |
#'
#' There are no further styling options.
#'
#' The context-agnostic alternative to using `theme()` is to use
#' [`theme_guide()`]:
#'
#' ```r
#' primitive_title(theme = theme_guide(
#'   title = element_text(),
#' ))
#' ```
#'
#' @examples
#' # A standard plot
#' p <- ggplot(mpg, aes(displ, hwy)) +
#'  geom_point()
#'
#' # Adding as secondary guides
#' p + guides(
#'   x.sec = primitive_title("Horizontal Title"),
#'   y.sec = primitive_title(c("along vertical", "Multiple tiles"))
#' )
#'
#' # 'Real' titles occur once per plot.
#' # Primitive titles repeat over facets and hide the 'real' title.
#' p + facet_wrap(~ drv) +
#'   guides(x = primitive_title("I am a repeated subtitle")) +
#'   labs(x = "I am the hidden real title")
primitive_title <- function(title = waiver(), angle = waiver(),
                            theme = NULL, position = waiver()) {
  if (!is_waive(angle)) {
    check_number_decimal(
      angle, min = -360.0, max = 360.0,
      allow_infinite = FALSE, allow_null = TRUE
    )
  }

  new_guide(
    title = NULL,
    my_title = title,
    angle = angle,
    theme = theme,
    position = position,
    available_aes = c("any", "x", "y", "r", "theta"),
    super = PrimitiveTitle
  )
}

# Class -------------------------------------------------------------------

#' @export
#' @rdname legendry_extensions
#' @format NULL
#' @usage NULL
PrimitiveTitle <- ggproto(
  "PrimitiveTitle", Guide,

  hashables = exprs(my_title),

  elements = list(
    position = list(title = "axis.title"),
    legend   = list(title = "legend.title")
  ),

  params = new_params(my_title = waiver(), angle = waiver()),

  extract_key = function(scale, aesthetic, ...) {
    # Need to keep track of limits for r/r.sec positions
    limits <- range(scale$continuous_range %||% scale$get_limits())
    data_frame0(!!aesthetic := c(-Inf, Inf), .value = limits)
  },

  extract_params = function(scale, params, title = waiver(), ...) {
    params$my_title <-
      scale$make_title(params$my_title, scale$name, title)
    primitive_extract_params(scale, params, ...)
  },

  transform = function(self, params, coord, panel_params) {
    if (is_theta(params$position)) {
      params$bbox  <- panel_params$bbox %||%
        list(c(x = c(0.0, 1.0), y = c(0.0, 1.0)))
      params$arc   <- panel_params$arc %||%
        c(0.0, 2.0 * pi)
      params$donut <- panel_params$inner_radius %||%
        c(0.0, 0.4)
    }
    params$key <-
      transform_key(params$key, params$position, coord, panel_params)
    params
  },

  setup_params = primitive_setup_params,

  setup_elements = primitive_setup_elements,

  measure_grobs = function(grobs, params, elements) {
    switch(
      params$position,
      top = , bottom = height_cm(grobs),
      left = , right = width_cm(grobs),
      get_attr(grobs, "offset", default = 0.0)
    )
  },

  build_title = function(label, elements, params) {
    label <- validate_labels(label)
    switch(
      params$position,
      theta = , theta.sec = draw_theta_title(label, elements, params),
      draw_cart_title(label, elements, params)
    )
  },

  draw = function(self, theme, position = NULL, direction = NULL,
                  params = self$params) {
    params <- replace_null(params, position = position, direction = direction)
    params <- self$setup_params(params)

    theme <- replace_null(theme, legend.title.position = params$position)

    elems <- self$setup_elements(params, self$elements, theme)
    title <- self$build_title(params$my_title, elems, params)
    size  <- self$measure_grobs(title, params, elements)

    primitive_grob(
      grob = title,
      size = unit(size, "cm"),
      position = params$position,
      name = "title"
    )
  }
)

# Helpers -----------------------------------------------------------------

draw_theta_title <- function(label, elements, params) {
  if (is.null(label)) {
    return(zeroGrob())
  }

  title <- elements$title
  position <- params$position
  hjust <- title$hjust
  offset <- elements$offset
  donut <- params$donut
  bbox <- params$bbox
  r <- if (position == "theta") donut[2L] else donut[1L]

  theta <- rescale(hjust, from = c(0.0, 1.0), to = params$arc)

  n_labels <- length(label)
  if (n_labels > 1L) {
    theta <- rescale(
      seq(0.0, 1.0, length.out = n_labels),
      from = c(0.0, 1.0), to = params$arc
    )
  }

  angle <- params$angle %|W|% NULL
  if (is.null(angle)) {
    angle <- title$angle
  } else {
    angle <- flip_text_angle(angle - rad2deg(theta))
  }
  rad <- deg2rad(angle)

  margin <- cm(max(title$margin))
  offset <- offset + margin

  x <- rescale(r * sin(theta) + 0.5, from = bbox$x)
  y <- rescale(r * cos(theta) + 0.5, from = bbox$y)

  if (position == "theta.sec") {
    theta <- theta + pi
  }

  hjust <- 0.5 - sin(theta + rad) / 2.0
  vjust <- 0.5 - cos(theta + rad) / 2.0

  x <- unit(x, "npc") + unit(offset * sin(theta), "cm")
  y <- unit(y, "npc") + unit(offset * cos(theta), "cm")

  grob <- element_grob(
    element = title,
    label = label,
    x = x, y = y,
    hjust = hjust, vjust = vjust,
    angle = angle
  )

  height <- if (inherits(grob, "textpath")) {
    measure_textpath_labels(grob)
  } else {
    measure_theta_labels(
      title, label, margin, theta + rad,
      list(hjust = hjust, vjust = vjust)
    )
  }
  attr(grob, "offset") <- height
  grob
}

draw_cart_title <- function(label, elements, params) {
  if (length(label) < 1L) {
    return(zeroGrob())
  }

  limits <- sort(params$key[[params$aesthetic]])
  limits <- oob_squish_infinite(limits)

  title    <- elements$title
  position <- params$position

  angle <- (params$angle %|W|% NULL) %||% title$angle

  singles <- lapply(label, function(lab) {
    element_grob(title, lab, margin_x = TRUE, margin_y = TRUE, angle = angle,
                 hjust = 0.5, vjust = 0.5)
  })
  widths  <- width_cm(singles)
  heights <- height_cm(singles)

  justs <- rotate_just(element = title)
  hjust <- justs$hjust
  vjust <- justs$vjust

  if (position %in% c("left", "right")) {
    x <- hjust
    y <- rescale(vjust, from = c(0.0, 1.0), to = limits)
  } else {
    x <- rescale(hjust, from = c(0.0, 1.0), to = limits)
    y <- vjust
  }

  n_labels <- length(label)
  if (n_labels > 1L) {
    if (position %in% c("top", "bottom")) {
      x <- seq(limits[1L], limits[2L], length.out = n_labels)
      hjust <- seq(0.0, 1.0, length.out = n_labels)
    } else {
      y <- seq(limits[1L], limits[2L], length.out = n_labels)
      vjust <- seq(0.0, 1.0, length.out = n_labels)
    }
  }

  x <- unit(x, "npc") + unit(widths  * (0.5 - hjust), "cm")
  y <- unit(y, "npc") + unit(heights * (0.5 - vjust), "cm")

  element_grob(
    title,
    label = label,
    x = x, y = y,
    hjust = 0.5, vjust = 0.5,
    angle = angle,
    margin_x = TRUE,
    margin_y = TRUE
  )
}

test_that("primitive_title works as axis", {

  base <- ggplot(mpg, aes(displ, hwy)) +
    geom_blank() +
    theme_test() +
    theme(
      panel.background = element_rect(fill = NA, colour = "grey80"),
      panel.grid.major = element_line(colour = "grey90"),
      panel.border = element_blank(),
      axis.line = element_line()
    )

  red_text <-
    theme(axis.title.y.right = element_text(colour = "red", vjust = 0.5))

  p <- base +
    guides(
      x     = primitive_title(c("foo", "bar", "baz")),
      x.sec = primitive_title("horizontal title"),
      y     = primitive_title(c("qux", "egg"), angle = 45.0),
      y.sec = primitive_title("foobar", theme = red_text)
    )

  vdiffr::expect_doppelganger("primitive_title cartesian", p)

  p <- base +
    coord_radial(start = 0.25 * pi, end = 1.75 * pi, inner.radius = 0.5) +
    guides(
      theta.sec = primitive_title(c("foo", "bar", "baz"), angle = 0.0),
      theta = primitive_title("horizontal title"),
      r     = primitive_title(c("qux", "egg"), angle = 90.0),
      r.sec = primitive_title("foobar", theme = red_text)
    )

  vdiffr::expect_doppelganger("primitive_title radial", p)
})

test_that("primitive_title works as legend", {

  p <- ggplot(mtcars) +
    aes(
      x = disp, y = mpg,
      colour = hp,
      fill = hp
    ) +
    geom_point() +
    guides(
      colour = primitive_title(title = "right title"),
      fill = primitive_title(title = "bottom title", position = "bottom")
    )

  vdiffr::expect_doppelganger("primitive_title legend", p)

})

test_that("draw_theta_title works", {

  elements <- list(
    title = theme_gray()$text,
    offset = 0.0
  )

  params <- list(
    position = "theta",
    donut = c(0.2, 0.4),
    arc = c(0.0, 1.0),
    bbox = list(x = c(0.0, 1.0), y = c(0.0, 1.0)),
    angle = 0.0
  )

  test <- withr::with_pdf(
    tempfile(fileext = ".pdf"),
    draw_theta_title("Foobar", elements, params)
  )

  expect_equal(test$children[[1L]]$rot, 331.0, tolerance = 1.0)

  params$position <- "theta.sec"
  params$angle <- NULL

  test <- withr::with_pdf(
    tempfile(fileext = ".pdf"),
    draw_theta_title("Foobar", elements, params)
  )

  expect_equal(test$children[[1L]]$rot, 0.0, tolerance = 1.0)
})

# Constructor -------------------------------------------------------------

#' Guide gizmo: histogram
#'
#' This guide displays a histogram of the aesthetic. If the aesthetic is
#' `colour` or `fill`, the shape will reflect this.
#'
#' @param hist One of the following:
#'   * `NULL` for computing histograms on the data values (default).
#'   * an atomic `<vector>` to feed to the `hist.fun` function.
#'   * A named `<list>` with `breaks` and `counts` numeric items, where the
#'   `breaks` item is exactly one element longer than the `counts` item. A
#'   typical way to construct such list is using the [`hist()`][graphics::hist]
#'   function. Please note that `<list>` input is expected in scale-transformed
#'   space, not original data space.
#' @param hist.args A `<list>` with additional arguments to the `hist.fun`
#'   argument. Only applies when `hist` is not provided as a `<list>` already.
#'   Please note that these arguments are only used for binning and counting:
#'   graphical arguments are ignored.
#' @param hist.fun A `<function>` to use for computing histograms when the
#'   `hist` argument is not provided as a list already.
#' @param just A `<numeric[1]>` between 0 and 1. Use 0 for bottom- or
#'   left-aligned histograms, use 1 for top- or right-aligned histograms and 0.5
#'   for centred histograms.
#' @param metric A `<character[1]>` either `"counts"` or `"density"` stating
#'   which field of the `<histogram>` class to display. The `"density"` metric
#'   might be more appropriate to display when the histogram breaks have
#'   non-constant intervals.
#' @inheritParams gizmo_density
#'
#' @details
#' Non-finite values such as `NA` and `NaN` are ignored while infinite values
#' such as `-Inf` and `Inf` are [squished][scales::oob_squish] to the limits.
#'
#' ## Styling options
#'
#' Below are the [theme][ggplot2::theme] options that determine the styling of
#' this guide.
#'
#' | **Theme setting** | **Type** | **Description** |
#' | ----------------- | -------- | --------------- |
#' | `legend.frame` | [`element_rect()`] | Outline drawn around the histogram itself. The `fill` setting is ignored. |
#' | `legend.key` | [`element_rect()`] | Background underneath the histogram area. |
#' | `legend.key.width` | [`unit()`] | Width of the histogram area. |
#' | `legend.key.height` | [`unit()`] | Height of the histogram area. |
#'
#' Please note that depending on the `direction` argument, the
#' `legend.key.width`/`legend.key.height` setting are expanded 5-fold if
#' originating from the global theme. To set these directly, you can use the
#' local `theme` argument in the guide.
#' These settings have shorthands in [`theme_guide()`]:
#'
#' ```r
#' gizmo_histogram(theme = theme_guide(
#'   frame = element_rect(),
#'   key = element_rect(),
#'   key.width = unit(5, "mm")
#'   key.height = unit(5, "cm")
#' ))
#' ```
#'
#' @return A `<GizmoHistogram>` object.
#' @family gizmos
#' @export
#'
#' @examples
#' # A standard plot
#' p <- ggplot(mpg, aes(displ, hwy, colour = cty)) +
#'   geom_point() +
#'   scale_colour_viridis_c()
#'
#' # Histogram from plot data
#' p + guides(colour = gizmo_histogram())
#'
#' # Using bins instead of gradient
#' p + guides(colour = gizmo_histogram("bins"))
#'
#' # Providing custom values to compute histogram
#' p + guides(colour = gizmo_histogram(hist = runif(1000, min = 5, max = 35)))
#'
#' # Providing precomputed histogram
#' p + guides(colour = gizmo_histogram(hist = hist(mpg$cty, breaks = 10)))
#'
#' # Alternatively, parameters may be passed through hist.args
#' p + guides(colour = gizmo_histogram(hist.arg = list(breaks = 10)))
gizmo_histogram <- function(
  key = waiver(),
  hist = NULL, hist.args = list(), hist.fun = graphics::hist,
  just = 1.0, oob = oob_keep, metric = "counts", alpha = NA,
  # standard arguments
  theme = NULL, position = waiver(), direction = NULL
) {
  hist <- suppress_hist_plot(enquo(hist))
  hist.args$plot <- hist.args$plot %||% FALSE

  check_number_decimal(just, min = 0.0, max = 1.0, allow_infinite = FALSE)
  check_argmatch(metric, c("counts", "density"))

  new_guide(
    key = key,
    hist      = hist,
    hist_args = hist.args,
    hist_fun  = hist.fun,
    just = just, oob  = oob, metric = metric, alpha = alpha,
    theme = theme, position = position, direction = direction,
    name = "histogram",
    super = GizmoHistogram
  )
}

# Class -------------------------------------------------------------------

#' @export
#' @rdname legendry_extensions
#' @format NULL
#' @usage NULL
GizmoHistogram <- ggproto(
  "GizmoHistogram", GizmoDensity,

  params = new_params(
    hist = NULL, hist_args = list(), hist_fun = graphics::hist,
    just = 0.5, nbin = 15L, oob = oob_keep, metric = "counts",
    alpha = NA, key = "sequence"
  ),

  extract_decor = function(scale, hist, hist_args, hist_fun, ...) {
    if (is.null(hist)) {
      return(NULL) # extract data later
    }
    if (is.atomic(hist)) {
      hist <- filter_finite(scale$transform(hist))
      hist <- inject(hist_fun(hist, !!!hist_args))
    }
    check_histogram(hist)
    hist
  },

  extract_params = function(scale, params, ...) {
    params <- GizmoDensity$extract_params(scale, params, ...)
    if (is.null(params$hist) && inherits(params$key, "key_bins")) {
      breaks <- sort(union(params$key$min, params$key$max))
      params$hist_args$breaks <- params$hist_args$breaks %||% breaks
    }
    params
  },

  get_layer_key = function(params, layers, data = NULL, ...) {
    hist <- params$decor %||% params$hist
    if (length(hist) == 0L) {
      values <-
        vec_c(!!!lapply(data, .subset2, params$aesthetic)) |>
        filter_finite()
      hist   <- inject(params$hist_fun(values, !!!params$hist_args))
      check_histogram(hist, params$metric)
    }
    params$decor  <- normalise_histogram(hist, params$metric)
    params$limits <- range(params$limits, params$decor$x)
    params
  }
)

# Helpers -----------------------------------------------------------------

normalise_histogram <- function(hist, metric = "counts") {
  x <- hist$breaks
  y <- hist[[metric]]

  xlim <- range(x, na.rm = TRUE, finite = TRUE)
  x    <- oob_squish_infinite(x, xlim)

  ylim <- range(y, na.rm = TRUE, finite = TRUE)
  y    <- oob_squish_infinite(y, ylim)

  list(
    x = rep(x, each = 2L),
    y = rescale_max(
      c(0.0, rep(y, each = 2L), 0.0),
      to = c(0.0, 0.9), from = ylim
    )
  )
}

check_histogram <- function(
  x, metric = "counts", arg = caller_arg(x), call = caller_env()
) {
  if (is_missing(x)) {
    cli::cli_abort("{.arg {arg}} cannot be missing.", call = call)
  }
  if (inherits(x, "histogram")) {
    # We'll trust this class
    return(x)
  }
  check_list_names(x, c("breaks", metric), arg = arg, call = call)

  if (length(x$breaks) != (length(x[[metric]]) + 1L)) {
    cli::cli_abort(c(
      "In the {.arg {arg}} argument, the {.field breaks} element should be \\
       exactly 1 longer than the {.field {metric}} element.",
      i = "{.code {arg}$breaks} has length {length(x$breaks)}.",
      i = "{.code {arg}${metric}} has length {length(x[[metric]])}."
    ), call = call)
  }
  check_length(x$breaks, min = 2L, arg = as_cli("{arg}$breaks"), call = call)
  check_bare_numeric(x$breaks,     arg = as_cli("{arg}$breaks"), call = call)
  check_bare_numeric(x$counts,     arg = as_cli("{arg}${metric}"), call = call)
  invisible()
}

suppress_hist_plot <- function(hist) {
  if (!quo_is_call(hist)) {
    return(eval_tidy(hist))
  }
  expr <- quo_get_expr(hist)
  if (expr[[1L]] != quote(hist)) {
    return(eval_tidy(hist))
  }
  expr <- call_match(expr, graphics::hist)
  expr <- call_modify(expr, plot = FALSE)
  hist <- quo_set_expr(hist, expr)
  eval_tidy(hist)
}

test_that("gizmo_histogram can compute histograms in various ways", {

  values <- mtcars$mpg
  target <- hist(mtcars$mpg, breaks = 10L, plot = FALSE)
  scale  <- scale_colour_gradient(limits = range(values))
  scale$train(values)
  fields <- c("breaks", "counts")

  guide <- gizmo_histogram(hist = target)
  expect_identical(guide$params$hist[fields], target[fields])

  guide <- gizmo_histogram(hist = values, hist.args = list(breaks = 10L))
  params <- guide$train(guide$params, scale, "colour")
  expect_identical(params$decor[fields], target[fields])

  p <- ggplot(mtcars, aes(drat, wt, colour = mpg)) + geom_point() +
    guides(colour = gizmo_histogram(hist.args = list(breaks = 10L)))
  b <- ggplot_build(p)
  result <- b$plot$guides$params[[1L]]$decor

  expect_identical(result$x, rep(target$breaks, each = 2L))
  expect_identical(
    result$y,
    c(0.0, rep(rescale_max(target$counts, to = c(0.0, 0.9)), each = 2L), 0.0)
  )

})

test_that("check_histogram throws appropriate errors", {

  expect_silent(
    check_histogram(hist(mtcars$mpg, plot = FALSE))
  )

  expect_error(
    check_histogram(arg = "x"),
    "cannot be missing"
  )
  expect_error(
    check_histogram(list(foo = 1L, bar = 2L)),
    "must have named"
  )
  expect_error(
    check_histogram(list(breaks = 1L, counts = 1L)),
    "should be exactly 1 longer"
  )
  expect_error(
    check_histogram(list(breaks = 1L, counts = integer())),
    "more than or equal to 2"
  )
})

test_that("hist(plot = TRUE) is suppressed", {
  skip_if_not_installed("ragg")

  # Normally hist should shade pixels in ragg's buffer
  cap <- ragg::agg_capture()
  hist(mtcars$mpg, plot = TRUE)
  img <- cap()
  dev.off()
  expect_gt(length(unique(as.vector(img))), 1L)

  # But using it in `gizmo_histogram()` should suppress that
  cap <- ragg::agg_capture()
  gizmo_histogram(hist = hist(mtcars$mpg, plot = TRUE))
  img <- cap()
  dev.off()
  expect_identical(unique(as.vector(img)), "white")
})

# Constructors -------------------------------------------------------------

#' Colour rings and arcs
#'
#' Similar to [`guide_colourbar()`][ggplot2::guide_colourbar], this guide
#' displays continuous `colour` or `fill` aesthetics. Instead of a bar, the
#' gradient in shown in a ring or arc, which can be convenient for cyclical
#' palettes such as some provided in the \pkg{scico} package.
#'
#' @param key A [standard key][key_standard] specification. Defaults to
#'   [`key_auto()`].
#' @param start,end A `<numeric[1]>` in radians specifying the offset of the
#'   starting and end points from 12 o'clock. The `NULL` default for `end`,
#'   internally defaults to `start + 2 * pi`.
#' @param outer_guide,inner_guide Guides to display on the outside and inside
#'   of the colour ring. Each guide can be specified using one of the following:
#'   * A `<Guide>` class object.
#'   * A `<function>` that returns a `<Guide>` class object.
#'   * A `<character>` naming such function, without the `guide_` or
#'     `primitive_` prefix.
#' @param
#' nbin A positive `<integer[1]>` determining how many colours to display.
#' @param reverse A `<logical[1]>` whether to reverse continuous guides.
#'   If `TRUE`, guides like colour bars are flipped. If `FALSE` (default),
#'   the original order is maintained.
#' @param show_labels A `<character[1]>` indicating for which guide labels
#'   should be shown. Can be one of `"outer"` (default), `"inner"`, `"both"` or
#'   `"none"`. Note that labels can only be omitted if the related guide
#'   has a label suppression mechanism.
#' @param vanilla A `<logical[1]>` whether to have the default style match
#'   the vanilla `guide_colourbar()` (`TRUE`) or take the theme
#'   verbatim (`FALSE`).
#' @param ... Arguments forwarded to the `outer_guide` and `inner_guide` if
#'   provided as functions or strings.
#' @inheritParams common_parameters
#'
#' @details
#' ## Styling options
#'
#' This guide is a hybrid composition guide, where the [theme][ggplot2::theme]
#' settings apply both this guide itself and the constituents it manages.
#' The constituents are linked below so you can find their 'Styling options'
#' sections. Note that `guide_axis_base()` is just a default that can be
#' swapped out.
#'
#' | **Constituent** | **Description** |
#' | ------------- | --------------- |
#' | [`guide_axis_base()`] | Makes up the tick marks and labels at inner and outer rings. |
#'
#' In addition to the constituent's theme setting, it also has the following
#' settings:
#'
#' | **Theme setting** | **Type** | **Description** |
#' | ----------------- | -------- | --------------- |
#' | `legend.background`| [`element_rect()`] | Background of the legend. |
#' | `legend.margin` | [`margin()`] | Padding around the legend. |
#' | `legend.key.width` | [`unit()`] | Radial thickness of the donut ring. |
#' | `legend.key.size` | [`unit()`] | Size of the legend. Multiplied by 5 to roughly match dimensions of colour bar guides. |
#' | `legend.frame` | [`element_rect()`] | Outline of the donut. The `fill` setting is ignored. |
#' | `legend.title` | [`element_text()`] | Title of the legend. |
#' | `legend.title.position` | `<character[1]>` | One of `"top"`, `"right"`, `"bottom"` or `"left"`.
#'
#' Styling options *per break* can be set in the [standard key][key_standard].
#' These override theme settings.
#'
#' The context-agnostic alternative to using `theme()` is to use
#' [`theme_guide()`]:
#'
#' ```r
#' guide_colring(theme = theme_guide(
#'   # Ring settings
#'   title = element_text(),
#'   title.position = "top",
#'   margin = margin(5),
#'   background = element_rect(),
#'   frame = element_rect(),
#'   key.size = unit(1, "cm"),
#'   key.width = unit(5, "mm"),
#'
#'   # Common options for `guide_axis_base()`
#'   line = element_line(),
#'   text = element_text(),
#'   ticks = element_line(),
#'   ticks.length = unit(5, "mm"),
#' ))
#' ```
#'
#' @return A `<Guide>` object.
#' @export
#' @family standalone guides
#'
#' @examples
#' # Rings works best with a cyclical palette
#' my_pal <- c("black", "tomato", "white", "dodgerblue", "black")
#'
#' p <- ggplot(mpg, aes(displ, hwy, colour = cty)) +
#'   geom_point() +
#'   scale_colour_gradientn(colours = my_pal)
#'
#' # Standard colour ring
#' p + guides(colour = "colring")
#'
#' # As an arc
#' p + guides(colour = guide_colring(
#'   start = 1.25 * pi, end = 2.75 * pi
#' ))
#'
#' # Removing the inner tick marks
#' p + guides(colour = guide_colring(inner_guide = "none"))
#'
#' # Include labels on the inner axis
#' p + guides(colour = guide_colring(show_labels = "both"))
#'
#' # Passing an argument to inner/outer guides
#' p + guides(colour = guide_colring(angle = 0))
guide_colring <- function(
  title = waiver(),
  key = "auto",
  start = 0.0,
  end = NULL,
  outer_guide = "axis_base",
  inner_guide = "axis_base",
  nbin = 300L,
  reverse = FALSE,
  show_labels = "outer",
  theme = NULL,
  vanilla = TRUE,
  position = waiver(),
  available_aes = c("colour", "fill"),
  ...
) {
  show_labels <- arg_match0(show_labels, c("both", "inner", "outer", "none"))
  show_labels <- list(
    inner = show_labels %in% c("inner", "both"),
    outer = show_labels %in% c("outer", "both")
  )

  defaults <- if (isTRUE(vanilla)) vanilla_colourbar_theme() else NULL

  coord <- coord_radial(
    start = start, end = end,
    expand = FALSE, inner.radius = 0.9
  )

  new_compose(
    list(inner = inner_guide, outer = outer_guide),
    title = title,
    coord = coord,
    args = list(...),
    key = key,
    nbin = nbin,
    reverse = reverse,
    theme = theme,
    theme_defaults = defaults,
    show_labels = show_labels,
    position = position,
    available_aes = available_aes,
    super = GuideColring
  )
}

# Class -------------------------------------------------------------------

#' @export
#' @rdname legendry_extensions
#' @format NULL
#' @usage NULL
GuideColring <- ggproto(
  "GuideColring", Compose,

  params = new_params(
    guides = list(), guide_params = list(),
    key = NULL, angle = waiver(), coord = NULL,
    nbin = 300L, alpha = NA, reverse = FALSE,
    show_labels = list(inner = FALSE, outer = TRUE),
    theme_defaults = NULL
  ),

  elements = list(
    frame = "legend.frame",
    size  = "legend.key.size",
    width = "legend.key.width",
    background = "legend.background",
    title = "legend.title",
    title_position = "legend.title.position",
    margin = "legend.margin"
  ),

  train = function(self, params = self$params, scale, aesthetic = NULL,
                   title = waiver(), ...) {
    params$guide_params$inner$position <- "theta.sec"
    params$guide_params$outer$position <- "theta"
    params <- Compose$train(params, scale, aesthetic, title, ...)
    params <- ggproto_parent(Guide, self)$train(params, scale, aesthetic)
    params
  },

  extract_key = function(scale, aesthetic, nbin, ...) {
    standard_extract_key(scale, aesthetic, key_sequence(nbin))
  },

  extract_decor = function(scale, aesthetic, ...) {
    data_frame(!!aesthetic := scale$get_limits(), group = 1L)
  },

  get_layer_key = function(params, layers, data = NULL, ...) {
    params <- ring_xy(params, params$aesthetic)

    params <- Compose$get_layer_key(params, layers, data, ...)
    if (isTRUE(params$reverse)) {
      params <- set_limits(params, rev(params$limits))
    }

    panels <- params$coord$setup_panel_params(
      scale_x_continuous(limits = params$limits),
      scale_y_continuous(limits = c(0.0, 1.0))
    )
    # Override parameters
    panels$inner_radius <- c(0.5, 0.5)
    panels$bbox <- params$bbox <-
      polar_bbox(panels$arc, 0.0, panels$inner_radius)

    params <- Compose$transform(params, params$coord, panels)
    params$arc  <- panels$arc %% (2.0 * pi)
    params$decor <- coord_munch(
      params$coord, params$decor,
      panels, segment_length = 0.04
    )
    params
  },

  build_frame = function(params, elements) {
    frame <- elements$frame
    if (is_blank(frame)) {
      frame <- list(colour = NA, linewidth = 0.0)
    }
    decor <- params$decor

    x <- unit(decor$x, "npc")
    y <- unit(decor$y, "npc")

    x <- unit.c(x, rev(x) - unit(sin(rev(decor$theta)) * elements$width, "cm"))
    y <- unit.c(y, rev(y) - unit(cos(rev(decor$theta)) * elements$width, "cm"))

    if (abs(diff(params$coord$arc %% (2.0 * pi))) < 1e-2) {
      id <- nrow(decor)[c(1L, 1L)]
    } else {
      id <- nrow(decor) * 2L
    }

    polygonGrob(
      x = x, y = y, id.lengths = id,
      gp = gpar(
        fill = NA,
        col = frame$colour,
        lwd = frame$linewidth * .pt,
        lty = frame$linetype
      )
    )
  },

  build_arc = function(params, elements, frame) {
    check_device("clippingPaths")
    arc <- params$coord$arc
    limits <- params$limits %||% range(params$key$.value)
    theta <- rescale(params$key$.value, to = arc, from = limits)
    difftheta <- diff(theta)
    start <- c(arc[1L], theta[-1L] - 0.5 * difftheta)
    end   <- c(theta[-length(theta)] + 0.5 * difftheta, arc[2L])

    theta <- vec_interleave(start, start, end, end)
    r <- rep(c(0.0, 1.0, 1.0, 0.0), length(start)) * elements$width

    x <- rescale(0.5 * sin(theta) + 0.5, from = params$bbox$x)
    y <- rescale(0.5 * cos(theta) + 0.5, from = params$bbox$y)

    x <- unit(x, "npc") - unit(sin(theta) * r, "cm")
    y <- unit(y, "npc") - unit(cos(theta) * r, "cm")

    colour <- params$key[[params$aesthetic]]
    ring <- polygonGrob(
      x = x, y = y, id.lengths = rep(4L, nrow(params$key)),
      gp = gpar(fill = colour, col = colour, lwd = 1.0),
      vp = viewport(clip = frame)
    )
    gTree(children = gList(ring, frame))
  },

  setup_elements = function(params, elements, theme) {
    elements$title <- setup_legend_title(theme, direction = params$direction)
    theme$legend.frame <- theme$legend.frame %||% element_blank()
    Guide$setup_elements(params, elements, theme)
  },

  override_elements = function(params, elements, theme) {
    elements$title_position <- elements$title_position %||%
      switch(params$direction, horizontal = "left", vertical = "top")
    check_position(
      elements$title_position,
      theta = FALSE,
      arg = "legend.title.position"
    )
    elements$width <- cm(elements$width)
    elements$size  <- cm(elements$size) * 5.0
    elements$margin <- elements$margin %||% margin()
    elements$background <- element_grob(elements$background)
    elements
  },

  draw = function(self, theme, position = NULL, direction = NULL,
                  params = self$params) {

    position  <- params$position  <- params$position  %||% position
    direction <- params$direction <- params$direction %||% direction
    position  <- switch(position, inside = "right", position)
    check_position(position, theta = FALSE)
    check_argmatch(direction, c("horizontal", "vertical"))

    theme <- theme + params$theme
    theme <- apply_theme_defaults(theme, params$theme_defaults)
    theme$legend.text.position <- "theta"

    elems <- self$setup_elements(params, self$elements, theme)
    elems <- self$override_elements(params, elems, theme)

    # Draw inner guide
    inner <- params$guide_params$inner
    inner$stack_offset <- unit(elems$width, "cm")
    inner$draw_label <- isTRUE(params$show_labels$inner)
    inner <-
      params$guides$inner$draw(theme, position, direction, params = inner)

    # Draw outer guide
    outer <- params$guide_params$outer
    outer$draw_label <- isTRUE(params$show_labels$outer)
    outer <-
      params$guides$outer$draw(theme, position, direction, params = outer)

    # Draw ring
    frame <- self$build_frame(params, elems)
    ring  <- self$build_arc(params, elems, frame)

    # Setup gtable
    asp <- with(params$bbox, diff(y) / diff(x))
    gt <- gtable(
      widths  = unit(elems$size * pmin(1.0 / asp, 1.0), "cm"),
      heights = unit(elems$size * pmin(asp, 1.0), "cm")
    ) |>
      gtable_add_grob(ring,  1L, 1L, name = "ring",  clip = "off") |>
      gtable_add_grob(inner, 1L, 1L, name = "inner", clip = "off") |>
      gtable_add_grob(outer, 1L, 1L, name = "outer", clip = "off")

    # Add padding, title, margin and background
    margin <-
      ring_margin(params$arc, outer$offset, elems$width + cm(inner$offset))
    title  <- self$build_title(params$title, elems)
    gt <- gtable_add_padding(gt, margin) |>
      self$add_title(
        title, elems$title_position,
        get_just(elems$title)
      ) |>
      gtable_add_padding(elems$margin)

    if (!is_zero(elems$background)) {
      gt <- gtable_add_grob(
        gt, elems$background,
        name = "background", clip = "off",
        t = 1L, r = -1L, b = -1L, l = 1L, z = -Inf
      )
    }

    gt
  }
)

# Helpers -----------------------------------------------------------------

ring_xy <- function(params, aesthetic) {
  if ("guide_params" %in% names(params)) {
    params$guide_params <-
      lapply(params$guide_params, ring_xy, aesthetic = aesthetic)
  }
  key <- params$key
  if (!is.null(key)) {
    key$x <- key$x %||% key$.value %||% key[[aesthetic]]
    key$y <- key$y %||% 1.0
    params$key <- key
  }
  decor <- params$decor
  if (!is.null(decor)) {
    decor$x <- decor$x %||% decor$.value %||% decor[[aesthetic]]
    decor$y <- decor$y %||% 1.0
    params$decor <- decor
  }
  params
}

ring_margin <- function(arc, outer = NULL, inner = NULL) {

  outer <- cm(outer %||% unit(0.0, "cm"))

  # If we have a full circle, apply outer padding to every margin
  if (abs(diff(arc)) >= 2.0 * pi) {
    return(unit(rep(outer, 4L), "cm"))
  }

  tol <- c(1.0, -1.0) * sqrt(.Machine$double.eps)
  margin <- rep(0.0, 4L)
  inner  <- cm(inner %||% unit(0.0, "cm"))

  # Left margin
  keep <- in_range(arc, c(1.0, 2.0) * pi + tol)
  if (in_arc(1.5 * pi, arc)) {
    margin[4L] <- outer
  } else if (any(keep)) {
    theta <- arc[keep]
    margin[4L] <- max(abs(cos(theta))) * outer
  } else if (all(in_range(arc, c(0.0, 1.0) * pi + tol))) {
    margin[4L] <- max(cos(arc)) * inner
  }

  # Right margin
  keep <- in_range(arc, c(0.0, 1.0) * pi + tol)
  if (in_arc(0.5 * pi, arc)) {
    margin[2L] <- outer
  } else if (any(keep)) {
    theta <- arc[keep]
    margin[2L] <- max(abs(cos(theta))) * outer
  } else if (all(in_range(arc, c(1.0, 2.0) * pi + tol))) {
    margin[2L] <- max(cos(arc)) * inner
  }

  # Bottom margin
  keep <- in_range(arc, c(0.5, 1.5) * pi + tol)
  if (in_arc(pi, arc)) {
    margin[3L] <- outer
  }  else if (any(keep)) {
    theta <- arc[keep]
    margin[3L] <- max(abs(sin(theta))) * outer
  } else if (all(in_arc(arc, c(1.5, 2.5) * pi + tol))) {
    margin[3L] <- max(sin(arc)) * inner
  }

  # Top margin
  keep <-
    in_range(arc, c(0.0, 0.5) * pi + c(0.0, tol[2L])) |
    in_range(arc, c(1.5, 2.0) * pi + c(tol[1L], 0.0))
  if (in_arc(0.0, arc)) {
    margin[1L] <- outer
  } else if (any(keep)) {
    theta <- arc[keep]
    margin[1L] <- max(abs(sin(theta))) * outer
  } else if (all(in_arc(arc, c(0.5, 1.5) * pi + tol))) {
    margin[1L] <- max(-sin(arc)) * inner
  }

  unit(margin, "cm")
}

test_that("guide_colring looks as it should", {

  # vdiffr's device doesn't support clipping paths yet, so we just ignore
  # warnings for now
  expect_doppelganger <- function(...) {
    suppressWarnings(vdiffr::expect_doppelganger(...))
  }

  p <- ggplot(mtcars, aes(disp, mpg, colour = drat)) +
    geom_point() +
    theme(
      legend.frame = element_rect(colour = "tomato"),
      legend.background = element_rect(colour = "limegreen")
    )

  outline <- compose_stack("axis_base", primitive_line(theme = theme(
    legend.axis.line = element_line(colour = "dodgerblue")
  )), theme = theme(legendry.guide.spacing = unit(0.0, "cm")))

  standard_ring <- guides(colour = guide_colring(
    nbin = 15L, outer_guide = outline, inner_guide = outline
  ))

  pring <- p + standard_ring

  expect_doppelganger(
    "standard ring", pring
  )

  pring <- p +
    guides(
      colour = guide_colring(
        nbin = 15L,
        outer_guide = outline,
        inner_guide = outline,
        show_labels = "inner"
      )
    )

  expect_doppelganger(
    "inner labels", pring
  )

  pring <- p + standard_ring + theme(
    legend.key.size = unit(2.0, "lines"),
    legend.key.width = unit(5.0, "mm")
  )

  expect_doppelganger(
    "resized ring", pring
  )

  pring <- p + standard_ring + theme(legend.key.width = rel(2.5))

  expect_doppelganger(
    "conical", pring
  )

  pring <- p + guides(colour = guide_colring(
    nbin = 15L, outer_guide = outline, inner_guide = outline,
    start = 0.25 * pi, end = 1.75 * pi
  ))

  expect_doppelganger(
    "open ring", pring
  )
})

test_that("ring_margin calculates margins correctly", {

  # Full ring should have outer margins everywhere
  test <- ring_margin(c(0.0, 2.0) * pi, outer = 2.0, inner = 1.0)
  expect_equal(as.numeric(test), c(2.0, 2.0, 2.0, 2.0))

  test <- ring_margin(c(0.0, 1.0) * pi, outer = 2.0, inner = 1.0)
  expect_equal(as.numeric(test), c(2.0, 2.0, 2.0, 0.0))

  test <- ring_margin(c(0.5, 1.5) * pi, outer = 2.0, inner = 1.0)
  expect_equal(as.numeric(test), c(0.0, 2.0, 2.0, 2.0))

  test <- ring_margin(c(1.0, 2.0) * pi, outer = 2.0, inner = 1.0)
  expect_equal(as.numeric(test), c(2.0, 0.0, 2.0, 2.0))

  test <- ring_margin(c(1.5, 0.5) * pi, outer = 2.0, inner = 1.0)
  expect_equal(as.numeric(test), c(2.0, 2.0, 0.0, 2.0))

  s2 <- sqrt(2.0)

  test <- ring_margin(c(0.25, 0.75) * pi, outer = 2.0, inner = 1.0)
  expect_equal(as.numeric(test), c(s2, 2.0, s2, s2 / 2.0))

  test <- ring_margin(c(0.75, 1.25) * pi, outer = 2.0, inner = 1.0)
  expect_equal(as.numeric(test), c(s2 / 2.0, s2, 2.0, s2))

  test <- ring_margin(c(1.25, 1.75) * pi, outer = 2.0, inner = 1.0)
  expect_equal(as.numeric(test), c(s2, s2 / 2.0, s2, 2.0))

  test <- ring_margin(c(1.75, 0.25) * pi, outer = 2.0, inner = 1.0)
  expect_equal(as.numeric(test), c(2.0, s2, s2 / 2.0, s2))

})

# Speciality keys ----------------------------------------------------------

#' Speciality keys
#'
#' @description
#' These functions are helper functions for working with keys in guides. The
#' functions described here are not widely applicable and may only apply
#' to a small subset of guides. As such, it is fine to adjust the arguments
#' of a speciality key, but swapping types is ill-advised.
#'
#' * `key_sequence()` is a function factory whose functions create a regularly
#'   spaced sequence between the limits of a scale. It is used in colour bar
#'   guides.
#' * `key_bins()` is a function factory whose function create a binned key
#'   given the breaks in the scale. It is used in colour steps guides.
#' * `key_upset()` is a function factory whose function creates an upset key
#'   from splitting the breaks in the scale. It is used in the upset guide.
#' * `key_symbols()` is a function factory whose function creates a key
#'   from the literal provided values. It is used in the symbols guide.
#'
#' @param n A positive `<integer[1]>` giving the number of colours to use for a
#'   gradient.
#' @param even.steps A `<logical[1]>` indicating whether the size of bins
#'   should be displayed as equal (`TRUE`) or proportional to their length in
#'   data space (`FALSE`).
#' @param show.limits A `<logical[1]>` stating whether the limits of the scale
#'   should be shown with labels and ticks (`TRUE`) or remain hidden (`FALSE`).
#'   Note that breaks coinciding with limits are shown regardless of this
#'   setting. The default, `NULL`, consults the scale's `show.limits` setting
#'   or defaults to `FALSE`.
#' @inheritParams key_group sep
#' @param order Order to set the upset layers in. One of the following:
#' * A `<character[n]>` giving pieces of split labels.
#' * An `<integer[n]>` giving the numerical order in which pieces of split
#'   labels should appear.
#' @param empty_label A `<character[1]>` giving a level label to assign to the
#'   breaks that match no values to the pieces of split labels. Can be `NULL`
#'   to omit labels for empty levels.
#' @param aesthetic A vector of values for the guide to represent equivalent to
#'   the `breaks` argument in scales. These will be mapped by the scale to
#'   positions. Alternatively, a `<numeric[n]>` vector to set positions
#'   directly. Positions are used to place symbols.
#' @param level A `<factor[n]>` or `<character[n]>` parallel to the `aesthetic`
#'   argument setting the label level of the symbol.
#' @param symbol (Optional) An `<integer[n]>` indexing the guide's
#'   `override.aes` parameter.
#' @param ... Additional graphical properties to set for each symbol. Valid
#'   properties are `colour`, `shape`, `size`, `fill` and `stroke`. These
#'   graphical properties have priority over properties derived via `symbol`
#'   or the theme.
#'
#' @return
#' A function.
#'
#' @name key_specialty
#' @family keys
#'
#' @examples
#' # An example scale
#' template <- scale_fill_viridis_c(limits = c(0, 10), breaks = c(2, 4, 6, 8))
#'
#' # Retrieving colourbar and colourstep keys
#' key_sequence()(template)
#' key_bins()(template)
#'
#' # Upset key with example scale
#' template <- scale_x_discrete(limits = c("A", "A,B", ""))
#' key_upset()(template)
#' # Putting 'B' in 1st level
#' key_upset(order = c("B", "A"))(template)
#' # Omit level for the empty break
#' key_upset(empty_label = NULL)(template)
#'
#' # Symbol key with example scale
#' template <- scale_x_discrete(limits = LETTERS[1:5])
#' key_symbols(aesthetic = LETTERS[1:3], level = 3:1)(template)
#' # Aesthetic can also be numeric
#' key_symbols(1:3, 3:1)(template)
#' # Setting level order via factors
#' ordered <- factor(c("X", "Y", "Z"), c("Y", "X", "Z"))
#' key_symbols(1:3, level = ordered)(template)
#' # Setting groups for symbols, for `guide_axis_symbols(override.aes)`
#' key_symbols(1:3, 1:3, symbol = c(1, 1, 2))(template)
#' # Passing individual graphical parameters
#' key_symbols(1:3, 3:1, colour = c("red", "green", "blue"))(template)
NULL

#' @export
#' @rdname key_specialty
key_sequence <- function(n = 15L) {
  force(n)
  check_number_whole(n, min = 2.0)
  function(scale, aesthetic = NULL) {
    aesthetic <- aesthetic %||% scale$aesthetics[1L]
    df <- map_sequence(scale = scale, aesthetic = aesthetic, nbin = n)
    class(df) <- c("key_sequence", "key_guide", class(df))
    df
  }
}

#' @export
#' @rdname key_specialty
key_bins <- function(even.steps = FALSE, show.limits = NULL) {
  force(even.steps)
  force(show.limits)
  function(scale, aesthetic = NULL) {
    aesthetic <- aesthetic %||% scale$aesthetics[1L]
    df <- binned_key(
      scale = scale, aesthetic = aesthetic,
      even_steps = even.steps, show_limits = show.limits
    )
    class(df) <- c("key_bins", "key_guide", class(df))
    df
  }
}

#' @export
#' @rdname key_specialty
key_upset <- function(
  sep = "[^[:alnum:]]+", order = NULL, empty_label = "Other"
) {
  check_string(sep)
  force(string)
  force(order)
  function(scale, aesthetic = NULL) {
    upset_from_split_label(
      scale = scale, aesthetic = aesthetic,
      sep = sep, order = order, empty_label = empty_label
    )
  }
}

#' @export
#' @rdname key_specialty
key_symbols <- function(aesthetic, level, symbol = NULL, ...) {
  if (is_integerish(level)) {
    level_universe <- sort(unique(level))
  } else {
    level_universe <- levels(level) %||% unique(level)
  }
  index <- match(level, level_universe)
  valid <- c("colour", "color", "shape", "size", "fill", "stroke")
  key <- data_frame0(
    aesthetic = aesthetic,
    .value = factor(level, level_universe),
    .col = index,
    .symbol = symbol,
    !!!extra_args(..., .valid_args = valid)
  )
  function(scale, aesthetic = NULL) {
    merge_symbol_key(scale, aesthetic, key)
  }
}

# Helpers -----------------------------------------------------------------

map_sequence <- function(scale, aesthetic, nbin = 15L, ...) {
  if (scale$is_discrete()) {
    cli::cli_abort("Cannot use {.fn key_sequence} for discrete scales.")
  }

  limits <- scale$get_limits()
  key <- seq(limits[1L], limits[2L], length.out = nbin)
  if (length(key) == 0L) {
    key <- vec_unique(limits)
  }
  key <- data_frame0(
    !!aesthetic := scale$map(key),
    .value  = key,
    .size  = length(key)
  )
  key
}

binned_key <- function(scale, aesthetic, even_steps, show_limits = NULL) {

  breaks <- scale$get_breaks()
  limits <- scale$get_limits()

  if (even_steps) {
    breaks <- parse_binned_breaks(scale, breaks, even.steps = even_steps)

    n <- length(breaks$bin_at)
    seq <- seq(0.0, n)
    seq <- rescale(seq, to = limits, from = c(0.0, n))

    key <- data_frame0(
      !!aesthetic := scale$map(breaks$bin_at),
      min = seq[-length(seq)],
      max = seq[-1L]
    )
    key <- vec_c(key, NA)
    key$.label <- key$.value <- NA

    key$.label[breaks$all %in% breaks$breaks] <- scale$get_labels(breaks$breaks)
    key$.value <- seq
  } else {
    all <- unique(sort(c(limits, breaks)))
    n <- length(all)
    bin_at <- (all[-1L] + all[-n]) / 2.0
    key <- data_frame0(
      !!aesthetic := scale$map(bin_at),
      min = all[-n],
      max = all[-1L]
    )
    key <- vec_c(key, NA)
    key$.label <- NA

    key$.label[all %in% breaks] <- scale$get_labels(breaks)
    key$.value <- all
  }

  show_limits <- show_limits %||% scale$show.limits %||% FALSE

  if (show_limits && (is.character(scale$labels) || is.numeric(scale$labels))) {
    cli::cli_warn(c(paste0(
      "{.arg show.limits} is ignored when {.arg labels} are given as a ",
      "{.cls character} vector."
    ), i = paste0(
      "Either add the limits to {.arg breaks} or provide a function for ",
      "{.arg labels}."
    )))
    show_limits <- FALSE
  }

  n <- nrow(key)
  if (show_limits) {
    key$.label[c(1L, n)] <- scale$get_labels(limits)
  }
  key$.value[is.na(key$.label)] <- NA
  key
}

parse_binned_breaks <- function(scale, breaks = scale$get_breaks(),
                                even.steps = TRUE) {
  breaks <- breaks[!is.na(breaks)]
  if (length(breaks) == 0L) {
    return(NULL)
  }
  breaks <- sort(breaks)
  if (is.numeric(breaks)) {
    limits <- scale$get_limits()
    if (!is.numeric(scale$breaks)) {
      breaks <- breaks[!breaks %in% limits]
    }
    all_breaks <- unique0(c(limits[1L], breaks, limits[2L]))
    bin_at <- all_breaks[-1L] - diff(all_breaks) / 2.0
  } else {
    if (isFALSE(even.steps)) {
      cli::cli_warn(paste0(
        "{.code even.steps = FALSE} is not supported when used with a ",
        "discrete scale."
      ))
    }
    bin_at <- breaks
    nums <- as.character(breaks)
    nums <- strsplit(gsub("\\(|\\)|\\[|\\]", "", nums), ",\\s?")
    nums <- as.numeric(unlist(nums, FALSE, FALSE))

    if (anyNA(nums)) {
      cli::cli_abort(c(
        "Breaks are not formatted correctly for a bin legend.",
        i = "Use {.code (<lower>, <upper>]} format to indicate bins."
      ))
    }
    all_breaks <- nums[c(1L, seq_along(breaks) * 2L)]
    limits <- all_breaks[ c(1L, length(all_breaks))]
    breaks <- all_breaks[-c(1L, length(all_breaks))]
  }
  list(
    breaks = breaks,
    limits = limits,
    bin_at = bin_at,
    all    = all_breaks
  )
}

upset_from_split_label <- function(
  scale, aesthetic, sep, order, empty_label = NULL
) {

  aesthetic <- aesthetic %||% scale$aesthetics[1L]
  key <- Guide$extract_key(scale, aesthetic)

  label <- strsplit(key$.label, split = sep)

  levels <- unique(unlist(label))
  if (is.character(order)) {
    levels <- c(intersect(order, levels), setdiff(levels, order))
  } else if (is.numeric(order)) {
    levels <- levels[union(order, seq_along(levels))]
  }
  levels <- factor(levels, levels)

  mtx <- t(vapply(label, function(x) levels %in% x, logical(length(levels))))
  colnames(mtx) <- levels

  empty <- rowSums(mtx) == 0.0
  if (any(empty) && !is.null(empty_label)) {
    empty[!empty] <- NA
    mtx <- cbind(mtx, empty)
    colnames(mtx)[ncol(mtx)] <- empty_label
    levels <- c(levels, factor(empty_label, empty_label))
  }

  row <- as.vector(row(mtx))
  col <- as.vector(col(mtx))
  df <- data_frame0(
    x = key$x[row],
    y = key$y[row],
    .col = col,
    .row = row,
    .value = levels[col],
    .symbol = as.vector(mtx)
  )

  class(df) <- c("key_guide", "key_upset", class(df))
  df
}

merge_symbol_key <- function(scale, aesthetic, key) {
  aesthetic <- aesthetic %||% scale$aesthetics[1L]
  if (!is.numeric(key$aesthetic)) {
    scale_key <- Guide$extract_key(scale, aesthetic)
    i <- match(key$aesthetic, scale_key$.value)
    key <- data_frame0(
      !!!key[setdiff(names(key), "aesthetic")],
      !!!scale_key[i, setdiff(names(scale_key), names(key))]
    )
  }
  key$.row <- key[[aesthetic]]
  key
}

test_that("key_sequence gives appropriate errors", {

  key <- key_sequence()

  expect_error(
    key(scale_x_discrete(), "x"),
    "for discrete scales"
  )

  expect_error(
    key_sequence(0.0)
  )
})

test_that("key_bins works with discrete scales", {
  skip_if(
    "even.steps" %in% fn_fmls_names(parse_binned_breaks),
    "Not implemented yet"
  )

  scale <- scale_x_discrete(limits = c("[0,1)", "[1,3)", "[3,4)"))

  test <- key_bins(even.steps = TRUE, show.limits = TRUE)(scale, "x")
  expect_identical(test$.label, as.character(c(0L, 1L, 3L, 4L)))
  expect_identical(test$min, c(0L:2L, NA))
  expect_identical(test$max, c(1L:3L, NA))

  test <- key_bins(even.steps = FALSE, show.limits = TRUE)(scale, "x")
  expect_identical(test$.label, as.character(c(0L, 1L, 3L, 4L)))
  expect_identical(test$min, c(0.0, 1.0, 3.0, NA))
  expect_identical(test$max, c(1.0, 3.0, 4.0, NA))

})

test_that("key_bins throws appropriate messages", {

  key <- key_bins(show.limits = TRUE)

  scale <- scale_x_continuous(
    limits = c(0.0, 10.0),
    breaks = c(2.0, 8.0),
    labels = c("2", "8")
  )

  expect_warning(
    key(scale, "x"),
    "is ignored"
  )
})

test_that("key_upset can set order", {
  scale <- scale_x_discrete(limits = c("X,Y", "X", "", "Z"))

  key <- key_upset()
  test <- key(scale)
  test <- test[test$.symbol & !is.na(test$.symbol), ]
  expect_identical(levels(test$.value), c("X", "Y", "Z", "Other"))

  key <- key_upset(order = c("Y", "X"))
  test <- key(scale)
  test <- test[test$.symbol & !is.na(test$.symbol), ]
  expect_identical(levels(test$.value), c("Y", "X", "Z", "Other"))

  key <- key_upset(order = c(3L, 1L))
  test <- key(scale)
  test <- test[test$.symbol & !is.na(test$.symbol), ]
  expect_identical(levels(test$.value), c("Z", "X", "Y", "Other"))
})

test_that("key_upset can set labels for empty levels", {
  scale <- scale_x_discrete(limits = c("X,Y", "X", "", "Z"))

  key <- key_upset(empty_label = "foo")
  test <- key(scale)
  expect_identical(levels(test$.value), c("X", "Y", "Z", "foo"))

  key <- key_upset(empty_label = NULL)
  test <- key(scale)
  expect_identical(levels(test$.value), c("X", "Y", "Z"))
})

test_that("key_symbols levels work as intended", {

  scale <- scale_x_discrete(limits = c("X", "Y", "Z"))
  key <- key_symbols(c("X", "Y", "Z"), c("C", "A", "B"))
  test <- key(scale)

  expect_identical(
    test[c(".value", ".col")],
    data_frame0(
      .value = factor(c("C", "A", "B"), levels = c("C", "A", "B")),
      .col = c(1L, 2L, 3L)
    )
  )

  key <- key_symbols(c("X", "Y", "Z"), factor(c("C", "A", "B")))
  test <- key(scale)
  expect_identical(
    test[c(".value", ".col")],
    data_frame0(
      .value = factor(c("C", "A", "B"), levels = c("A", "B", "C")),
      .col = c(3L, 1L, 2L)
    )
  )

  key <- key_symbols(c("X", "Y", "Z"), c(2L, 3L, 1L))
  test <- key(scale)
  expect_identical(
    test[c(".value", ".col")],
    data_frame0(
      .value = factor(c("2", "3", "1"), levels = c("1", "2", "3")),
      .col = c(2L, 3L, 1L)
    )
  )
})

test_that("gizmo_density can compute density in various ways", {

  values <- mtcars$mpg
  target <- density(mtcars$mpg, n = 10L)
  scale  <- scale_colour_gradient(limits = range(values))
  scale$train(values)

  guide <- gizmo_density(density = target)
  expect_identical(guide$params$density[c("x", "y")], target[c("x", "y")])

  guide <- gizmo_density(density = values, density.args = list(n = 10L))
  params <- guide$train(guide$params, scale, "colour")
  expect_identical(params$decor[c("x", "y")], target[c("x", "y")])

  p <- ggplot(mtcars, aes(drat, wt, colour = mpg)) + geom_point() +
    guides(colour = gizmo_density(density.args = list(n = 10L)))
  b <- ggplot_build(p)
  result <- b$plot$guides$params[[1L]]$decor

  expect_identical(result$x, target$x)
  expect_identical(result$y, rescale_max(target$y, to = c(0.0, 0.9)))

})

test_that("gizmo_density can handle sequential and bin keys", {

  scale <- scale_colour_viridis_c(limits = c(10.0, 30.0))
  scale$train(c(10.0, 30.0))
  colours <- scale$map(seq(10.0, 30.0, length.out = 5L))

  guide  <- gizmo_density(key = key_sequence(n = 5L))
  params <- guide$train(guide$params, scale, "colour")

  expect_identical(params$key$.value, seq(10.0, 30.0, by = 5L))
  expect_identical(params$key$colour, colours)

  scale <- scale_colour_viridis_c(limits = c(10.0, 30.0))
  scale$train(c(0.0, 40.0))

  params <- guide$train(guide$params, scale, "colour")
  expect_identical(params$key$.value, c(9.98, seq(10.0, 30.0, by = 5L), 30.02))
  expect_identical(params$key$colour, c("#7F7F7F", colours, "#7F7F7F"))

  scale <- scale_colour_viridis_c(limits = c(10.0, 30.0))
  scale$train(c(10.0, 30.0))
  colours <- scale$map(seq(12.5, 27.5, length.out = 4L))

  guide  <- gizmo_density(key = key_bins())
  params <- guide$train(guide$params, scale, "colour")

  expect_identical(params$key$min, c(10.0, 15.0, 20.0, 25.0, NA))
  expect_identical(params$key$max, c(15.0, 20.0, 25.0, 30.0, NA))
  expect_identical(params$key$colour, c(colours, NA))

  scale <- scale_colour_viridis_c(limits = c(10.0, 30.0))
  scale$train(c(0.0, 40.0))

  params <- guide$train(guide$params, scale, "colour")
  key <- vec_slice(params$key, !is.na(params$key$colour))

  expect_identical(key$colour, c("#7F7F7F", colours, "#7F7F7F"))
  expect_identical(key$min, c(-Inf, 10.0, 15.0, 20.0, 25.0, 30.0))
  expect_identical(key$max, c(10.0, 15.0, 20.0, 25.0, 30.0, Inf))

})

test_that("check_density throws appropriate errors", {

  expect_silent(
    check_density(density(mtcars$mpg))
  )

  expect_error(
    check_density(arg = "x"),
    "cannot be missing"
  )
  expect_error(
    check_density(list(foo = 1.0, bar = 2.0)),
    "must have named"
  )
  expect_error(
    check_density(list(x = 1.0, y = 1.0)),
    "at least length 2"
  )
  expect_error(
    check_density(list(x = 1L:3L, y = 1L:2L)),
    "must be of equal length"
  )
})

.label_params <- c(
  "text_colour", "colour",
  "text_size", "size",
  "face", "hjust", "vjust", "angle",
  "lineheight"
)
.line_params <- c(
  "line_colour",    "colour",
  "line_linewidth", "linewidth",
  "line_linetype",  "linetype"
)
.rect_params <- c(
  "rect_colour",    "colour",
  "rect_linewidth", "linewidth",
  "rect_linetype",  "linetype",
  "rect_fill",      "fill"
)
.point_params <- c(
  "point_colour", "colour",
  "point_size",  "size",
  "point_fill", "fill",
  "shape", "stroke"
)
.element_params <- unique(
  c(.label_params, .line_params, .rect_params, .point_params)
)

# Collects elemental properties from key into list, taking into account
# some simple inheritance from a global pool of properties, i.e.
# for text elements, the `text_colour` takes precedence over `colour`.
element_key_properties <- function(key, type, ...) {
  props <- switch(
    type,
    point = list(
      colour     = key$.point_colour   %||% key$.colour,
      size       = key$.point_size     %||% key$.size,
      fill       = key$.point_fill     %||% key$.fill,
      shape      = key$.shape,
      stroke     = key$.stroke
    ),
    rect = list(
      colour     = key$.rect_colour    %||% key$.colour,
      linewidth  = key$.rect_linewidth %||% key$.linewidth,
      linetype   = key$.rect_linetype  %||% key$.linetype,
      fill       = key$.rect_fill      %||% key$.fill
    ),
    line = list(
      colour     = key$.line_colour    %||% key$.colour,
      linewidth  = key$.line_linewidth %||% key$.linewidth,
      linetype   = key$.line_linetype  %||% key$.linetype
    ),
    text = list(
      colour     = key$.text_colour    %||% key$.colour,
      size       = key$.text_size      %||% key$.size,
      face       = key$.face,
      hjust      = key$.hjust,
      vjust      = key$.vjust,
      angle      = key$.angle,
      lineheight = key$.lineheight
    ),
    list()
  )
  extra <- lapply(list2(...), rep0, length.out = nrow(key))
  props <- replace_null(props, !!!extra)
  props[lengths(props) > 0]
}

destructure_element <- function(element) {
  if (!S7::S7_inherits(element)) {
    return(element)
  }
  S7::props(element)
}

element_classes <- function(..., allow_null = TRUE) {
  type <- do.call(c, list(...))
  classes <- character()
  if (allow_null) {
    classes <- c(classes, "NULL")
  }
  if ("rect" %in% type) {
    classes <- c(classes, "ggplot2::element_rect", "element_rect")
  }
  if ("line" %in% type) {
    classes <- c(classes, "ggplot2::element_line", "element_line")
  }
  if ("text" %in% type) {
    classes <- c(classes, "ggplot2::element_text", "element_text")
  }
  if ("blank" %in% type) {
    classes <- c(classes, "ggplot2::element_blank", "element_blank")
  }
  if ("point" %in% type) {
    classes <- c(classes, "ggplot2::element_point")
  }
  if ("polygon" %in% type) {
    classes <- c(classes, "ggplot2::element_polygon")
  }
  classes
}

# Constructor -------------------------------------------------------------

#' Guide primitive: fence
#'
#' This function constructs a fence [guide primitive][guide-primitives]. The
#' customisation options are easier to understand if we view fence 'post' as the
#' vertical pieces of a real world fence, and the 'rail' as the horizontal
#' pieces.
#'
#' @inheritParams primitive_bracket
#' @param rail A `<character[1]>` giving an option for how to display fence
#'   railing. Can be either `"none"` (default) to display no railings, `"inner"`
#'   to draw one rail closer to the plot panel, `"outer"` to display one rail
#'   farther from the plot panel, or `"both"` to sandwich the labels between
#'   rails.
#' @param levels_post,levels_rail A list of `<element_line>` objects to
#'   customise how fence posts and rails are displayed at every level.
#'
#' @return A `<PrimitiveFence>` primitive guie that can be used inside other
#'   guides.
#' @family primitives
#' @export
#'
#' @details
#' ## Styling options
#'
#' Below are the [theme][ggplot2::theme] options that determine the styling of
#' this guide, which may differ depending on whether the guide is used in
#' an axis or in a legend context.
#'
#' The possible `{position}` suffixes mentioned below are `x`, `x.top`,
#' `x.bottom`, `y`, `y.left`, `y.right`. The `theta` and `r` position suffixes
#' in \pkg{ggplot2} are *not* obeyed in \pkg{legendry}.
#'
#' | **Theme setting** | **Context** | **Type** | **Description** |
#' | ----------------- | ----------- | -------- | --------------- |
#' | `legendry.fence`  | Both | [`element_line()`] | Line segments for both 'post' and 'rail' segments |
#' | `legendry.fence.post` | Both | [`element_line()`] | Line segments orthogonal to the scale |
#' | `legendry.fence.rail` | Both | [`element_line()`] | Line segments parallel to the scale |
#' | `axis.text.{position}` | Axis | [`element_text()`] | The text labels at the fence. |
#' | `legend.text` | Legend | [`element_text()`] | The text labels at the fence. |
#'
#' Styling options *per level* can be set in the `levels_post`, `levels_rail`
#' and `levels_text` arguments. These override theme settings.
#'
#' Styling options *per range* can be set in the [range key][key_range].
#' The `line` and `text` prefixed properties are prioritised for the fence
#' and text respectively. The 'post' and 'rail' distinction does not apply
#' at the 'per range' settings. These override theme settings and the
#' 'per level' settings.
#'
#' The context-agnostic alternative to using `theme()` is to use
#' [`theme_guide()`]:
#'
#' ```r
#' primitive_fence(theme = theme_guide(
#'   fence = element_line(),
#'   fence.post = element_line(),
#'   fence.rail = element_line(),
#'   text = element_text()
#' ))
#' ```
#'
#' @examples
#' # A standard plot
#' p <- ggplot(mpg, aes(interaction(drv, year), displ)) +
#'   geom_point()
#'
#' key <- key_range_manual(c(2, 4), c(5, 6), c("A", "B"))
#'
#' # Adding as secondary guides
#' p + guides(
#'   x.sec = primitive_fence(rail = "inner"),
#'   y.sec = primitive_fence(key = key, rail = "outer")
#' )
primitive_fence <- function(
  key = "range_auto",
  rail = "none",
  angle = waiver(),
  oob = "squish",
  drop_zero = TRUE,
  pad_discrete = 0.5,
  levels_text = NULL,
  levels_post = NULL,
  levels_rail = NULL,
  theme = NULL,
  position = waiver()
) {

  key <- resolve_key(key)
  oob <- arg_match0(oob, c("squish", "censor", "none"))
  rail <- arg_match0(rail, c("none", "inner", "outer", "both"))
  check_bool(drop_zero)
  check_number_decimal(pad_discrete, allow_infinite = FALSE)
  check_list_of(
    levels_text, element_classes("text", "blank"),
    allow_null = TRUE
  )
  check_list_of(
    levels_post, element_classes("line", "blank"),
    allow_null = TRUE
  )
  check_list_of(
    levels_rail, element_classes("line", "blank"),
    allow_null = TRUE
  )

  new_guide(
    key = key,
    oob = oob,
    rail = rail,
    angle = angle,
    drop_zero = drop_zero,
    pad_discrete = pad_discrete,
    levels_text = levels_text,
    levels_post = levels_post,
    levels_rail = levels_rail,
    theme = theme,
    position = position,
    available_aes = c("any", "x", "y", "r", "theta"),
    super = PrimitiveFence
  )
}

# Class -------------------------------------------------------------------

#' @export
#' @rdname legendry_extensions
#' @format NULL
#' @usage NULL
PrimitiveFence <- ggproto(
  "PrimitiveFence", Guide,

  params = new_params(
    key = NULL, oob = "squish", drop_zero = TRUE,
    pad_discrete = 0.5, angle = waiver(),
    levels_text = NULL, levels_post = NULL, levels_rail = NULL,
    rail = "none"
  ),

  hashables = exprs(key, decor),

  elements = list(
    position = list(
      text = "axis.text",
      post = I("legendry.fence.post"),
      rail = I("legendry.fence.rail")
    ),
    legend = list(
      text = "legend.text",
      post = I("legendry.fence.post"),
      rail = I("legendry.fence.rail")
    )
  ),

  extract_key = range_extract_key,

  extract_params = extract_range_params,

  extract_decor = function(scale, aesthetic, position, key, ...) {

    levels <- sort(unique(key$.level))
    key <- vec_slice(key, key$.draw)
    if (nrow(key) < 1L) {
      return(NULL)
    }

    # Take unique positions by level
    split <- vec_split(key, key$.level)

    decor <- lapply(split$val, function(df) {
      aes <- vec_interleave(df$start, df$end)
      df <- vec_rep_each(df[setdiff(names(df), c("start", "end"))], 2)
      df[[aesthetic]] <- aes
      df
    })

    level_end <- rep(split$key, list_sizes(decor))
    decor <- vec_c(!!!decor)
    decor$.level <- min(levels)
    decor$.level_end <- level_end
    decor <- vec_slice(decor, order(decor$.level_end, decor[[aesthetic]]))

    # We don't want fencepost of outer pieces poke through the railing of
    # the inner pieces.
    for (lvl in levels[-1L]) {
      lower <- which(key$.level == lvl - 1L)
      current <- which(decor$.level_end >= lvl)
      if (length(current) < 1L || length(lower) < 1L) {
        next
      }
      trim <- in_ranges(
        decor[[aesthetic]][current],
        start = key$start[lower],
        end   = key$end[lower]
      )
      decor$.level[current[trim]] <- lvl
    }
    keep <- !duplicated(decor[c(aesthetic, ".level")], fromLast = TRUE)
    vec_slice(decor, keep)
  },

  transform = function(self, params, coord, panel_params) {
    params$key <-
      transform_key(params$key, params$position, coord, panel_params)
    params$decor <-
      transform_key(params$decor, params$position, coord, panel_params)
    params$bbox <- panel_params$bbox %||%
      list(x = c(0.0, 1.0), y = c(0.0, 1.0))
    params
  },

  setup_params = setup_range_params,

  setup_elements = primitive_setup_elements,

  build_fence = function(key, decor, elements, params) {

    levels   <- unique(c(key$.level, decor$.level, decor$.level_end))
    nlevels  <- length(levels)
    position <- params$position

    text_levels <- rep0(params$levels_text, length.out = nlevels)
    post_levels <- rep0(params$levels_post, length.out = nlevels)
    rail_levels <- rep0(params$levels_rail, length.out = nlevels)

    rail <- vec_slice(key, key$.draw)
    key <- justify_ranges(key, levels, elements$text, text_levels)

    if (is_theta(position)) {
      add  <- if (position == "theta.sec") pi else 0.0
      key  <- polar_xy(key, key$r,   key$theta  + add, params$bbox)
      rail <- polar_xy(rail, rail$r, rail$theta + add, params$bbox)
    }

    decor$.level <- match(decor$.level, levels)
    decor$.level_end <- match(decor$.level_end, levels)
    rail$.level <- match(rail$.level, levels)

    measure <- switch(
      position,
      left = , right = width_cm,
      top = , bottom = height_cm,
      get_size_attr
    )

    angle <- params$angle %|W|% NULL
    text <- angle_labels(elements$text, angle, position)
    offset <- elements$offset
    sizes <- numeric(nlevels + 1L)
    grobs <- vector("list", nlevels)

    for (i in seq_len(nlevels)) {

      labels <- draw_labels(
        vec_slice(key, key$.level == levels[[i]]),
        combine_elements(text_levels[[i]], text),
        angle = angle, offset = offset, position = position
      )
      sizes[i + 1L] <- measure(labels)
      offset <- offset + sizes[i + 1L]

      fencepost <- draw_fencepost(
        vec_slice(decor, decor$.level_end == i),
        combine_elements(post_levels[[i]], elements$post),
        sizes = sizes[1L:(i + 1L)],
        offset = offset, position = position
      )

      fencerail <- draw_fencerail(
        vec_slice(rail, rail$.level == i),
        combine_elements(rail_levels[[i]], elements$rail),
        sizes = sizes[1L:(i + 1L)],
        offset = offset, position = position,
        side = params$rail, bbox = params$bbox
      )

      grobs[[i]] <- grobTree(fencepost, fencerail, labels)
    }

    sizes <- sizes[-1L]
    if (position %in% c("top", "left")) {
      grobs <- rev(grobs)
      sizes <- rev(sizes)
    }

    attr(grobs, "size") <- sizes
    grobs
  },

  draw = function(self, theme, position = NULL, direction = NULL,
                  params = self$params) {
    params <- replace_null(params, position = position, direction = direction)
    params <- self$setup_params(params)

    elems <- self$setup_elements(params, self$elements, theme)
    fence <- self$build_fence(params$key, params$decor, elems, params)

    if (length(fence) < 1L) {
      return(zeroGrob())
    }

    primitive_grob(
      grob = fence,
      size = unit(get_size_attr(fence), "cm"),
      position = params$position,
      name = "fence"
    )
  }

)

# Helpers -----------------------------------------------------------------

draw_fencerail <- function(rail, element, sizes, offset, position, side, bbox) {
  if (side == "none" || nrow(rail) < 1L || is_blank(element)) {
    return(NULL)
  }

  if (is_theta(position)) {
    n <- as.integer(round(rail$thetaend - rail$theta) / (pi / 45.0))
    n <- pmax(n, 2L)

    theta <- Map(seq, rail$theta, rail$thetaend, length.out = n)
    i     <- rep(seq_along(theta), lengths(theta))

    add <- as.integer(position == "theta.sec")
    xy <- data_frame0(
      theta = unlist(theta) + add * pi,
      r = rail$r[i],
      i = i
    )
    xy <- polar_xy(xy, xy$r, xy$theta, bbox)

    if (side == "inner") {
      r <- unit(rep(offset - sizes[rail$.level + 1L], n), "cm")
    } else if (side == "outer") {
      r <- unit(rep(offset, sum(n)), "cm")
    } else {
      r <- unit(c(
        rep(offset - sizes[rail$.level + 1L], n),
        rep(offset, sum(n))
      ), "cm")
      xy$i <- c(1L, xy$i[-1L] != xy$i[-nrow(xy)])
      xy <- vec_c(xy, xy)
      xy$i <- cumsum(xy$i)
    }
    if (add == 1L) {
      r <- r * -1.0
    }

    args <- list(
      x = unit(xy$x, "npc") + sin(xy$theta) * r,
      y = unit(xy$y, "npc") + cos(xy$theta) * r,
      id.lengths = vec_unrep(xy$i)$times
    )
  } else {

    aes <- switch(position, top = , bottom = "x", left = , right = "y", "theta")
    aesend <- paste0(aes, "end")

    mark <- vec_interleave(rail[[aes]], rail[[aesend]])
    if (side == "inner") {
      tick <- rep(0.0, length(mark))
    } else if (side == "outer") {
      tick <- rep(1.0, length(mark))
    } else {
      tick <- rep(c(0.0, 1.0), each = length(mark))
      mark <- c(mark, mark)
    }
    mark <- unit(mark, "npc")

    tick <- switch(
      position,
      top = , right = unit(0.0 + tick, "npc"),
      unit(1.0 - tick, "npc")
    )

    args <- list(x = tick, y = mark, id.lengths = rep(2L, length(tick) / 2L))
    if (position %in% c("top", "bottom")) {
      args <- flip_names(args)
    }

  }

  props <- element_key_properties(rail, "line")

  inject(element_grob(element, !!!args, !!!props))
}

draw_fencepost <- function(decor, element, sizes, offset, position) {
  if (nrow(decor) < 1L || is_blank(element)) {
    return(NULL)
  }

  levels <- vec_interleave(decor$.level, decor$.level_end + 1L)

  if (is_theta(position)) {
    add <- as.integer(position == "theta.sec")

    angle <- rep(decor$theta, each = 2L) + add * pi
    x     <- rep(decor$x,     each = 2L)
    y     <- rep(decor$y,     each = 2L)
    length <- cumsum(sizes)[levels] + offset - sum(sizes)
    if (add == 1L) {
      length <- length * -1L
    }
    length <- unit(length, "cm")

    args <- list(
      x = unit(x, "npc") + sin(angle) * length,
      y = unit(y, "npc") + cos(angle) * length
    )
  } else {
    aes <- switch(position, top = , bottom = "x", left = , right = "y", "theta")
    mark <- unit(rep(decor[[aes]], each = 2L), "npc")

    tick <- unit(offset - cumsum(sizes)[levels], "cm")
    tick <- switch(
      position,
      top = , right = unit(1.0, "npc") - tick,
      unit(0.0, "npc") + tick
    )

    args <- list(x = tick, y = mark)
    if (position %in% c("top", "bottom")) {
      args <- flip_names(args)
    }
  }
  id <- rep(2L, nrow(decor))

  props <- element_key_properties(decor, "line")

  inject(element_grob(element, id.lengths = id, !!!args, !!!props))
}

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @import ggplot2
#' @import grid
#' @import gtable
#' @import rlang
#' @import scales
#' @import vctrs
#' @importFrom lifecycle deprecated
## usethis namespace: end
NULL

#' @name legendry_extensions
#' @title `ggproto` objects in \pkg{legendry}
#'
#' @description The \pkg{legendry} package relies on an extension system of
#'   \pkg{ggplot2} through [`ggproto`][ggplot2::ggproto] class objects, which
#'   allow cross-package inheritance of objects such as geoms, stats, facets,
#'   scales and coordinate systems. For the purpose of making plots, users are
#'   invited to wholly ignore these objects, since interacting with these
#'   objects is preferred through various constructor functions. The
#'   \pkg{legendry} package introduces a new `<Guide>` ggproto class to support
#'   variations on axes, legends and colourbars.
#'
#' @seealso The documentation over at [`ggproto`][ggplot2::ggproto()].
#' @keywords internal
NULL

#' @name common_parameters
#' @title common parameters in \pkg{legendry}
#'
#' @description
#' This is a collection of common parameters so they needn't be re-documented
#' each time.
#'
#' @param title
#' One of the following to indicate the title of the guide:
#' * A `<character[1]>` or `<expression[1]>` to set a custom title.
#' * `NULL` to not display any title.
#' * [`waiver()`][ggplot2::waiver()] (default) to take the name of the scale
#'   object or the name specified in [`labs()`][ggplot2::labs] as the title.
#'
#' @param theme
#' A [`<theme>`][ggplot2::theme] object to style the guide individually or
#' differently from the plot's theme settings. The `theme` argument in the
#' guide overrides and is combined with the plot's theme.
#'
#' @param position
#' A `<character[1]>` giving the location of the guide. Can be one of `"top"`,
#' `"bottom"`, `"left"` or `"right"`.
#'
#' @param order
#' A positive `<integer[1]>` that specifies the order of this guide among
#' multiple guides. This controls in which order guides are merged if there
#' are multiple guides for the same position. If `0` (default), the order is
#' determined by a hashing indicative settings of a guide.
#'
#' @param available_aes
#' A `<character>` vector listing the aesthetics for which this guide can
#' be build.
#'
#' @param direction
#' A `<character[1]>` indicating the direction of the guide. Can be on of
#' `"horizontal"` or `"vertical"`.
#'
#' @param angle
#' A specification for the text angle. Compared to setting the `angle` argument
#' in [`element_text()`][ggplot2::element_text], this argument uses some
#' heuristics to automatically pick the `hjust` and `vjust` that you
#' probably want. Can be one of the following:
#' * `NULL` to take angles and justification settings directly from the theme.
#' * [`waiver()`][ggplot2::waiver] to allow reasonable defaults in special
#'   cases.
#' * A `<numeric[1]>` between -360 and 360 for the text angle in degrees.
#'
#' @param override.aes A named `<list>` specifying aesthetic parameters of the
#'   key glyphs. See details and examples in
#'   [`guide_legend()`][ggplot2::guide_legend()].
#'
#' @param call
#' A [call][rlang::topic-error-call] to display in messages.
#'
#' @keywords internal
NULL

new_params <- function(...) {
  required <- list(
    title = waiver(),
    theme = NULL,
    name  = character(),
    position = waiver(),
    direction = NULL,
    order = 0L,
    hash = character(0L)
  )
  dots <- list2(...)
  for (i in names(dots)) required[i] <- dots[i]
  required
}

.trbl <- c("top", "right", "bottom", "left")

standard_extract_key <- function(scale, aesthetic, key, ...) {
  key <- resolve_key(key %||% "auto")
  if (inherits(key, "key_guide") && !inherits(key, "key_standard")) {
    return(key) # probably not a standard key, no need to treat
  }
  if (is.function(key)) {
    key <- key(scale, aesthetic)
  }
  if (is_empty(key)) {
    return(key)
  }
  if ("aesthetic" %in% names(key)) {
    key$aesthetic <-
      scale_transform(key$aesthetic, scale, map = TRUE, "aesthetic")
    key$.value <-
      scale_transform(key$.value, scale, map = FALSE, "value")
  }
  key <- rename(key, "aesthetic", aesthetic)
  key <- validate_key_types(key)

  if (is.numeric(key$.value) && !is_asis(key$.value)) {
    range <- scale$continuous_range %||% scale$get_limits()
    key <- vec_slice(key, is.finite(oob_censor_any(key$.value, range)))
  }

  key
}

test_that("new_params works as intended", {

  params <- new_params(foo = "bar")
  expect_named(
    params,
    c(names(Guide$params), "foo")
  )

})

test_that("standard_extract_key works as intended", {

  scale <- scale_x_continuous(limits = c(0.0, 10.0), breaks = c(1.0, 5.0, 10.0))
  key <- standard_extract_key(scale, "x", key = "auto")
  expect_snapshot(key)

  key <- standard_extract_key(scale, "x", key = key_manual(I(c(0.3, 0.7))))
  expect_identical(key$x, I(c(0.3, 0.7)))

})

#' @name guide-gizmos
#' @title Guide gizmos
#'
#' @description
#' Guide gizmos are a speciality guide components that are very specific to
#' one or a few aesthetics to display.
#'
#' Typically they can be [composed][guide-composition] with other guides or
#' guide [primitives][guide-primitives] to form a complete guide.
NULL

# Constructor -------------------------------------------------------------

#' Guide primitive: labels
#'
#' This function constructs a labels [guide primitive][guide-primitives].
#'
#' @param key A [standard key][key_standard] specification. See more information
#'   in the linked topic.
#' @param n.dodge An positive `<integer[1]>` setting the number of layers text
#'   labels can occupy to avoid overlapping labels.
#' @param check.overlap A `<logical[1]>` indicating whether to check for and
#'   omit overlapping text. If `TRUE`, first, last and middle labels are
#'   recursively prioritised in that order. If `FALSE`, all labels are drawn.
#' @inheritParams common_parameters
#'
#' @return A `<PrimitiveLabels>` primitive guide that can be used inside other
#'   guides.
#' @export
#' @family primitives
#'
#' @details
#' ## Styling options
#'
#' Below are the [theme][ggplot2::theme] options that determine the styling of
#' this guide, which may differ depending on whether the guide is used in
#' an axis or in a legend context.
#'
#' The possible `{position}` suffixes mentioned below are `x`, `x.top`,
#' `x.bottom`, `y`, `y.left`, `y.right`. The `theta` and `r` position suffixes
#' in \pkg{ggplot2} are *not* obeyed in \pkg{legendry}.
#'
#' | **Theme setting** | **Context** | **Type** | **Description** |
#' | ----------------- | ----------- | -------- | --------------- |
#' | `axis.text.{position}` | Axis | [`element_text()`] | The text labels. |
#' | `legend.text` | Legend | [`element_text()`] | The text labels. |
#'
#' Styling options *per break* can be set in the [standard key][key_standard].
#' The `text` prefixed properties are prioritised. These override theme
#' settings.
#'
#' The context-agnostic alternative to using `theme()` is to use
#' [`theme_guide()`]:
#'
#' ```r
#' primitive_labels(theme = theme_guide(
#'   text = element_line()
#' ))
#' ```
#'
#' @examples
#' # A standard plot
#' p <- ggplot(mpg, aes(displ, hwy)) +
#'  geom_point()
#'
#' # Adding as secondary guides
#' p + guides(
#'   x.sec = primitive_labels(),
#'   y.sec = primitive_labels(n.dodge = 2)
#' )
primitive_labels <- function(key = NULL, angle = waiver(), n.dodge = 1L,
                             check.overlap = FALSE,
                             theme = NULL, position = waiver()) {
  if (!is_waive(angle)) {
    check_number_decimal(
      angle, min = -360.0, max = 360.0,
      allow_infinite = FALSE, allow_null = TRUE
    )
  }
  check_number_whole(n.dodge, min = 1.0)
  check_bool(check.overlap)

  new_guide(
    angle = angle,
    n_dodge = n.dodge,
    check_overlap = check.overlap,
    key = key,
    theme = theme,
    position = position,
    available_aes = c("any", "x", "y", "r", "theta"),
    super = PrimitiveLabels
  )
}


# Class -------------------------------------------------------------------

#' @export
#' @rdname legendry_extensions
#' @format NULL
#' @usage NULL
PrimitiveLabels <- ggproto(
  "PrimitiveLabels", Guide,

  params = new_params(
    angle = waiver(), n_dodge = 1L, check_overlap = FALSE, key = NULL
  ),

  elements = list(
    position = list(text = "axis.text"),
    legend   = list(text = "legend.text")
  ),

  hashables = exprs(key$.label),

  extract_params = primitive_extract_params,

  extract_key = standard_extract_key,

  transform = function(self, params, coord, panel_params) {
    params$key <-
      transform_key(params$key, params$position, coord, panel_params)
    params
  },

  setup_params = primitive_setup_params,

  setup_elements = primitive_setup_elements,

  override_elements = function(params, elements, theme) {
    elements$text <- angle_labels(elements$text, params$angle, params$position)
    elements
  },

  build_labels = function(key, elements, params) {
    if (".type" %in% names(key)) {
      key <- vec_slice(key, key$.type == "major")
    }

    n_labels <- nrow(key)

    if (n_labels < 1L || is_blank(elements$text)) {
      return(list(zeroGrob()))
    }

    dodge_value <- rep_len(seq_len(params$n_dodge %||% 1L), n_labels)
    dodge_index <- unname(split(seq_len(n_labels), dodge_value))
    angle <- params$angle %|W|% NULL

    offset <- elements$offset
    grobs  <- list()

    for (i in seq_along(dodge_index)) {
      index <- dodge_index[[i]]
      grob  <- draw_labels(
        vec_slice(key, index), elements$text, angle, offset,
        params$position, check_overlap = params$check_overlap
      )
      offset <- offset + get_size_attr(grob)
      grobs[[i]] <- grob
    }
    if (params$position %in% c("top", "left")) grobs <- rev(grobs)

    attr(grobs, "offset") <- offset - elements$offset
    grobs
  },

  measure_grobs = function(grobs, params, elements) {
    switch(
      params$position,
      top = , bottom = height_cm(grobs),
      left = , right = width_cm(grobs),
      attr(grobs, "offset")
    )
  },

  draw = function(self, theme, position = NULL, direction = NULL,
                  params = self$params) {

    params <- replace_null(params, position = position, direction = direction)
    params <- self$setup_params(params)

    elems  <- self$setup_elements(params, self$elements, theme)
    elems  <- self$override_elements(params, elems, theme)
    labels <- self$build_labels(params$key, elems, params)
    size   <- self$measure_grobs(labels, params, elems)

    primitive_grob(labels, unit(size, "cm"), params$position, "labels")
  }

)

# Helpers -----------------------------------------------------------------

draw_labels <- function(key, element, angle, offset,
                        position, check_overlap = NULL) {

  n_breaks  <- length(key$.label)
  if (n_breaks < 1L || is_blank(element)) {
    return(zeroGrob())
  }

  margin_x <- switch(position, left = , right = TRUE, FALSE)
  margin_y <- switch(position, top = , bottom = TRUE, FALSE)
  hjust <- vjust <-  NULL

  check_overlap <- check_overlap %||% FALSE
  if (check_overlap) {
    order <- label_priority(n_breaks)
    key <- vec_slice(key, order)
  }

  just <- rotate_just(angle, element = element)
  x <- switch(position, left = , right = just$hjust, key$x)
  y <- switch(position, top = , bottom = just$vjust, key$y)
  just <- NULL

  # Resolve positions
  x <- rep_len(x, n_breaks)
  y <- rep_len(y, n_breaks)
  if (!is.unit(x)) x <- unit(x, "npc")
  if (!is.unit(y)) y <- unit(y, "npc")

  labels <- validate_labels(key$.label)

  # Theta labels
  theta <- get_theta(key, position)
  if (is_theta(position)) {
    if (is_null(angle)) {
      # Single verbatim angle from theme
      angle <- element$angle
    } else {
      # Adaptive angle based on theta
      angle <- flip_text_angle(angle - rad2deg(theta))
    }
    angle_radians <- deg2rad(angle)
    text_radians <- angle_radians + theta

    margin <- cm(max(element$margin))
    offset <- offset + margin

    x <- x + unit(offset * sin(theta), "cm")
    y <- y + unit(offset * cos(theta), "cm")

    hjust <- 0.5 - sin(text_radians) / 2.0
    vjust <- 0.5 - cos(text_radians) / 2.0
  } else if (all(c("theta", ".length") %in% names(key))) {
    shift <- function(theta, range, value) {
      unit(ifelse(in_arc(theta, range * pi), value, 0), "cm")
    }
    length <- key$.length
    switch(
      position,
      top =    {x <- x + shift(theta, c(-0.5, 0.5), sin(theta) * length)},
      right =  {y <- y + shift(theta, c(0.0,  1.0), cos(theta) * length)},
      bottom = {x <- x + shift(theta, c(0.5,  1.5), sin(theta) * length)},
      left =   {y <- y + shift(theta, c(1.0,  2.0), cos(theta) * length)}
    )
  }

  props <- element_key_properties(
    key, "text",
    hjust = hjust,
    vjust = vjust,
    angle = angle
  )

  grob <- inject(element_grob(
    element = element,
    label = labels,
    x = x, y = y,
    !!!props,
    margin_x = margin_x,
    margin_y = margin_y,
    check.overlap = check_overlap
  ))

  if (!is_theta(position)) {
    return(grob)
  }
  # For composition purposes we need to keep track of the text size along
  # the radial axis, not just simply width/height.
  height <- if (inherits(grob, "textpath")) {
    measure_textpath_labels(grob)
  } else {
    measure_theta_labels(element, labels, margin, text_radians, props)
  }
  attr(grob, "size") <- height
  grob
}

measure_textpath_labels <- function(grob) {
  labels <- grob$textpath$label
  height <- map_dbl(labels, function(x) attr(x, "metrics")$height)
  max(height) * .in2cm
}

measure_theta_labels <- function(element, labels, margin, angle, params) {

  n_labels <- length(labels)
  hjust <- params$hjust %||% 0.5
  vjust <- params$vjust %||% 0.5
  size  <- params$size

  singles <- lapply(seq_len(n_labels), function(i) {
    element_grob(
      element, label = labels[[i]],
      margin = margin(),
      size = size[[i]]
    )
  })

  widths  <- width_cm(singles)
  heights <- height_cm(singles)

  xmin <- widths * -hjust
  xmax <- widths * (1.0 - hjust)

  ymin <- heights * -vjust
  ymax <- heights * (1.0 - vjust)

  x <- vec_interleave(xmin, xmin, xmax, xmax)
  y <- vec_interleave(ymin, ymax, ymax, ymin)

  angle <- rep(angle, each = 4L)
  max(x * sin(angle) + y * cos(angle), na.rm = TRUE) + max(cm(margin))
}

label_priority <- function(n) {
  if (n <= 0L) {
    return(numeric(0L))
  }
  c(1L, n, label_priority_between(1L, n))
}

label_priority_between <- function(min, max) {
  n <- max - min + 1L
  if (n <= 2L) {
    return(numeric(0L))
  }
  mid <- min - 1L + (n + 1L) %/% 2L
  c(mid, label_priority_between(min, mid), label_priority_between(mid, max))
}

get_theta <- function(key, position) {
  key$theta %||% rep(
    switch(
      position,
      top    = 0.0,
      bottom = 1.0,
      left   = 1.5,
      right  = 0.5
    ) * pi,
    nrow(key)
  )
}

test_that("primitive_labels works as axis", {

  base <- ggplot(mpg, aes(displ, hwy)) +
    geom_blank() +
    theme_test() +
    theme(
      panel.background = element_rect(fill = NA, colour = "grey80"),
      panel.grid.major = element_line(colour = "grey90"),
      panel.border = element_blank(),
      axis.line = element_line()
    )

  p <- base +
    guides(
      x     = primitive_labels(),
      x.sec = primitive_labels(n.dodge = 2L),
      y     = primitive_labels(angle = 45.0),
      y.sec = primitive_labels(
        theme = theme(axis.text = element_text(colour = "red"))
      )
    )

  vdiffr::expect_doppelganger("primitive_labels cartesian", p)

  p <- base +
    coord_radial(start = 0.25 * pi, end = 1.75 * pi, inner.radius = 0.5) +
    guides(
      theta     = primitive_labels(),
      theta.sec = primitive_labels(angle = 0.0),
      r         = primitive_labels(n.dodge = 2L),
      r.sec     = primitive_labels(
        theme = theme(axis.text = element_text(colour = "red"))
      )
    )

  vdiffr::expect_doppelganger("primitive_labels radial", p)

})

test_that("primitive_labels works as legend", {

  p <- ggplot(mtcars) +
    aes(
      x = disp, y = mpg,
      colour = hp,
      fill = hp
    ) +
    geom_point() +
    guides(
      colour = primitive_labels(),
      fill = primitive_labels(
        key = key_manual(c(200.0, 300.0)), position = "bottom"
      )
    )

  vdiffr::expect_doppelganger("primitive_labels legend", p)

})

# Helper tests ------------------------------------------------------------

test_that("draw_labels sets priorities", {

  elem <- theme_gray()$text
  key <- data_frame0(
    x = c(0.0, 0.25, 0.5, 0.75, 1.0),
    y = 1.0,
    .label = c("A", "B", "C", "D", "E")
  )

  test <- withr::with_pdf(
    tempfile(fileext = ".pdf"),
    draw_labels(key, elem, 0.0, 0.0, "bottom", check_overlap = TRUE)
  )
  # Expect outer labels first
  expect_identical(test$children[[1L]]$label, c("A", "E", "C", "B", "D"))

})

test_that("draw_labels can draw theta labels", {

  elem <- theme_gray()$text

  key <- data_frame0(
    x = c(0.0, 0.25, 0.5, 0.75, 1.0),
    y = 1.0,
    .label = c("A", "B", "C", "D", "E")
  )
  key$theta <- key$x
  key$r <- 0.4

  test <- withr::with_pdf(
    tempfile(fileext = ".pdf"),
    draw_labels(key, elem, 0.0, 0.0, "theta")
  )
  expect_s3_class(test, "titleGrob")
  expect_equal(attr(test, "size"), 0.364, tolerance = 1e-3)

})

test_that("label angles works", {

  grid <- expand.grid(angle = seq(0.0, 360.0, by = 45.0), position = .trbl)
  result <- Map(
    angle_labels,
    angle = grid$angle, position = grid$position,
    MoreArgs = list(element = theme_gray()$text)
  )

  grid$hjust <- map_dbl(result, `[[`, i = "hjust")
  grid$vjust <- map_dbl(result, `[[`, i = "vjust")
  expect_snapshot(grid)
})

#' Theme wrapper for guides
#'
#' This function does three things different than [`theme()`].
#'
#' 1. It has shorthand names for various guide settings, similar to
#'   [`theme_sub_legend()`] and [`theme_sub_axis()`]
#' 2. It simultaneously sets theme elements for axes and legends. This makes
#'   it contextually agnostic. For example, setting `theme_guide(text)` will
#'   populate `legend.text` and also `axis.text`.
#' 3. It includes \pkg{legendry} specific settings.
#'
#' The first two things make it very good for setting the `guide_*(theme)`
#' arguments. The second thing makes it very bad for setting plot-wide or
#' global themes.
#'
#' @param text An [`<element_text>`][ggplot2::element_text] setting both
#'   `legend.text` and `axis.text` elements.
#' @param line An [`<element_line>`][ggplot2::element_line] setting both
#'   `legend.axis.line` and `axis.line` elements.
#' @param title An [`<element_text>`][ggplot2::element_text] setting both
#'   `legend.title` and `axis.title` elements.
#' @param subtitle An [`<element_text>`][ggplot2::element_text] setting both
#'   `legendry.legend.subtitle` and `legendry.axis.subtitle` elements.
#' @param text.position,title.position,subtitle.position One of `"top"`,
#'   `"right"`, `"bottom"` or `"right"` setting the following elements:
#'
#'   * `text.position`: sets only `legend.text.position`.
#'   * `title.position`: sets only `legend.title.position`.
#'   * `subtitle.position` sets both `legendry.legend.subtitle.position` and
#'   `legendry.axis.subtitle.position`
#' @param ticks An [`<element_line>`][ggplot2::element_line] setting both
#'   `axis.ticks` and `legend.ticks` elements.
#' @param minor.ticks An [`<element_line>`][ggplot2::element_line] setting
#'   `legendry.legend.minor.ticks` and all 6 of the
#'   `axis.ticks.minor.{r/theta/x.top/x.bottom/y.left/y.right}` elements.
#' @param mini.ticks An [`<element_line>`][ggplot2::element_line] setting both
#'   `legendry.legend.mini.ticks` and `legendry.axis.mini.ticks` elements.
#' @param ticks.length,minor.ticks.length,mini.ticks.length A
#'   [`<unit[1]>`][grid::unit()] setting the following elements:
#'
#'   * `ticks.length`: sets both `legend.ticks.length` and `axis.ticks.length`.
#'   * `minor.ticks.length` sets both `axis.minor.ticks.length` and
#'   `legendry.legend.minor.ticks.length`.
#'   * `mini.ticks.length` sets both `legendry.axis.mini.ticks.length` and
#'   `legendry.legend.mini.ticks.length`.
#' @param spacing,group.spacing,table.spacing A [`<unit[1]>`][grid::unit()]
#'   settingthe `legendry.guide.spacing`, `legendry.group.spacing` and
#'   `legendry.table.spacing` theme elements respectively.
#' @param key An [`<element_rect>`][ggplot2::element_rect] setting the
#'   `legend.key` element.
#' @param key.size,key.width,key.height A [`<unit>`][grid::unit()] setting the
#'   `legend.key.size`, `legend.key.width` and `legend.key.height` elements
#'   respectively.
#' @param key.spacing,key.spacing.x,key.spacing.y A [`<unit[1]>`][grid::unit()]
#'   setting the `legend.key.spacing`, `legend.key.spacing.x` and
#'   `legend.key.spacing.y` elements respectively.
#' @param key.margin A [`<margin>`][ggplot2::margin] setting the margin around
#'   legend glyphs.
#' @param key.justification A [`<numeric[2]>`] passed to the
#'   `legend.key.justification` setting.
#' @param frame An [`<element_rect>`][ggplot2::element_rect] setting the
#'   `legend.frame` element.
#' @param byrow A `<logical[1]>` setting the `legend.byrow` element.
#' @param background An [`<element_rect>`][ggplot2::element_rect] setting the
#'   `legend.background` element.
#' @param margin A [`<margin>`][ggplot2::margin] setting the `legend.margin`
#'   element.
#' @param bracket An [`<element_line>`][ggplot2::element_line] setting the
#'   `legendry.bracket` element.
#' @param bracket.size A [`<unit[1]>`][grid::unit()] setting the
#'   `legendry.bracket.size` element.
#' @param box An [`<element_rect>`][ggplot2::element_rect] setting the
#'   `legendry.box` element.
#' @param fence,fence.post,fence.rail An
#'   [`<element_line>`][ggplot2::element_line] setting the `legendry.fence`,
#'   `legendry.fence.post` and `legendry.fence.rail` respectively.
#' @param zebra.light,zebra.dark An [`<element_rect>`][ggplot2::element_rect]
#'   setting the `legendry.zebra.light` and `legendry.zebra.dark` settings
#'   respectively.
#' @param point An [`<element_point>`][ggplot2::element_point] setting the
#'   `legendry.point` element.
#' @param connector An [`<elemenet_line>`][ggplot2::element_line] setting the
#'   `legendry.connector` element.
#'
#' @return A `<theme>` object that can be provided to a guide.
#' @export
#'
#' @examples
#' red_ticks <- theme_guide(
#'   ticks = element_line(colour = "red", linewidth = 0.5)
#' )
#'
#' # Both axis and colourbar gain red ticks
#' ggplot(mpg, aes(displ, hwy, colour = cty)) +
#'   geom_point() +
#'   guides(
#'     colour = guide_colourbar(theme = red_ticks),
#'     x = guide_axis(theme = red_ticks)
#'   )
theme_guide <- function(
  text = NULL,
  line = NULL,

  title = NULL,
  subtitle = NULL,

  text.position = NULL,
  title.position = NULL,
  subtitle.position = NULL,

  ticks = NULL,
  minor.ticks = NULL,
  mini.ticks = NULL,

  ticks.length = NULL,
  minor.ticks.length = NULL,
  mini.ticks.length = NULL,

  spacing = NULL,
  group.spacing = NULL,
  table.spacing = NULL,

  key = NULL,
  key.size = NULL,
  key.width = NULL,
  key.height = NULL,
  key.spacing = NULL,
  key.spacing.x = NULL,
  key.spacing.y = NULL,
  key.margin = NULL,
  key.justification = NULL,

  frame = NULL,
  byrow = NULL,
  background = NULL,
  margin = NULL,

  bracket = NULL,
  bracket.size = NULL,
  box = NULL,
  fence = NULL,
  fence.post = NULL,
  fence.rail = NULL,
  zebra.light = NULL,
  zebra.dark = NULL,
  point = NULL,
  connector = NULL
) {

  theme <- list(
    legend.text = text,
    axis.text = text,

    axis.line = line,
    legend.axis.line = line,

    legend.title = title,
    axis.title = title,

    legendry.legend.subtitle = subtitle,
    legendry.axis.subtitle = subtitle,

    legend.title.position = title.position,
    legend.text.position = text.position,

    legendry.legend.subtitle.position = subtitle.position,
    legendry.axis.subtitle.position = subtitle.position,

    legend.ticks = ticks,
    axis.ticks = ticks,

    legend.ticks.length = ticks.length,
    axis.ticks.length = ticks.length,

    legendry.legend.minor.ticks = minor.ticks,
    axis.minor.ticks.r = minor.ticks,
    axis.minor.ticks.theta = minor.ticks,
    axis.minor.ticks.x.top = minor.ticks,
    axis.minor.ticks.x.bottom = minor.ticks,
    axis.minor.ticks.y.left = minor.ticks,
    axis.minor.ticks.y.right = minor.ticks,

    legendry.legend.minor.ticks.length = minor.ticks.length,
    axis.minor.ticks.length = minor.ticks.length,

    legendry.axis.mini.ticks = mini.ticks,
    legendry.legend.mini.ticks = mini.ticks,

    legendry.axis.mini.ticks.length = mini.ticks.length,
    legendry.legend.mini.ticks.length = mini.ticks.length,

    legendry.guide.spacing = spacing,
    legendry.group.spacing = group.spacing,
    legendry.table.spacing = table.spacing,

    legend.key = key,
    legend.key.spacing = key.spacing,
    legend.key.spacing.x = key.spacing.x,
    legend.key.spacing.y = key.spacing.y,
    legend.key.size = key.size,
    legend.key.width = key.width,
    legend.key.height = key.height,
    legend.key.justification = key.justification,
    legendry.legend.key.margin = key.margin,

    legend.frame = frame,
    legend.byrow = byrow,
    legend.background = background,
    legend.margin = margin,

    legendry.bracket = bracket,
    legendry.bracket.size = bracket.size,
    legendry.box = box,
    legendry.fence = fence,
    legendry.fence.post = fence.post,
    legendry.fence.rail = fence.rail,
    legendry.zebra.light = zebra.light,
    legendry.zebra.dark = zebra.dark,
    legendry.point = point,
    legendry.connector = connector
  )
  theme <- theme[lengths(theme) > 0L]
  theme(!!!theme)
}


register_legendry_elements <- function() {

  tree <- get_element_tree()
  line <- tree$line$class
  rect <- tree$rect$class
  text <- tree$text$class
  marg <- tree$plot.margin$class

  register_theme_elements(
    legendry.bracket.size = unit(2.0, "mm"),
    legendry.bracket = element_line(),
    legendry.fence = element_line(),
    legendry.fence.post = element_line(),
    legendry.fence.rail = element_line(),
    legendry.box = element_rect(colour = "white"),
    legendry.legend.minor.ticks = element_line(),
    legendry.legend.minor.ticks.length = rel(0.75),
    legendry.legend.mini.ticks = element_line(),
    legendry.legend.mini.ticks.length = rel(0.5),
    legendry.legend.subtitle = element_text(size = rel(0.9)),
    legendry.legend.subtitle.position = "top",
    legendry.legend.key.margin = NULL,
    legendry.axis.mini.ticks = element_line(),
    legendry.axis.mini.ticks.length = rel(0.5),
    legendry.guide.spacing = unit(2.25, "pt"),
    legendry.group.spacing = rel(2.0),
    legendry.axis.subtitle = NULL,
    legendry.axis.subtitle.position = c("left", "top", "bottom", "right"),
    legendry.symbol = element_point(),
    legendry.connector = element_line(),
    legendry.zebra.light = NULL,
    legendry.zebra.dark = NULL,
    legendry.table.vlines = element_blank(),
    legendry.table.hlines = element_blank(),
    legendry.table.spacing = rel(0.5),
    element_tree = list(
      legendry.bracket.size = el_def("unit"),
      legendry.bracket = el_def(line, "line"),
      legendry.fence = el_def(line, "line"),
      legendry.fence.post = el_def(line, "legendry.fence"),
      legendry.fence.rail = el_def(line, "legendry.fence"),
      legendry.box = el_def(rect, "strip.background"),
      legendry.legend.minor.ticks = el_def(line, "legend.ticks"),
      legendry.legend.minor.ticks.length = el_unit("legend.ticks.length"),
      legendry.legend.mini.ticks = el_def(line, "legendry.legend.minor.ticks"),
      legendry.legend.mini.ticks.length =
        el_unit("legendry.legend.minor.ticks.length"),
      legendry.legend.subtitle = el_def(text, "legend.title"),
      legendry.legend.subtitle.position = el_def("character"),
      legendry.legend.key.margin = el_def(marg, "legend.margin"),
      legendry.axis.mini.ticks = el_def(line, "axis.ticks"),
      legendry.axis.mini.ticks.length = el_unit("axis.minor.ticks.length"),
      legendry.guide.spacing = el_unit("axis.ticks.length"),
      legendry.group.spacing = el_unit("legend.key.spacing"),
      legendry.axis.subtitle = el_def(text, "axis.text"),
      legendry.axis.subtitle.position = el_def("character"),
      legendry.symbol = el_def(element_point, "point"),
      legendry.connector = el_def(line, "line"),
      legendry.zebra.light = el_def(rect, "plot.background"),
      legendry.zebra.dark  = el_def(rect, "panel.background"),
      legendry.table.vlines = el_def(line, "axis.line"),
      legendry.table.hlines = el_def(line, "axis.line"),
      legendry.table.spacing = el_def(c("unit", "rel"), "spacing")
    )
  )
}

on_load(register_legendry_elements())

el_unit <- function(...) el_def(c("unit", "rel"), ...)

test_that("theme elements can be registered", {

  expect_in("legendry.bracket", names(get_element_tree()))

  ggplot2::reset_theme_settings()

  expect_false("legendry.bracket" %in% names(get_element_tree()))

  register_legendry_elements()

  expect_in("legendry.bracket", names(get_element_tree()))

})

test_that("theme elements can be registered", {

  expect_in("legendry.bracket", names(get_element_tree()))

  ggplot2::reset_theme_settings()

  expect_false("legendry.bracket" %in% names(get_element_tree()))

  register_legendry_elements()

  expect_in("legendry.bracket", names(get_element_tree()))

})

test_that("all arguments of theme_guide are used", {

  fmls <- fn_fmls_names(theme_guide)
  args <- set_names(seq_along(fmls), fmls)
  theme <- inject(theme_guide(!!!args))
  expect_setequal(unlist(theme), seq_along(fmls))

})

# Constructor -------------------------------------------------------------

#' Compose guides in a cross
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' This guide composition has a central guide optionally surrounded by other
#' guides on all four sides.
#'
#' @param key A [standard key][key_standard] specification. The key is shared
#'   among all guides that have `NULL` keys themselves. See more information
#'   in the linked topic.
#' @param centre,left,right,top,bottom Guides to use in
#'   [composition][guide-composition] per position. Each guide can be
#'   specified as one of the following:
#'   * A `<Guide>` class object.
#'   * A `<function>` that returns a `<Guide>` class object.
#'   * A `<character>` naming such a function, without the `guide_` or
#'   `primitive_` prefix.
#' @param reverse A `<logical[1]>` whether to reverse continuous guides.
#'   If `TRUE`, guides like colour bars are flipped. If `FALSE` (default),
#'   the original order is maintained.
#' @param args A `<list>` of arguments to pass to guides that are given either
#'   as a function or as a string.
#' @param complete A `<logical[1]>` whether to treat the composition as a
#'   complete guide. If `TRUE`, a title and margin are added to the result.
#'   If `FALSE` (default), no titles and margins are added.
#' @param theme A [`<theme>`][ggplot2::theme] object to style the guide
#'   individually of differently from the plot's theme settings. The `theme`
#'   arguments in the guide overrides, and is combined with, the plot's theme.
#' @param theme_defaults A `<list>` of theme elements to override undeclared
#'   theme arguments.
#' @param position Where this guide should be drawn: one of `"top"`, `"bottom"`,
#'   `"left"`, or `"right"`.
#' @inheritParams common_parameters
#'
#' @details
#' ## Styling options
#'
#' Below are the [theme][ggplot2::theme] options that determine the styling of
#' this guide.
#'
#' | **Theme setting** | **Type** | **Description** |
#' | ----------------- | -------- | --------------- |
#' | `legend.title` | [`element_text()`] | The title of the legend. |
#' | `legend.title.position` | `<character[1]>` | One of `"top"`, `"right"`, `"bottom"` or `"left"`. |
#' | `legend.text.position` | `<character[1]>` | One of `"top"`, `"right"`, `"bottom"` or `"left"`. |
#' | `legend.margin` | [`margin()`] | Padding around the legend. |
#' | `legend.background` | [`element_rect()`] | Background of the legend. |
#'
#' There are no further styling options.
#'
#' The context-agnostic alternative to using `theme()` is to use
#' [`theme_guide()`]:
#'
#' ```r
#' compose_crux(theme = theme_guide(
#'   title = element_text(),
#'   title.position = "top",
#'   text.position = "right",
#'   margin = margin(5),
#'   background = element_rect()
#' ))
#' ```
#'
#' @return A `<ComposeCrux>` guide object.
#' @export
#' @family composition
#'
#' @examples
#' # Roughly recreating a colour bar with extra text on top and bottom
#' crux <- compose_crux(
#'   centre = gizmo_barcap(), left = "axis_base",
#'   right = "axis_base",
#'   top = primitive_title("A lot"),
#'   bottom = primitive_title("A little")
#' )
#'
#' ggplot(mpg, aes(displ, hwy)) +
#'   geom_point(aes(colour = cty)) +
#'   guides(colour = crux)
compose_crux <- function(
  key = NULL,
  centre = "none",
  left = "none",
  right = "none",
  top = "none",
  bottom = "none",
  args = list(),
  complete = FALSE,
  theme = NULL,
  theme_defaults = list(),
  reverse = FALSE,
  order = 0L,
  title = waiver(),
  position = waiver(),
  available_aes = NULL
) {
  check_bool(complete)
  check_bool(reverse)
  new_compose(
    guides = list(
      centre = centre,
      left   = left,
      right  = right,
      top    = top,
      bottom = bottom
    ),
    reverse = reverse,
    complete = complete,
    args  = args,
    title = title,
    key   = key,
    theme = theme,
    theme_defaults = theme_defaults,
    available_aes = available_aes,
    order = order,
    position = position,
    name = "crux_composition",
    super = ComposeCrux
  )
}

# Class -------------------------------------------------------------------

#' @export
#' @rdname legendry_extensions
#' @format NULL
#' @usage NULL
ComposeCrux <- ggproto(
  "ComposeCrux", Compose,

  params = c(
    Compose$params,
    list(complete = FALSE, reverse = FALSE, theme_defaults = list())
  ),

  elements = list(
    title_position = "legend.title.position",
    text_position = "legend.text.position",
    title = "legend.title",
    margin = "legend.margin",
    background = "legend.background"
  ),

  setup_elements = function(params, elements, theme) {
    elements$title <- setup_legend_title(theme, direction = params$direction)
    theme <- theme + params$theme
    Guide$setup_elements(params, elements, theme)
  },

  override_elements = function(params, elements, theme) {
    elements$title_position <- elements$title_position %||%
      switch(params$direction, horizontal = "left", vertical = "top")
    elements$text_position <- elements$text_position %||%
      switch(params$direction, horizontal = "bottom", vertical = "right")

    check_position(
      elements$title_position,
      theta = FALSE, arg = "legend.title.position"
    )
    elements
  },

  draw = function(
    self, theme, position = NULL,
    direction = NULL, params = self$params
  ) {
    n_guides <- length(params$guides)
    if (n_guides < 1L) {
      return(zeroGrob())
    }

    direction <- params$direction <- params$direction %||% direction
    position  <- params$position  <- params$position  %||% position
    position  <- switch(position, inside = "right", position)
    check_position(position)
    check_argmatch(direction, c("horizontal", "vertical"))

    theme <- theme + params$theme
    theme <- replace_null(theme, !!!params$theme_defaults)
    elems <- self$setup_elements(params, self$elements, theme)
    elems <- self$override_elements(params, elems, theme)

    directions <- c(
      top = "horizontal", bottom = "horizontal",
      left = "vertical", right = "vertical", centre = direction
    )
    text_positions <- c(
      top = "top", bottom = "bottom", left = "left", right = "right",
      centre = elems$text_position
    )

    if (isTRUE(params$reverse)) {
      params <- set_limits(params, rev(params$limits))
    }

    grobs <- vector("list", n_guides)
    names(grobs) <- names(params$guides)

    for (i in names(params$guides)) {
      pars <- params$guide_params[[i]]
      pars$position  <- unname(switch(i, centre = position, i))
      pars$direction <- unname(directions[i])

      grobs[[i]] <- params$guides[[i]]$draw(
        theme = theme + theme(legend.text.position = text_positions[i]),
        params = pars
      )
    }

    gt <- grobs$centre
    align  <- try_alignment(params$guides$centre, gt)

    if (!is_zero(grobs$top)) {
      gt <- gtable_add_rows(gt, grobs$top$height, 0L)
      gt <- gtable_add_grob(
        gt, grobs$top, t = 1L, l = align$h[1L], r = align$h[2L],
        clip = "off", name = "top-guide"
      )
      align$v[1L] <- align$v[1L] + 1.0
    }
    if (!is_zero(grobs$bottom)) {
      gt <- gtable_add_rows(gt, grobs$bottom$height, pos = -1L)
      gt <- gtable_add_grob(
        gt, grobs$bottom, t = -1L, l = align$h[1L], r = align$h[2L],
        clip = "off", name = "bottom-guide"
      )
      align$v[2L] <- align$v[2L] - 1.0
    }
    if (!is_zero(grobs$left)) {
      gt <- gtable_add_cols(gt, grobs$left$width, pos = 0L)
      gt <- gtable_add_grob(
        gt, grobs$left, t = align$v[1L], b = align$v[2L], l = 1L,
        clip = "off", name = "left-guide"
      )
    }
    if (!is_zero(grobs$right)) {
      gt <- gtable_add_cols(gt, grobs$right$width, pos = -1L)
      gt <- gtable_add_grob(
        gt, grobs$right, t = align$v[1L], b = align$v[2L], l = -1L,
        clip = "off", name = "right-guide"
      )
    }
    if (params$complete) {
      gt <- self$add_title(
        gt,
        title = self$build_title(params$title, elems, params),
        position = elems$title_position,
        get_just(elems$title)
      )
      if (!is.null(elems$margin)) {
        gt <- gtable_add_padding(gt, elems$margin)
      }
      if (!is_zero(elems$background)) {
        gt <- gtable_add_grob(
          gt, element_grob(elems$background), name = "background",
          clip = "off", t = 1L, r = -1L, b = -1L, l = 1L, z = -Inf
        )
      }
    }
    gt
  }
)

try_alignment <- function(guide, gt) {
  v <- gt$align$vertical   %||% c(1.0, -1.0)
  h <- gt$align$horizontal %||% c(1.0, -1.0)
  if (inherits(guide, "GuideLegend")) {
    layout <- gt$layout[grepl("bar|key", gt$layout$name), ]
    v <- range(layout$t, layout$b)
    h <- range(layout$l, layout$r)
    v[2L] <- v[2L] - nrow(gt) - 1L
    h[2L] <- h[2L] - ncol(gt) - 1L

  }
  list(v = v, h = h)
}

#' Segment keys
#'
#' @description
#' These functions are helper functions for working with segment data as keys
#' in guides. They all share the goal of creating a guide key, but have
#' different methods:
#'
#' * `key_segment_manual()` directly uses user-provided vectors to set segments.
#' * `key_segment_map()` makes mappings from a `<data.frame>` to set segments.
#' * `key_dendro()` is a specialty case for coercing dendrogram data to
#'   segments. Be aware that setting the key alone cannot affect the scale
#'   limits, and will give misleading results when used incorrectly!
#'
#' @param value,value_end A vector that is interpreted to be along the scale
#'   that the guide codifies.
#' @param oppo,oppo_end A vector that is interpreted to be orthogonal to the
#'   `value` and `value_end` variables.
#' @param data A `<data.frame>` or similar object coerced by
#'   [`fortify()`][ggplot2::fortify] to a `<data.frame>`, in which the `mapping`
#'   argument is evaluated.
#' @param dendro A data structure that can be coerced to a dendrogram through
#'   the [`as.dendrogram()`][stats::as.dendrogram()] function. When `NULL`
#'   (default) an attempt is made to search for such data in the scale.
#' @param type A string, either `"rectangle"` or `"triangle"`, indicating the
#'   shape of edges between nodes of the dendrogram.
#' @param ...
#' The `...` parameter has two purposes.
#' 1. In `key_segments_map()` it is [`<data-masking>`][rlang::topic-data-mask].
#'   A set of mappings similar to those provided to [`aes()`][ggplot2::aes],
#'   which will be evaluated in the `data` argument. These *must* contain
#'   `value` and `oppo` mappings.
#' 2. In other keys, `...` can be used to transfer graphical properties to the
#'   individual breaks of a guide. For example, using `colour = "blue"` will
#'   draw parts of the guides associated with breaks in blue. There is a shallow
#'   hierarchy in that `line_colour` is the specific property for segment
#'   elements, but others inherit from the main `colour` setting. Likewise,
#'   `linewidth` and `linetype` have specific variants for line elements.
#' @param .call A [call][rlang::topic-error-call] to display in messages.
#'
#' @export
#' @name key_segments
#' @family keys
#' @return
#' For `key_segments_manual()` and `key_segments_map()`, a `<data.frame>` with
#' the `<key_range>` class.
#'
#' @examples
#' # Giving vectors directly
#' key_segment_manual(
#'   value = 0:1, value_end = 2:3,
#'   oppo  = 1:0, oppo_end  = 3:2
#' )
#'
#' # Taking columns of a data frame
#' data <- data.frame(x = 0:1, y = 1:0, xend = 2:3, yend = 3:2)
#' key_segment_map(data, value = x, oppo = y, value_end = xend, oppo_end = yend)
#'
#' # Using dendrogram data
#' clust <- hclust(dist(USArrests), "ave")
#' key_dendro(clust)(scale_x_discrete())
key_segment_manual <- function(value, oppo, value_end = value,
                               oppo_end = oppo, ...) {
  df <- data_frame0(
    value = value, oppo = oppo,
    value_end = value_end, oppo_end = oppo_end,
    !!!extra_args(..., .valid_args = .line_params),
    .error_call = current_env()
  )
  check_columns(df, c("value", "oppo"))
  class(df) <- c("key_segment", "key_guide", class(df))
  df
}

#' @rdname key_segments
#' @export
key_segment_map <- function(data, ..., .call = caller_env()) {

  mapping <- aes(!!!enquos(...))

  df <- eval_aes(
    data, mapping,
    required = c("value", "oppo"),
    optional = c("value_end", "oppo_end", .line_params),
    call = .call, arg_mapping = "mapping", arg_data = "data"
  )

  df <- rename(df, .line_params, paste0(".", .line_params))
  class(df) <- c("key_segment", "key_guide", class(df))
  df

}

#' @rdname key_segments
#' @export
key_dendro <- function(dendro = NULL, type = "rectangle", ..., .call = NULL) {
  call <- .call %||% current_call()
  dots <- extra_args(..., .valid_args = .line_params)
  force(dendro)
  function(scale, aesthetic = NULL, ...) {
    key <- extract_dendro(scale$scale$clust %||% dendro, type = type)
    data_frame0(key, !!!dots, .error_call = call)
  }
}

# Extractor ---------------------------------------------------------------

segment_extract_key <- function(scale, aesthetic, key, ...) {
  key <- standard_extract_key(scale, aesthetic, key, ...)

  # Backtransform AsIs variables
  range <- scale$continuous_range %||% scale$get_limits()
  key$value     <- descale(key$value, range)
  key$value_end <- descale(key$value_end, range)

  remove_vars <- character()

  # Interleave values
  value_vars <- c("value", "value_end")
  if (all(value_vars %in% names(key))) {
    value <- vec_interleave(key$value, key$value_end)
    remove_vars <- c(remove_vars, value_vars)
  }

  # Interleave opposites
  oppo_vars <- c("oppo", "oppo_end")
  if (all(oppo_vars %in% names(key))) {
    oppo <- vec_interleave(key$oppo, key$oppo_end)
    remove_vars <- c(remove_vars, oppo_vars)
  }

  # Reconstruct key
  key[remove_vars] <- NULL
  new_key <- data_frame0(value = value, oppo = oppo)
  i <- rep(vec_seq_along(key), each = 2L)
  new_key[names(key)] <- vec_slice(key, i)
  new_key$group <- new_key$group %||% i
  new_key$oppo  <- rescale(new_key$oppo, from = range(new_key$oppo, 0.0))

  # Normalise key column names
  if (aesthetic %in% c("x", "y")) {
    new_names <- union(aesthetic, c("x", "y"))
    new_key <- rename(new_key, c("value", "oppo"), new_names)
  } else {
    new_key <- rename(new_key, "value", aesthetic)
    new_key$.value <- new_key[[aesthetic]]
  }
  new_key
}

# Dendrogram utilities ----------------------------------------------------

# Simplified version of `stats:::plotNode`.
# It only looks for the segments and ignores labels and most other attributes.
extract_dendro <- function(tree, type = "rectangle") {

  # Check arguments
  whole_tree <- tree <- as_dendrogram(tree)

  # Initialise stuff
  depth <- 0L
  llimit <- list()
  x1 <- i <- 1L
  x2 <- number_of_members(tree)
  KK <- kk <- integer()

  mtx <- setup_segments(stats::nobs(tree), type)
  col_order <- attr(mtx, "order", exact = TRUE)
  row_index <- attr(mtx, "offset", exact = TRUE)
  row_step  <- length(row_index)

  repeat {
    depth <- depth + 1L
    inner <- !stats::is.leaf(tree) && x1 != x2

    node <- node_limit(x1, x2, tree)
    llimit[[depth]] <- node$limit

    ymax <- attr(tree, "height")
    xmax <- node$x

    if (inner) {
      for (k in seq_along(tree)) {
        child <- tree[[k]]

        ymin <- attr(child, "height") %||% 0.0
        xmin <- node$limit[k] + (attr(child, "midpoint") %||% 0.0)

        mtx[i + row_index, ] <- c(xmax, ymax, xmin, ymin)[col_order]
        i <- i + row_step
      }
      if (length(tree) > 0L) {
        KK[depth] <- length(tree)
        kk[depth] <- 1L
        x1 <- node$limit[1L]
        x2 <- node$limit[2L]
        tree <- tree[[1L]]
      }
    } else {
      repeat {
        # We climb back up the tree until we have unvisited children
        depth <- depth - 1L
        if (!depth || kk[depth] < KK[depth]) {
          break
        }
      }
      if (!depth) {
        break
      }
      length(kk) <- depth
      kk[depth] <- k <- kk[depth] + 1L
      x1 <- llimit[[depth]][k]
      x2 <- llimit[[depth]][k + 1L]
      tree <- whole_tree[[kk]]
    }
  }
  as.data.frame(mtx)
}

setup_segments <- function(n_obs, type = "triangle") {
  type <- arg_match0(type, c("rectangle", "triangle"))
  if (type == "triangle") {
    n_segments <- 2L * n_obs - 2L
    i_increment <- 1L
    i_offset <- 0L
    coord_order <- c(1L, 2L, 3L, 4L)
  } else {
    n_segments <- 4L * n_obs - 4L
    i_increment <- 2L
    i_offset <- 0L:1L
    # xmax -> xmin -> ymax -> ymax -> xmin -> xmin -> ymax -> ymin
    coord_order <- c(1L, 3L, 2L, 2L, 3L, 3L, 2L, 4L)
  }

  mtx <- matrix(NA_real_, n_segments, ncol = 4L)

  attributes(mtx) <- list(
    offset = i_offset,
    increment = i_increment,
    order = coord_order,
    dim = dim(mtx)
  )
  colnames(mtx) <- c("value", "oppo", "value_end", "oppo_end")
  mtx
}

# Copy of `stats:::.memberDend()`
number_of_members <- function(tree) {
  attr(tree, "x.member") %||% attr(tree, "members") %||% 1L
}

# Simplified version of `stats:::plotNodeLimit`,
# It has `center = FALSE` build-in.
node_limit <- function(x1, x2, subtree) {
  inner <- !stats::is.leaf(subtree) && x1 != x2
  if (inner) {
    K <- length(subtree)
    limit <- integer(K)
    xx1 <- x1
    for (k in 1L:K) {
      xx1 <- xx1 + number_of_members(subtree[[k]])
      limit[k] <- xx1
    }
  } else {
    limit <- x2
  }
  limit <- c(x1, limit)
  mid <- attr(subtree, "midpoint")
  center <- inner && !is.numeric(mid)
  x <- if (center) mean(c(x1, x2)) else x1 + (mid %||% 0L)
  list(x = x, limit = limit)
}

as_dendrogram <- function(tree) {
  # Check arguments
  try_fetch(
    stats::as.dendrogram(tree),
    error = function(cnd) {
      cli::cli_abort(
        "Could not find or coerce {.arg dendro} argument.",
        parent = cnd
      )
    }
  )
}

test_that("key_segment_manual works as intended", {

  test <- key_segment_manual(
    c("A", "B"), 1L:2L, c("B",  "C"),
    3L:4L, linetype = 1L:2L
  )
  expect_s3_class(test, "key_segment")

  expect_identical(test$value, c("A", "B"))
  expect_identical(test$value_end, c("B", "C"))
  expect_identical(test$oppo, 1L:2L)
  expect_identical(test$oppo_end, 3L:4L)
  expect_identical(test$.linetype, 1L:2L)
})

test_that("key_segment_map works as intended", {

  data <- data_frame0(
    x = c("A", "B"), y = 1L:2L, xend = c("B", "C"), yend = 3L:4L
  )

  test <- key_segment_map(
    data, value = x, oppo = y, value_end = xend, oppo_end = yend,
    linetype = 1L:2L, color = "blue"
  )

  expect_identical(test$value, c("A", "B"))
  expect_identical(test$value_end, c("B", "C"))
  expect_identical(test$oppo, 1L:2L)
  expect_identical(test$oppo_end, 3L:4L)
  expect_identical(test$.linetype, 1L:2L)
  expect_identical(test$.colour, c("blue", "blue"))
})

test_that("key_dendro works as intended", {

  dummy <- scale_x_discrete()
  d <- hclust(dist(matrix(1L:9L, 3L)))
  ptype <- data.frame(
    value = double(), oppo = double(),
    value_end = double(), oppo_end = double()
  )

  test <- key_dendro(d, type = "rectangle")(dummy)
  expect_vector(test, ptype, size = 8L)

  test <- key_dendro(d, type = "triangle")(dummy)
  expect_vector(test, ptype, size = 4L)

})

test_that("segment_extract_key works as intended", {

  key <- key_segment_manual(
    value = 1.0, oppo = 0.0,
    value_end = I(0.9), oppo_end = 1.0
  )

  sc <- scale_x_continuous(limits = c(0.0, 10.0))
  test <- segment_extract_key(sc, "x", key)
  expect_equal(test$x, c(1.0, 9.0))

  sc <- scale_colour_gradient(limits = c(0.0, 10.0))
  test <- segment_extract_key(sc, "colour", key)
  expect_equal(test$.value, c(1.0, 9.0))

})

# Constructor -------------------------------------------------------------

#' Circle size guide
#'
#' This guide displays the sizes of points as a series of circles. It is
#' typically paired with [`geom_point()`][ggplot2::geom_point] with
#' [`draw_key_point()`][ggplot2::draw_key_point] glyphs.
#'
#' @param key A [standard key][key_standard] specification. Defaults to
#'   [`key_auto()`]. See more information in the linked topic.
#' @param hjust,vjust A `<numeric[1]>` between 0 and 1 giving the horizontal
#'   and vertical justification, respectively, of the central shapes. It is
#'   recommended `hjust = 0.5` when text is placed on the left or right and
#'   `vjust = 0.5` is recommended when text is placed on top or in the bottom.
#' @param text_position A string, one of `"ontop"`, `"top"`, `"right"`,
#'   `"bottom"`, or `"left"` do describe the placement of labels. The
#'   default (`NULL`), will take the `legend.text.position` theme setting.
#' @param clip_text A `<logical[1]>` whether to give text in the `"ontop"`
#'   position a small rectangle of background colour.
#' @inheritParams common_parameters
#'
#' @return A `<GuideCircles>` object.
#' @export
#' @family standalone guides
#' @details
#' Please note that the default size scales scale to area, not radius, so
#' equidistant breaks will appear at irregularly spaced positions due to
#' labelling the diameter of a circle.
#'
#' This graph was designed with standard round [shapes][ggplot2::scale_shape]
#' in mind, i.e. shapes 1, 16, 19 and 21. For that reason, `shape = 1` is the
#' default `override.aes` argument. Other shapes will probably be drawn but the
#' quality of their alignment and label placement may be unsatisfactory.
#'
#' ## Styling options
#'
#' Below are the [theme][ggplot2::theme] options that determine the styling of
#' this guide.
#'
#' | **Theme setting** | **Type** | **Description** |
#' | ----------------- | -------- | --------------- |
#' | `legend.background`| [`element_rect()`] | Background of the legend. |
#' | `legend.margin` | [`margin()`] | Padding around the legend. |
#' | `legend.key` | [`element_rect()`] | Background of the key area underneath circles. |
#' | `legend.text` | [`element_text()`] | Labels displayed next to tick marks or on top of circles. |
#' | `legendry.legend.key.margin` | [`margin()`] | Padding between circles and edge of key area. |
#' | `legend.ticks` | [`element_line()`] | Tick marks connecting circle to label. |
#' | `legend.title` | [`element_text()`] | Title of the legend. |
#' | `legend.title.position` | `<character[1]>` | One of `"top"`, `"right"`, `"bottom"` or `"left"`.
#'
#' Styling options *per break* can be set in the [standard key][key_standard].
#' These override theme settings.
#'
#' The context-agnostic alternative to using `theme()` is to use
#' [`theme_guide()`]:
#'
#' ```r
#' guide_circles(theme = theme_guide(
#'   text = element_text(),
#'   title = element_text(),
#'   title.position = "top",
#'   margin = margin(5),
#'   key = element_rect(),
#'   key.margin = margin(5),
#'   background = element_rect(),
#'   ticks = element_line(),
#' ))
#' ```
#'
#' @examples
#' # A standard plot
#' p <- ggplot(mtcars, aes(disp, mpg)) +
#'   geom_point(aes(size = hp), alpha = 0.3)
#'
#' # By default, the sizes aren't large enough to make this guide clear
#' p + scale_size_area(guide = "circles")
#'
#' # Update with a more approrpriate scale
#' p <- p +
#'   scale_size_area(
#'     max_size = 30,
#'     limits = c(0, NA),
#'     breaks = c(0, 25, 100, 250)
#'   )
#' p + guides(size = "circles")
#'
#' # Horizontal orientation
#' p + guides(size = guide_circles(
#'   vjust = 0.5, hjust = 0, text_position = "bottom"
#' ))
#'
#' # Alternative text placement
#' p + guides(size = guide_circles(
#'   text_position = "ontop",
#'   clip_text = TRUE
#' ))
#'
#' # More styling options
#' p + guides(size = guide_circles(override.aes = aes(colour = "red")))+
#'   theme(
#'     # Key background
#'     legend.key = element_rect(colour = "black", fill = 'white'),
#'     # Padding around central shapes
#'     legendry.legend.key.margin = margin(1, 1, 1, 1, "cm"),
#'     legend.ticks = element_line(colour = "blue"),
#'     legend.text.position = "left"
#'   )
guide_circles <- function(
  key = NULL,
  title = waiver(),
  theme = NULL,
  hjust = 0.5,
  vjust = 0.0,
  text_position = NULL,
  clip_text = FALSE,
  override.aes = list(shape = 1L),
  position = waiver(),
  direction = NULL
) {

  check_number_decimal(hjust, min = 0.0, max = 1.0, allow_infinite = FALSE)
  check_number_decimal(vjust, min = 0.0, max = 1.0, allow_infinite = FALSE)
  check_bool(clip_text)
  check_argmatch(text_position, c(.trbl, "ontop"), allow_null = TRUE)

  new_guide(
    key = key,
    title = title,
    theme = theme,
    hjust = hjust,
    vjust = vjust,
    text_position = text_position,
    clip_text = clip_text,
    override.aes = rename_aes(override.aes),
    position = position,
    direction = direction,
    super = GuideCircles
  )
}

# Class -------------------------------------------------------------------

#' @export
#' @rdname legendry_extensions
#' @format NULL
#' @usage NULL
GuideCircles <- ggproto(
  "GuideCircles", Guide,

  params = new_params(key = "auto", hjust = 0.5, vjust = 0.0, clip_text = TRUE,
                      text_position = "ontop", override.aes = list()),

  elements = list(
    background = "legend.background",
    margin     = "legend.margin",
    key        = "legend.key",
    text       = "legend.text",
    padding    = "legendry.legend.key.margin",
    ticks      = "legend.ticks",
    title      = "legend.title",
    title_position = "legend.title.position"
  ),

  extract_key = standard_extract_key,

  extract_params = function(scale, params, title = waiver(), ...) {
    params$title    <- scale$make_title(params$title, scale$name, title)
    params$position <- params$position %|W|% NULL
    params$limits   <- scale$get_limits()
    params
  },

  process_layers = function(self, params, ...) {
    # We transfer key properties to the `override.aes` to transfer them to
    # the glyphs drawn for the legend.
    props <- element_key_properties(params$key, "point")
    props <- props[setdiff(names(props), names(params$key))]
    params$override.aes <- list2(!!!params$override.aes, !!!props)
    GuideLegend$process_layers(params, ...)
  },

  build_decor = function(decor, grobs, elements, params) {
    key <- params$key
    key <- vec_slice(key, order(-key$size))

    glyphs <- lapply(
      decor, draw_circles,
      hjust = params$hjust,
      vjust = params$vjust
    )

    x <- glyphs[[1L]]$xpos
    y <- glyphs[[1L]]$ypos

    key_bg <- element_grob(elements$key)
    padding <- cm(elements$padding)
    width <- width_cm(glyphs[[1L]]$width)
    height <- height_cm(glyphs[[1L]]$height)
    width  <- unit(c(padding[4L], width,  padding[2L]), "cm")
    height <- unit(c(padding[1L], height, padding[3L]), "cm")

    position <- params$text_position %||% elements$text_position
    if (position == "ontop") {
      text <- Map(
        element_grob, label = key$.label, x = x, y = y,
        MoreArgs = list(element = elements$text)
      )
      if (isTRUE(params$clip_text)) {
        glyphs <- censor_text_background(
          x, y, text, glyphs,
          vjust = elements$text$vjust, colour = elements$key$fill
        )
      }
      grob <- gTree(children = inject(gList(!!!glyphs, !!!text)))
    } else {
      ticks <- draw_circle_ticks(
        elements$ticks, x, y, padding, position,
        element_key_properties(key, "line")
      )

      x <- switch(position, left = , right = NULL, x)
      y <- switch(position, top = , bottom = NULL, y)

      text <- inject(element_grob(
        elements$text, x = x, y = y, label = key$.label,
        margin_x = position %in% c("left", "right"),
        margin_y = position %in% c("top", "bottom"),
        !!!element_key_properties(key, "text")
      ))
      grob <- gTree(children = inject(gList(ticks, !!!glyphs)))
    }

    gt <- gtable(widths = width, heights = height) |>
      gtable_add_grob(
        key_bg, t = 1L, l = 1L, b = 3L, r = 3L,
        clip = "off", name = "key-bg"
      ) |>
      gtable_add_grob(
        grob, t = 2L, l = 2L,
        clip = "off", name = "circles"
      )
    if (position == "ontop") {
      return(gt)
    }

    gt <- switch(
      position,
      left   = gtable_add_cols(gt, unit(width_cm(text),  "cm"),  0L),
      right  = gtable_add_cols(gt, unit(width_cm(text),  "cm"), -1L),
      bottom = gtable_add_rows(gt, unit(height_cm(text), "cm"), -1L),
      top    = gtable_add_rows(gt, unit(height_cm(text), "cm"),  0L)
    )
    gt <- gtable_add_grob(
      gt, text, clip = "off", name = "labels",
      t = switch(position, top  = 1L, bottom = 4L, 2L),
      l = switch(position, left = 1L, right  = 4L, 2L)
    )
    gt
  },

  setup_elements = function(params, elements, theme) {
    theme <- theme + params$theme
    theme$legend.axis.line <-
      theme$legend.axis.line %||% calc_element("panel.grid", theme)

    # Setup title
    title_position <- calc_element("legend.title.position", theme) %||%
      switch(params$direction, horizontal = "left", vertical = "top")
    elements$title <- setup_legend_title(
      theme,
      title_position,
      params$direction
    )

    text_position <- params$text_position %||%
      calc_element("legend.text.position", theme) %||% "right"
    if (text_position != "ontop") {
      elements$text <- setup_legend_text(theme, text_position, params$direction)
    }

    is_char <- map_lgl(elements, is.character)
    elements[is_char] <- lapply(elements[is_char], calc_element, theme = theme)
    elements$title_position <- title_position
    elements$text_position  <- text_position
    elements
  },

  draw = function(self, theme, position = NULL, direction = NULL,
                  params = self$params) {

    params <- replace_null(params, position = position, direction = direction)

    elems  <- self$setup_elements(params, self$elements, theme)

    title <- self$build_title(params$title, elems, params)
    grob  <- self$build_decor(params$decor, NULL, elems, params)
    grobs <- list(circles = grob, title = title)

    self$assemble_drawing(grobs, params = params, elements = elems)
  },

  assemble_drawing = function(self, grobs, layout = NULL, sizes = NULL,
                              params = list(), elements = list()) {

    # Add title
    gt <- self$add_title(
      grobs$circles, grobs$title, elements$title_position,
      rotate_just(element = elements$title)
    )

    # Add padding
    padding <- rep(0.0, 4L)
    padding[c(1L, 3L)] <- height_cm(elements$margin[c(1L, 3L)])
    padding[c(2L, 4L)] <-  width_cm(elements$margin[c(2L, 4L)])
    gt <- gtable_add_padding(gt, unit(padding, "cm"))

    # Add background
    background <- element_grob(elements$background)
    if (!is_zero(background)) {
      gt <- gtable_add_grob(
        gt, background, name = "background", clip = "off",
        t = 1L, r = -1L, b = -1L, l = 1L, z = -Inf
      )
    }
    gt
  }
)


# Helpers -----------------------------------------------------------------

censor_text_background <- function(x, y, text, background,
                                   vjust = 0.5, padding = 0.1,
                                   colour = "white") {

  width  <- width_cm(text)  + padding
  height <- height_cm(text) + padding

  mask <- rectGrob(
    x = x, y = y - unit((vjust - 0.5) * height, "cm"),
    width  = unit(width, "cm"),
    height = unit(height, "cm"),
    gp = gpar(fill = colour, col = NA, lty = 0.0),
    vp = viewport(clip = "on")
  )

  c(background, list(mask))
}

draw_circle_ticks <- function(element, x, y, padding, position, props) {

  n <- length(x)

  xend <- switch(
    position,
    left  = unit(rep(0.0, n), "npc") - unit(padding[4L], "cm"),
    right = unit(rep(1.0, n), "npc") + unit(padding[2L], "cm"),
    x
  )

  yend <- switch(
    position,
    top    = unit(rep(1.0, n), "npc") + unit(padding[1L], "cm"),
    bottom = unit(rep(0.0, n), "npc") - unit(padding[3L], "cm"),
    y
  )

  interleave <- vec_interleave(seq_len(n), seq_len(n) + n)
  x <- unit.c(x, xend)[interleave]
  y <- unit.c(y, yend)[interleave]

  inject(element_grob(
    element, x = x, y = y,
    id.lengths = rep(2L, n),
    !!!props
  ))
}

draw_circles <- function(decor, vjust = 0.0, hjust = 0.5) {

  data <- vec_slice(decor$data, decor$data$.draw %||% TRUE)
  order <- order(-data$size)
  glyph <- lapply(seq_len(nrow(data))[order], function(i) {
    decor$draw_key(vec_slice(data, i), decor$params, 0.0)
  })
  size  <- map_dbl(glyph, function(x) max(x$gp$fontsize %||% 0.0))
  shape <- map_dbl(glyph, function(x) if (is.numeric(x$pch)) x$pch[1L] else 1L)

  # These are some arcane incantations I've long forgotten why they work
  size <- size * magic_num * pch_mult[shape]
  ydiv <-  pch_div0[shape] * (1.0 - vjust) + pch_div1[shape] * vjust
  xdiv <- (pch_div0[shape] * (1.0 - hjust) + pch_div1[shape] * hjust) *
    asp[shape]

  yoffset <- (max(size) - size) / ydiv
  xoffset <- (max(size) - size) / xdiv

  ypos <- (vjust - 0.5) * 2.0
  ypos <- unit(ynpc[shape], "npc") + unit(ypos * yoffset, "pt")

  xpos <- (hjust - 0.5) * 2.0
  xpos <- unit(0.5, "npc") + unit(xpos * xoffset, "pt")

  glyph <- Map(function(grob, x, y) {
    vp <- editViewport(grob$vp %||% viewport(), x = x, y = y)
    editGrob(grob, vp = vp)
  }, grob = glyph, x = xpos, y = ypos)

  if (all(shape %in% c(0L, 7L, 12L, 14L, 15L, 22L))) {
    xpos <- xpos + unit((hjust - 0.5) * -size, "pt")
    ypos <- ypos + unit((vjust - 0.5) * -size, "pt")
  } else {
    angle <- 1.5 * pi - atan2(vjust * 2.0 - 1.0, hjust * 2.0 - 1.0)
    xpos <- xpos + unit(sin(angle) * 0.5 * size, "pt")
    ypos <- ypos + unit(cos(angle) * 0.5 * size * asp[shape], "pt")
  }

  gTree(
    height = unit(max(size), "pt"),
    width  = unit(max(size * asp[shape]), "pt"),
    xpos = xpos, ypos = ypos,
    children = inject(gList(!!!glyph))
  )
}

# Magic constants ---------------------------------------------------------

# Don't bother asking me to implement other shapes. I've already forgotten
# why the calculations do what they do.

magic_num <- .pt / .stroke

pch_div <- rep(
  c(2.0, 3.0, 2.0, 1.5, 2.0, 3.0, 2.0, 3.0, 1.5),
  c(1L, 1L, 3L, 1L, 10L, 1L, 6L, 1L, 1L)
)

pch_div0 <- pch_div
pch_div1 <- pch_div
pch_div1[c(2L, 17L, 24L)] <- 1.5
pch_div1[c(6L, 25L)] <- 3.0

ynpc <- rep(0.5, 25L)
ynpc[c(2L, 17L, 24L)] <- 0.35
ynpc[c(6L, 25L)] <- 0.65

asp <- rep(1.0, 25L)
asp[c(2L, 6L, 17L, 24L, 25L)] <- 2.0 / sqrt(3.0)

pch_mult <- c(
  1.0, 1.166, 1.41466666666667, 1.0, 1.41466666666667, 1.166, 1.0,
  1.0, 1.41466666666667, 1.0, 1.55466666666667, 1.0, 1.0, 1.0, 1.0, 1.0, 1.166,
  1.0, 1.0, 0.666666666666667, 1.0, 0.886, 1.25333333333333, 1.166, 1.166
)

test_that("label placement is ok regardless of hjust or vjust", {

  p <- ggplot(data.frame(x = c(4.0, 12.0, 25.0))) +
    geom_point(aes(x, x, size = x)) +
    scale_size_area(
      limits = c(0.0, 25.0),
      breaks = c(0.0, 3.0, 12.0, 25.0),
      max_size = 20L,
      guide = guide_circles()
    )

  build  <- ggplot_build(p)
  guide  <- build$plot$guides$get_guide("size")
  params <- build$plot$guides$get_params("size")
  params[c("position", "direction")] <- list("right", "vertical")

  grid <- vec_expand_grid(hjust = c(0.0, 0.5, 1.0), vjust = c(0.0, 0.5, 1.0))

  grobs <- lapply(vec_seq_along(grid), function(i) {
    tmp <- params
    tmp[c("hjust", "vjust")] <- as.list(grid[i, ])
    guide$draw(
      theme_get() + theme(legend.text.position = "ontop"),
      params = tmp
    )
  })

  gt <- gtable(unit(rep(1.0, 3L), "null"), unit(rep(1.0, 3L), "null"))
  gt <- gtable_add_grob(
    gt, grobs,
    t = grid$vjust * 2.0 + 1.0,
    l = grid$hjust * 2.0 + 1.0
  )

  vdiffr::expect_doppelganger(
    "guide_circles text placement",
    gt
  )

  grid <- data_frame0(
    text  = c("top", "right", "bottom", "left"),
    hjust = c(1.0, 0.5, 0.0, 0.5),
    vjust = c(0.5, 0.0, 0.5, 1.0)
  )

  grobs <- lapply(vec_seq_along(grid), function(i) {
    tmp <- params
    tmp[c("text_position", "hjust", "vjust")] <- as.list(grid[i, ])
    guide$draw(
      theme_get(),
      params = tmp
    )
  })

  gt <- gtable(unit(rep(1.0, 2.0), "null"), unit(rep(1.0, 2.0), "null"))
  gt <- gtable_add_grob(
    gt, grobs,
    t = c(1L, 2L, 1L, 2L),
    l = c(1L, 1L, 2L, 2L)
  )

  vdiffr::expect_doppelganger(
    "guide_circles text locations",
    gt
  )
})

test_that("guide_circles handles override.aes properly", {
  p <- guide_circles(override.aes = list(color = "blue", pch = 19L))
  expect_identical(p$params$override.aes, list(colour = "blue", shape = 19L))

  expect_snapshot_warning(
    p <- guide_circles(override.aes = list(colour = "black", color = "red"))
  )
  expect_identical(p$params$override.aes, list(colour = "black", colour = "red"))
})

# Constructor -------------------------------------------------------------

#' Nested axis guide
#'
#' This axis guide gives extra range annotations to position scales. It can
#' be used to infer nesting structure from labels or annotate ranges.
#'
#' @param key One of the following:
#'   * A [range key][key_range] specification. If not `key = "range_auto"`,
#'     additional labels will be inserted to represent point values.
#'   * A `<character[1]>` passed to the [`key_range_auto(sep)`][key_range_auto]
#'     argument. An exception is made when the string is a valid key
#'     specification.
#' @param regular_key A [standard key][key_standard] specification for the
#'   appearance of regular tick marks.
#' @param type
#' Appearance of ranges. One of the following:
#' * `"box"` to put text in boxes.
#' * `"bracket"` (default) to text over brackets.
#' * `"fence"` to put text near fences.
#' @param subtitle Passed on to [`primitive_title(title)`][primitive_title].
#'   Follow the linked topic for more details.
#' @inheritParams common_parameters
#' @inheritParams primitive_line
#' @inheritParams primitive_ticks
#' @inheritParams primitive_bracket
#' @param ... Arguments passed on to [`primitive_bracket()`],
#'   [`primitive_box()`] or [`primitive_fence()`] (depending on the `type`
#'   argument).
#'
#' @details
#' To offer other keys the opportunity to display ranges alongside
#' regular-looking labels, the `regular_key` argument can be used to setup a
#' separate key for display in between the ticks and ranges.
#'
#' By default, the [`key = "range_auto"`][key_range] will incorporate the 0th
#' level labels inferred from the scale's labels. These labels will look like
#' regular labels.
#'
#' ## Styling options
#'
#' Because this guide is pure composite guide, the [theme][ggplot2::theme]
#' options that govern the styling are determined by its constituents. They are
#' linked below so you can find their 'Styling options' sections.
#'
#' | **Primitive** | **Context** | **Description** |
#' | ------------- | ----------- | --------------- |
#' | [`compose_stack`] | Always | Stacks the other primitives. |
#' | [`primitive_line()`] | Always | Makes up the axis line. |
#' | [`primitive_ticks()`] | Always | Makes up the tick marks. |
#' | [`primitive_bracket()`] | `type = "bracket"` | Range display as brackets. |
#' | [`primitive_box()`] | `type = "box"` | Range display as boxes. |
#' | [`primitive_fence()`] | `type = "fence"` | Range display as fences. |
#' | [`primitive_title()`] | `subtitle = <...>` | Used for displaying subtitles. |
#' | [`primitive_labels()`] | `key != "range_auto"` | Used for displaying range-less, 0th level labels |
#'
#' Styling options *per range* can be set in the [range key][key_range].
#' These override theme settings.
#'
#' The context-agnostic alternative to using `theme()` is to use
#' [`theme_guide()`]:
#'
#' ```r
#' guide_axis_nested(theme = theme_guide(
#'   # Common options
#'   line = element_line(),
#'   text = element_text(),
#'   ticks = element_line(),
#'   ticks.length = unit(5, "mm"),
#'
#'   # For brackets
#'   bracket = element_line(),
#'   bracket.size = unit(5, "mm"),
#'
#'   # For boxes
#'   box = element_rect(),
#'
#'   # For fences
#'   fence = element_line(),
#'   fence.post = element_line(),
#'   fence.rail = element_line(),
#'
#'   # For subtitle (not main title)
#'   title = element_text()
#' ))
#' ```
#'
#' @return A `<Guide>` object.
#' @export
#' @family standalone guides
#'
#' @examples
#' # A plot with nested categories on the x-axis
#' p <- ggplot(mpg, aes(interaction(drv, cyl), hwy)) +
#'   geom_boxplot()
#'
#' p + guides(x = "axis_nested")
#'
#' # Apply styling to brackets
#' p + guides(x = "axis_nested") +
#'   theme_guide(bracket = element_line("red", linewidth = 1))
#'
#' # Don't drop nesting indicators that have 0-width
#' p + guides(x = guide_axis_nested(drop_zero = FALSE))
#'
#' # Change additional padding for discrete categories
#' p + guides(x = guide_axis_nested(pad_discrete = 0))
#'
#' # Change bracket type
#' p + guides(x = guide_axis_nested(bracket = "curvy"))
#'
#' # Use boxes instead of brackets + styling of boxes
#' p + guides(x = guide_axis_nested(type = "box")) +
#'   theme_guide(box = element_rect("limegreen", "forestgreen"))
#'
#' # Using fences instead of brackets + styling of fences
#' p + guides(x = guide_axis_nested(type = "fence", rail = "inner")) +
#'   theme_guide(
#'     fence.post = element_line("tomato"),
#'     fence.rail = element_line("dodgerblue")
#'   )
#'
#' # Use as annotation of a typical axis
#' # `regular_key` controls display of typical axis
#' ggplot(mpg, aes(displ, hwy)) +
#'   geom_point() +
#'   guides(x = guide_axis_nested(
#'     key = key_range_manual(
#'       start = 2:3,
#'       end = 5:6,
#'       name = c("First", "Second")
#'     ),
#'     regular_key = key_manual(c(2, 2.5, 3, 5, 7))
#'   ))
guide_axis_nested <- function(
  key   = "range_auto",
  regular_key = "auto",
  type  = "bracket",
  title = waiver(),
  subtitle = NULL,
  theme = NULL,
  angle = waiver(),
  cap   = "none",
  bidi  = FALSE,
  oob   = "squish",
  drop_zero = TRUE,
  pad_discrete = NULL,
  levels_text = NULL,
  ...,
  order = 0L,
  position = waiver()
) {

  theme <- theme %||% theme()
  theme$legendry.guide.spacing <-
    theme$legendry.guide.spacing %||% unit(0.0, "cm")

  pad_discrete <- pad_discrete %||% switch(type, fence = 0.5, 0.4)

  guides <- list(
    primitive_line(cap = cap, position = position),
    primitive_ticks(bidi = bidi, position = position)
  )

  nesting <- switch(
    arg_match0(type, c("bracket", "box", "fence")),
    bracket = primitive_bracket,
    box     = primitive_box,
    fence   = primitive_fence
  )

  if (is_string(key) && !is_key_string(key)) {
    key <- key_range_auto(sep = key)
  }
  # Some of the `nesting` guides may already absorb label-duty
  automatic_labels <-
    identical(key, "range_auto") ||
    inherits(key, "key_range_auto_function") ||
    is.null(key)
  if (!automatic_labels) {
    # If not absorbed, roll out a formal primitive for labels
    guides <- c(guides, list(primitive_labels(angle = angle)))
  }

  # Populate nesting guide
  nesting <- nesting(
    key = key %||% "range_auto", angle = angle,
    oob = oob, drop_zero = drop_zero, pad_discrete = pad_discrete,
    levels_text = levels_text, ...
  )
  guides <- c(guides, list(nesting))

  # Add optional subtitle
  if (!is.null(subtitle)) {
    guides <- c(guides, list(primitive_title(subtitle)))
  }

  compose_stack(
    !!!guides,
    key = regular_key %||% "auto",
    side.titles = NULL, drop = 3L:4L, title = title, theme = theme,
    order = order, available_aes = c("any", "x", "y", "r", "theta"),
    position = position
  )
}

# Constructor -------------------------------------------------------------

#' Custom legend guide
#'
#' This legend closely mirrors `ggplot2::guide_legend()`, but has two
#' adjustments. First, `guide_legend_base()` supports a `design` argument
#' for a more flexible layout. Secondly, the `legend.spacing.y` theme element
#' is observed verbatim instead of overruled.
#'
#' @param key A [standard key][key_standard] specification. Defaults to
#'   [`key_auto()`]. See more information in the linked topic.
#' @param design Specification of the legend layout. One of the following:
#'   * `NULL` (default) to use the layout algorithm of
#'     [`guide_legend()`][ggplot2::guide_legend()].
#'   * A `<character[1]>` string representing a cell layout wherein `#` defines
#'     an empty cell. See examples.
#'   * A `<matrix[n, m]>` representing a cell layout wherein `NA` defines an
#'     empty cell. See examples. Non-string atomic vectors will be treated with
#'     `as.matrix()`.
#' @param nrow,ncol A positive `<integer[1]>` setting the desired dimensions of
#'   the legend layout. When `NULL` (default), the dimensions will be derived
#'   from the `design` argument or fit to match the number of keys.
#' @param reverse A `<logical[1]>` whether the order of keys should be inverted.
#' @inheritParams common_parameters
#'
#' @details
#' ## Styling options
#'
#' Below are the [theme][ggplot2::theme] options that determine the styling of
#' this guide. Note that these are the same for [`ggplot2::guide_legend()`].
#'
#' | **Theme setting** | **Type** | **Description** |
#' | ----------------- | -------- | --------------- |
#' | `legend.background`| [`element_rect()`] | Background of the legend. |
#' | `legend.margin` | [`margin()`] | Padding around the legend. |
#' | `legend.text` | [`element_text()`] | Labels displayed next to keys. |
#' | `legend.text.position` | `<character[1]>` | One of `"top"`, `"right"`, `"bottom"` or `"left"`. |
#' | `legend.title` | [`element_text()`] | Title of the legend. |
#' | `legend.title.position` | `<character[1]>` | One of `"top"`, `"right"`, `"bottom"` or `"left"`. |
#' | `legend.key` | [`element_rect()`] | Background of the key areas. |
#' | `legend.key.height` | [`unit()`] | Height of keys. |
#' | `legend.key.width` | [`unit()`] | Width of keys. |
#' | `legend.key.justification` | `<numeric[2]>` | Justification for placing legend keys in excess space. |
#' | `legend.key.spacing.x` | [`unit()`] | Horizontal spacing between keys. |
#' | `legend.key.spacing.y` | [`unit()`] | Vertical spacing between keys. Taken literally. |
#' | `legend.byrow` | `<logical[1]>` | Row-order key filling (`TRUE`) or column-order (`FALSE`) |
#'
#' The context-agnostic alternative to using `theme()` is to use
#' [`theme_guide()`]:
#'
#' ```r
#' guide_legend_base(theme = theme_guide(
#'   text = element_text(),
#'   text.position = "right",
#'   title = element_text(),
#'   title.position = "top",
#'   key = element_rect(),
#'   key.height = unit(5, "mm"),
#'   key.width = unit(5, "mm"),
#'   key.justification = c(0.5, 0.5),
#'   key.spacing.x = unit(5, "mm"),
#'   key.spacing.y = unit(5, "mm"),
#'   byrow = TRUE,
#'   margin = margin(5),
#'   background = element_rect(),
#' ))
#' ```
#'
#' @return A `<GuideLegend>` object.
#' @export
#' @family standalone guides
#' @family legend guides
#'
#' @examples
#' # A dummy plot
#' p <- ggplot(data.frame(x = 1:3, type = c("tic", "tac", "toe"))) +
#'   aes(x, x, shape = type) +
#'   geom_point(na.rm = TRUE) +
#'   scale_shape_manual(values = c(1, 4, NA))
#'
#' # A design string, each character giving a cell value.
#' # Newlines separate rows, white space is ignored.
#' design <- "
#'   123
#'   213
#'   321
#' "
#'
#' # Alternatively, the same can be specified using a matrix directly
#' # design <- matrix(c(1, 2, 3, 2, 1, 3, 3, 2, 1), 3, 3, byrow = TRUE)
#'
#' p + guides(shape = guide_legend_base(design = design))
#'
#' # Empty cells can be created using `#`
#' design <- "
#'   #2#
#'   1#3
#' "
#'
#' # Alternatively:
#' # design <- matrix(c(NA, 1, 2, NA, NA, 3), nrow = 2)
#'
#' p + guides(shape = guide_legend_base(design = design))
guide_legend_base <- function(
  key = NULL,
  title = waiver(),
  theme = NULL,
  design = NULL,
  nrow = NULL,
  ncol = NULL,
  reverse = FALSE,
  override.aes = list(),
  position = NULL,
  direction = NULL,
  order = 0L
) {

  check_position(position, theta = FALSE, inside = TRUE, allow_null = TRUE)
  check_argmatch(direction, c("horizontal", "vertical"), allow_null = TRUE)
  check_bool(reverse)
  check_number_whole(nrow, min = 1.0, allow_null = TRUE)
  check_number_whole(ncol, min = 1.0, allow_null = TRUE)

  design <- validate_design(design, allow_null = TRUE)
  if (!is.null(design)) {
    ignored <- c(
      if (!is.null(nrow)) "nrow",
      if (!is.null(ncol)) "ncol"
    )
    if (length(ignored) > 0L) {
      cli::cli_warn(
        "The {.and {.arg {ignored}}} argument{?s} {?is/are} ignored \\
        when the {.arg design} argument is provided."
      )
    }
    nrow <- NULL
    ncol <- NULL
  }

  new_guide(
    key = key,
    title = title,
    design = design,
    nrow = nrow,
    ncol = ncol,
    override.aes = rename_aes(override.aes),
    reverse = reverse,
    theme = theme,
    position = position,
    direction = direction,
    order = order,
    super = GuideLegendBase
  )
}

# Class -------------------------------------------------------------------

#' @export
#' @rdname legendry_extensions
#' @format NULL
#' @usage NULL
GuideLegendBase <- ggproto(
  "GuideLegendBase", GuideLegend,

  params = new_params(
    override.aes = list(), reverse = FALSE,
    key = NULL, nrow = NULL, ncol = NULL, design = NULL
  ),

  extract_key = standard_extract_key,

  draw = function(self, theme, position = NULL, direction = NULL,
                  params = self$params) {
    # We ensure we know the 'byrow' setting from the beginning
    params$byrow <- params$theme$legend.byrow %||% theme$legend.byrow %||% FALSE
    ggproto_parent(Guide, self)$draw(
      theme = theme, position = position,
      direction = direction, params = params
    )
  },

  setup_params = function(params) {
    params$direction <- arg_match0(
      params$direction,
      c("horizontal", "vertical"),
      arg_nm = "direction"
    )
    params$n_breaks <- nrow(params$key)
    # We embed the design into the key as `.row`/`.col` columns
    params$key <- apply_design(
      params$key, params$design,
      params$nrow, params$ncol,
      params$direction, params$byrow
    )
    params
  },

  setup_elements = function(params, elements, theme) {

    theme <- theme + params$theme
    params$theme <- NULL

    text_position <- theme$legend.text.position %||% "right"
    elements$text <- setup_legend_text(theme, text_position)

    title_position <- theme$legend.title.position %||%
      switch(params$direction, vertical = "top", horizontal = "left")
    elements$title <- setup_legend_title(theme, title_position)

    elements <- Guide$setup_elements(params, elements, theme)
    elements[c("text_position", "title_position")] <-
      list(text_position, title_position)
    elements
  },

  build_decor = function(decor, grobs, elements, params) {
    decor <- render_legend_glyphs(
      index = seq_len(params$n_breaks),
      decor = decor, background = elements$key,
      default_size = c(elements$width_cm, elements$height_cm) * 10.0
    )
    decor <- decor[params$key$.index]
    names(decor) <- paste("key", params$key$.row, params$key$.col, sep = "-")
    decor
  },

  measure_grobs = function(grobs, params, elements) {

    # Get width of keys per column
    col <- params$key$.col
    widths <- map_dbl(grobs$decor, `[[`, i = "width")
    widths <- pmax(by_group(widths, col, max), elements$width_cm)

    # Weave in width of labels, depending on label position
    label_widths  <- by_group(width_cm(grobs$labels), col, max)
    widths <- switch(
      elements$text_position,
      left  = list(label_widths, widths),
      right = list(widths, label_widths),
      list(pmax(label_widths, widths))
    )
    widths <- vec_interleave(!!!widths, elements$spacing_x %||% 0.0)
    widths <- widths[-length(widths)] # Remove last spacer

    # Get height of keys per row
    row <- params$key$.row
    heights <- map_dbl(grobs$decor, `[[`, i = "height")
    heights <- pmax(by_group(heights, row, max), elements$height_cm)

    # Weave in height of labels, depending on label position
    label_heights <- by_group(height_cm(grobs$labels), row, max)
    heights <- switch(
      elements$text_position,
      top    = list(label_heights, heights),
      bottom = list(heights, label_heights),
      list(pmax(label_heights, heights))
    )
    heights <- vec_interleave(!!!heights, elements$spacing_y %||% 0.0)
    heights <- heights[-length(heights)] # Remove last spacer

    list(widths = widths, heights = heights)
  },

  arrange_layout = function(key, sizes, params, elements) {

    row <- key$.row
    col <- key$.col

    # Account for spacing in between keys
    key_row <- row * 2L - 1L
    key_col <- col * 2L - 1L

    # Resolve position of labels relative to keys
    position <- elements$text_position
    key_row <- key_row + switch(position, top  = row, bottom = row - 1L, 0L)
    lab_row <- key_row + switch(position, top  = -1L, bottom = 1L,       0L)
    key_col <- key_col + switch(position, left = col, right  = col - 1L, 0L)
    lab_col <- key_col + switch(position, left = -1L, right  = 1L,       0L)

    data_frame0(
      key_row = key_row,
      key_col = key_col,
      label_row = lab_row,
      label_col = lab_col
    )
  }
)

# Helpers -----------------------------------------------------------------

render_legend_glyphs <- function(index, decor, background, default_size) {
  lapply(index, function(i) {
    glyphs <- lapply(decor, function(dec) {
      data <- vec_slice(dec$data, i)
      if (!(data$.draw %||% TRUE)) {
        return(zeroGrob())
      }
      key <- dec$draw_key(data, dec$params, default_size)
      set_key_size(key, data$linewidth, data$size, default_size / 10.0)
    })
    gTree(
      width    = max(map_dbl(glyphs, get_width_attr),  0.0,  na.rm = TRUE),
      height   = max(map_dbl(glyphs, get_height_attr), 0.0,  na.rm = TRUE),
      children = inject(gList(background, !!!glyphs))
    )
  })
}

set_key_size <- function(key, lwd = NULL, size = NULL, default = NULL) {
  width  <- get_width_attr(key, default = NULL)
  height <- get_height_attr(key, default = NULL)
  if (!is.null(width) && !is.null(height)) {
    return(key)
  }
  if (!is.null(size) || !is.null(lwd)) {
    size <- size[1L] %||% 0.0 %|NA|% 0.0
    lwd  <- lwd[1L]  %||% 0.0 %|NA|% 0.0
    size <- (size + lwd) / 10.0
  } else {
    size <- NULL
  }
  attr(key, "width")  <- width  %||% size %||% default[1L]
  attr(key, "height") <- height %||% size %||% default[2L]
  key
}

apply_design <- function(
  key, design = NULL, nrow = NULL, ncol = NULL,
  direction = "horizontal", byrow = FALSE
) {
  n_breaks <- nrow(key)

  # Handle case where there is no design, à la ggplot2::guide_legend
  if (is.null(design)) {
    if (is.null(nrow) && is.null(ncol)) {
      if (direction == "horizontal") {
        nrow <- ceiling(n_breaks / 5.0)
      } else {
        ncol <- ceiling(n_breaks / 20.0)
      }
    }
    nrow <- nrow %||% ceiling(n_breaks / ncol)
    ncol <- ncol %||% ceiling(n_breaks / nrow)

    design <- matrix(
      seq_len(nrow * ncol),
      nrow = nrow, ncol = ncol,
      byrow = byrow
    )
  }

  max_design <- max(design, na.rm = TRUE)
  if (isTRUE(max_design < n_breaks)) {
    cause <- if (is.null(design)) {
      "{.arg nrow} * {.arg ncol} ({nrow * ncol}) is insufficient "
    } else {
      "The {.arg design} argument has insufficient levels "
    }
    cli::cli_warn(
      paste0(cause, "to accommodate the number of breaks ({n_breaks}).")
    )
  }

  key$.index <- seq_len(nrow(key))

  index <- match(design, key$.index)
  rows <- as.vector(row(design))
  cols <- as.vector(col(design))

  key <- vec_slice(key, index)
  key$.row <- rows
  key$.col <- cols
  vec_slice(key, !is.na(key$.index))
}

validate_design <- function(design = NULL, trim = TRUE, allow_null = TRUE) {
  if (is.null(design)) {
    if (allow_null) {
      return(NULL)
    }
    cli::cli_abort("The {.arg design} argument cannot be {.code NULL}.")
  }
  design <- parse_design_character(design)
  if (!is.matrix(design) && is.atomic(design)) {
    design <- as.matrix(design)
  }
  check_matrix(design)
  if (typeof(design) == "character") {
    design[design == "#"] <- NA
  }
  levels <- unique(sort(design))
  design <- matrix(
    match(design, levels),
    nrow = nrow(design),
    ncol = ncol(design)
  )

  if (trim) {
    filled <- !is.na(design)
    design <- design[rowSums(filled) > 0L, colSums(filled) > 0L, drop = FALSE]
  }

  if (!is.numeric(levels)) {
    attr(design, "levels") <- levels
  }

  design
}

parse_design_character <- function(design, call = caller_env()) {

  if (!is.character(design)) {
    return(design)
  }

  # Check is here to ensure scalar character
  check_string(design, allow_empty = FALSE, call = call)

  # Inspired by patchwork::as_areas()
  design <- trimws(strsplit(design, "\n", fixed = TRUE)[[1L]])
  design <- strsplit(design[nzchar(design)], "", fixed = TRUE)

  nrow <- length(design)
  ncol <- lengths(design)
  if (length(unique(ncol)) != 1L) {
    cli::cli_abort(
      "The {.arg design} argument must be rectangular.",
      call = call
    )
  }

  matrix(
    unlist(design, FALSE, FALSE),
    nrow = nrow, ncol = ncol[1L], byrow = TRUE
  )
}

test_that("guide_legend_base can parse different designs", {

  guide <- guide_legend_base()
  expect_null(guide$params$design)

  example_design <- matrix(1L:4L, nrow = 2L, byrow = TRUE)

  guide <- guide_legend_base(design = example_design)
  expect_identical(guide$params$design, example_design)

  guide <- guide_legend_base(design = "AB\nCD")
  expect_equal(guide$params$design, example_design, ignore_attr = TRUE)

  guide <- guide_legend_base(design = 1L:4L)
  expect_identical(guide$params$design, as.matrix(1L:4L))

  # Check warning is emitted when there is a conflict
  expect_warning(
    guide_legend_base(design = example_design, ncol = 2L),
    "`ncol` argument is ignored"
  )

  # Check error is thrown when design is invalid
  expect_error(
    guide_legend_base(design = "AB\nCDE"),
    "must be rectangular"
  )

})

test_that("design application is correct", {

  data <- data.frame(value = 1L:8L)

  # Default horizontal
  test <- apply_design(data, direction = "horizontal")
  expect_identical(
    test,
    data.frame(
      value = 1L:8L,
      .index = 1L:8L,
      .row = rep(1L:2L, 4L),
      .col = rep(1L:4L, each = 2L)
    )
  )

  # Default vertical
  test <- apply_design(data, direction = "vertical")
  expect_identical(
    test,
    data.frame(value = 1L:8L, .index = 1L:8L, .row = 1L:8L, .col = rep(1L, 8L))
  )

  # Vertical with fixed columns
  test <- apply_design(data, ncol = 2L, direction = "vertical")
  expect_identical(
    test,
    data.frame(
      value = 1L:8L,
      .index = 1L:8L,
      .row = rep(1L:4L, 2L),
      .col = rep(1L:2L, each = 4L)
    )
  )

  # Custom design
  design <- matrix(c(1L:3L, 8L, NA, 4L, 7L:5L), nrow = 3L)
  test <- apply_design(data, design = design)
  expect_identical(
    test,
    data.frame(
      value  = c(1L, 2L, 3L, 8L, 4L, 7L, 6L, 5L),
      .index = c(1L, 2L, 3L, 8L, 4L, 7L, 6L, 5L),
      .row   = c(1L, 2L, 3L, 1L, 3L, 1L, 2L, 3L),
      .col   = rep(1L:3L, c(3L, 2L, 3L))
    )
  )

  # Warning about flawed design
  design <- matrix(c(1L:3L, NA, NA, 4L, 7L:5L), nrow = 3L)
  expect_warning(
    apply_design(data, design = design),
    "insufficient levels"
  )

  # Warning about flawed ncol/nrow
  expect_warning(
    apply_design(data, nrow = 2L, ncol = 2L),
    "insufficient levels"
  )
})

test_that("guide_legend_base can draw a custom design", {

  design <- "1#7\n2#6\n345"

  df <- data.frame(x = LETTERS[1L:7L])

  p <- ggplot(df, aes(x, x, fill = x)) +
    geom_tile()

  vdiffr::expect_doppelganger(
    "standard legend design",
    p + guides(fill = guide_legend_base())
  )

  vdiffr::expect_doppelganger(
    "custom legend design",
    p + guides(fill = guide_legend_base(design = design))
  )
})

# Bracket types -----------------------------------------------------------

#' Bracket options
#'
#' These functions construct various sorts of brackets. They construct a matrix
#' that can be supplied as the `bracket` argument in [`primitive_bracket()`].
#'
#' @param n An `integer(1)` number of points to use for the bracket.
#' @param curvature A `numeric(1)` that controls the curliness of the bracket.
#'   More precisely, it is used to construct the sequence
#'   `seq(-curvature, curvature, length.out = n)` over which the logistic or
#'   arctangent functions is evaluated.
#' @param angle A `numeric(1)`: the angle in degrees for which a circle piece
#'   is drawn. For `bracket_curvy()`, an angle between 180 and 270.
#'
#' @return A `<matrix[n, 2]>` with coordinates for points on the brackets.
#' @name bracket_options
#'
#' @details
#' When designing custom bracket shapes, the expectation is both columns are
#' are a number between 0 and 1. The first column follows the direction of the
#' guide whereas the second column is orthogonal to that direction.
#'
#' @examples
#' plot(bracket_sigmoid(), type = 'l')
NULL

#' @export
#' @describeIn bracket_options A simple line as bracket. Has `n = 2` points.
bracket_line <- function() {
  cbind(c(0.0, 1.0), 0.5)
}

#' @export
#' @describeIn bracket_options A square bracket. Has `n = 4` points.
bracket_square <- function() {
  cbind(c(1.0, 1.0, 0.0, 0.0), c(1.0, 0.0, 0.0, 1.0))
}

#' @export
#' @describeIn bracket_options A chevron (V-shape) that makes a bracket. Has
#'   `n = 3` points.
bracket_chevron <- function() {
  cbind(c(0.0, 0.5, 1.0), c(1.0, 0.0, 1.0))
}

#' @export
#' @describeIn bracket_options One circular arc that makes a bracket.
bracket_round <- function(angle = 180.0, n = 100L) {
  t <- deg2rad(seq(0.0, angle, length.out = n) - 0.5 * angle + 180.0)
  y <- rescale(cos(t))
  x <- rescale(sin(t))
  cbind(x, y)
}

#' @export
#' @describeIn bracket_options Two sigmoid curves stacked on top of one another
#'   to form a bracket.
bracket_sigmoid <- function(curvature = 10.0, n = 100L) {
  half_n <- round(n / 2L)
  y <- c(seq(1.0, 0.0, length.out = half_n), seq(0.0, 1.0, length.out = half_n))
  x <- 1.0 / (1.0 + exp(-seq(-curvature, curvature, length.out = half_n)))
  x <- rescale(x)
  x <- c(x, 1.0 + x) / 2.0
  cbind(x, y)
}

#' @export
#' @describeIn bracket_options Two arctangent curves stacked on top of one
#'   another to form a bracket.
bracket_atan <- function(curvature = 5.0, n = 100L) {
  half_n <- round(n / 2L)
  y <- c(seq(1.0, 0.0, length.out = half_n), seq(0.0, 1.0, length.out = half_n))
  x <- atan(seq(-curvature, curvature, length.out = half_n))
  x <- rescale(x)
  x <- c(x, 1.0 + x) / 2.0
  cbind(x, y)
}

#' @export
#' @describeIn bracket_options Four circular arcs that make a bracket.
bracket_curvy <- function(angle = 225.0, n = 100L) {
  quarter_n <- round(n / 4L)
  angle <- pmax(pmin(angle, 270.0), 180.0) / 180.0

  t <- c(
    seq(0.5, angle, length.out = quarter_n),
    seq(angle + 1.0, 1.5, length.out = quarter_n)
  ) * pi
  y <- cos(t) + rep(c(0.0, cos(angle * pi) * 2.0), each = quarter_n)
  x <- sin(t) + rep(c(0.0, sin(angle * pi) * 2.0), each = quarter_n)
  y <- rescale(y)
  x <- rescale(x, to = c(0.5, 1.0))
  x <- c(x, 1.0 - rev(x))
  y <- c(y, rev(y))
  cbind(x, y)
}

# Helpers -----------------------------------------------------------------

resolve_bracket <- function(x, arg = caller_arg(x), call = caller_env()) {
  if (is.character(x)) {
    x <- paste0("bracket_", x)
    fun <- find_global(x, env = global_env(), mode = "function")
    if (!is.function(fun)) {
      cli::cli_abort("Cannot find function: {.fn {x}}.", call = call)
    }
    x <- fun
  }
  if (is.function(x)) {
    msg <- "{.arg {arg}} must return {type}, not {obj_type_friendly(x)}."
    x <- x()
  } else {
    msg <- "{.arg {arg}} must be {type}, not {obj_type_friendly(x)}"
  }
  if (is.matrix(x) && ncol(x) %in% c(2L, 3L) && nrow(x) > 1L) {
    return(x)
  }
  type <- as_cli("a {.cls matrix}")
  if (!ncol(x) %in% c(2L, 3L)) {
    type <- as_cli("a {.cls matrix} with 2 or 3 columns")
    msg <- c(msg, "The provided {.arg {arg}} has {ncol(x)} column{?s}.")
  }
  if (nrow(x) < 2L) {
    type <- as_cli("a {.cls matrix} with 2 or more rows")
    msg <- c(msg, "The provided {.arg {arg}} has {nrow(x)} row{?s}.")
  }
  cli::cli_abort(msg, call = call)
}

transform_bracket <- function(bracket, position, coord, panel_params) {
  if (is_empty(bracket)) {
    return(bracket)
  }

  bbox  <- panel_params$bbox %||% list(x = c(0.0, 1.0), y = c(0.0, 1.0))

  if (is_theta(position)) {
    bbox  <- panel_params$bbox %||% list(x = c(0.0, 1.0), y = c(0.0, 1.0))
    range <- panel_params$r.range
    if (position == "theta") {
      other <-
        rescale(1.0 + bracket$offset * 0.1, to = range, from = c(0.0, 1.0))
    } else {
      other <-
        rescale(0.0 - bracket$offset * 0.1, to = range, from = c(0.0, 1.0))
    }
  } else {
    other <- switch(position, top = , right = -Inf, Inf)
  }

  bracket$x <- bracket$x %||% other
  bracket$y <- bracket$y %||% other
  bracket   <- coord_munch(coord, bracket, panel_params)

  if (!is_theta(position)) {
    return(bracket)
  }
  donut <- panel_params$inner_radius
  r <- donut[as.numeric(position == "theta") + 1L]
  bracket <- polar_xy(bracket, r, bracket$theta, bbox)

  if (abs(diff(donut)) < 1e-2) {
    return(bracket)
  }
  if (position == "theta") {
    bracket$offset <-
      (rescale(bracket$r, to = c(0.0, 1.0), from = donut) - 1.0) * 10.0
  } else {
    bracket$theta <- bracket$theta + pi
    bracket$offset <-
      abs(rescale(bracket$r, to = c(0.0, 1.0), from = donut) * 10.0)
  }
  bracket
}

test_that("every bracket function returns a matrix", {

  test <- bracket_line()
  expect_true(is.matrix(test))
  expect_identical(dim(test), c(2L, 2L))

  test <- bracket_square()
  expect_true(is.matrix(test))
  expect_identical(dim(test), c(4L, 2L))

  test <- bracket_chevron()
  expect_true(is.matrix(test))
  expect_identical(dim(test), c(3L, 2L))

  test <- bracket_round()
  expect_true(is.matrix(test))
  expect_identical(dim(test), c(100L, 2L))

  test <- bracket_sigmoid()
  expect_true(is.matrix(test))
  expect_identical(dim(test), c(100L, 2L))

  test <- bracket_atan()
  expect_true(is.matrix(test))
  expect_identical(dim(test), c(100L, 2L))

  test <- bracket_curvy()
  expect_true(is.matrix(test))
  expect_identical(dim(test), c(100L, 2L))

})

test_that("resolve_bracket throws appropriate errors", {
  expect_error(
    resolve_bracket("foobar"),
    "Cannot find function"
  )
  expect_error(
    resolve_bracket(mtcars),
    "must be a"
  )
  expect_error(
    resolve_bracket(matrix(NA, 2L, 1L)),
    "2 or 3 columns"
  )
  expect_error(
    resolve_bracket(matrix(NA, 0L, 2L)),
    "2 or more rows"
  )
})

test_that("transform_bracket works for theta positions", {

  expect_null(transform_bracket(NULL))

  bracket <- data.frame(
    x = c(1.0, 2.0),
    offset = 0.0
  )
  panel <- list(
    bbox = list(x = c(0.0, 1.0), y = c(0.0, 1.0)),
    inner_radius = c(0.2, 0.4),
    r.range = c(0.0, 1.0),
    theta.range = c(0.0, 4.0),
    arc = c(0.0, 2.0 * pi),
    direction = 1.0
  )

  coord <- coord_radial()

  test <- transform_bracket(bracket, "theta", coord, panel)
  expect_gte(nrow(test), 40L)
  expect_equal(test$r[1L], 0.4)

  test <- transform_bracket(bracket, "theta.sec", coord, panel)
  expect_lte(nrow(test), 10L)
  expect_equal(test$r[1L], 0.2)
})

# Constructor -------------------------------------------------------------

#' Guide primitive: line
#'
#' This function constructs a line [guide primitive][guide-primitives].
#'
#' @param cap A method to cap the axes. One of the following:
#'   * A `<character[1]>` with one of the following:
#'     * `"none"` to perform no capping.
#'     * `"both"` to cap the line at both ends at the most extreme breaks.
#'     * `"upper"` to cap the line at the upper extreme break.
#'     * `"lower"` to cap the line at the lower extreme break.
#'   * A `<logical>[1]`, where `TRUE` is equivalent to `"both"` and `FALSE`
#'   is equivalent to `"none"` in the options above.
#'   * A sorted `<numeric>[2n]` with an even number of members. The lines
#'   will be drawn between every odd-even pair.
#'   * A `<function>` that takes the scale's breaks as the first argument, the
#'   scale's limits as the second argument and returns a `<numeric>[2n]` as
#'   described above.
#' @inheritParams primitive_labels
#'
#' @return A `PrimitiveLine` primitive guide that can be used inside other
#'   guides.
#' @export
#' @family primitives
#'
#' @details
#' ## Styling options
#'
#' Below are the [theme][ggplot2::theme] options that determine the styling of
#' this guide, which may differ depending on whether the guide is used in
#' an axis or in a legend context.
#'
#' | **Theme setting** | **Context** | **Type** | **Description** |
#' | ----------------- | ----------- | -------- | --------------- |
#' | `axis.line.{x/y}.{position}` | Axis | [`element_line()`] | The axis line. |
#' | `legend.axis.line` | Legend | [`element_line()`] | The axis line. |
#'
#' There are no other styling options.
#' The context-agnostic alternative to using `theme()` is to use
#' [`theme_guide()`]:
#'
#' ```r
#' primitive_line(theme = theme_guide(
#'   line = element_line()
#' ))
#' ```
#'
#' @examples
#' # A standard plot
#' p <- ggplot(mpg, aes(displ, hwy)) +
#'   geom_point() +
#'   theme(axis.line = element_line())
#'
#' # Adding as secondary guides
#' p + guides(
#'   x.sec = primitive_line(),
#'   y.sec = primitive_line(cap = "both")
#' )
primitive_line <- function(key = NULL, cap = "none", theme = NULL,
                           position = waiver()) {
  new_guide(
    key = key,
    cap = check_cap_arg(cap),
    theme = theme,
    position = position,
    available_aes = c("any", "x", "y", "r", "theta"),
    super = PrimitiveLine
  )
}

# Class -------------------------------------------------------------------

#' @export
#' @rdname legendry_extensions
#' @format NULL
#' @usage NULL
PrimitiveLine <- ggproto(
  "PrimitiveLine", Guide,

  params = new_params(cap = c(-Inf, Inf), key = NULL),

  elements = list(
    position = list(line = "axis.line"),
    legend   = list(line = "legend.axis.line")
  ),

  hashables = exprs(decor[[aesthetic]], get0("position")),

  extract_key = standard_extract_key,

  extract_decor = function(scale, aesthetic, position, cap, key, ...) {
    limits <- scale$continuous_range %||% scale$get_limits()
    if (is.function(cap)) {
      if (length(fn_fmls_names(cap)) == 1L) {
        cap <- cap(key[[aesthetic]])
      } else {
        cap <- cap(key[[aesthetic]], limits)
      }
    }

    decor <- data_frame(!!aesthetic := cap)
    if (aesthetic %in% c("x", "y")) {
      opposite <- setdiff(c("x", "y"), aesthetic)
      position <-
        switch(position, theta = "left", theta.sec = "right", position)
      value <- if (position %in% c("top", "right")) -Inf else Inf
      decor[[opposite]] <- value
    } else {
      value <- scale$oob(decor[[aesthetic]], range = limits)
      decor[[aesthetic]] <- value
    }

    group <- seq_len(ceiling(nrow(decor) / 2L))
    decor$group <- rep(group, each = 2L, length.out = nrow(decor))
    decor
  },

  extract_params = primitive_extract_params,

  transform = function(self, params, coord, panel_params) {
    params$decor <- coord_munch(coord, params$decor, panel_params)
    params
  },

  setup_elements = primitive_setup_elements,

  build_decor = function(decor, grobs, elements, params) {
    if (is_empty(decor)) {
      return(zeroGrob())
    }
    x <- y <- NULL
    if ("theta" %in% names(decor)) {
      theta  <- decor$theta + as.numeric(params$position == "theta.sec") * pi
      offset <- elements$offset
      x <- unit(decor$x, "npc") + unit(sin(theta) * offset, "cm")
      y <- unit(decor$y, "npc") + unit(cos(theta) * offset, "cm")
    }
    if (!all(c("x", "y") %in% names(decor))) {
      value <- guide_rescale(decor[[params$aesthetic]], params$limits)
      if (params$position %in% c("left", "right")) {
        y <- unit(value, "npc")
        x <- as.numeric(params$position == "left") |>
          rep_len(length.out = length(y)) |>
          unit("npc")
      } else {
        x <- unit(value, "npc")
        y <- as.numeric(params$position == "bottom") |>
          rep_len(length.out = length(x)) |>
          unit("npc")
      }
    }
    element_grob(
      elements$line,
      x = x %||% unit(decor$x, "npc"),
      y = y %||% unit(decor$y, "npc"),
      id.lengths = vctrs::vec_unrep(decor$group)$times
    )
  },

  draw = function(self, theme, position = NULL, direction = NULL,
                  params = self$params) {

    params <- replace_null(params, position = position, direction = direction)

    elems <- self$setup_elements(params, self$elements, theme)
    decor <- self$build_decor(params$decor, grobs, elems, params)

    primitive_grob(
      grob = decor,
      size = unit(0.0, "cm"),
      position = params$position,
      name = "line"
    )
  }
)

# Helpers -----------------------------------------------------------------

check_cap_arg <- function(cap, call = caller_env()) {
  if (is.logical(cap)) {
    check_bool(cap)
    cap <- if (cap) "both" else "none"
  }
  if (is.character(cap)) {
    cap <- switch(
      cap,
      none  =
        c(-Inf, Inf),
      both  =
        function(breaks, limits) range(breaks, na.rm = TRUE),
      upper =
        function(breaks, limits) c(limits[1L], max(breaks, na.rm = TRUE)),
      lower =
        function(breaks, limits) c(min(breaks, na.rm = TRUE), limits[2L]),
      arg_match0(cap, c("none", "both", "upper", "lower"))
    )
  }
  if (!is.function(cap) && !vctrs::vec_is(cap)) {
    stop_input_type(cap, what = as_cli("a {.cls function} or {.cls vector}"))
  }
  cap
}

test_that("check_cap_arg wors", {

  expect_type(check_cap_arg(TRUE), "closure")
  expect_identical(check_cap_arg(FALSE), c(-Inf, Inf))
  expect_type(check_cap_arg("upper"), "closure")
  expect_snapshot_error(check_cap_arg(NA))
  expect_error(
    check_cap_arg(environment()),
    "not an environment"
  )

})

test_that("primitive_line works as axis line", {

  base <- ggplot(mpg, aes(displ, hwy)) +
    geom_blank() +
    theme_test() +
    theme(
      panel.background = element_rect(fill = NA, colour = "grey80"),
      panel.grid.major = element_line(colour = "grey90"),
      panel.border = element_blank(),
      axis.line = element_line()
    )

  p <- base +
    guides(
      x     = primitive_line(),
      x.sec = primitive_line(cap = "both"),
      y     = primitive_line(cap = c(15.0, 25.0, 35.0, 40.0)),
      y.sec = primitive_line(cap = function(breaks, limits) {
        c(min(breaks, na.rm = TRUE), limits[2L])
      })
    )

  vdiffr::expect_doppelganger("primitive_line cartesian", p)

  p <- base +
    coord_radial(start = 0.25 * pi, end = 1.75 * pi, inner.radius = 0.5) +
    guides(
      theta.sec = primitive_line(),
      theta     = primitive_line(cap = "both"),
      r         = primitive_line(cap = c(15.0, 25.0, 35.0, 40.0)),
      r.sec     = primitive_line(cap = function(breaks, limits) {
        c(min(breaks, na.rm = TRUE), limits[2L])
      })
    )

  vdiffr::expect_doppelganger("primitive_line radial", p)

})

test_that("primitive_line works as legend", {

  p <- ggplot(mtcars) +
    aes(
      x = disp, y = mpg,
      colour = hp,
      fill = hp
    ) +
    geom_point() +
    guides(
      colour = primitive_line(),
      fill = primitive_line(position = "bottom")
    )


  vdiffr::expect_doppelganger("primitive_line legend", p)

})

# Constructor -------------------------------------------------------------

#' Grouped legend
#'
#' This legend resembles `ggplot2::guide_legend()`, but has the ability to
#' keep groups in blocks with their own titles.
#'
#' @param key A [group key][key_group] specification. Defaults to
#'   `key_group_split()` to split labels to find groups.
#' @param nrow,ncol A positive `<integer[1]>` setting the desired dimensions of
#'   the legend layout. Either `nrow` or `ncol` can be set, but not both,
#' @inheritParams common_parameters
#'
#' @details
#' ## Styling options
#'
#' Below are the [theme][ggplot2::theme] options that determine the styling of
#' this guide.
#'
#' | **Theme setting** | **Type** | **Description** |
#' | ----------------- | -------- | --------------- |
#' | `legendry.legend.subtitle` | [`element_text()`] | Title of groups in the legend. |
#' | `legendry.legend.subtitle.position` | `<character[1]>` | One of `"top"`, `"right"`, `"bottom"` or `"left"`. |
#' | `legendry.group.spacing` | [`unit()`] | Spacing in between groups of keys. |
#' | `legend.background`| [`element_rect()`] | Background of the legend. |
#' | `legend.margin` | [`margin()`] | Padding around the legend. |
#' | `legend.text` | [`element_text()`] | Labels displayed next to keys. |
#' | `legend.text.position` | `<character[1]>` | One of `"top"`, `"right"`, `"bottom"` or `"left"`. |
#' | `legend.title` | [`element_text()`] | Title of the legend. |
#' | `legend.title.position` | `<character[1]>` | One of `"top"`, `"right"`, `"bottom"` or `"left"`. |
#' | `legend.key` | [`element_rect()`] | Background of the key areas. |
#' | `legend.key.height` | [`unit()`] | Height of keys. |
#' | `legend.key.width` | [`unit()`] | Width of keys. |
#' | `legend.key.justification` | `<numeric[2]>` | Justification for placing legend keys in excess space. |
#' | `legend.key.spacing.x` | [`unit()`] | Horizontal spacing between keys. |
#' | `legend.key.spacing.y` | [`unit()`] | Vertical spacing between keys. Taken literally. |
#' | `legend.byrow` | `<logical[1]>` | Row-order key filling (`TRUE`) or column-order (`FALSE`) |
#'
#' The context-agnostic alternative to using `theme()` is to use
#' [`theme_guide()`]:
#'
#' ```r
#' guide_legend_group(theme = theme_guide(
#'   subtitle = element_text(),
#'   subtitle.position = "top",
#'   group.spacing = unit(5, "mm"),
#'   text = element_text(),
#'   text.position = "right",
#'   title = element_text(),
#'   title.position = "top",
#'   key = element_rect(),
#'   key.height = unit(5, "mm"),
#'   key.width = unit(5, "mm"),
#'   key.justification = c(0.5, 0.5),
#'   key.spacing.x = unit(5, "mm"),
#'   key.spacing.y = unit(5, "mm"),
#'   byrow = TRUE,
#'   margin = margin(5),
#'   background = element_rect(),
#' ))
#' ```
#'
#' @return A `<GuideLegend>` object.
#' @export
#' @family standalone guides
#' @family legend guides
#'
#' @examples
#' # Standard plot for selection of `msleep`
#' df <- msleep[c(9, 28, 11, 5, 34, 54, 64, 24, 53), ]
#'
#' p <- ggplot(df) +
#'   aes(bodywt, awake, colour = paste(order, name)) +
#'   geom_point()
#'
#' # By default, groups are inferred from the name
#' p + guides(colour = "legend_group")
#'
#' # You can also use a look-up table for groups
#' # The lookup table can be more expansive than just the data:
#' # We're using the full 'msleep' data here instead of the subset
#' lut <- key_group_lut(msleep$name, msleep$order)
#'
#' p + aes(colour = name) +
#'   guides(colour = guide_legend_group(key = lut))
#'
#' # `nrow` and `ncol` apply within groups
#' p + guides(colour = guide_legend_group(nrow = 1))
#'
#' # Groups are arranged according to `direction`
#' p + guides(colour = guide_legend_group(ncol = 1, direction = "horizontal")) +
#'   theme(legend.title.position = "top")
#'
#' # Customising the group titles
#' p + guides(colour = "legend_group") +
#'   theme(
#'     legendry.legend.subtitle.position = "left",
#'     legendry.legend.subtitle = element_text(
#'       hjust = 1, vjust = 1, size = rel(0.9),
#'       margin = margin(t = 5.5, r = 5.5)
#'     )
#'   )
#'
#' # Changing the spacing between groups
#' p + guides(colour = "legend_group") +
#'   theme(legendry.group.spacing = unit(0, "cm"))
guide_legend_group <- function(
  key = "group_split",
  title = waiver(),
  override.aes = list(),
  nrow = NULL,
  ncol = NULL,
  theme = NULL,
  position = NULL,
  direction = NULL,
  order = 0L
) {

  check_position(position, theta = FALSE, inside = TRUE, allow_null = TRUE)
  check_argmatch(direction, c("horizontal", "vertical"), allow_null = TRUE)
  check_number_whole(nrow, min = 1.0, allow_null = TRUE)
  check_number_whole(ncol, min = 1.0, allow_null = TRUE)
  check_exclusive(nrow, ncol)

  new_guide(
    key = key,
    title = title,
    theme = theme,
    override.aes = rename_aes(override.aes),
    nrow = nrow,
    ncol = ncol,
    order = order,
    available_aes = "any",
    name = "legend_group",
    direction = direction,
    position = position,
    super = GuideLegendGroup
  )
}

# Class -------------------------------------------------------------------

#' @export
#' @rdname legendry_extensions
#' @format NULL
#' @usage NULL
GuideLegendGroup <- ggproto(
  "GuideLegendGroup", GuideLegendBase,

  elements = list2(
    !!!GuideLegendBase$elements,
    subtitle_spacing = "legendry.group.spacing",
    subtitle = "legendry.legend.subtitle",
    subtitle_position = "legendry.legend.subtitle.position"
  ),

  # Typical legend + group column
  hashables = exprs(title, key$.label, name, key$.group),

  merge = function(self, params, new_guide, new_params) {
    # The merge hashables guarantee group equality
    new_params$key$.group <- NULL
    ggproto_parent(GuideLegendBase, self)$merge(params, new_guide, new_params)
  },

  setup_params = function(params) {
    params$direction <- direction <- arg_match0(
      params$direction,
      c("horizontal", "vertical"),
      arg_nm = "direction"
    )
    params$n_breaks <- nrow(params$key)
    params$groups <-
      group_design(params$key, params$nrow, params$ncol, direction)
    params$key <-
      apply_group_design(params$key, params$groups, direction, params$byrow)
    params
  },

  setup_elements = function(params, elements, theme) {
    theme <- theme + params$theme
    params$theme <- NULL

    subtitle_position <- theme$legendry.legend.subtitle.position %||% "top"
    elements$subtitle <-
      setup_legend_title(theme, subtitle_position, element = elements$subtitle)

    elements <- GuideLegendBase$setup_elements(params, elements, theme)
    elements$subtitle_position <- subtitle_position
    elements
  },

  override_elements = function(params, elements, theme) {
    elements <- GuideLegendBase$override_elements(params, elements, theme)
    elements$subtitle_spacing <- convertUnit(
      elements$subtitle_spacing %||% unit(0.0, "cm"),
      "cm", valueOnly = TRUE
    )
    elements
  },

  build_title = function(label, elements, params) {
    main <- Guide$build_title(label, elements, params)
    subtitles <- lapply(
      params$groups$key,
      function(lab) {
        sub <- element_grob(
          elements$subtitle, label = lab,
          margin_x = TRUE, margin_y = TRUE
        )
        sub$name <- grobName(sub, "guide.subtitle")
        sub
      }
    )
    list(main = main, subtitles = subtitles)
  },

  measure_grobs = function(grobs, params, elements) {
    measures <- GuideLegendBase$measure_grobs(grobs, params, elements)
    measures$sub_widths  <- width_cm( grobs$title$subtitles)
    measures$sub_heights <- height_cm(grobs$title$subtitles)
    measures
  },

  arrange_layout = function(key, sizes, params, elements) {

    layout <- GuideLegendBase$arrange_layout(key, sizes, params, elements)

    group <- as.integer(key$.group)

    key_row <- layout$key_row
    key_col <- layout$key_col
    lab_row <- layout$label_row
    lab_col <- layout$label_col

    t <- by_group(pmin(key_row, lab_row), group, min)
    b <- by_group(pmax(key_row, lab_row), group, max)
    l <- by_group(pmin(key_col, lab_col), group, min)
    r <- by_group(pmax(key_col, lab_col), group, max)

    widths     <- sizes$widths
    heights    <- sizes$heights
    sub_width  <- by_group(sizes$sub_widths,  l, max)
    sub_height <- by_group(sizes$sub_heights, t, max)
    spacing    <- elements$subtitle_spacing

    position <- elements$subtitle_position
    aligned_top  <- all(t == t[1L])
    aligned_left <- all(l == l[1L])
    if (position != "top" && aligned_top) {
      b[] <- max(b) # align bottom
    }
    if (position != "left" && aligned_left) {
      r[] <- max(r) # align right
    }

    subtitle_cell  <- switch(position, top = t, left = l, bottom = b, right = r)
    cells  <- unique(subtitle_cell)
    subtitle_cell <- subtitle_cell + match(subtitle_cell, cells)



    topleft <- position %in% c("top", "left")
    if (topleft) {
      subtitle_cell <- subtitle_cell - 1L
      spacing_index <- subtitle_cell - 1L
    } else {
      spacing_index <- subtitle_cell + 1L
    }

    just <- get_just(elements$subtitle)
    insert <- if (topleft) insert_before else insert_after

    row_add <- col_add <- 0L
    if (position %in% c("top", "bottom")) {
      row_add <- findInterval(key_row, cells, left.open = !topleft)
      t <- b <- subtitle_cell
      heights <- insert(heights, cells, sub_height)
      heights <- set_within(heights, spacing_index, spacing)
      end <- unique(r)
      start <- unique(l)

      if (aligned_top) {
        widths <- set_within(widths, start - 1L, spacing)
      }
      widths <- insert_spillover(
        widths, start, end,
        sub_width, position, just$hjust
      )

      index <- reeindex(length(widths), start, end)
      key_col <- index[key_col]
      lab_col <- index[lab_col]
      l <- index[l] - 1L
      r <- index[r] + 1L
    } else {
      col_add <- findInterval(key_col, cells, left.open = !topleft)
      l <- r <- subtitle_cell
      widths <- insert(widths, cells, sub_width)
      widths <- set_within(widths, spacing_index, spacing)

      start <- unique(t)
      end   <- unique(b)

      if (aligned_left) {
        heights <- set_within(heights, start - 1L, spacing)
      }
      heights <- insert_spillover(
        heights, start, end,
        sub_height, position, just$vjust
      )

      index <- reeindex(length(heights), start, end)
      key_row <- index[key_row]
      lab_row <- index[lab_row]
      t <- index[t] - 1L
      b <- index[b] + 1L
    }

    key_row <- key_row + row_add
    lab_row <- lab_row + row_add
    key_col <- key_col + col_add
    lab_col <- lab_col + col_add

    groups <- params$groups
    groups[, c("t", "r", "b", "l")] <- list(t, r, b, l)

    df <- cbind(key, key_row, key_col, label_row = lab_row, label_col = lab_col)
    list(layout = df, heights = heights, widths = widths, groups = groups)
  },

  assemble_drawing = function(self, grobs, layout, sizes, params, elements) {
    widths <- unit(layout$widths, "cm")
    if (isTRUE(elements$stretch_x)) {
      widths[unique0(layout$layout$key_col)] <- elements$key_width
    }
    heights <- unit(layout$heights, "cm")
    if (isTRUE(elements$stretch_y)) {
      heights[unique0(layout$layout$key_row)] <- elements$key_height
    }
    groups <- layout$groups
    layout <- layout$layout
    gt <- gtable(widths = widths, heights = heights)

    if (!is_zero(grobs$decor)) {
      gt <- gtable_add_grob(
        gt, grobs$decor, name = names(grobs$decor),
        t = layout$key_row, l = layout$key_col,
        clip = "off"
      )
    }
    if (!is_zero(grobs$labels)) {
      gt <- gtable_add_grob(
        gt, grobs$labels, name = names(grobs$labels) %||%
          paste("label", layout$label_row, layout$label_col, sep = "-"),
        t = layout$label_row, l = layout$label_col,
        clip = "off"
      )
    }
    if (!is_zero(grobs$title$subtitles)) {
      gt <- gtable_add_grob(
        gt, grobs$title$subtitles, name = names(grobs$title$subtitles) %||%
          paste0("subtitle-", seq_along(grobs$title$subtitles)),
        t = groups$t, r = groups$r, b = groups$b, l = groups$l, clip = "off"
      )
    }
    gt <- self$add_title(gt, grobs$title$main, elements$title_position,
                         get_just(elements$title))
    gt <- gtable_add_padding(gt, unit(elements$padding, "cm"))
    if (!is_zero(elements$background)) {
      gt <- gtable_add_grob(
        gt, elements$background, t = 1L, r = -1L, b = -1L, l = 1L, z = -Inf,
        name = "background", clip = "off"
      )
    }
    gt
  }
)

# Helpers -----------------------------------------------------------------

group_design <- function(key, nrow = NULL, ncol = NULL,
                         direction = "vertical") {
  groups <- vec_count(key$.group)
  groups <- vec_slice(groups, order(match(groups$key, key$.group)))
  n <- groups$count

  if (is.null(nrow) && is.null(ncol)) {
    if (direction == "horizontal") {
      nrow <- ceiling(n / 5.0)
    } else {
      ncol <- ceiling(n / 20.0)
    }
  }

  groups$nrow <- nrow %||% ceiling(n / ncol)
  groups$ncol <- ceiling(n / groups$nrow)
  groups
}

apply_group_design <- function(
  key, groups, direction = "vertical", byrow = FALSE
) {

  nrow <- rep(groups$nrow, groups$count)
  ncol <- rep(groups$ncol, groups$count)

  index <- seq_len(sum(groups$count))
  sub_index <- vec_ave(index, key$.group, seq_along)

  if (byrow) {
    row <- ceiling(sub_index / ncol)
    col <- (sub_index - 1L) %% ncol + 1L
  } else {
    row <- (sub_index - 1L) %% nrow + 1L
    col <- ceiling(sub_index / nrow)
  }

  if (direction == "vertical") {
    row <- row + rep(cumsum(c(0L, groups$nrow[-nrow(groups)])), groups$count)
  } else {
    col <- col + rep(cumsum(c(0L, groups$ncol[-nrow(groups)])), groups$count)
  }

  key$.index <- index
  key$.row   <- row
  key$.col   <- col
  key
}

set_within <- function(x, i, value) {
  i <- i[i > 0L & i <= length(x)]
  x[i] <- value
  x
}

insert_spillover <- function(size, start, end, extra, position, just = NULL) {
  cumsize <- cumsum(size)
  extra_size <- pmax(0.0, extra - (cumsize[end] - c(0.0, cumsize)[start]))
  just <- (just %||% 0.5) * c(1.0, -1.0) + c(0.0, 1.0)

  if (position %in% c("left", "right")) {
    just <- rev(just)
  }

  size <- insert_before(size, start, extra_size * just[1L])
  insert_after(size, end + match(start, start), extra_size * just[2L])
}

reeindex <- function(n, start, end) {
  index <- seq_len(n)
  new_index <- insert_before(index, start, NA)
  new_index <- insert_after(new_index, end + match(start, start), NA)
  match(index, new_index)
}

test_that("guide_legend_group can merge two legends", {

  df <- data.frame(x = 1L:3L, f = c("A,a", "A,b", "Z,x"))
  p <- ggplot(df, aes(x, x, color = f, shape = f)) +
    geom_point() +
    guides(
      color = "legend_group",
      shape = "legend_group"
    )
  gd <- get_guide_data(p, "color")
  expect_in(
    c("colour", "shape", ".group"),
    names(gd)
  )
  expect_identical(nrow(gd), 3L)
  expect_identical(gd$.group, factor(c("A", "A", "Z"), levels = c("A", "Z")))
})

test_that("guide_legend_group works in both direction with all subtitles", {

  df <- msleep[c(9L, 28L, 11L, 5L, 34L, 54L, 24L, 53L), ]

  base <- ggplot(df, aes(bodywt, awake)) +
    geom_point(aes(colour = paste0(order, ".", name))) +
    scale_colour_discrete(
      name = "Animals",
      guide = guide_legend_group(ncol = 2L)
    ) +
    theme_test() +
    theme(
      legend.key = element_rect(colour = NA, fill = "grey90"),
      legend.title.position = "top"
    )

  vdiffr::expect_doppelganger(
    "right-toptitle",
    base + theme(
      legend.position = "right",
      legendry.legend.subtitle.position = "top"
    )
  )

  vdiffr::expect_doppelganger(
    "right-lefttitle",
    base + theme(
      legend.position = "right",
      legendry.legend.subtitle.position = "left"
    )
  )

  vdiffr::expect_doppelganger(
    "right-righttitle",
    base + theme(
      legend.position = "right",
      legendry.legend.subtitle.position = "right"
    )
  )

  vdiffr::expect_doppelganger(
    "right-bottomtitle",
    base + theme(
      legend.position = "right",
      legendry.legend.subtitle.position = "bottom"
    )
  )

  vdiffr::expect_doppelganger(
    "bottom-toptitle",
    base + theme(
      legend.position = "bottom",
      legendry.legend.subtitle.position = "top"
    )
  )

  vdiffr::expect_doppelganger(
    "bottom-lefttitle",
    base + theme(
      legend.position = "bottom",
      legendry.legend.subtitle.position = "left",
      legendry.legend.subtitle = element_text(angle = 90.0, hjust = 1.0)
    )
  )

  vdiffr::expect_doppelganger(
    "bottom-righttitle",
    base + theme(
      legend.position = "bottom",
      legendry.legend.subtitle.position = "right",
      legendry.legend.subtitle = element_text(angle = 270.0)
    )
  )

  vdiffr::expect_doppelganger(
    "bottom-bottomtitle",
    base + theme(
      legend.position = "bottom",
      legendry.legend.subtitle.position = "bottom"
    )
  )
})

# Constructor -------------------------------------------------------------

#' Guide gizmo: kernel density estimate
#'
#' This guide displays a kernel density estimate (KDE) of the aesthetic. If the
#' aesthetic is `colour` or `fill`, the shape will reflect this.
#'
#' @param key A [sequence key][key_sequence] or [binned key][key_bins]
#'   specification. Internally defaults to a sequence key when the scale is
#'   continuous and a binned key when the scale is binned.
#' @param density One of the following:
#'  * `NULL` for using kernel density estimation on the data values (default).
#'  * a `<numeric>` vector to feed to the `density.fun` function.
#'  * A named `<list>` with `x` and `y` numeric elements of equal length, such
#'    as one returned by using the [`density()`][stats::density] function.
#'    Please note that `<list>` input is expected in scale-transformed space,
#'    not original data space.
#' @param density.args A `<list>` with additional arguments to the
#'   `density.fun` argument. Only applies when `density` is not provided as a
#'   `<list>`. already.
#' @param density.fun A `<function>` to use for kernel density estimation when
#'   the `density` argument is not provided as a list already.
#' @param just A `<numeric[1]>` between 0 and 1. Use 0 for bottom- or
#'   left-aligned densities, use 1 for top- or right-aligned densities and 0.5
#'   for violin shapes.
#' @inheritParams gizmo_barcap
#'
#' @details
#' Non-finite values such as `NA` and `NaN` are ignored while infinite values
#' such as `-Inf` and `Inf` are [squished][scales::oob_squish] to the limits.
#'
#' ## Styling options
#'
#' Below are the [theme][ggplot2::theme] options that determine the styling of
#' this guide.
#'
#' | **Theme setting** | **Type** | **Description** |
#' | ----------------- | -------- | --------------- |
#' | `legend.frame` | [`element_rect()`] | Outline drawn around the density itself. The `fill` setting is ignored. |
#' | `legend.key` | [`element_rect()`] | Background underneath the density area. |
#' | `legend.key.width` | [`unit()`] | Width of the density area. |
#' | `legend.key.height` | [`unit()`] | Height of the density area. |
#'
#' Please note that depending on the `direction` argument, the
#' `legend.key.width`/`legend.key.height` setting are expanded 5-fold if
#' originating from the global theme. To set these directly, you can use the
#' local `theme` argument in the guide.
#' These settings have shorthands in [`theme_guide()`]:
#'
#' ```r
#' gizmo_density(theme = theme_guide(
#'   frame = element_rect(),
#'   key = element_rect(),
#'   key.width = unit(5, "mm")
#'   key.height = unit(5, "cm")
#' ))
#' ```
#'
#' @return A `<GizmoDensity>` object.
#' @family gizmos
#' @export
#'
#' @examples
#' # A standard plot
#' p <- ggplot(mpg, aes(displ, hwy, colour = cty)) +
#'   geom_point() +
#'   scale_colour_viridis_c()
#'
#' # Density from plot data
#' p + guides(colour = gizmo_density())
#'
#' # Using bins instead of gradient
#' p + guides(colour = gizmo_density("bins"))
#'
#' # Providing custom values to compute density of
#' p + guides(colour = gizmo_density(density = runif(1000, min = 5, max = 35)))
#'
#' # Providing a precomputed density
#' p + guides(colour = gizmo_density(density = density(mpg$cty, adjust = 0.5)))
#'
#' # Alternatively, parameters may be passed through density.args
#' p + guides(colour = gizmo_density(density.args = list(adjust = 0.5)))
gizmo_density <- function(
  key = waiver(),
  density = NULL, density.args = list(), density.fun = stats::density,
  just = 0.5, oob = "keep", alpha = NA,
  # standard arguments
  theme = NULL, position = waiver(), direction = NULL
) {

  check_number_decimal(just, min = 0.0, max = 1.0, allow_infinite = FALSE)

  new_guide(
    key = key,
    density      = density,
    density_args = density.args,
    density_fun  = density.fun,
    just = just, oob = oob, alpha = alpha,
    name = "density",
    theme = theme, position = position, direction = direction,
    super = GizmoDensity
  )
}

# Class -------------------------------------------------------------------

#' @export
#' @rdname legendry_extensions
#' @format NULL
#' @usage NULL
GizmoDensity <- ggproto(
  "GizmoDensity", Guide,

  params = new_params(
    density = NULL, density_args = list(), density_fun = stats::density,
    just = 0.5, nbin = 15L, oob = oob_keep, alpha = NA, key = "sequence"
  ),

  elements = list(
    frame  = "legend.frame",
    key    = "legend.key",
    width  = "legend.key.width",
    height = "legend.key.height"
  ),

  extract_key = function(scale, aesthetic, key, ...) {
    key <- key %|W|% if (inherits(scale, "ScaleBinned")) "bins" else "sequence"
    key <-  resolve_key(key %||% "sequence")
    if (is.function(key)) {
      key <- disallow_even_steps(key)
      key <- key(scale, aesthetic)
    }
    key
  },

  extract_decor = function(scale, density, density_args, density_fun, ...) {
    if (is.null(density)) {
      return(NULL)
    }
    if (is.atomic(density)) {
      density <- filter_finite(scale$transform(density))
      density <- inject(density_fun(density, !!!density_args))
    }
    check_density(density)
    density
  },

  extract_params = function(scale, params, ...) {

    params$position <- params$position %|W|% NULL

    key    <- params$key
    aes    <- params$aesthetic
    limits <- scale$get_limits()
    range  <- scale$range$range

    if ((range[1L] < limits[1L]) || (range[2L] > limits[2L])) {
      # Collect out-of-bounds values
      add <- abs(diff(limits)) / 1000.0
      oob <- c(-1.0, 1.0) * add + limits
      map <- scale$map(oob)
      aes <- params$aesthetic

      limits <- range(limits, oob)
      if (all(c("min", "max") %in% names(params$key))) {
        n <- max(which(!is.na(key[[aes]])))
        key <- data_frame0(
          !!aes := c(map[1L], key[[aes]], map[2L]),
          min = c(-Inf, key$min, key$max[n]),
          max = c(key$min[1L], key$max, Inf)
        )
      } else {
        key <- data_frame0(
          !!aes := c(map[1L], key[[aes]],  map[2L]),
          .value = c(oob[1L], key$.value,  oob[2L])
        )
      }
    }
    if (grepl("colour|color|fill", aes)) {
      key[[aes]] <- alpha(key[[aes]], params$alpha)
    }

    params$limits <- limits
    params$key <- key
    params
  },

  get_layer_key = function(params, layers, data = NULL, ...) {
    density <- params$decor %||% params$density
    if (length(density) == 0L) {
      values  <-
        vec_c(!!!lapply(data, .subset2, params$aesthetic)) |>
        filter_finite()
      density <- inject(params$density_fun(values, !!!params$density_args))
      check_density(density)
    }
    params$decor  <- normalise_density(density)
    params$limits <- range(params$limits, params$decor$x)
    params
  },

  setup_params = function(params) {
    limits <- params$limits
    key <- params$key
    key <- vec_slice(key, !is.na(key[[params$aesthetic]]))
    if (all(c("min", "max") %in% names(key))) {
      min <- key$min
      min[which.min(min)] <- -Inf
      min <- guide_rescale(min, limits)

      max <- key$max
      max[which.max(max)] <- Inf
      max <- guide_rescale(key$max, limits)
      key$mid <- (max + min) / 2.0
      key$height <- abs(max - min)
    } else {
      key$.value <- guide_rescale(key$.value, limits)
    }
    params$key <- key
    params$decor$x <- guide_rescale(params$decor$x, params$limits)
    params
  },

  setup_elements = function(params, elements, theme) {
    theme$legend.frame <- theme$legend.frame %||% element_blank()
    if (params$direction == "horizontal") {
      theme$legend.key.width  <- (theme$legend.key.width %||% rel(1.0)) * 5.0
    } else {
      theme$legend.key.height <- (theme$legend.key.height %||% rel(1.0)) * 5.0
    }
    Guide$setup_elements(params, elements, theme)
  },

  build_frame = function(params, elems) {

    decor <- params$decor
    just <- c(params$just, 1.0 - params$just)

    poly_args <- list(
      x = c(decor$x, rev(decor$x)),
      y = c((1.0 - decor$y) * just[1L], 1.0 - (1.0 - rev(decor$y)) * just[2L])
    )

    if (params$direction == "vertical") {
      poly_args <- rename(poly_args, c("x", "y"), c("y", "x"))
    }

    frame <- element_grob(elems$frame)
    gp <- frame$gp %||% gpar(col = NA, fill = NA)

    polygonGrob(x = poly_args$x, y = poly_args$y, gp = gp)
  },

  fill_frame = function(frame, params, elems) {
    key <- params$key
    if (!any(c("colour", "fill") %in% names(key))) {
      return(frame)
    }

    if (all(c("min", "max") %in% names(key))) {
      check_device("clippingPaths")

      args <- list(
        x = 0.5, y = key$mid, height = key$height,
        vjust = 0.5, hjust = 0.5,
        gp = gpar(fill = key[[params$aesthetic]], col = NA)
      )
      if (params$direction == "horizontal") {
        args <- flip_names(args)
        args$height <- grobHeight(frame)
      } else {
        args$width  <- grobWidth(frame)
      }
      args$vp <- viewport(clip = frame)
      grob <- inject(rectGrob(!!!args))
      return(grob)
    }

    check_device("gradients", call = expr(gizmo_density()))
    grad_args <- list(
      x1 = 0.0, x2 = 1.0, y1 = 0.5, y2 = 0.5,
      colours = key$colour %||% key$fill, stops = key$.value
    )
    if (params$direction == "vertical") {
      grad_args <- flip_names(grad_args)
    }
    gradient <- inject(linearGradient(!!!grad_args))
    gradient <- editGrob(frame, gp = gpar(fill = gradient, col = NA))
    gradient
  },

  draw = function(self, theme, position = NULL, direction = NULL,
                  params = self$params) {
    params <- replace_null(params, position = position, direction = direction)
    params <- self$setup_params(params)

    elems <- self$setup_elements(params, self$elements, theme)

    frame    <- self$build_frame(params, elems)
    gradient <- self$fill_frame(frame, params, elems)
    key <- element_grob(elems$key)

    gt <- gtable(widths = elems$width, heights = elems$height)
    gt <- gtable_add_grob(
      gt, list(key, frame, gradient), t = 1L, l = 1L, clip = "off",
      name = c("background", "frame", "density")
    )
  }
)

# Helpers -----------------------------------------------------------------

normalise_density <- function(density) {

  xlim <- range(density$x, na.rm = TRUE, finite = TRUE)
  density$x <- oob_squish_infinite(density$x, xlim)

  ylim <- range(density$y, na.rm = TRUE, finite = TRUE)
  density$y <- oob_squish_infinite(density$y, ylim)

  density$y <- rescale_max(density$y, to = c(0.0, 0.9), from = ylim)
  density
}

check_density <- function(x, arg = caller_arg(x), call = caller_env()) {
  if (is_missing(x)) {
    cli::cli_abort("{.arg {arg}} cannot be missing.", call = call)
  }
  if (inherits(x, "density")) {
    # We'll trust this class
    return(x)
  }
  check_list_names(x, c("x", "y"), arg = arg, call = call)
  if (length(x$x) != length(x$y) || length(x$x) < 2L) {
    if (length(x$x) < 2L || length(x$y) < 2L) {
      extra <- " and at least length 2."
    } else {
      extra <- "."
    }
    cli::cli_abort(c(
      paste0("The {.field x} and {.field y} elements in the {.arg {arg}} ",
             "argument must be of equal length{extra}"),
      i = "{.code {arg}$x} has length {length(x$x)}.",
      i = "{.code {arg}$y} has length {length(x$y)}."
    ), call = call)
  }
  check_bare_numeric(x$x, arg = as_cli("{arg}$x"), call = call)
  check_bare_numeric(x$y, arg = as_cli("{arg}$y"), call = call)
}

disallow_even_steps <- function(fun, call = caller_env()) {
  even_steps <- environment(fun)$even.steps
  if (isTRUE(even_steps)) {
    cli::cli_warn(
      "Cannot use {.code even.steps = TRUE} in this guide.",
      call = call
    )
    environment(fun)$even.steps <- FALSE
  }
  fun
}

# Constructor -------------------------------------------------------------

#' Cross legend guide
#'
#' This is a legend type similar to [`guide_legend()`][ggplot2::guide_legend()]
#' that displays crosses, or: interactions, between two variables.
#'
#' @param key One of the following key specifications:
#'   * A [group split][key_group_split] specification when using the legend to
#'     display a compound variable like `paste(var1, var2)`.
#'   * A [standard key][key_standard] specification, like [`key_auto()`], when
#'     crossing two separate variables across two scales.
#' @param swap A `<logical[1]>` which when `TRUE` exchanges the column and row
#'   variables in the displayed legend.
#' @param col_text An `<element_text>` object giving adjustments to text for
#'   the column labels. Can be `NULL` to display column labels in equal fashion
#'   to the row labels.
#' @param reverse A `<logical[2]>` whether the order of the keys should be
#'   inverted, where the first value controls the row order and second value
#'   the column order. Input as `<logical[1]>` will be recycled.
#' @param row_title,col_title One of the following to indicate subtitles
#'   spanning the rows and columns of the guide:
#'   * A `<character[1]>` or `<expression[1]>` to set a custom title.
#'   * `NULL` to not display any title.
#'   * [`waiver()`][ggplot2::waiver()] to propagate subtitles from
#'     merging guides (default).
#' @param subtitle_position A named list of 4
#'   [text elements][ggplot2::element_text()], having the names `"top"`,
#'   `"right"`, `"bottom"` and `"left`. These govern the display of subtitles
#'   when placed in any of these positions relative to the keys. See
#'   [`position_text()`] for a convenient helper.
#' @inheritParams common_parameters
#'
#' @details
#' ## Styling options
#'
#' Below are the [theme][ggplot2::theme] options that determine the styling of
#' this guide. Note that these are almost the same for
#' [`ggplot2::guide_legend()`].
#'
#' | **Theme setting** | **Type** | **Description** |
#' | ----------------- | -------- | --------------- |
#' | `legend.background`| [`element_rect()`] | Background of the legend. |
#' | `legend.margin` | [`margin()`] | Padding around the legend. |
#' | `legend.text` | [`element_text()`] | Labels displayed next to keys. |
#' | `legend.text.position` | `<character[1]>` | One of `"top"`, `"right"`, `"bottom"` or `"left"`. |
#' | `legend.title` | [`element_text()`] | Title of the legend. |
#' | `legend.title.position` | `<character[1]>` | One of `"top"`, `"right"`, `"bottom"` or `"left"`. |
#' | `legend.key` | [`element_rect()`] | Background of the key areas. |
#' | `legend.key.height` | [`unit()`] | Height of keys. |
#' | `legend.key.width` | [`unit()`] | Width of keys. |
#' | `legend.key.justification` | `<numeric[2]>` | Justification for placing legend keys in excess space. |
#' | `legend.key.spacing.x` | [`unit()`] | Horizontal spacing between keys. |
#' | `legend.key.spacing.y` | [`unit()`] | Vertical spacing between keys. Taken literally. |
#'
#' The context-agnostic alternative to using `theme()` is to use
#' [`theme_guide()`]:
#'
#' ```r
#' guide_legend_cross(theme = theme_guide(
#'   text = element_text(),
#'   text.position = "right",
#'   title = element_text(),
#'   title.position = "top",
#'   key = element_rect(),
#'   key.height = unit(5, "mm"),
#'   key.width = unit(5, "mm"),
#'   key.justification = c(0.5, 0.5),
#'   key.spacing.x = unit(5, "mm"),
#'   key.spacing.y = unit(5, "mm"),
#'   margin = margin(5),
#'   background = element_rect(),
#' ))
#' ```
#'
#' @return A `<GuideLegend>` object.
#' @export
#' @family standalone guides
#' @family legend guides
#'
#' @examples
#' # Standard use for single aesthetic. The default is to split labels to
#' # disentangle aesthetics that are already crossed (by e.g. `paste()`)
#' ggplot(mpg, aes(displ, hwy)) +
#'   geom_point(aes(colour = paste(year, drv))) +
#'   guides(colour = "legend_cross")
#'
#' # If legends should be merged between identical aesthetics, both need the
#' # same legend type.
#' ggplot(mpg, aes(displ, hwy)) +
#'   geom_point(aes(colour = paste(year, drv), shape = paste(year, drv))) +
#'   guides(colour = "legend_cross", shape = "legend_cross")
#'
#' # Crossing two aesthetics requires a shared title and `key = "auto"`. The
#' # easy way to achieve this is to predefine a shared guide.
#' my_guide <- guide_legend_cross(key = "auto", title = "My title")
#'
#' ggplot(mpg, aes(displ, hwy)) +
#'   geom_point(aes(colour = drv, shape = factor(year))) +
#'   guides(colour = my_guide, shape  = my_guide)
#'
#' # You can cross more than 2 aesthetics but not more than 2 unique aesthetics.
#' ggplot(mpg, aes(displ, hwy)) +
#'   geom_point(aes(colour = drv, shape = factor(year), size = factor(drv))) +
#'   scale_size_ordinal() +
#'   guides(colour = my_guide, shape = my_guide, size = my_guide)
#'
#' # You can merge an aesthetic that is already crossed with an aesthetic that
#' # contributes to only one side of the cross.
#' ggplot(mpg, aes(displ, hwy)) +
#'   geom_point(aes(colour = paste(year, drv), shape  = drv)) +
#'   guides(
#'     colour = guide_legend_cross(title = "My Title"),
#'     shape  = guide_legend_cross(title = "My Title", key = "auto")
#'   )
guide_legend_cross <- function(
  key = NULL,
  title = waiver(),
  row_title = waiver(),
  col_title = waiver(),
  swap = FALSE,
  col_text = element_text(angle = 90.0, vjust = 0.5),
  subtitle_position =
    position_text(angle = c(0.0, -90.0, 0.0, 90.0), hjust = 0.5),
  override.aes = list(),
  reverse = FALSE,
  theme = NULL,
  position = NULL,
  direction = NULL,
  order = 0L
) {

  check_position(position, theta = FALSE, inside = TRUE, allow_null = TRUE)
  check_argmatch(direction, c("horizontal", "vertical"), allow_null = TRUE)
  check_bool(swap)
  check_list_of(
    subtitle_position,
    element_classes("text", "blank"),
    allow_null = TRUE
  )
  check_length(subtitle_position, exact = 4L, allow_null = TRUE)
  check_inherits(
    col_text, element_classes("text", "blank"),
    allow_null = TRUE
  )

  if (length(reverse) == 1L) {
    check_bool(reverse)
  } else {
    check_length(reverse, exact = 2L)
    check_bool(reverse[1L])
    check_bool(reverse[2L])
  }

  dim_order <- if (swap) c("col", "row") else c("row", "col")

  new_guide(
    key = key,
    title = title,
    row_title = row_title,
    col_title = col_title,
    dim_order = dim_order,
    override.aes = rename_aes(override.aes),
    subtitle_position = subtitle_position,
    col_text = col_text,
    reverse = reverse,
    theme = theme,
    position = position,
    direction = direction,
    order = order,
    super = GuideLegendCross
  )
}



# Class -------------------------------------------------------------------

GuideLegendCross <- ggproto(

  "GuideLegendCross", GuideLegendBase,

  params = new_params(
    override.aes = list(), reverse = FALSE,
    row_title = waiver(), col_title = waiver(),
    key = NULL, dim_order = c("row", "col"),
    subtitle_position = list(),
    col_text = NULL
  ),

  hashables = exprs(title, "GuideLegendCross"),

  elements = list2(
    !!!GuideLegendBase$elements,
    subtitle = "legendry.legend.subtitle"
  ),

  extract_key = function(scale, aesthetic, key = NULL,
                         dim_order = c("row", "col"), ...) {

    key <- standard_extract_key(scale, aesthetic, key %||% "group_split", ...)
    grouping <- c(".label", ".group")

    # If we don't have grouping columns yet,
    # we cannot start filling in the layout
    if (!all(grouping %in% names(key))) {
      return(key)
    }

    # Start filling in layout, to be finalised later
    row  <- unique0(key[grouping[dim_order == "row"]][[1L]])
    col  <- unique0(key[grouping[dim_order == "col"]][[1L]])
    grid <- vec_expand_grid(row = row, col = col)

    # Repeat key to match layout
    i    <- vec_match(grid, rename(key[grouping], grouping, dim_order))
    key  <- vec_slice(key, i)

    # Fill in locations
    key$.row_label <- grid$row
    key$.col_label <- grid$col
    key
  },

  merge = function(self, params, new_guide, new_params) {

    old_key <- params$key
    new_key <- new_params$key

    columns   <- c(".row_label", ".col_label")
    old_ready <- all(columns %in% names(old_key))
    new_ready <- all(columns %in% names(new_key))

    if (!old_ready) {
      params$key <- cross_merge_incomplete(old_key, new_key, params$dim_order)
    } else if (new_ready) {
      params$key <- cross_merge_complete(old_key, new_key)
    } else {
      params$key <- cross_merge_partial(old_key, new_key)
    }

    params$row_title <- params$row_title %|W|% new_params$row_title
    params$col_title <- params$col_title %|W|% new_params$col_title

    params$override.aes <-
      merge_legend_override(params$override.aes, new_params$override.aes)
    list(guide = self, params = params)
  },

  setup_params = function(params) {

    key <- params$key
    params$n_breaks <- n_breaks <- nrow(key)

    key$.index <- vec_seq_along(key)
    key$.row <- match_self(key$.row_label %||% seq_len(n_breaks))
    key$.col <- match_self(key$.col_label %||% rep_len(1L, n_breaks))

    reverse <- rep_len(params$reverse, 2L)
    if (reverse[1L]) {
      nrows <- max(key$.row, na.rm = TRUE)
      key$.row <- nrows - key$.row + 1L
    }
    if (reverse[2L]) {
      ncols <- max(key$.col, na.rm = TRUE)
      key$.col <- ncols - key$.col + 1L
    }

    params$key <- key
    params
  },

  setup_elements = function(params, elements, theme) {

    theme <- theme + params$theme
    params$theme <- NULL

    text_position  <- theme$legend.text.position  %||% "right"
    title_position <- theme$legend.title.position %||%
      switch(params$direction, vertical = "top", horizontal = "left")
    elements$title <- setup_legend_title(theme, title_position)

    # Resolve text positions
    row <- intersect(c("right", "left"), text_position)[1L] %|NA|% "right"
    col <- intersect(c("top", "bottom"), text_position)[1L] %|NA|% "bottom"

    # Resolve text theming
    elements$text_row <- setup_legend_text(theme, row)
    elements$text_col <- combine_elements(
      params$col_text,
      setup_legend_text(theme, col)
    )

    if (!is.null(params$row_title %|W|% NULL)) {
      elements$subtitle_row <- combine_elements(
        switch(
          row,
          right = params$subtitle_position[["right"]],
          params$subtitle_position[["left"]]
        ),
        setup_legend_title(theme, row, element = elements$subtitle)
      )
    }

    if (!is.null(params$col_title %|W|% NULL)) {
      elements$subtitle_col <- combine_elements(
        switch(
          col,
          top = params$subtitle_position[["top"]],
          params$subtitle_position[["bottom"]]
        ),
        setup_legend_title(theme, col, element = elements$subtitle)
      )
    }

    elements <- Guide$setup_elements(params, elements, theme)
    elements[c("row_position", "col_position")] <- list(row, col)
    elements$title_position <- title_position
    elements
  },

  build_labels = function(key, elements, params) {

    # Render row labels first
    rows <- vec_slice(key, !duplicated(key$.row))
    rows$.label <- rows$.row_label %||% rows[[".label"]]
    elem <- list(text = elements[["text_row"]])
    rows <- GuideLegendBase$build_labels(rows, elem, params)

    # Then column labels follow
    cols <- vec_slice(key, !duplicated(key$.col))
    cols$.label <- cols$.col_label %||% cols[[".label"]]
    elem <- list(text = elements[["text_col"]])
    cols <- GuideLegendBase$build_labels(cols, elem, params)

    # We don't combine them yet, as they need to be measured separately later
    list(rows = rows, cols = cols)
  },

  build_title = function(label, elements, params) {
    main <- GuideLegend$build_title(label, elements, params)
    row <- element_grob(
      elements$subtitle_row %||% element_blank(),
      label = params$row_title,
      margin_x = TRUE,
      margin_y = TRUE
    )
    col <- element_grob(
      elements$subtitle_col %||% element_blank(),
      label = params$col_title,
      margin_x = TRUE,
      margin_y = TRUE
    )
    list(main = main, col = col, row = row)
  },

  measure_grobs = function(grobs, params, elements) {

    # Get width of keys per column
    col <- params$key[[".col"]]
    widths <- map_dbl(grobs$decor, `[[`, i = "width")
    widths <- pmax(by_group(widths, col, max), elements$width_cm)

    # Weave in width of labels
    widths <- pmax(widths, width_cm(grobs$labels$cols))
    widths <- vec_interleave(elements$spacing_x %||% 0.0, widths)[-1L]
    label_width <- max(width_cm(grobs$labels$rows))
    widths <- switch(
      elements$row_position,
      left = c(label_width, widths),
      c(widths, label_width)
    )

    # Get height of keys per row
    row <- params$key[[".row"]]
    heights <- map_dbl(grobs$decor, `[[`, i = "height")
    heights <- pmax(by_group(heights, row, max), elements$height_cm)

    # Weave in heights of labels
    heights <- pmax(heights, height_cm(grobs$labels$rows))
    heights <- vec_interleave(elements$spacing_y %||% 0.0, heights)[-1L]
    label_height <- max(height_cm(grobs$labels$cols))
    heights <- switch(
      elements$col_position,
      top = c(label_height, heights),
      c(heights, label_height)
    )

    list(widths = widths, heights = heights)
  },

  arrange_layout = function(key, sizes, params, elements) {

    # Account for spacing in between keys
    key_row <- key[[".row"]] * 2L - 1L
    key_col <- key[[".col"]] * 2L - 1L

    lab_row <- max(key_row) + 1L
    lab_col <- max(key_col) + 1L

    if (elements$row_position == "left") {
      key_col <- key_col + 1L
      lab_col <- 1L
    }

    if (elements$col_position == "top") {
      key_row <- key_row + 1L
      lab_row <- 1L
    }

    cols <- unique(key_col)
    rows <- unique(key_row)

    lab_row <- c(rows, rep(lab_row, length(cols)))
    lab_col <- c(rep(lab_col, length(rows)), cols)

    list(
      key_row = key_row, key_col = key_col,
      label_row = lab_row, label_col = lab_col
    )
  },

  assemble_drawing = function(self, grobs, layout, sizes, params, elements) {

    widths <- unit(sizes$widths, "cm")
    if (isTRUE(elements$stretch_x)) {
      widths[unique0(layout$key_col)] <- elements$key_width
    }

    heights <- unit(sizes$heights, "cm")
    if (isTRUE(elements$stretch_y)) {
      heights[unique0(layout$key_row)] <- elements$key_height
    }

    gt <- gtable(widths = widths, heights = heights)

    # Add keys
    if (!is_zero(grobs$decor)) {
      gt <- gtable_add_grob(
        gt, grobs$decor,
        name = names(grobs$decor),
        clip = "off",
        t = layout$key_row, r = layout$key_col,
        b = layout$key_row, l = layout$key_col
      )
    }

    # Add labels
    grobs$labels <- c(grobs$labels$rows, grobs$labels$cols)
    if (!is_zero(grobs$labels)) {
      gt <- gtable_add_grob(
        gt, grobs$labels,
        name = paste("label", layout$label_row, layout$label_col, sep = "-"),
        clip = "off",
        t = layout$label_row, r = layout$label_col,
        b = layout$label_row, l = layout$label_col
      )
    }

    # Add row title
    if (!is_zero(grobs$title$row)) {
      row_size <- unit(width_cm(grobs$title$row), "cm")
      place <- switch(elements$row_position, right = c(-1L, -1L), c(0L, 1L))
      gt <- gtable_add_cols(gt, row_size, pos = place[1L])
      gt <- gtable_add_grob(
        gt, grobs$title$row,
        name = "row-subtitle", clip = "off",
        t = min(layout$key_row),
        b = max(layout$key_row),
        l = place[2L], r = place[2L]
      )
      if (elements$row_position == "left") {
        # Account that we've added a column in front.
        # Needed for accurate placement of column subtitle.
        layout$key_col <- layout$key_col + 1L
      }
    }

    # Add col title
    if (!is_zero(grobs$title$col)) {
      col_size <- unit(height_cm(grobs$title$col), "cm")
      place <- switch(elements$col_position, top = c(0L, 1L), c(-1L, -1L))
      gt <- gtable_add_rows(gt, col_size, pos = place[1L])
      gt <- gtable_add_grob(
        gt, grobs$title$col,
        name = "col-subtitle", clip = "off",
        t = place[2L], b = place[2L],
        l = min(layout$key_col),
        r = max(layout$key_col)
      )
      if (elements$col_position == "top") {
        # Account that we've added a row on top.
        # Needed for accurate placement of column subtitle.
        layout$key_row <- layout$key_row + 1L
      }
    }

    # Increase size if titles are large
    if (!is_zero(grobs$title$col)) {
      title_width <- width_cm(grobs$title$col)

      # Compute excess title width
      widths <- width_cm(gt$widths)
      i <- range(layout$key_col)
      key_width <- sum(widths[i[1L]:i[2L]])
      excess_width <- title_width - key_width

      if (excess_width > 0.0) {
        just <- rotate_just(element = elements$subtitle_col)$hjust

        # Find out how much space needs to added to the right
        right_width <- sum(after(widths, i[2L]))
        right_width <- pmax(((1.0 - just) * excess_width) - right_width, 0.0)

        # Find out how much space needs to added to the left
        left_width <- sum(before(widths, i[1L]))
        left_width <- pmax((just * excess_width) - left_width, 0.0)

        # Add actual space
        gt <- gtable_add_cols(gt, unit(right_width, "cm"), pos = -1L)
        gt <- gtable_add_cols(gt, unit(left_width,  "cm"), pos = 0L)
      }
    }

    # Increase size if titles are large
    if (!is_zero(grobs$title$row)) {
      title_height <- height_cm(grobs$title$row)

      # Compute excess title height
      heights <- height_cm(gt$heights)
      i <- range(layout$key_row)
      key_height <- sum(heights[i[1L]:i[2L]])
      excess_height <- title_height - key_height

      if (excess_height > 0.0) {
        just <- rotate_just(element = elements$subtitle_row)$vjust

        # Find out how much space needs to added on top
        top_height <- sum(before(heights, i[1L]))
        top_height <- pmax(((1.0 - just) * excess_height) - top_height, 0.0)

        # Find out how much space needs to added on the bottom
        bottom_height <- sum(after(heights, i[2L]))
        bottom_height <- pmax((just * excess_height) - bottom_height, 0.0)

        # Add actual space
        gt <- gtable_add_rows(gt, unit(bottom_height, "cm"), pos = -1L)
        gt <- gtable_add_rows(gt, unit(top_height, "cm"), pos = 0L)
      }
    }

    gt <- self$add_title(
      gt, grobs$title$main, elements$title_position,
      rotate_just(element = elements$title)
    )

    # Add padding and background
    gt <- gtable_add_padding(gt, unit(elements$padding, "cm"))
    if (!is_zero(elements$background)) {
      gt <- gtable_add_grob(
        gt, elements$background,
        name = "background", clip = "off",
        t = 1L, r = -1L, b = -1L, l = 1L, z = -Inf
      )
    }
    gt
  }
)

# Public helpers ----------------------------------------------------------

#' Helper to position text
#'
#' This is a helper function for use in [`guide_legend_cross()`]. It creates
#' a list of text elements, each corresponding the top, right, bottom or left
#' positions. Input is given for that order as well.
#'
#' @param angle A `<numeric[1-4]>` value setting the angle of the text.
#' @param hjust,vjust A `<numeric[1-4]>` between 0 and 1 to set the horizontal
#'   justification (`hjust`) or vertical justification (`vjust`) for the text.
#' @param ... Other arguments passed to the `element` function.
#' @param element An `<function>` creating an object that inherits from the
#'   [`<element_text>`][ggplot2::element_text] class.
#'
#' @details
#' Input other than the `element` argument will be recycled to length 4. `NA`
#' and 0-length input will be dropped.
#'
#' @returns A named list of `element` objects of length 4. The list has the
#'   names `"top"`, `"right"`, `"bottom"` and `"left"`.
#' @export
#'
#' @examples
#' # Red text turning along with position
#' position_text(angle = c(0, -90, 180, 90), colour = "red")
position_text <- function(
  angle = NA,
  hjust = NA,
  vjust = NA,
  ...,
  element = element_text
) {

  input <- list2(angle = angle, hjust = hjust, vjust = vjust, ...)

  # Check for invalid arguments
  nms <- names(input)
  extra <- setdiff(nms, fn_fmls_names(element))
  if (length(extra) > 0L) {
    cli::cli_warn(
      "Unknown argument{?/s}: {.and {.arg {extra}}}",
      call = current_call()
    )
    input <- input[setdiff(nms, extra)]
  }

  input <- input[lengths(input) > 0L]
  input <- data_frame0(!!!lapply(input, rep, length.out = 4L))

  output <- lapply(vec_seq_along(input), function(i) {
    args  <- as.list(vec_slice(input, i))
    valid <- !map_lgl(args, function(x) is.null(x) || is.na(x))
    if (is.list(args$margin) && is_margin(args$margin[[1L]])) {
      args$margin <- args$margin[[1L]]
    }
    args  <- args[valid]
    inject(element(!!!args))
  })
  names(output) <- .trbl
  output
}

# Helpers -----------------------------------------------------------------

cross_merge_complete <- function(old, new) {
  columns <- c(".row_label", ".col_label")
  if (!identical(old[columns], new[columns])) {
    old_aes <- colnames(old)[!startsWith(colnames(old), ".")]
    new_aes <- colnames(new)[!startsWith(colnames(new), ".")]
    cli::cli_abort(
      "Cannot merge legends for {.field {old_aes}} and {.field {new_aes}}."
    )
  }
  data_frame0(!!!defaults(old, new))
}

cross_merge_partial <- function(old, new) {
  new_aes <- colnames(new)[!startsWith(colnames(new), ".")]

  row_match <- match(old$.row_label, new$.label)
  col_match <- match(old$.col_label, new$.label)

  index <- if (anyNA(row_match)) col_match else row_match
  if (anyNA(index)) {
    old_aes <- colnames(old)[!startsWith(colnames(old), ".")]
    cli::cli_abort(c(
      "Cannot match legend for {.field {new_aes}} aesthetic{?s}.",
      i = "The labels mismatch those of the {.field {old_aes}} aesthetic{?s}."
    ))
  }
  old[new_aes] <- new[index, new_aes]
  old
}

cross_merge_incomplete <- function(old, new, order = c("row", "col")) {
  if (identical(old$.label, new$.label)) {
    return(data_frame0(!!!defaults(old, new)))
  }

  grid <- vec_expand_grid(
    old = vec_seq_along(old),
    new = vec_seq_along(new)
  )

  old <- vec_slice(old, grid$old)
  new <- vec_slice(new, grid$new)

  order <- paste0(".", order, "_label")
  old[order[1L]] <- old$.label
  new[order[2L]] <- new$.label
  data_frame0(!!!defaults(old, new))
}

merge_legend_override <- function(old, new) {
  new <- c(old, new)
  dup <- duplicated(names(new))
  if (any(dup)) {
    cli::cli_warn(
      "Duplicated {.arg override.aes} are ignored: {.field {names(new)[dup]}}."
    )
  }
  vec_slice(new, !dup)
}

test_that("legend cross labels can be placed anywhere", {

  p <- ggplot(data_frame0(colour = c("A:a", "B:a", "C:a", "A:b", "C:b"))) +
    geom_point(aes(1L:5L, 1L:5L, colour = colour)) +
    scale_colour_discrete(guide = guide_legend_cross())

  build <- ggplot_build(p)
  guide  <- build$plot$guides$get_guide("colour")
  params <- build$plot$guides$get_params("colour")
  params[c("position", "direction")] <- list("right", "vertical")

  sets <- list(
    c("left", "bottom"),
    c("left", "top"),
    c("right", "bottom"),
    c("right", "top")
  )

  grobs <- lapply(sets, function(set) {
    guide$draw(
      theme_get() + theme(legend.text.position = set),
      params = params
    )
  })

  gt <- gtable(unit(c(0.5, 0.5), "null"), unit(c(0.5, 0.5), "null"))
  gt <- gtable_add_grob(
    gt, grobs,
    t = c(2L, 1L, 2L, 1L), l = c(1L, 1L, 2L, 2L)
  )

  vdiffr::expect_doppelganger(
    "legend cross orientations",
    gt
  )
})

test_that("cross legend can be constructed from single scale", {
  df <- data_frame0(
    x = 1L:5L, y = 1L:5L,
    z = c("A:1", "A:2", "B:2", "C:1", "C:2")
  )

  p <- ggplot(df, aes(x, y, colour = z)) +
    geom_point() +
    guides(colour = "legend_cross")

  vdiffr::expect_doppelganger(
    "legend cross single scale",
    p
  )

  p <- ggplot(df, aes(x, y, colour = z)) +
    geom_point() +
    guides(colour = guide_legend_cross(reverse = c(TRUE, TRUE)))

  vdiffr::expect_doppelganger(
    "legend cross with double reverse",
    p
  )
})

test_that("cross legend can be constructed from dual scales", {

  df <- data_frame0(
    x = 1L:5L, y = 1L:5L,
    v = c("A", "A", "B", "C", "C"),
    w = c("1", "2", "2", "1", "2")
  )

  guide <- guide_legend_cross(title = "cross legend", key = "auto")

  p <- ggplot(df, aes(x, y, colour = v, shape = w)) +
    geom_point() +
    scale_colour_discrete(guide = guide) +
    scale_shape_discrete(guide = guide)

  vdiffr::expect_doppelganger(
    "legend cross two scales",
    p
  )

  guide <- guide_legend_cross(title = "cross legend", key = "auto",
                              swap = TRUE)

  p <- ggplot(df, aes(x, y, colour = v, shape = w)) +
    geom_point() +
    scale_colour_discrete(guide = guide) +
    scale_shape_discrete(guide = guide)

  vdiffr::expect_doppelganger(
    "legend cross two scales swapped order",
    p
  )
})

test_that("subtitles are placed correctly", {

  placement <- position_text(
    angle  = c(0.0, 270.0, 180.0, 90.0),
    colour = c("orchid", "tomato", "dodgerblue", "limegreen"),
    hjust  = 0.5
  )

  df <- data_frame0(
    x = 1L:5L, y = 1L:5L,
    z = c("A:1", "A:2", "B:2", "C:1", "C:2")
  )

  p <- ggplot(df, aes(x, y, colour = z)) +
    geom_point() +
    guides(colour = guide_legend_cross(
      row_title = "Row Title",
      col_title = "Column Title",
      subtitle_position = placement
    ))

  vdiffr::expect_doppelganger(
    "legend cross subtitles bottom right",
    p
  )

  p <- ggplot(df, aes(x, y, colour = z)) +
    geom_point() +
    guides(colour = guide_legend_cross(
      row_title = "Row Title",
      col_title = "Column Title",
      subtitle_position = placement,
      theme = theme(
        legend.text.position = c("top", "left")
      )
    ))

  vdiffr::expect_doppelganger(
    "legend cross subtitles top left",
    p
  )
})

test_that("merge strategies work as intended", {

  df <- data_frame0(
    x = 1L:5L, y = 1L:5L,
    v = c("A", "A", "B", "C", "C"),
    w = c("1", "2", "2", "1", "2")
  )

  guide <- guide_legend_cross(title = "cross legend", key = "auto")

  # Uses the 'incomplete' strategy
  p <- ggplot(df, aes(x, y, colour = v, shape = w)) +
    geom_point() +
    scale_colour_discrete(guide = guide) +
    scale_shape_discrete(guide = guide)

  build <- ggplot_build(p)
  key <- build$plot$guides$get_params(1L)$key
  expect_identical(key$.row_label, c("A", "A", "B", "B", "C", "C"))
  expect_identical(key$.col_label, c("1", "2", "1", "2", "1", "2"))

  # Uses the 'partial' strategy
  p <- ggplot(df, aes(x, y, colour = paste(v, w), shape = w)) +
    geom_point() +
    scale_colour_discrete(guide = guide_legend_cross(title = "cross legend")) +
    scale_shape_discrete(guide = guide)

  build <- ggplot_build(p)
  key <- build$plot$guides$get_params(1L)$key
  expect_identical(key$.row_label, factor(rep(c("1", "2"), each = 3L)))
  expect_identical(key$.col_label, factor(c("A", "B", "C", "A", "B", "C")))
  # The B-1 combination does not exist in the data
  expect_true(is.na(key$colour[2L]))
  expect_false(is.na(key$shape[2L]))

  # Uses the 'complete' strategy
  guide <- guide_legend_cross(title = "cross legend")
  p <- ggplot(df, aes(x, y, colour = paste(v, w), shape = paste(v, w))) +
    geom_point() +
    scale_colour_discrete(guide = guide) +
    scale_shape_discrete(guide = guide)

  build <- ggplot_build(p)
  key <- build$plot$guides$get_params(1L)$key
  expect_identical(key$.row_label, factor(rep(c("1", "2"), each = 3L)))
  expect_identical(key$.col_label, factor(c("A", "B", "C", "A", "B", "C")))
  # The B-1 combination does not exist in the data
  expect_true(is.na(key$colour[2L]))
  expect_true(is.na(key$shape[2L]))

  # Edge cases
  a <- data.frame(foo = 1L:2L, .row_label = 1L:2L, .col_label = 1L:2L)
  b <- data.frame(bar = 3L:4L, .row_label = 3L:4L, .col_label = 3L:4L)
  expect_error(cross_merge_complete(a, b), "Cannot merge")
  expect_error(cross_merge_partial(a, b), "Cannot match")
  d <- data_frame0(qux = 1L:2L, .label = c("A", "B"))
  expect_identical(d, cross_merge_incomplete(d, d))
})

# Constructor -------------------------------------------------------------

#' Guide primitives: boxes
#'
#' This function constructs a boxes [guide primitive][guide-primitives].
#'
#' @inheritParams primitive_bracket
#' @param min_size A [`<grid::unit[1]>`][grid::unit] setting the minimal size
#'   of a box.
#' @param levels_box A list of `<element_rect>` objects to customise how
#'   boxes appear at every level.
#'
#' @return A `<PrimitiveBox>` primitive guide that can be used inside other
#'   guides.
#' @family primitives
#' @export
#'
#' @details
#' ## Styling options
#'
#' Below are the [theme][ggplot2::theme] options that determine the styling of
#' this guide, which may differ depending on whether the guide is used in
#' an axis or in a legend context.
#'
#' The possible `{position}` suffixes mentioned below are `x`, `x.top`,
#' `x.bottom`, `y`, `y.left`, `y.right`. The `theta` and `r` position suffixes
#' in \pkg{ggplot2} are *not* obeyed in \pkg{legendry}.
#'
#' | **Theme setting** | **Context** | **Type** | **Description** |
#' | ----------------- | ----------- | -------- | --------------- |
#' | `legendry.box` | Both | [`element_rect()`] | The boxes themselves |
#' | `axis.text.{position}`\* | Axis | [`element_text()`] | The text in the boxes. |
#' | `legend.text` | Legend | [`element_text()`] | The text in the boxes. |
#'
#' Styling options *per level* can be set in the `levels_box` and `levels_text`
#' arguments. These override theme settings.
#'
#' Styling options *per range* can be set in the [range key][key_range].
#' The `rect` and `text` prefixed properties are prioritised for the boxes and
#' text respectively. These override theme settings and 'per level' settings.
#'
#' The context-agnostic alternative to using `theme()` is to use
#' [`theme_guide()`]:
#'
#' ```r
#' primitive_box(theme = theme_guide(
#'   box = element_rect(),
#'   text = element_text()
#' ))
#' ```
#'
#' @examples
#' # A standard plot
#' p <- ggplot(mpg, aes(interaction(drv, year), displ)) +
#'  geom_point()
#'
#' key <- key_range_manual(c(2, 4), c(5, 6), c("A", "B"))
#'
#' # Adding as secondary guides
#' p + guides(
#'   x.sec = primitive_box(),
#'   y.sec = primitive_box(key = key)
#' )
primitive_box <- function(
  key = "range_auto",
  angle = waiver(),
  oob = "squish",
  drop_zero = TRUE,
  pad_discrete = 0.4,
  min_size = NULL,
  levels_box = NULL,
  levels_text = NULL,
  theme = NULL,
  position = waiver()
) {

  key <- resolve_key(key)
  oob <- arg_match0(oob, c("squish", "censor", "none"))
  check_bool(drop_zero)
  check_number_decimal(pad_discrete, allow_infinite = FALSE)
  check_list_of(
    levels_box, element_classes("rect", "blank"),
    allow_null = TRUE
  )
  check_list_of(
    levels_text, element_classes("text", "blank"),
    allow_null = TRUE
  )

  new_guide(
    key = key,
    oob = oob,
    angle = angle,
    drop_zero = drop_zero,
    pad_discrete = pad_discrete,
    min_size = min_size,
    levels_box = levels_box,
    levels_text = levels_text,
    theme = theme,
    position = position,
    available_aes = c("any", "x", "y", "r", "theta"),
    super = PrimitiveBox
  )
}

# Class -------------------------------------------------------------------

#' @export
#' @rdname legendry_extensions
#' @format NULL
#' @usage NULL
PrimitiveBox <- ggproto(
  "PrimitiveBox", Guide,

  hashables = exprs(key, decor),

  params = new_params(
    key = NULL, oob = "squish", drop_zero = TRUE,
    pad_discrete = 0.4, angle = waiver(), min_size = NULL,
    levels_text = NULL, levels_box = NULL
  ),

  elements = list(
    position = list(text = "axis.text",   box = I("legendry.box")),
    legend   = list(text = "legend.text", box = I("legendry.box"))
  ),

  extract_key = range_extract_key,

  extract_params = extract_range_params,

  extract_decor = function(scale, aesthetic, key, ...) {
    key <- vec_slice(key, key$.draw)
    decor <- key[setdiff(names(key), c("start", "end"))]
    decor$group <- seq_len(nrow(key))
    decor <- vec_rep_each(decor, 2)
    decor[[aesthetic]] <- vec_interleave(key$start, key$end)
    decor
  },

  transform = function(self, params, coord, panel_params) {
    params$key <-
      transform_key(params$key, params$position, coord, panel_params)
    params$bbox <- panel_params$bbox %||% list(x = c(0.0, 1.0), y = c(0.0, 1.0))
    if (!is_empty(params$decor)) {
      other <-
        switch(params$position, bottom = , left = , theta.sec = -Inf, Inf)
      params$decor <- replace_null(params$decor, x = other, y = other)
      params$decor <- coord_munch(coord, params$decor, panel_params)
      if (params$position == "theta.sec") {
        params$decor$theta <- params$decor$theta + pi
      }
    }
    params
  },

  setup_params = setup_range_params,

  setup_elements = primitive_setup_elements,

  build_box = function(key, decor, elements, params) {

    levels   <- unique(c(key$.level, decor$.level))
    nlevels  <- length(levels)
    position <- params$position

    # Recycle custom elements per level to appropriate length
    box_levels  <- rep0(params$levels_box,  length.out = nlevels)
    text_levels <- rep0(params$levels_text, length.out = nlevels)

    # Justify labels along their ranges
    key <- justify_ranges(key, levels, elements$text, text_levels)

    if (is_theta(position)) {
      add <- if (position == "theta.sec") pi else 0.0
      key <- polar_xy(key, key$r, key$theta + add, params$bbox)
    }

    grobs  <- vector("list", nlevels)
    offset <- elements$offset
    angle  <- params$angle %|W|% NULL
    min_size <- cm(params$min_size %||% 0.2)
    sizes <- numeric(nlevels)
    text  <- angle_labels(elements$text, angle, position)

    measure <- switch(
      position,
      left = , right = width_cm,
      top = , bottom = height_cm,
      get_size_attr
    )

    for (i in seq_len(nlevels)) {

      # Render text
      labels <- draw_labels(
        vec_slice(key, key$.level == levels[[i]]),
        combine_elements(text_levels[[i]], text),
        angle = angle, offset = offset, position = position
      )
      sizes[i] <- max(measure(labels), min_size)

      # Render box
      box <- draw_box(
        vec_slice(decor, decor$.level == levels[[i]]),
        combine_elements(box_levels[[i]], elements$box),
        size = sizes[i], offset = offset, position = position
      )

      offset <- offset + sizes[i]
      grobs[[i]] <- grobTree(box, labels)
    }

    if (position %in% c("top", "left")) {
      grobs <- rev(grobs)
      sizes <- rev(sizes)
    }

    attr(grobs, "size") <- sizes
    grobs
  },

  draw = function(self, theme, position = NULL, direction = NULL,
                  params = self$params) {
    params <- replace_null(params, position = position, direction = direction)
    params <- self$setup_params(params)

    elems <- self$setup_elements(params, self$elements, theme)
    box <- self$build_box(params$key, params$decor, elems, params)

    if (length(box) < 1L) {
      return(zeroGrob())
    }

    primitive_grob(
      grob = box,
      size = unit(get_size_attr(box), "cm"),
      position = params$position,
      name = "box"
    )

  }
)

# Helpers -----------------------------------------------------------------

draw_box <- function(decor, element, size, offset, position) {
  if (nrow(decor) < 2L || is_blank(element)) {
    return(zeroGrob())
  }
  aes <- switch(position, top = , bottom = "x", left = , right = "y", "theta")
  rle <- new_rle(decor$group)
  props <- element_key_properties(vec_slice(decor, rle$start), "rect")

  if (is_theta(position)) {
    rev <- vec_slice(decor, rev(vec_seq_along(decor)))
    x <- unit(c(decor$x, rev$x), "npc")
    y <- unit(c(decor$y, rev$y), "npc")
    theta  <- c(decor$theta, rev$theta)
    offset <- rep(c(0.0, size) + offset, each = nrow(decor))
    x <- x + unit(sin(theta) * offset, "cm")
    y <- y + unit(cos(theta) * offset, "cm")
    id <- c(decor$group, rev$group)
    element <- destructure_element(element)
    gp <- gg_par(
      col  = props$colour    %||% element$colour,
      fill = props$fill      %||% element$fill,
      lwd  = props$linewidth %||% element$linewidth,
      lty  = props$linetype  %||% element$linetype,
      linejoin = element$linejoin
    )
    grob <- polygonGrob(x = x, y = y, id = id, gp = gp)
    return(grob)
  }

  start <- decor[[aes]][rle$start]
  end   <- decor[[aes]][rle$end]
  min <- pmin(start, end)
  max <- pmax(start, end)
  args <- list(
    x = min, width = max - min,
    hjust = 0.0, vjust = 0.5,
    y = 0.5, height = 1.0
  )
  if (position %in% c("left", "right")) {
    args <- flip_names(args)
  }
  inject(element_grob(element, !!!args, !!!props))
}

# Constructor -------------------------------------------------------------

#' Symbol and upset axis guide
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' @description
#' These axis guides can be used for set annotations of discrete categories.
#' The upset guide displays set intersections in matrix and is can be used
#' to replace Venn/Euler diagrams. The symbol guide also displays a matrix of
#' symbols, but requires manually specifying them.
#'
#' @param key
#' One of the following:
#' * An [upset key][key_upset] specification. For `guide_axis_upset`, specifying
#'   a `<character[n]>` is also passed to the `key_upset(order)` argument. An
#'   exception is made when the string is a valid key specification.
#' * A [symbol key][key_symbols] specification.
#'
#' @param connect
#' One of the following:
#' * A `<data.frame>` containing the following columns:
#'     * `value_start`, `value_end` Scale break values or `<numeric[n]>` values
#'       connecting along the axis.
#'     * `level_start`, `level_end` must be `<integer[n]>` values connecting
#'       perpendicular to the axis.
#'     * (Optional) columns for graphical parameters: `colour`, `linewidth`
#'       and `linetype`.
#' * A `<character[1]>` keyword for upset guides or logical symbols:
#'     * `"perpendicular"`: connect `TRUE` symbols (set membership for upset)
#'       perpendicular to the axis.
#'     * `"parallel"`: connect `TRUE` symbols parallel to the axis. This is
#'       *not* appropriate for upset guides.
#'
#' @param override.aes
#' A named `<list>` specifying graphical properties of points to apply to
#' symbols. Every element must either be length 1 or match the number of symbols
#' determined by the key. 3 symbols are used in upset guides or for logical
#' symbolism. Otherwise the number of unique values to the `key_symbols(symbol)`
#' argument determines the number of symbols.
#'
#' @inheritParams common_parameters
#'
#' @details
#' The upset axis does not predetermine the order the categories. If any sorting
#' based on set size needs to occur, the scale is the correct tool to handle
#' this task.
#'
#' ## Styling options
#'
#' Several styling options are provided in the theme, while individualised
#' styling is discussed below the table.
#'
#' | **Theme setting** | **Type** | **Description** |
#' | ----------------- | -------- | --------------- |
#' | `legendry.axis.subtitle` | [`element_text()`] | Titles on the side labelling levels. |
#' | `legendry.axis.subtitle.position` | `<character[1]>` | One of `"top"`, `"right"`, `"bottom"` or `"left"`. |
#' | `legendry.zebra.light` | [`element_rect()`] | Row shading. |
#' | `legendry.zebra.dark` | [`element_rect()`] | Alternate row shading. |
#' | `legendry.table.spacing` | [`rel()`]/[`unit()`] | Padding between levels. |
#' | `legendry.point` | [`element_point()`] | Styling of the symbols |
#' | `legendry.connector` | [`element_line()`] Drawing the `connect` lines. |
#'
#' Moreover, styling options *per group* of symbols can be set via the
#' `override.aes` argument. These override theme settings.
#'
#' Styling options *per symbol* can be set in [`key_symbols()`] via the `...`
#' argument. These override theme settings and 'per group' settings.
#'
#' Styling options *per line* in line connectors can be set by including
#' graphical parameters as columns in the `connect = <data.frame>` argument.
#' These override theme settings.
#'
#' The context-agnostic alternative to using `theme()` is to use
#' [`theme_guide()`]:
#'
#' ```r
#' guide_axis_symbols(theme = theme_guide(
#'   subtitle = element_line(),
#'   subtitle.position = "left",
#'   zebra.light = element_rect(),
#'   zebra.dark = element_rect(),
#'   table.spacing = unit(5, "mm"),
#'   point = element_point(),
#'   connector = element_line()
#' ))
#' ```
#'
#'
#' @returns A `<Guide>` object.
#' @export
#' @family standalone guides
#'
#' @examples
#' # Example plot
#' p <- ggplot(mpg, aes(paste(drv, year))) +
#'   geom_bar()
#'
#' # A standard upset axis might not have right order of levels
#' p + guides(x = "axis_upset")
#'
#' # The levels can be manually adjusted to taste
#' p + guides(x = guide_axis_upset(c("1999", "2008", "4", "f", "r")))
#'
#' # The connections can be turned off to just show symbols
#' p + guides(x = guide_axis_upset(connect = NULL))
#'
#' # The style can be changed per group of symbols.
#' # We need to give 3 colours to also cover NA-breaks
#' p + guides(x = guide_axis_upset(
#'   override.aes = list(colour = c("purple", "orange", NA))
#' ))
#'
#' # For symbol guides you have to manually specify where you want symbols and
#' # connection lines appear.
#' p + guides(x = guide_axis_symbols(
#'   key_symbols(
#'     aesthetic = c("4 1999", "4 2008", "r 1999"),
#'     level = c("Lvl 1", "Lvl 2", "Lvl 3")
#'   ),
#'   connect = data.frame(
#'     value_start = "4 2008", value_end = "r 1999",
#'     level_start = 2, level_end = 3
#'   )
#' ))
guide_axis_symbols <- function(
  key = NULL,
  connect = NULL,
  title = waiver(),
  theme = NULL,
  override.aes = list(),
  position = waiver(),
  direction = NULL,
  call = NULL
) {
  call <- call %||% current_call()
  if (is.null(key)) {
    cli::cli_abort(
      "The {.arg key} argument is required. You can set a manual key \\
      using {.fn key_symbols}.",
      call = call
    )
  }

  check_connect_arg(connect)

  new_guide(
    key = key,
    title = title,
    theme = theme,
    connect = connect,
    override.aes = override.aes,
    position = position,
    direction = direction,
    available_aes = c("x", "y", "any"),
    call = call,
    super = GuideAxisSymbols
  )
}

#' @rdname guide_axis_symbols
#' @export
guide_axis_upset <- function(
  key = "upset",
  connect = "perpendicular",
  title = waiver(),
  theme = NULL,
  override.aes = list(),
  position = waiver(),
  direction = NULL,
  call = NULL
) {

  if (is_character(key) && !(length(key) == 1L && is_key_string(key))) {
    key <- key_upset(order = key)
  }

  call <- call %||% current_call()
  if (is.null(key)) {
    cli::cli_abort(
      "The {.arg key} argument is required. You can set a key \\
      using {.fn key_upset}.",
      call = call
    )
  }

  check_connect_arg(connect)


  guide_axis_symbols(
    key = key,
    title = title,
    theme = theme,
    override.aes = override.aes,
    connect = connect,
    position = position,
    direction = direction,
    call = call
  )
}

# Class -------------------------------------------------------------------

#' @export
#' @rdname legendry_extensions
#' @format NULL
#' @usage NULL
GuideAxisSymbols <- ggproto(
  "GuideAxisSymbols", Guide,

  params = new_params(
    key = "upset", connect = NULL, override.aes = list(), call = NULL
  ),

  elements = list(
    position  = list(
      text    = I("legendry.axis.subtitle"),
      text_position = I("legendry.axis.subtitle.position"),
      light   = I("legendry.zebra.light"),
      dark    = I("legendry.zebra.dark"),
      hlines  = I("legendry.table.hlines"),
      vlines  = I("legendry.table.vlines"),
      spacing = I("legendry.table.spacing"),
      point   = I("legendry.symbol"),
      connector = I("legendry.connector")
    ),
    legend = list(
      text    = I("legendry.legend.subtitle"),
      text_position = I("legendry.legend.subtitle.position"),
      light   = I("legendry.zebra.light"),
      dark    = I("legendry.zebra.dark"),
      hlines  = I("legendry.table.hlines"),
      vlines  = I("legendry.table.vlines"),
      spacing = I("legendry.table.spacing"),
      point   = I("legendry.symbol"),
      connector = I("legendry.connector")
    )
  ),

  transform = function(self, params, coord, panel_params) {
    for (i in c("key", "decor")) {
      params[[i]] <-
        transform_key(params[[i]], params$position, coord, panel_params)
    }
    params
  },

  extract_key = function(scale, aesthetic, key, ...) {
    key <- standard_extract_key(scale, aesthetic, key, ...)
    key$.symbol <- key$.symbol %||% 1L
    key <- vec_slice(key, !is.na(key$.symbol))
    symbol <- key$.symbol
    if (is.logical(symbol)) {
      index <- match(symbol, c(TRUE, FALSE, NA), nomatch = 3L)
    } else if (is_integerish(symbol)) {
      index <- as.integer(symbol)
    } else {
      index <- match(symbol, levels(symbol) %||% sort(unique(symbol)))
    }
    key$.index <- index
    key
  },

  # Decor in the symbol/upset guide are connections between symbols
  extract_decor = function(scale, aesthetic, key, connect, ...) {
    if (is.null(connect)) {
      return()
    }

    # Custom, user-defined connections
    if (is.data.frame(connect)) {

      level <- vec_interleave(connect$level_start, connect$level_end)
      value <- vec_interleave(connect$value_start, connect$value_end)
      value <- scale$map(value)

      required_cols <- c("value_start", "value_end", "level_start", "level_end")
      extra_cols <- setdiff(names(connect), required_cols)
      extra <- NULL
      if (length(extra_cols) > 0L) {
        extra <- vec_rep_each(connect[extra_cols], 2L)
        extra <- rename_aes(extra, arg = "connect")
        names(extra) <- paste0(".", names(extra))
      }
      connect <- data_frame0(
        !!aesthetic := value,
        .level = level,
        .id = rep(vec_seq_along(connect), each = 2L),
        !!!extra
      )
      return(connect)
    }

    if (!is_logical(key$.symbol)) {
      cli::cli_abort(
        "{.arg connect} cannot be {.val {connect}} when key symbols are \\
        not logical."
      )
    }

    # Upset connections
    major <- switch(connect, perpendicular = ".row", ".col")
    minor <- setdiff(c(".row", ".col"), major)

    connect <- vec_split(key, key[[major]])
    connect <- lapply(connect$val, function(df) {

      hits <- df$.symbol & !is.na(df$.symbol)
      if (sum(hits) < 2L) {
        return(NULL)
      }
      i <- which(df$.symbol)
      minor <- df[[minor]][i]
      i <- i[c(which.max(minor), which.min(minor))]
      vec_slice(df, i)

    })
    connect <- vec_rbind(!!!connect)
    connect$.id <- rep(seq(nrow(connect) / 2L), each = 2L)
    connect
  },

  extract_params = function(scale, params, key, ...) {
    # Ensure override.aes is setup with the right lengths
    override <- params$override.aes
    if (is.logical(params$key$.symbol)) {
      override$shape <- override$shape %||% c(19L, 1L, 12L)
      n <- 3L
    } else {
      n <- max(params$key$.index)
    }

    wrong <- which(!(lengths(override) %in% c(1L, n)))
    if (length(wrong)) {
      problems <- paste0("override.aes$", names(override)[wrong])
      lens <- lengths(override)[wrong]
      lens <- paste0(
        "size", if (length(lens) > 1L) "s", " ",
        oxford_comma(lens, final = "and")
      )
      cli::cli_abort(
        "Can't recycle {.and {.arg {problems}}} ({lens}) to size {n}.",
        call = params$call
      )
    }
    override <- rename_aes(override)
    params$override.aes <- df_list(!!!override, .size = n)
    params
  },

  setup_params = function(params) {
    key <- params$key
    key$.symbol <- key$.symbol %||% TRUE
    if (params$position %in% c("top", "bottom")) {
      key$.row <- key$.row %||% 1L
    } else {
      key$.col <- key$.col %||% 1L
    }
    params$key <- key
    params
  },

  setup_elements = function(params, elements, theme) {
    primitive_setup_elements(params, elements, theme)
  },

  override_elements = function(params, elements, theme) {
    elements$text_position <-
      switch(
        params$position,
        top = , bottom = setdiff(elements$text_position, c("top", "bottom")),
        setdiff(elements$text_position, c("left", "right"))
      )[[1L]]
    type <- if (any(params$aesthetic %in% c("x", "y"))) "axis" else "legend"
    elements$text <- elements$title <-
      setup_side_title(theme, elements$text_position, type)
    elements
  },

  build_labels = function(key, elements, params) {
    labels <- levels(key$.value)
    if (length(labels) < 1L) {
      return(NULL)
    }
    lapply(labels, function(lab) {
      element_grob(elements$text, label = lab, margin_x = TRUE, margin_y = TRUE)
    })
  },

  build_decor = function(self, decor, grobs, elements, params) {
    key <- params$key
    key <- vec_slice(key, !is.na(key$.symbol))
    levels <- vec_group_loc(key$.col)
    levels <- vec_slice(levels, order(levels$key))
    point <- elements$point

    groups <- vec_group_id(key$.symbol)
    groups[is.na(key$.symbol)] <- NA
    n_groups <- attr(groups, "n")

    override <- params$override.aes
    x <- switch(params$position, top = , bottom = key$x, rep(0.5, nrow(key)))
    y <- switch(params$position, left = , right = key$y, rep(0.5, nrow(key)))

    points <- lapply(levels$loc, function(level) {
      grobs <- list()
      size_tracker <- point$size
      for (group_id in seq_len(n_groups)) {
        member <- level[groups[level] == group_id]
        if (length(member) < 1L) {
          next
        }
        key_members <- vec_slice(key, member)
        props <- element_key_properties(
          key_members, "point",
          !!!lapply(override, vec_slice, i = key_members$.index)
        )
        size_tracker <- max(size_tracker, props$size)
        grob <- inject(element_grob(
          point,
          x = unit(x[member], "native"),
          y = unit(y[member], "native"),
          !!!props
        ))
        grobs <- c(grobs, list(grob))
      }
      gTree(children = inject(gList(!!!grobs)), size = size_tracker)
    })

    size <- map_dbl(points, function(x) x$size)
    size <- unit(size + height_cm(elements$spacing) * 10.0, "mm")
    along <- seq_along(points)

    zebra <- list(element_grob(elements$light), element_grob(elements$dark))
    zebra <- rep_len(zebra, length(size))

    if (params$position %in% c("top", "bottom")) {
      gt <- gtable(widths = unit(1.0, "npc"), heights = size) |>
        gtable_add_grob(
          points, l = 1L, t = along, z = 3L,
          name = paste0("symbols-", along)
        ) |>
        gtable_add_grob(
          zebra, l = 1L, t = along, z = 0L,
          name = paste0("zebra-", along)
        )
    } else {
      gt <- gtable(widths = size, heights = unit(1.0, "npc")) |>
        gtable_add_grob(
          points, l = along, t = 1L, z = 3L,
          name = paste0("symbols-", along)
        ) |>
        gtable_add_grob(
          zebra, l = along, t = 1L, z = 0L,
          name = paste0("zebra-", along)
        )
    }
    gt
  },

  measure_grobs = function(grobs, params, elements) {
    if (params$position %in% c("top", "bottom")) {
      lab_height <- height_cm(grobs$labels %||% unit(0.0, "cm"))
      tab_height <- height_cm(grobs$decor$heights)
      pmax(lab_height, tab_height)
    } else {
      lab_width <- width_cm(grobs$labels %||% unit(0.0, "cm"))
      tab_width <- width_cm(grobs$decor$widths)
      pmax(lab_width, tab_width)
    }
  },

  assemble_drawing = function(grobs, layout, sizes, params, elems) {
    connectors <- draw_connectors(params$decor, params, elems, sizes)

    labels <- grobs$labels
    along  <- seq_along(labels)
    table  <- grobs$decor

    if (params$position %in% c("top", "bottom")) {
      width <- unit(max(width_cm(labels)), "cm")
      table$heights <- unit(sizes, "cm")
      if (!is_zero(connectors)) {
        table <- gtable_add_grob(
          table, connectors, l = 1L, t = 1L, b = -1L,
          name = "connectors", clip = "off", z = 2L
        )
      }
      if (elems$text_position == "left") {
        table <- table |>
          gtable_add_cols(c(-1.0, 1.0) * width, pos = 0L) |>
          gtable_add_grob(
            labels, t = along, l = 2L,
            name = paste0("label-", along)
          )
      } else {
        table <- table |>
          gtable_add_cols(c(1.0, -1.0) * width, pos = -1L) |>
          gtable_add_grob(
            labels, t = along, l = 2L,
            name = paste0("label-", along)
          )
      }
    } else {
      height <- unit(max(height_cm(labels)), "cm")
      table$widths <- unit(sizes, "cm")
      if (!is_zero(connectors)) {
        table <- gtable_add_grob(
          table, connectors, l = 1L, r = -1L, t = 1L,
          name = "connectors", clip = "off", z = 2L
        )
      }
      if (elems$text_position == "bottom") {
        table <- table |>
          gtable_add_rows(c(1.0, -1.0) * height, pos = -1L) |>
          gtable_add_grob(
            labels, t = 2L, l = along,
            name = paste0("label-", along),
            clip = "off"
          )
      } else {
        table <- table |>
          gtable_add_rows(c(-1.0, 1.0) * height, pos = 0L) |>
          gtable_add_grob(
            labels, t = 2L, l = along,
            name = paste0("label-", along),
            clip = "off"
          )
      }
    }
    table
  }
)

check_connect_arg <- function(
  connect, arg = caller_arg(connect), env = caller_env()
) {
  if (is.null(connect)) {
    return(invisible())
  }
  if (is.character(connect)) {
    arg_match0(
      connect, c("perpendicular", "parallel"),
      arg_nm = arg, error_call = env
    )
    return(invisible())
  }
  cols <- c("value_start", "value_end", "level_start", "level_end")
  check_columns(connect, cols)
  for (col in cols[3L:4L]) {
    check_object(
      connect[[col]],
      is_integerish,
      what = "{.cls integer}",
      arg = paste0(arg, "$", col),
      call = env
    )
  }
  invisible()
}

draw_connectors <- function(decor, params, elems, sizes) {
  if (is.null(params$connect) || is.null(decor)) {
    return(zeroGrob())
  }

  levels <- decor[[".level"]] %||% decor[[".col"]]
  if (params$position %in% c("top", "bottom")) {
    oppo <- sum(sizes) - cumsum(sizes) + sizes / 2.0
    x <- unit(decor$x, "native")
    y <- unit(oppo[levels], "cm")
  } else {
    oppo <- c(0.0, cumsum(sizes[-length(sizes)])) + sizes / 2.0
    x <- unit(oppo[levels], "cm")
    y <- unit(decor$y, "native")
  }

  element_grob(
    elems$connector,
    x = x, y = y,
    id.lengths = vec_unrep(decor$.id)$times,
    colour = decor[[".colour"]],
    linewidth = decor[[".linewidth"]],
    linetype = decor[[".linetype"]]
  )
}

test_that("connectors works", {

  expect_silent(check_connect_arg("perpendicular"))
  expect_snapshot_error(
    check_connect_arg(data.frame(foo = 1L:2L))
  )


  df <- data.frame(
    x    = c(3.0, 3.0),
    y    = c(3.0, 3.0),
    .col = c(2L, 3L),
    .row = c(3L, 3L),
    .id  = c(1L, 1L),
    .symbol = TRUE
  )

  line <- draw_connectors(
    df,
    list(connect = "perpendicular", position = "top"),
    sizes = c(1.0, 1.0, 1.0),
    elems = list(connector = element_line())
  )
  expect_s3_class(line, "polyline")
  expect_equal(as.numeric(line$x), c(3.0, 3.0))
  expect_equal(as.numeric(line$y), c(1.5, 0.5))

  line <- draw_connectors(
    df,
    list(connect = "perpendicular", position = "left"),
    sizes = c(1.0, 1.0, 1.0),
    elems = list(connector = element_line())
  )
  expect_equal(as.numeric(line$x), c(1.5, 2.5))
  expect_equal(as.numeric(line$y), c(3.0, 3.0))

  line <- draw_connectors(
    df,
    list(connector = NULL, position = "left"),
    sizes = c(1.0, 1.0, 1.0),
    elems = list(connector = element_line())
  )
  expect_true(is_zero(line))
})

test_that("guide_axis_symbols does input checks", {
  expect_snapshot_error(guide_axis_symbols())
})

test_that("guide_axis_upset treats input correctly", {
  expect_snapshot_error(guide_axis_upset(key = NULL))
  expect_silent(guide_axis_upset())
})

test_that("guide_axis_upset forwards key order argument", {
  g <- guide_axis_upset(c("foo", "qux", "bar"))
  key <- g$params$key(scale_x_discrete(limits = c("foo,bar", "qux,bar", "")))
  expect_identical(levels(key$.value), c("foo", "qux", "bar", "Other"))
})

test_that("guide_axis_upset gives informative error for `override.aes`", {
  override <- list(size = 5.0, colour = c("red", "blue", "green", "purple"))
  p <- ggplot() +
    scale_x_discrete(
      limits = c("foo,bar", "bar"),
      guide = guide_axis_upset(override.aes = override)
    )
  expect_snapshot_error(ggplotGrob(p))
})

test_that("guide_axis_symbols and guide_axis_upset can be drawn", {
  df <- data_frame0(
    x = c("", "A", "A,B", "B,C", "A,C"),
    y = c("X;Y", "X;Z", "Y;Z", "X;Y;Z", "")
  )

  symbol_key <- key_symbols(
    rep(1L:5L, c(1L:4L, 2L)),
    5L - c(1L, 1L, 2L, 1L, 2L, 3L, 1L, 2L, 3L, 4L, 1L, 2L),
    c(1L, 2L, 1L, 2L, 2L, 1L, 1L, 2L, 2L, 1L, 2L, 1L),
    size = 3.0
  )

  connector <- data_frame0(
    value_start = 1.0, value_end = 4.0,
    level_start = 4L, level_end = 1L,
    colour = "red"
  )

  p <- ggplot(df, aes(x, y)) +
    geom_point() +
    guides(
      x = guide_axis_upset(
        theme = theme(
          legendry.symbol = element_point(size = 3.0),
          legendry.connector = element_line(linetype = "dotted")
        )
      ),
      y = guide_axis_upset(
        key_upset(empty_label = NULL, order = c("Z", "Y", "X")),
        connect = "parallel",
        override.aes = list(shape = c(15L, 0L, 2L))
      ),
      x.sec = guide_axis_symbols(
        symbol_key,
        override.aes = list(colour = c("red", "blue")),
        connect = connector,
        theme = theme(legendry.axis.subtitle.position = "right")
      ),
      y.sec = guide_axis_symbols(
        key_symbols(df$y, level = LETTERS[seq_len(nrow(df))]),
        connect = NULL
      )
    )

  vdiffr::expect_doppelganger("upset and symbol guides", p)
})

# Constructor -------------------------------------------------------------

#' Compose guides as stack
#'
#' @description
#' This guide can stack other guides.
#'
#' @param ... Guides to stack in [composition][guide-composition]. Each guide
#'   can be specified as one of the following:
#'   * A `<Guide>` class object.
#'   * A `<function>` that returns a `<Guide>` class object.
#'   * A `<character[1]>` naming such a function, without the `guide_` or
#'   `primitive_` prefix.
#' @param key A [standard key][key_standard] specification. The key is shared
#'   among all guides that have `NULL` keys themselves. See more information
#'   in the linked topic.
#' @inheritParams guide-composition
#' @param side.titles A `<character>` giving labels for titles displayed on the
#'   side of the stack. Set to `NULL` to display no side titles. If `waiver()`,
#'   an attempt is made to extract the titles from the guides and use these
#'   as side titles.
#' @param drop An `<integer>` giving the indices of guides that should be
#'   dropped when a facet requests no labels to be drawn at axes in between
#'   panels. The default, `NULL`, will drop every guide except the first.
#' @inheritParams common_parameters
#'
#' @details
#' ## Styling options
#'
#' Below are the [theme][ggplot2::theme] options that determine the styling of
#' this guide.
#'
#' | **Theme setting** | **Type** | **Description** |
#' | ----------------- | -------- | --------------- |
#' | `legendry.axis.subtitle` | [`element_text()`] | Display of the `side.titles` argument's labels. |
#' | `legendry.axis.subtitle.position` | `<character[1]>` | One of `"top"`, `"right"`, `"bottom"` or `"left"`. |
#' | `legendry.guide.spacing` | [`unit()`] | Spacing between guides. |
#'
#' There are no further styling options.
#'
#' The context-agnostic alternative to using `theme()` is to use
#' [`theme_guide()`]:
#'
#' ```r
#' compose_stack(theme = theme_guide(
#'   subtitle = element_text(),
#'   subtitle.position = "left",
#'   guide.spacing = unit(5, "mm")
#' ))
#' ```
#'
#' @return A `<ComposeStack>` guide object.
#' @export
#' @family composition
#'
#' @examples
#' ggplot() +
#'   geom_function(fun = dnorm, xlim = c(-3, 3)) +
#'   guides(x = compose_stack(
#'     "axis", "axis",
#'     side.titles = c("first", "second")
#'   )) +
#'   # Add margin to make room for side titles
#'   theme(plot.margin = margin(5.5, 5.5, 5.5, 11))
compose_stack <- function(
  ..., args = list(),
  key = NULL, title = waiver(), side.titles = waiver(),
  angle = waiver(), theme = NULL, order = 0L, drop = NULL,
  position = waiver(), available_aes = NULL
) {
  new_compose(
    guides = list2(...),
    title = title,
    theme = theme,
    key = key,
    angle = angle,
    side_titles = side.titles,
    drop = drop,
    available_aes = available_aes,
    order = order,
    position = position,
    name = "stack_composition",
    super = ComposeStack
  )
}

# Class -------------------------------------------------------------------

#' @export
#' @rdname legendry_extensions
#' @format NULL
#' @usage NULL
ComposeStack <- ggproto(
  "ComposeStack", Compose,

  params = new_params(
    guides = list(), guide_params = list(),
    key = NULL, side_titles = NULL, angle = waiver(), drop = NULL
  ),

  elements = list(
    side_titles   = "legendry.axis.subtitle",
    side_position = "legendry.axis.subtitle.position",
    spacing       = "legendry.guide.spacing"
  ),

  train = function(self, params = self$params, scale, aesthetic = NULL, ...) {
    params$side_titles <- get_side_titles(
      params$side_titles, params$guide_params, call = expr(compose_stack())
    )
    Compose$train(params = params, scale = scale, aesthetic = aesthetic, ...)
  },

  transform = function(self, params, coord, panel_params) {
    params <- Compose$transform(params, coord, panel_params)
    if (!is.null(params$side_titles)) {
      params$sides <- get_sides(coord, panel_params)
    }
    params
  },

  setup_elements = function(self, params, elements, theme) {
    side_position <- NULL
    if (!is.null(params$side_titles)) {
      side_position <- elements$side_position
      if (!side_position %in% .trbl) {
        side_position <- calc_element(elements$side_position, theme)
      }
      type <- if (any(params$aesthetic %in% c("x", "y"))) "axis" else "legend"
      side_position <- switch(
        params$position,
        top = , bottom = , theta = , theta.sec =
          setdiff(side_position, c("top", "bottom")),
        left = , right =
          setdiff(side_position, c("left", "right"))
      )
      if (length(side_position) > 1L) {
        if (type == "axis") {
          side_position <- side_position[1L]
        } else {
          # When we are a non-position guide, we don't want to interfere
          # with the title, so we try to avoid placing the side titles there
          title_position <- calc_element("legend.title.position", theme) %||%
            switch(params$direction, vertical = "top", horizontal = "left")
          side_position <- setdiff(side_position, title_position)[1L]
        }
      }
      check_argmatch(side_position, .trbl)
      elements$side_titles <- setup_side_title(theme, side_position, type)
    }
    elements <-
      ggproto_parent(self, Guide)$setup_elements(params, elements, theme)
    elements$side_position <- side_position %||% elements$side_position
    elements
  },

  override_elements = function(params, elements, theme) {
    if (!is_theta(params$position)) {
      elements$spacing <- cm(elements$spacing)
    }


    elements
  },

  build_title = function(label, elements, params) {
    if (length(label) < 1L) {
      return(NULL)
    }

    side  <- elements$side_position
    sides <- params$sides %||% get_sides()
    sides <- vec_slice(sides, sides$position == params$position)
    sides <- vec_slice(sides, sides$side == side)

    element <- elements$side_titles

    x <- switch(side, left = , right = sides$x, element$hjust)
    y <- switch(side, top = , bottom = sides$y, element$vjust)

    hjust <- switch(side, left = 1.0, right  = 0.0, element$hjust)
    vjust <- switch(side, top  = 0.0, bottom = 1.0, element$vjust)

    lapply(label, function(lab) {
      element_grob(
        element, lab, x = x, y = y,
        margin_x = TRUE, margin_y = TRUE,
        hjust = hjust, vjust = vjust
      )
    })
  },

  draw = function(self, theme, position = NULL, direction = NULL,
                  params = self$params) {
    n_guides <- length(params$guides)
    if (n_guides < 1L) {
      return(zeroGrob())
    }

    theme <- theme + params$theme
    position <- params$position <- params$position  %||% position
    position <- switch(position, inside = "right", position)
    check_position(position)
    params$guide_params <-
      set_list_element(params$guide_params, "position", position)
    direction <- params$direction <- params$direction %||% direction

    elems <- self$setup_elements(params, self$elements, theme)
    elems <- self$override_elements(params, elems, theme)

    guide_index <- seq_len(n_guides)
    grobs  <- vector("list", n_guides)
    draw_label <- !isFALSE(params$draw_label %||% TRUE)

    keep <- rep(TRUE, n_guides)
    if (!draw_label && length(params$drop) > 0L) {
      drop <- intersect(params$drop %||% guide_index[-1L], guide_index)
      keep[drop] <- FALSE
    }

    if (is_theta(position)) {
      stack_offset <- unit(cm(params$stack_offset %||% 0.0), "cm")
      offset <- stack_offset
      offset_ranges <- vector("list", n_guides)
      guide_index <- guide_index[keep]

      for (i in guide_index) {
        pars <- params$guide_params[[i]]
        pars$draw_label <- params$draw_label
        pars$stack_offset <- offset
        grob <- params$guides[[i]]$draw(
          theme = theme, position = position, direction = direction,
          params = pars
        )
        if (!is.null(grob$offset) && !is_zero(grob)) {
          offset_ranges[[i]] <- unit.c(offset, grob$offset)
          offset <- unit(cm(offset + elems$spacing + grob$offset), "cm")
        }
        grobs[[i]] <- grob
      }
      keep <- !map_lgl(grobs, is_zero)
      if (!any(keep)) {
        return(zeroGrob())
      }
      offset <- offset - stack_offset
      grobs <- grobs[keep]
      side_titles <-
        theta_side_titles(params$side_titles, elems, params, offset_ranges)
      grobs <- inject(gList(!!!grobs, side_titles))
      grob  <- gTree(offset = offset, children = grobs)
      return(grob)
    }

    side_titles <- self$build_title(params$side_titles, elems, params)
    lengths <- NULL

    for (i in guide_index) {
      pars <- params$guide_params[[i]]
      if (vec_size(pars$key) == length(lengths)) {
        pars$key$.length <- pars$key$.length %||% lengths
      }
      pars$draw_label <- draw_label
      grobs[[i]] <- params$guides[[i]]$draw(
        theme = theme, position = position, direction = direction, params = pars
      )
      lengths <- get_attr(grobs[[i]], "lengths")
    }

    keep <- keep & !map_lgl(grobs, is_zero)
    grobs <- grobs[keep]
    if (length(grobs) == 0L) {
      return(zeroGrob())
    }
    side_titles <- side_titles[keep]

    origin  <- unit(as.numeric(position %in% c("left", "bottom")), "npc")
    just    <- opposite_position(position)
    along   <- seq_along(grobs)
    widths  <- width_cm(grobs)
    heights <- height_cm(grobs)

    if (position %in% c("top", "left")) {
      along   <- rev(along)
      widths  <- rev(widths)
      heights <- rev(heights)
    }

    if (position %in% c("bottom", "top")) {
      gt <- gtable(widths = unit(1.0, "npc"), heights = unit(heights, "cm")) |>
        gtable_add_grob(
          grobs, t = along, l = 1L,
          name = "stack", clip = "off"
        ) |>
        add_side_titles(side_titles, params$position) |>
        gtable_add_row_space(height = unit(elems$spacing, "cm"))
      vp <- viewport(y = origin, height = grobHeight(gt), just = just)
    } else {
      gt <- gtable(widths = unit(widths, "cm"), heights = unit(1.0, "npc")) |>
        gtable_add_grob(
          grobs, t = 1L, l = along,
          name = "stack", clip = "off"
        ) |>
        add_side_titles(side_titles, params$position) |>
        gtable_add_col_space(width = unit(elems$spacing, "cm"))
      vp <- viewport(x = origin, width = grobWidth(gt), just = just)
    }

    absoluteGrob(
      grob = gList(gt), vp = vp,
      width = gtable_width(gt), height = gtable_height(gt)
    )
  }
)


# Helpers -----------------------------------------------------------------

get_side_titles <- function(side_titles, params, call = caller_env()) {

  if (is_waive(side_titles)) {
    side_titles <- lapply(params, function(p) (p$title %|W|% NULL) %||% NA)
    side_titles <- unlist(lapply(side_titles, `[[`, i = 1L), recursive = FALSE)
    if (all(is.na(side_titles))) {
      side_titles <- names(params)
    }
  }
  if (is.null(side_titles)) {
    return(NULL)
  }
  if (!is_waive(side_titles) && length(side_titles) != length(params)) {
    cli::cli_abort(c(
      "Must have a number of {.arg side.titles} equal to the number of guides.",
      i = "There are {length(side_titles)} side titles.",
      i = "There are {length(params)} guides."
    ), call = call)
  }
  side_titles
}

add_side_titles <- function(gt, titles, position) {
  if (is.null(titles)) {
    return(gt)
  }
  along <- seq_along(titles)
  t <- switch(position, left = , right = 1L, top  = rev(along), bottom = along)
  l <- switch(position, top = , bottom = 1L, left = rev(along), right  = along)
  gtable_add_grob(gt, titles, t = t, l = l, clip = "off", name = "side titles")
}

theta_side_titles <- function(label, elements, params, ranges) {
  if (length(label) < 1L) {
    return(zeroGrob())
  }

  ranges <- lapply(lapply(ranges, cm), cumsum)
  min <- map_dbl(ranges, min)
  max <- map_dbl(ranges, max)

  element <- elements$side_titles
  just <- justify_range(min, max, element$vjust)

  side <- elements$side_position
  sides <- vec_slice(params$sides, params$sides$side == side)
  position <- params$position

  origin_index <- switch(position, theta = which.max, which.min)(sides$r)
  origin <- vec_slice(sides, origin_index)

  rad <- origin$theta

  if (position == "theta") {
    dir <- switch(side, left = 1.0, right = -1.0)
  } else {
    rad <- rad + pi
    dir <- switch(side, left = -1.0, right = 1.0)
  }

  hjust <- switch(side, left = 1.0, right = 0.0, elements$hjust)
  margin <- max(cm(element$margin)) * dir

  xoffset <- sin(rad) * just - cos(rad) * margin
  yoffset <- cos(rad) * just + sin(rad) * margin

  x <- unit(origin$x, "npc") + unit(xoffset, "cm")
  y <- unit(origin$y, "npc") + unit(yoffset, "cm")

  angle <- -rad2deg(origin$theta) %% 360.0
  flip  <- angle > 90.0 & angle < 270.0
  if (flip) {
    angle <- angle + 180.0
    hjust <- 1.0 - hjust
  }

  element_grob(
    element, label, x = x, y = y,
    hjust = hjust,
    angle = angle
  )
}

get_sides <- function(coord = NULL, panel_params = NULL) {

  if (is.null(panel_params)) {
    x <- c(0.0, 1.0)
    y <- c(0.0, 1.0)
  } else {
    x <- panel_params$x.range %||%
      switch(coord$theta, x = panel_params$theta.range, panel_params$r.range)
    y <- panel_params$y.range %||%
      switch(coord$theta, y = panel_params$theta.range, panel_params$r.range)
  }

  df <- data_frame0(
    position = rep(.trbl, each = 2L),
    side     = rep(.trbl[c(4L, 2L, 1L, 3L)], 2L),
    x = x[c(1L, 2L, 2L, 2L, 1L, 2L, 1L, 1L)],
    y = y[c(2L, 2L, 2L, 1L, 1L, 1L, 2L, 1L)],
    group = 1L:8L
  )
  if (is.null(coord)) {
    return(df)
  }
  coord_munch(coord, df, panel_params)
}

test_that("compose_stack works as axis line", {

  base <- ggplot(mpg, aes(displ, hwy)) +
    geom_blank() +
    theme_test() +
    theme(
      panel.background = element_rect(fill = NA, colour = "grey80"),
      panel.grid.major = element_line(colour = "grey90"),
      panel.border = element_blank(),
      axis.line = element_line()
    )

  spacer <- primitive_spacer(space = unit(2.25, "pt"))
  stack <- compose_stack(
    "line", "ticks", "labels", spacer, "ticks", "line",
    side.titles = c("", "A", "", "", "B", ""),
    theme = theme(legendry.guide.spacing = unit(0.0, "pt"))
  )

  p <- base +
    guides(x = stack, x.sec = stack, y = stack, y.sec = stack)

  vdiffr::expect_doppelganger("compose_stack cartesian", p)

  p <- base +
    coord_radial(start = 0.25 * pi, end = 1.75 * pi, inner.radius = 0.5) +
    guides(theta = stack, theta.sec = stack, r = stack, r.sec = stack)

  vdiffr::expect_doppelganger("compose_stack radial", p)

})

test_that("get_side_titles throws appropriate warning", {
  expect_error(
    get_side_titles("", list()),
    "Must have a number"
  )
})

test_that("theta_side_titles does not error", {

  elements <- list(
    side_titles = theme_gray()$text,
    side_position = "left"
  )

  test <- withr::with_pdf(
    tempfile(fileext = ".pdf"),
    theta_side_titles(
      label = c("A", "B"),
      elements,
      ranges = list(c(0.0, 0.4), c(0.0, 0.4)),
      params = list(
        position = "theta",
        sides = data_frame0(
          theta = c(5.5, 0.8),
          r     = c(0.4, 0.4),
          position = c("left", "left"),
          group = 1L:2L,
          x = c(0.2, 0.6),
          y = c(0.9, 0.75),
          side = c("left", "left")
        )
      )
    )

  )

  expect_s3_class(test, "titleGrob")
})

check_columns <- function(data, columns, call = caller_env(),
                          arg = caller_arg(data)) {
  check_data_frame(data, arg = arg, call = call)
  if (all(columns %in% names(data))) {
    return(invisible())
  }
  missing <- setdiff(columns, names(data))
  cli::cli_abort(
    "The {.field {missing}} column{?s} {?is/are} required in {.arg {arg}}.",
    call = call
  )
}

check_list_names <- function(data, names, call = caller_env(),
                             arg = caller_arg(data)) {
  check_object(data, is.list, what = "a {.cls list}", arg = arg, call = call)
  if (all(names %in% names(data))) {
    return(invisible())
  }
  missing <- setdiff(names, names(data))
  a <- if (length(missing) == 1L) "a" else ""
  cli::cli_abort(paste0(
    "The {.arg {arg}} argument must have {a} named {.field {missing}} ",
    "element{?s}."
  ), call = call)
}

check_list_of <- function(x, class, allow_null = FALSE,
                          call = caller_env(), arg = caller_arg(x)) {
  problems <- character()
  if (!missing(x)) {
    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
    if (is.list(x)) {
      fail <- !map_lgl(x, inherits, what = class)
      if (!any(fail)) {
        return(invisible(NULL))
      }
      problems <- map_chr(x[fail], obj_type_friendly)
      problems <- paste0(arg, "[[", which(fail), "]] is ", problems)
      names(problems) <- rep("x", length(problems))
      if (length(problems) > 5L) {
        problems <- c(problems[1L:5L], x = "...and more mismatches.")
      }
    }
  }

  class <- map_chr(class, function(x) as_cli("{.cls {x}}"))
  end <- if (is.list(x)) "." else paste0(", not ", obj_type_friendly(x), ".")

  message <- sprintf(
    "`%s` must be %s%s",
    arg, as_cli("a {.cls list} object with {.or {class}} elements"),
    end
  )
  message <- c(message, problems)
  abort(message, call = call, arg = arg)
}

check_grob <- function(x, allow_null = FALSE, call = caller_env(),
                       arg = caller_arg(x)) {
  if (!missing(x)) {
    if (is.grob(x)) {
      return(invisible(NULL))
    }
    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
  }
  stop_input_type(
    x, as_cli("a {.cls grob} object"),
    allow_null = allow_null, arg = arg, call = call
  )
}

check_unit <- function(x, allow_null = FALSE, allow_rel = FALSE,
                       call = caller_env(), arg = caller_arg(x)) {
  if (!missing(x)) {
    if (is.unit(x)) {
      return(invisible(NULL))
    }
    if (allow_null && is_null(x) || allow_rel && is_rel(x)) {
      return(invisible(NULL))
    }
  }
  stop_input_type(
    x, as_cli("a {.cls unit} object"),
    allow_null = allow_null, arg = arg, call = call
  )
}

check_bare_numeric <- function(x, ..., allow_null = FALSE,
                               arg = caller_arg(x), call = caller_env()) {
  if (!missing(x)) {
    if (is_bare_numeric(x)) {
      return(invisible(NULL))
    }
    if (allow_null && is_null(x)) {
      return(invisible())
    }
  }
  stop_input_type(
    x, as_cli("a bare {.cls numeric} vector"), ...,
    allow_null = allow_null, arg = arg, call = call
  )
}

check_object <- function(
  x, check_fun, what, ...,
  allow_null = FALSE, arg = caller_arg(x), call = caller_env()
) {
  if (!missing(x)) {
    if (check_fun(x)) {
      return(invisible(NULL))
    }
    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
  }
  stop_input_type(
    x, as_cli(what), ...,
    allow_null = allow_null, arg = arg, call = call
  )
}

check_length <- function(
  x, min = NULL, max = NULL, exact = NULL,
  allow_null = FALSE, arg = caller_arg(x), call = caller_env()
) {
  if (!missing(x)) {
    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
    length <- length(x)
    if (!is.null(min %||% max)) {
      between <- c(min %||% 0L, max %||% Inf)
      if (in_range(length, between)) {
        return(invisible(NULL))
      }
    }
    if (!is.null(exact) && length %in% exact) {
      return(invisible(NULL))
    }
  }
  if (is.null(min) && is.null(max) && is.null(exact)) {
    return(invisible(NULL))
  }
  type <- if (typeof(x) == "list") "list" else "vector"
  if (!is.null(min) && !is.null(max)) {
    what <- "a {type} with {.field length} between {min} and {max}"
  } else if (!is.null(min)) {
    what <- "a {type} with {.field length} more than or equal to {min}"
  } else if (!is.null(max)) {
    if (max > 1L) {
      what <- "a {type} with {.field length} less than or equal to {max}"
    } else {
      what <- "a value with {.field length} less than or equal to {max}"
    }
  } else if (!is.null(exact)) {
    if (any(exact > 1L)) {
      what <- "a {type} with {.field length} equal to {.or {exact}}"
    } else {
      what <- "a single value with {.field length} equal to {.or {exact}}"
    }
  }
  obj <- obj_type_friendly(x, value = FALSE)
  msg <- paste0(
    "{.arg {arg}} must be {as_cli(what)}, ",
    "not {obj} with length {length(x)}."
  )
  cli::cli_abort(msg, call = call)
}

check_inherits <- function(
  x, class, what = NULL, ...,
  allow_null = FALSE, arg = caller_arg(x), call = caller_env()
) {
  if (!missing(x)) {
    if (inherits(x, class)) {
      return(invisible(NULL))
    }
    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
  }
  what <- what %||% paste(
    "a", oxford_comma(paste0("{.cls ", class, "}")), "object"
  )

  stop_input_type(
    x, as_cli(what), ...,
    allow_null = allow_null, arg = arg, call = call
  )
}

check_argmatch <- function(
  x, options,
  ...,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (!missing(x)) {
    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
    if (is.character(x)) {
      arg_match0(x, options, arg_nm = arg, error_call = call)
      return(invisible(NULL))
    }
  }

  stop_input_type(
    x, "a single string", ...,
    allow_na = FALSE, allow_null = allow_null,
    arg = arg, call = call
  )
}

check_position <- function(
  x, options = .trbl, theta = TRUE, inside = FALSE,
  ...,
  allow_null = FALSE,
  arg = caller_arg(x), call = caller_env()
) {
  if (!missing(x)) {
    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
    if (is.character(x)) {
      if (theta) {
        options <- c(options, "theta", "theta.sec")
      }
      if (inside) {
        options <- c(options, "inside")
      }
      arg_match0(x, options, arg_nm = arg, error_call = call)
      return(invisible(NULL))
    }
  }

  stop_input_type(
    x, "a single string", ...,
    allow_na = FALSE, allow_null = allow_null,
    arg = arg, call = call
  )
}

check_unique <- function(x, arg = caller_arg(x), call = caller_env()) {
  if (!vec_duplicate_any(x)) {
    return(invisible())
  }
  dups <- unique(x[vec_duplicate_detect(x)])
  more <- if (length(dups) > 5L) " and more." else "."
  dups <- dups[seq_len(pmin(5L, length(dups)))]
  n <- length(dups)
  cli::cli_abort(c(
    "{.arg {arg}} must only have unique values.",
    i = paste0("Example {cli::qty(n)}duplicate{?s}: {.and {.val {dups}}}", more)
  ))
}

check_exclusive <- function(
  x, y, required = FALSE,
  x_arg = caller_arg(x), y_arg = caller_arg(y), call = caller_env()
) {
  x_present <- !(is_missing(x) || is.null(x))
  y_present <- !(is_missing(y) || is.null(y))
  if (xor(x_present, y_present)) {
    return(invisible())
  }
  if (required && !x_present && !y_present) {
    cli::cli_abort(
      "Either the {.arg {x_arg}} or {.arg {y_arg}} argument is required.",
      call = call
    )
  }
  if (!x_present && !y_present) {
    return(invisible())
  }
  cli::cli_abort(c(
    "The {.arg {x_arg}} and {.arg {y_arg}} arguments are mutually exclusive.",
    i = "Please use one, but not both."
  ), call = call)
}

check_matrix <- function(
  x, allow_null = FALSE, zero_dim = FALSE,
  arg = caller_arg(x), call = caller_env()
) {
  check_object(
    x, is.matrix, "a {.cls matrix}", allow_null = allow_null,
    arg = arg, call = call
  )

  # Test dimensions
  dim <- dim(x)
  valid_dim <- length(dim) == 2L && !anyNA(dim) &&
    all(dim >= (0L + as.integer(!zero_dim)))

  if (valid_dim) {
    return(invisible(NULL))
  }

  cli::cli_abort(
    "The {.arg {arg}} argument has invalid dimensions: {.value {dim}}.",
    call = call
  )
}

test_that("check_columns throws appropriate errors", {
  expect_silent(check_columns(mtcars, c("mpg", "cyl")))
  expect_error(
    check_columns(mtcars, c("foo", "bar")),
    "columns are required"
  )
})

test_that("check_list_names throws appropriate errors", {

  lst <- as.list(mtcars)
  expect_silent(check_list_names(lst, c("mpg", "cyl")))
  expect_error(
    check_list_names(lst, c("foo", "bar")),
    "must have named"
  )

})

test_that("check_unit throws appropriate errors", {

  expect_silent(check_unit(unit(1.0, "cm")))
  expect_silent(check_unit(NULL, allow_null = TRUE))
  expect_error(
    check_unit(10.0),
    "must be"
  )

})

test_that("check_bare_numeric throws appropriate errors", {

  expect_silent(check_bare_numeric(c(10L, 20L)))
  expect_silent(check_bare_numeric(NULL, allow_null = TRUE))
  expect_error(
    check_bare_numeric(unit(1.0, "cm")),
    "must be a bare"
  )

})

test_that("check_object throws appropriate errors", {

  expect_silent(
    check_object(mtcars, is.data.frame, "data.frame")
  )
  expect_silent(
    check_object(NULL, is.data.frame, "data.frame", allow_null = TRUE)
  )
  expect_error(
    check_object(mtcars, is.environment, "an environment"),
    "must be an environment"
  )

})

test_that("check_length throws appropriate errors", {

  expect_silent(check_length(NULL, allow_null = TRUE))
  expect_silent(check_length(1L:2L, min = 1L))
  expect_silent(check_length(1L:2L, max = 2L))
  expect_silent(check_length(1L:2L, min = 1L, max = 3L))
  expect_silent(check_length(1L:2L, exact = 2L))

  expect_error(
    check_length(1L:2L, min = 3L),
    "more than or equal to 3"
  )
  expect_error(
    check_length(1L:2L, max = 1L),
    "less than or equal to 1"
  )
  expect_error(
    check_length(1L:2L, min = 3L, max = 5L),
    "between 3 and 5"
  )
  expect_error(
    check_length(1L:2L, exact = c(1L, 3L)),
    "equal to 1 or 3"
  )
  expect_error(
    check_length(1L:2L, exact = 1L),
    "equal to 1"
  )

})

test_that("check_inherits throws appropriate errors", {

  expect_silent(check_inherits(mtcars, "data.frame"))
  expect_silent(check_inherits(NULL, "data.frame", allow_null = TRUE))
  expect_error(
    check_inherits(1L:2L, "data.frame"),
    "must be a "
  )

})

test_that("check_argmatch throws appropriate errors", {

  expect_silent(check_argmatch("A", c("A", "B")))
  expect_silent(check_argmatch(NULL, c("A", "B"), allow_null = TRUE))
  expect_snapshot_error(check_argmatch("C", c("A", "B")))
  expect_error(
    check_argmatch(mtcars, c("A", "B")),
    "must be a single string"
  )

})

test_that("check_grob throws appropriate errors", {

  expect_silent(check_grob(pointsGrob()))
  expect_silent(check_grob(NULL, allow_null = TRUE))
  expect_error(
    check_grob("A"),
    "must be a"
  )

})

test_that("check_exclusive throws appropriate errors", {

  expect_silent(check_exclusive(NULL, NULL))
  expect_error(
    check_exclusive(NULL, NULL, required = TRUE),
    "argument is required"
  )

  expect_silent(check_exclusive(1.0, NULL))
  expect_silent(check_exclusive(NULL, 1.0))
  expect_error(
    check_exclusive(1.0, 1.0),
    "arguments are mutually exclusive"
  )

})

test_that("check_unique throws appropriate errors", {
  expect_silent(check_unique(LETTERS[1L:3L]))
  expect_error(
    check_unique(c("A", "A", "B")),
    "Example duplicate"
  )
})

test_that("check_list_of throws appropriate errors", {

  nums <- c("integer", "double", "numeric")
  expect_silent(check_list_of(list(1L, 2L, 3L), nums))
  expect_silent(check_list_of(NULL, allow_null = TRUE))

  expect_error(
    check_list_of(list(1L, "A", 3L), nums),
    "is the string \"A\""
  )
  expect_error(
    check_list_of(12L, nums),
    "not the number 12"
  )
})

# guide_legend_manual() constructor works as expected

    Code
      guide_legend_manual("foo", layers = list("invalid input"))
    Condition
      Error in `guide_legend_manual()`:
      ! `layers` must be a <list> object with <LayerInstance> elements.
      x layers[[1]] is the string "invalid input"

---

    Code
      guide_legend_manual(c("foo", "bar"), colour = c("red", "green", "blue"))
    Condition
      Error in `guide_legend_manual()`:
      ! Can't recycle `labels` (size 2) to match `colour` (size 3).

# guide_axis_plot() rejects incompatible plots

    Code
      guide_axis_plot(plot = "foobar")
    Condition
      Error in `guide_axis_plot()`:
      ! `plot` must be a <ggplot> object, not the string "foobar".

---

    Code
      guide_axis_plot(plot + coord_radial())
    Condition
      Error in `guide_axis_plot()`:
      ! The `plot` argument cannot have a non-linear coordinate system.

---

    Code
      guide_axis_plot(plot + facet_wrap(~cyl))
    Condition
      Error in `guide_axis_plot()`:
      ! The `plot` argument cannot have facets of class <FacetWrap>.
      i Only `facet_null()` is supported.

---

    Code
      ggplot_build(wrong_plot)
    Condition
      Error in `transform()`:
      ! Non-linear coordinates do not support `guide_axis_plot()`.

# connectors works

    The value_start, value_end, level_start, and level_end columns are required in `connect`.

# guide_axis_symbols does input checks

    The `key` argument is required. You can set a manual key using `key_symbols()`.

# guide_axis_upset treats input correctly

    The `key` argument is required. You can set a key using `key_upset()`.

# guide_axis_upset gives informative error for `override.aes`

    Can't recycle `override.aes$colour` (size 4) to size 3.

# key_group_split works correctly

    Code
      key_group_split(sep = ":")(scale, "colour")
    Condition
      Error in `key_group_split()`:
      ! Cannot split the guide's label.
      i It must be a <character> vector, not a list.

# range_from_label can extract ranges

    Code
      test
    Output
        start end .label .level
      1     1   1      A      0
      2     2   2      B      0
      3     3   3      C      0
      4     1   2      1      1
      5     3   3      2      1

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shapes-brackets.R
\name{bracket_options}
\alias{bracket_options}
\alias{bracket_line}
\alias{bracket_square}
\alias{bracket_chevron}
\alias{bracket_round}
\alias{bracket_sigmoid}
\alias{bracket_atan}
\alias{bracket_curvy}
\title{Bracket options}
\usage{
bracket_line()

bracket_square()

bracket_chevron()

bracket_round(angle = 180, n = 100L)

bracket_sigmoid(curvature = 10, n = 100L)

bracket_atan(curvature = 5, n = 100L)

bracket_curvy(angle = 225, n = 100L)
}
\arguments{
\item{angle}{A \code{numeric(1)}: the angle in degrees for which a circle piece
is drawn. For \code{bracket_curvy()}, an angle between 180 and 270.}

\item{n}{An \code{integer(1)} number of points to use for the bracket.}

\item{curvature}{A \code{numeric(1)} that controls the curliness of the bracket.
More precisely, it is used to construct the sequence
\code{seq(-curvature, curvature, length.out = n)} over which the logistic or
arctangent functions is evaluated.}
}
\value{
A \verb{<matrix[n, 2]>} with coordinates for points on the brackets.
}
\description{
These functions construct various sorts of brackets. They construct a matrix
that can be supplied as the \code{bracket} argument in \code{\link[=primitive_bracket]{primitive_bracket()}}.
}
\details{
When designing custom bracket shapes, the expectation is both columns are
are a number between 0 and 1. The first column follows the direction of the
guide whereas the second column is orthogonal to that direction.
}
\section{Functions}{
\itemize{
\item \code{bracket_line()}: A simple line as bracket. Has \code{n = 2} points.

\item \code{bracket_square()}: A square bracket. Has \code{n = 4} points.

\item \code{bracket_chevron()}: A chevron (V-shape) that makes a bracket. Has
\code{n = 3} points.

\item \code{bracket_round()}: One circular arc that makes a bracket.

\item \code{bracket_sigmoid()}: Two sigmoid curves stacked on top of one another
to form a bracket.

\item \code{bracket_atan()}: Two arctangent curves stacked on top of one
another to form a bracket.

\item \code{bracket_curvy()}: Four circular arcs that make a bracket.

}}
\examples{
plot(bracket_sigmoid(), type = 'l')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compose-ontop.R
\name{compose_ontop}
\alias{compose_ontop}
\title{Compose guides on top of one another}
\usage{
compose_ontop(
  ...,
  args = list(),
  key = NULL,
  title = waiver(),
  angle = waiver(),
  theme = NULL,
  order = 0L,
  position = waiver(),
  available_aes = NULL
)
}
\arguments{
\item{...}{Guides to stack in \link[=guide-composition]{composition}. Each guide
can be specified as one of the following:
\itemize{
\item A \verb{<Guide>} class object.
\item A \verb{<function>} that returns a \verb{<Guide>} class object.
\item A \verb{<character[1]>} naming such a function, without the \code{guide_} or
\code{primitive_} prefix.
}}

\item{args}{A \verb{<list>} of arguments to pass to guides that are given either
as a function or as a string.}

\item{key}{A \link[=key_standard]{standard key} specification. The key is shared
among all guides that have \code{NULL} keys themselves. See more information
in the linked topic.}

\item{title}{One of the following to indicate the title of the guide:
\itemize{
\item A \verb{<character[1]>} or \verb{<expression[1]>} to set a custom title.
\item \code{NULL} to not display any title.
\item \code{\link[ggplot2:waiver]{waiver()}} (default) to take the name of the scale
object or the name specified in \code{\link[ggplot2:labs]{labs()}} as the title.
}}

\item{angle}{A specification for the text angle. Compared to setting the \code{angle} argument
in \code{\link[ggplot2:element_text]{element_text()}}, this argument uses some
heuristics to automatically pick the \code{hjust} and \code{vjust} that you
probably want. Can be one of the following:
\itemize{
\item \code{NULL} to take angles and justification settings directly from the theme.
\item \code{\link[ggplot2:waiver]{waiver()}} to allow reasonable defaults in special
cases.
\item A \verb{<numeric[1]>} between -360 and 360 for the text angle in degrees.
}}

\item{theme}{A \code{\link[ggplot2:theme]{<theme>}} object to style the guide individually or
differently from the plot's theme settings. The \code{theme} argument in the
guide overrides and is combined with the plot's theme.}

\item{order}{A positive \verb{<integer[1]>} that specifies the order of this guide among
multiple guides. This controls in which order guides are merged if there
are multiple guides for the same position. If \code{0} (default), the order is
determined by a hashing indicative settings of a guide.}

\item{position}{A \verb{<character[1]>} giving the location of the guide. Can be one of \code{"top"},
\code{"bottom"}, \code{"left"} or \code{"right"}.}

\item{available_aes}{A \verb{<character>} giving aesthetics that must match the
the guides.}
}
\value{
A \verb{<ComposeOntop>} composite guide object.
}
\description{
This guide can place other guides on top of one another.
}
\details{
\subsection{Styling options}{

There are no styling options in \code{theme()} for this composition.
}
}
\examples{
# Using the ontop composition to get two types of ticks with different
# lengths
ggplot(mpg, aes(displ, hwy)) +
  geom_point() +
  guides(x = compose_ontop(
    guide_axis_base(
      key_manual(c(2, 4, 6)),
      theme = theme(
        axis.ticks = element_line(colour = "limegreen"),
        axis.ticks.length = unit(11, "pt")
      )
    ),
    guide_axis_base(
      key_manual(c(3, 5, 7)),
      theme = theme(
        axis.ticks = element_line(colour = "tomato"),
        axis.ticks.length = unit(5.5, "pt")
      )
    )
  ))
}
\seealso{
Other composition:
\code{\link[=compose_crux]{compose_crux()}},
\code{\link[=compose_sandwich]{compose_sandwich()}},
\code{\link[=compose_stack]{compose_stack()}},
\code{\link{guide-composition}}
}
\concept{composition}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/guide-axis-nested.R
\name{guide_axis_nested}
\alias{guide_axis_nested}
\title{Nested axis guide}
\usage{
guide_axis_nested(
  key = "range_auto",
  regular_key = "auto",
  type = "bracket",
  title = waiver(),
  subtitle = NULL,
  theme = NULL,
  angle = waiver(),
  cap = "none",
  bidi = FALSE,
  oob = "squish",
  drop_zero = TRUE,
  pad_discrete = NULL,
  levels_text = NULL,
  ...,
  order = 0L,
  position = waiver()
)
}
\arguments{
\item{key}{One of the following:
\itemize{
\item A \link[=key_range]{range key} specification. If not \code{key = "range_auto"},
additional labels will be inserted to represent point values.
\item A \verb{<character[1]>} passed to the \code{\link[=key_range_auto]{key_range_auto(sep)}}
argument. An exception is made when the string is a valid key
specification.
}}

\item{regular_key}{A \link[=key_standard]{standard key} specification for the
appearance of regular tick marks.}

\item{type}{Appearance of ranges. One of the following:
\itemize{
\item \code{"box"} to put text in boxes.
\item \code{"bracket"} (default) to text over brackets.
\item \code{"fence"} to put text near fences.
}}

\item{title}{One of the following to indicate the title of the guide:
\itemize{
\item A \verb{<character[1]>} or \verb{<expression[1]>} to set a custom title.
\item \code{NULL} to not display any title.
\item \code{\link[ggplot2:waiver]{waiver()}} (default) to take the name of the scale
object or the name specified in \code{\link[ggplot2:labs]{labs()}} as the title.
}}

\item{subtitle}{Passed on to \code{\link[=primitive_title]{primitive_title(title)}}.
Follow the linked topic for more details.}

\item{theme}{A \code{\link[ggplot2:theme]{<theme>}} object to style the guide individually or
differently from the plot's theme settings. The \code{theme} argument in the
guide overrides and is combined with the plot's theme.}

\item{angle}{A specification for the text angle. Compared to setting the \code{angle} argument
in \code{\link[ggplot2:element_text]{element_text()}}, this argument uses some
heuristics to automatically pick the \code{hjust} and \code{vjust} that you
probably want. Can be one of the following:
\itemize{
\item \code{NULL} to take angles and justification settings directly from the theme.
\item \code{\link[ggplot2:waiver]{waiver()}} to allow reasonable defaults in special
cases.
\item A \verb{<numeric[1]>} between -360 and 360 for the text angle in degrees.
}}

\item{cap}{A method to cap the axes. One of the following:
\itemize{
\item A \verb{<character[1]>} with one of the following:
\itemize{
\item \code{"none"} to perform no capping.
\item \code{"both"} to cap the line at both ends at the most extreme breaks.
\item \code{"upper"} to cap the line at the upper extreme break.
\item \code{"lower"} to cap the line at the lower extreme break.
}
\item A \verb{<logical>[1]}, where \code{TRUE} is equivalent to \code{"both"} and \code{FALSE}
is equivalent to \code{"none"} in the options above.
\item A sorted \verb{<numeric>[2n]} with an even number of members. The lines
will be drawn between every odd-even pair.
\item A \verb{<function>} that takes the scale's breaks as the first argument, the
scale's limits as the second argument and returns a \verb{<numeric>[2n]} as
described above.
}}

\item{bidi}{A \verb{<logical[1]>}: whether ticks should be drawn bidirectionally
(\code{TRUE}) or in a single direction (\code{FALSE}, default).}

\item{oob}{A method for dealing with out-of-bounds (oob) ranges. Can be one
of \code{"squish"}, \code{"censor"} or \code{"none"}.}

\item{drop_zero}{A \verb{<logical[1]>} whether to drop near-zero width ranges
(\code{TRUE}, default) or preserve them (\code{FALSE}).}

\item{pad_discrete}{A \verb{<numeric[1]>} giving the amount ranges should be
extended when given as a discrete variable. This is applied after
the \code{drop_zero} setting.}

\item{levels_text}{A list of \verb{<element_text>} objects to customise how
text appears at every level.}

\item{...}{Arguments passed on to \code{\link[=primitive_bracket]{primitive_bracket()}},
\code{\link[=primitive_box]{primitive_box()}} or \code{\link[=primitive_fence]{primitive_fence()}} (depending on the \code{type}
argument).}

\item{order}{A positive \verb{<integer[1]>} that specifies the order of this guide among
multiple guides. This controls in which order guides are merged if there
are multiple guides for the same position. If \code{0} (default), the order is
determined by a hashing indicative settings of a guide.}

\item{position}{A \verb{<character[1]>} giving the location of the guide. Can be one of \code{"top"},
\code{"bottom"}, \code{"left"} or \code{"right"}.}
}
\value{
A \verb{<Guide>} object.
}
\description{
This axis guide gives extra range annotations to position scales. It can
be used to infer nesting structure from labels or annotate ranges.
}
\details{
To offer other keys the opportunity to display ranges alongside
regular-looking labels, the \code{regular_key} argument can be used to setup a
separate key for display in between the ticks and ranges.

By default, the \code{\link[=key_range]{key = "range_auto"}} will incorporate the 0th
level labels inferred from the scale's labels. These labels will look like
regular labels.
\subsection{Styling options}{

Because this guide is pure composite guide, the \link[ggplot2:theme]{theme}
options that govern the styling are determined by its constituents. They are
linked below so you can find their 'Styling options' sections.\tabular{lll}{
   \strong{Primitive} \tab \strong{Context} \tab \strong{Description} \cr
   \code{\link{compose_stack}} \tab Always \tab Stacks the other primitives. \cr
   \code{\link[=primitive_line]{primitive_line()}} \tab Always \tab Makes up the axis line. \cr
   \code{\link[=primitive_ticks]{primitive_ticks()}} \tab Always \tab Makes up the tick marks. \cr
   \code{\link[=primitive_bracket]{primitive_bracket()}} \tab \code{type = "bracket"} \tab Range display as brackets. \cr
   \code{\link[=primitive_box]{primitive_box()}} \tab \code{type = "box"} \tab Range display as boxes. \cr
   \code{\link[=primitive_fence]{primitive_fence()}} \tab \code{type = "fence"} \tab Range display as fences. \cr
   \code{\link[=primitive_title]{primitive_title()}} \tab \verb{subtitle = <...>} \tab Used for displaying subtitles. \cr
   \code{\link[=primitive_labels]{primitive_labels()}} \tab \code{key != "range_auto"} \tab Used for displaying range-less, 0th level labels \cr
}


Styling options \emph{per range} can be set in the \link[=key_range]{range key}.
These override theme settings.

The context-agnostic alternative to using \code{theme()} is to use
\code{\link[=theme_guide]{theme_guide()}}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{guide_axis_nested(theme = theme_guide(
  # Common options
  line = element_line(),
  text = element_text(),
  ticks = element_line(),
  ticks.length = unit(5, "mm"),

  # For brackets
  bracket = element_line(),
  bracket.size = unit(5, "mm"),

  # For boxes
  box = element_rect(),

  # For fences
  fence = element_line(),
  fence.post = element_line(),
  fence.rail = element_line(),

  # For subtitle (not main title)
  title = element_text()
))
}\if{html}{\out{</div>}}
}
}
\examples{
# A plot with nested categories on the x-axis
p <- ggplot(mpg, aes(interaction(drv, cyl), hwy)) +
  geom_boxplot()

p + guides(x = "axis_nested")

# Apply styling to brackets
p + guides(x = "axis_nested") +
  theme_guide(bracket = element_line("red", linewidth = 1))

# Don't drop nesting indicators that have 0-width
p + guides(x = guide_axis_nested(drop_zero = FALSE))

# Change additional padding for discrete categories
p + guides(x = guide_axis_nested(pad_discrete = 0))

# Change bracket type
p + guides(x = guide_axis_nested(bracket = "curvy"))

# Use boxes instead of brackets + styling of boxes
p + guides(x = guide_axis_nested(type = "box")) +
  theme_guide(box = element_rect("limegreen", "forestgreen"))

# Using fences instead of brackets + styling of fences
p + guides(x = guide_axis_nested(type = "fence", rail = "inner")) +
  theme_guide(
    fence.post = element_line("tomato"),
    fence.rail = element_line("dodgerblue")
  )

# Use as annotation of a typical axis
# `regular_key` controls display of typical axis
ggplot(mpg, aes(displ, hwy)) +
  geom_point() +
  guides(x = guide_axis_nested(
    key = key_range_manual(
      start = 2:3,
      end = 5:6,
      name = c("First", "Second")
    ),
    regular_key = key_manual(c(2, 2.5, 3, 5, 7))
  ))
}
\seealso{
Other standalone guides:
\code{\link[=guide_axis_annotation]{guide_axis_annotation()}},
\code{\link[=guide_axis_base]{guide_axis_base()}},
\code{\link[=guide_axis_dendro]{guide_axis_dendro()}},
\code{\link[=guide_axis_plot]{guide_axis_plot()}},
\code{\link[=guide_axis_symbols]{guide_axis_symbols()}},
\code{\link[=guide_circles]{guide_circles()}},
\code{\link[=guide_colbar]{guide_colbar()}},
\code{\link[=guide_colring]{guide_colring()}},
\code{\link[=guide_colsteps]{guide_colsteps()}},
\code{\link[=guide_legend_base]{guide_legend_base()}},
\code{\link[=guide_legend_cross]{guide_legend_cross()}},
\code{\link[=guide_legend_group]{guide_legend_group()}},
\code{\link[=guide_legend_manual]{guide_legend_manual()}}
}
\concept{standalone guides}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/guide-colsteps.R
\name{guide_colsteps}
\alias{guide_colsteps}
\title{Custom colour steps guide}
\usage{
guide_colsteps(
  title = waiver(),
  key = "bins",
  first_guide = "axis_base",
  second_guide = "axis_base",
  shape = "triangle",
  size = NULL,
  show = NA,
  alpha = NA,
  reverse = FALSE,
  suppress_labels = "second",
  oob = scales::oob_keep,
  theme = NULL,
  position = waiver(),
  vanilla = TRUE,
  available_aes = c("colour", "fill")
)
}
\arguments{
\item{title}{One of the following to indicate the title of the guide:
\itemize{
\item A \verb{<character[1]>} or \verb{<expression[1]>} to set a custom title.
\item \code{NULL} to not display any title.
\item \code{\link[ggplot2:waiver]{waiver()}} (default) to take the name of the scale
object or the name specified in \code{\link[ggplot2:labs]{labs()}} as the title.
}}

\item{key}{A \link[=key_bins]{bins key} specificiation. Defaults to
\code{key_bins(even.steps = FALSE, show.limits = NULL)}. Changing the
arguments to \code{key_bins()} is fine, but changing the key type is not
advised.}

\item{first_guide, second_guide}{Guides to flank the colour steps. Each guide
can be specified using one of the following:
\itemize{
\item A \verb{<Guide>} class object.
\item A \verb{<function>} that returns a \verb{<Guide>} class object.
\item A \verb{<character>} naming such a function, without the \code{guide_} or
\code{primitive_} prefix.
}

The \code{first_guide} will be placed at the location specified by the
\code{legend.text.position} theme setting. The \code{second_guide} will be placed
opposite that position. When \code{second_guide} has a label suppression
mechanism, no labels will be drawn for that guide.}

\item{shape}{A \link[=cap_options]{cap} specification by providing one of the
following:
\itemize{
\item A cap \verb{<function>}, such as \code{cap_triangle()}.
\item A \verb{<character[1]>} naming a cap function without the '\code{cap_}'-prefix,
e.g. \code{"round"}.
\item A two column \verb{<matrix[n, 2]>} giving coordinates for a cap, like those
created by cap functions such as \code{cap_arch()}.
}}

\item{size}{A \code{\link[grid:unit]{<unit>}} setting the size of the cap. When
\code{NULL} (default), cap size will be proportional to the \code{shape} coordinates
and the \code{legend.key.size} theme setting.}

\item{show}{A \verb{<logical>} to control how caps are displayed at the ends
of the bar. When \code{TRUE}, caps are always displayed. When \code{FALSE}, caps
are never displayed. When \code{NA} (default), caps are displayed when the
data range exceed the limits. When given as \verb{<logical[2]>}, \code{show[1]}
controls the display at the lower end and \code{show[2]} at the upper end.}

\item{alpha}{A \verb{<numeric[1]>} between 0 and 1 setting the colour transparency
of the bar. Use \code{NA} to preserve the alpha encoded in the colour itself.}

\item{reverse}{A \verb{<logical[1]>} whether to reverse continuous guides.
If \code{TRUE}, guides like colour bars are flipped. If \code{FALSE} (default),
the original order is maintained.}

\item{suppress_labels}{A \verb{<character>} vector giving any of \code{"text"} and
\code{"opposite"} for the parallel guides. The guide(s) listed here will not
draw labels if they support a label suppression mechanism.}

\item{oob}{An out-of-bounds handling function that affects the cap colour.
Can be one of the following:
\itemize{
\item A \verb{<function>} like \code{\link[scales:oob_squish]{oob_squish}}.
\item A \verb{<character[1]>} naming such a function without the '\code{oob}'-prefix,
such as \code{"keep"}.
}}

\item{theme}{A \code{\link[ggplot2:theme]{<theme>}} object to style the guide individually or
differently from the plot's theme settings. The \code{theme} argument in the
guide overrides and is combined with the plot's theme.}

\item{position}{A \verb{<character[1]>} giving the location of the guide. Can be one of \code{"top"},
\code{"bottom"}, \code{"left"} or \code{"right"}.}

\item{vanilla}{A \verb{<logical[1]>} whether to have the default style match
the vanilla \code{guide_colourbar()} (\code{TRUE}) or take the theme
verbatim (\code{FALSE}).}

\item{available_aes}{A \verb{<character>} vector listing the aesthetics for which this guide can
be build.}
}
\value{
A \verb{<Guide>} object
}
\description{
Similar to \code{\link[ggplot2:guide_coloursteps]{guide_coloursteps()}}, this guide
displays continuous \code{colour} or \code{fill} aesthetics. It has additional options
to display caps at the end of the bar, depending on out-of-bounds values.
}
\details{
As steps are rendered as clipped rectangles, it is important to use a
graphics device that can render clipped paths. This can be checked by using
\code{\link[ggplot2:check_device]{check_device("clippingPaths")}}.
\subsection{Styling options}{

Because this guide is pure composite guide, the \link[ggplot2:theme]{theme}
options that govern the styling are determined by its constituents. They are
linked below so you can find their 'Styling options' sections. Note that
\code{guide_axis_base()} is just a default that can be swapped out.\tabular{ll}{
   \strong{Constituent} \tab \strong{Description} \cr
   \code{\link{compose_sandwich}} \tab Combines the bar with two side-guides. \cr
   \code{\link[=gizmo_stepcap]{gizmo_stepcap()}} \tab Makes up the colour bar. \cr
   \code{\link[=guide_axis_base]{guide_axis_base()}} \tab Makes up the tick marks and labels. \cr
}


Styling options \emph{per break} can be set in the \link[=key_standard]{standard key}.
These override theme settings.

The context-agnostic alternative to using \code{theme()} is to use
\code{\link[=theme_guide]{theme_guide()}}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{guide_colsteps(theme = theme_guide(
  # Composition settings
  title = element_text(),
  title.position = "top",
  text.position = "right",
  margin = margin(5),
  background = element_rect(),

  # Steps settings
  frame = element_rect(),
  key.width = unit(5, "mm")
  key.height = unit(5, "cm")

  # Common options for `guide_axis_base()`
  line = element_line(),
  text = element_text(),
  ticks = element_line(),
  ticks.length = unit(5, "mm"),
))
}\if{html}{\out{</div>}}
}
}
\examples{
p <- ggplot(mpg, aes(displ, hwy)) +
  geom_point(aes(colour = cty))

# The colour steps show caps when values are out-of-bounds
p + scale_colour_viridis_b(
  limits = c(10, NA),
  guide = "colsteps"
)

# It also shows how oob values are handled
p + scale_colour_viridis_b(
  limits = c(10, 30), oob = scales::oob_censor,
  guide = "colsteps"
)

# Adjusting the type of cap
p + scale_colour_viridis_b(
  limits = c(10, 30),
  guide = guide_colsteps(shape = "round")
)

# The default is to use the breaks as-is
p + scale_colour_viridis_b(
  limits = c(10, 30), breaks = c(10, 20, 25),
  guide = "colsteps"
)

# But the display can be set to use evenly spaced steps
p + scale_colour_viridis_b(
  limits = c(10, 30), breaks = c(10, 20, 25),
  guide = guide_colsteps(key = key_bins(even.steps = TRUE))
)

# Using tick marks by swapping side guides
p + scale_colour_viridis_b(
  guide = guide_colsteps(
    first_guide  = "axis_base",
    second_guide = "axis_base"
  )
)
}
\seealso{
Other standalone guides:
\code{\link[=guide_axis_annotation]{guide_axis_annotation()}},
\code{\link[=guide_axis_base]{guide_axis_base()}},
\code{\link[=guide_axis_dendro]{guide_axis_dendro()}},
\code{\link[=guide_axis_nested]{guide_axis_nested()}},
\code{\link[=guide_axis_plot]{guide_axis_plot()}},
\code{\link[=guide_axis_symbols]{guide_axis_symbols()}},
\code{\link[=guide_circles]{guide_circles()}},
\code{\link[=guide_colbar]{guide_colbar()}},
\code{\link[=guide_colring]{guide_colring()}},
\code{\link[=guide_legend_base]{guide_legend_base()}},
\code{\link[=guide_legend_cross]{guide_legend_cross()}},
\code{\link[=guide_legend_group]{guide_legend_group()}},
\code{\link[=guide_legend_manual]{guide_legend_manual()}}
}
\concept{standalone guides}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/guide-legend-group.R
\name{guide_legend_group}
\alias{guide_legend_group}
\title{Grouped legend}
\usage{
guide_legend_group(
  key = "group_split",
  title = waiver(),
  override.aes = list(),
  nrow = NULL,
  ncol = NULL,
  theme = NULL,
  position = NULL,
  direction = NULL,
  order = 0L
)
}
\arguments{
\item{key}{A \link[=key_group]{group key} specification. Defaults to
\code{key_group_split()} to split labels to find groups.}

\item{title}{One of the following to indicate the title of the guide:
\itemize{
\item A \verb{<character[1]>} or \verb{<expression[1]>} to set a custom title.
\item \code{NULL} to not display any title.
\item \code{\link[ggplot2:waiver]{waiver()}} (default) to take the name of the scale
object or the name specified in \code{\link[ggplot2:labs]{labs()}} as the title.
}}

\item{override.aes}{A named \verb{<list>} specifying aesthetic parameters of the
key glyphs. See details and examples in
\code{\link[ggplot2:guide_legend]{guide_legend()}}.}

\item{nrow, ncol}{A positive \verb{<integer[1]>} setting the desired dimensions of
the legend layout. Either \code{nrow} or \code{ncol} can be set, but not both,}

\item{theme}{A \code{\link[ggplot2:theme]{<theme>}} object to style the guide individually or
differently from the plot's theme settings. The \code{theme} argument in the
guide overrides and is combined with the plot's theme.}

\item{position}{A \verb{<character[1]>} giving the location of the guide. Can be one of \code{"top"},
\code{"bottom"}, \code{"left"} or \code{"right"}.}

\item{direction}{A \verb{<character[1]>} indicating the direction of the guide. Can be on of
\code{"horizontal"} or \code{"vertical"}.}

\item{order}{A positive \verb{<integer[1]>} that specifies the order of this guide among
multiple guides. This controls in which order guides are merged if there
are multiple guides for the same position. If \code{0} (default), the order is
determined by a hashing indicative settings of a guide.}
}
\value{
A \verb{<GuideLegend>} object.
}
\description{
This legend resembles \code{ggplot2::guide_legend()}, but has the ability to
keep groups in blocks with their own titles.
}
\details{
\subsection{Styling options}{

Below are the \link[ggplot2:theme]{theme} options that determine the styling of
this guide.\tabular{lll}{
   \strong{Theme setting} \tab \strong{Type} \tab \strong{Description} \cr
   \code{legendry.legend.subtitle} \tab \code{\link[ggplot2:element_text]{element_text()}} \tab Title of groups in the legend. \cr
   \code{legendry.legend.subtitle.position} \tab \verb{<character[1]>} \tab One of \code{"top"}, \code{"right"}, \code{"bottom"} or \code{"left"}. \cr
   \code{legendry.group.spacing} \tab \code{\link[=unit]{unit()}} \tab Spacing in between groups of keys. \cr
   \code{legend.background} \tab \code{\link[ggplot2:element_rect]{element_rect()}} \tab Background of the legend. \cr
   \code{legend.margin} \tab \code{\link[ggplot2:margin]{margin()}} \tab Padding around the legend. \cr
   \code{legend.text} \tab \code{\link[ggplot2:element_text]{element_text()}} \tab Labels displayed next to keys. \cr
   \code{legend.text.position} \tab \verb{<character[1]>} \tab One of \code{"top"}, \code{"right"}, \code{"bottom"} or \code{"left"}. \cr
   \code{legend.title} \tab \code{\link[ggplot2:element_text]{element_text()}} \tab Title of the legend. \cr
   \code{legend.title.position} \tab \verb{<character[1]>} \tab One of \code{"top"}, \code{"right"}, \code{"bottom"} or \code{"left"}. \cr
   \code{legend.key} \tab \code{\link[ggplot2:element_rect]{element_rect()}} \tab Background of the key areas. \cr
   \code{legend.key.height} \tab \code{\link[=unit]{unit()}} \tab Height of keys. \cr
   \code{legend.key.width} \tab \code{\link[=unit]{unit()}} \tab Width of keys. \cr
   \code{legend.key.justification} \tab \verb{<numeric[2]>} \tab Justification for placing legend keys in excess space. \cr
   \code{legend.key.spacing.x} \tab \code{\link[=unit]{unit()}} \tab Horizontal spacing between keys. \cr
   \code{legend.key.spacing.y} \tab \code{\link[=unit]{unit()}} \tab Vertical spacing between keys. Taken literally. \cr
   \code{legend.byrow} \tab \verb{<logical[1]>} \tab Row-order key filling (\code{TRUE}) or column-order (\code{FALSE}) \cr
}


The context-agnostic alternative to using \code{theme()} is to use
\code{\link[=theme_guide]{theme_guide()}}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{guide_legend_group(theme = theme_guide(
  subtitle = element_text(),
  subtitle.position = "top",
  group.spacing = unit(5, "mm"),
  text = element_text(),
  text.position = "right",
  title = element_text(),
  title.position = "top",
  key = element_rect(),
  key.height = unit(5, "mm"),
  key.width = unit(5, "mm"),
  key.justification = c(0.5, 0.5),
  key.spacing.x = unit(5, "mm"),
  key.spacing.y = unit(5, "mm"),
  byrow = TRUE,
  margin = margin(5),
  background = element_rect(),
))
}\if{html}{\out{</div>}}
}
}
\examples{
# Standard plot for selection of `msleep`
df <- msleep[c(9, 28, 11, 5, 34, 54, 64, 24, 53), ]

p <- ggplot(df) +
  aes(bodywt, awake, colour = paste(order, name)) +
  geom_point()

# By default, groups are inferred from the name
p + guides(colour = "legend_group")

# You can also use a look-up table for groups
# The lookup table can be more expansive than just the data:
# We're using the full 'msleep' data here instead of the subset
lut <- key_group_lut(msleep$name, msleep$order)

p + aes(colour = name) +
  guides(colour = guide_legend_group(key = lut))

# `nrow` and `ncol` apply within groups
p + guides(colour = guide_legend_group(nrow = 1))

# Groups are arranged according to `direction`
p + guides(colour = guide_legend_group(ncol = 1, direction = "horizontal")) +
  theme(legend.title.position = "top")

# Customising the group titles
p + guides(colour = "legend_group") +
  theme(
    legendry.legend.subtitle.position = "left",
    legendry.legend.subtitle = element_text(
      hjust = 1, vjust = 1, size = rel(0.9),
      margin = margin(t = 5.5, r = 5.5)
    )
  )

# Changing the spacing between groups
p + guides(colour = "legend_group") +
  theme(legendry.group.spacing = unit(0, "cm"))
}
\seealso{
Other standalone guides:
\code{\link[=guide_axis_annotation]{guide_axis_annotation()}},
\code{\link[=guide_axis_base]{guide_axis_base()}},
\code{\link[=guide_axis_dendro]{guide_axis_dendro()}},
\code{\link[=guide_axis_nested]{guide_axis_nested()}},
\code{\link[=guide_axis_plot]{guide_axis_plot()}},
\code{\link[=guide_axis_symbols]{guide_axis_symbols()}},
\code{\link[=guide_circles]{guide_circles()}},
\code{\link[=guide_colbar]{guide_colbar()}},
\code{\link[=guide_colring]{guide_colring()}},
\code{\link[=guide_colsteps]{guide_colsteps()}},
\code{\link[=guide_legend_base]{guide_legend_base()}},
\code{\link[=guide_legend_cross]{guide_legend_cross()}},
\code{\link[=guide_legend_manual]{guide_legend_manual()}}

Other legend guides:
\code{\link[=guide_legend_base]{guide_legend_base()}},
\code{\link[=guide_legend_cross]{guide_legend_cross()}},
\code{\link[=guide_legend_manual]{guide_legend_manual()}}
}
\concept{legend guides}
\concept{standalone guides}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legendry-package.R
\name{common_parameters}
\alias{common_parameters}
\title{common parameters in \pkg{legendry}}
\arguments{
\item{title}{One of the following to indicate the title of the guide:
\itemize{
\item A \verb{<character[1]>} or \verb{<expression[1]>} to set a custom title.
\item \code{NULL} to not display any title.
\item \code{\link[ggplot2:waiver]{waiver()}} (default) to take the name of the scale
object or the name specified in \code{\link[ggplot2:labs]{labs()}} as the title.
}}

\item{theme}{A \code{\link[ggplot2:theme]{<theme>}} object to style the guide individually or
differently from the plot's theme settings. The \code{theme} argument in the
guide overrides and is combined with the plot's theme.}

\item{position}{A \verb{<character[1]>} giving the location of the guide. Can be one of \code{"top"},
\code{"bottom"}, \code{"left"} or \code{"right"}.}

\item{order}{A positive \verb{<integer[1]>} that specifies the order of this guide among
multiple guides. This controls in which order guides are merged if there
are multiple guides for the same position. If \code{0} (default), the order is
determined by a hashing indicative settings of a guide.}

\item{available_aes}{A \verb{<character>} vector listing the aesthetics for which this guide can
be build.}

\item{direction}{A \verb{<character[1]>} indicating the direction of the guide. Can be on of
\code{"horizontal"} or \code{"vertical"}.}

\item{angle}{A specification for the text angle. Compared to setting the \code{angle} argument
in \code{\link[ggplot2:element_text]{element_text()}}, this argument uses some
heuristics to automatically pick the \code{hjust} and \code{vjust} that you
probably want. Can be one of the following:
\itemize{
\item \code{NULL} to take angles and justification settings directly from the theme.
\item \code{\link[ggplot2:waiver]{waiver()}} to allow reasonable defaults in special
cases.
\item A \verb{<numeric[1]>} between -360 and 360 for the text angle in degrees.
}}

\item{override.aes}{A named \verb{<list>} specifying aesthetic parameters of the
key glyphs. See details and examples in
\code{\link[ggplot2:guide_legend]{guide_legend()}}.}

\item{call}{A \link[rlang:topic-error-call]{call} to display in messages.}
}
\description{
This is a collection of common parameters so they needn't be re-documented
each time.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/guide-legend-manual.R
\name{guide_legend_manual}
\alias{guide_legend_manual}
\title{Manual legend}
\usage{
guide_legend_manual(
  labels,
  ...,
  layers = list(geom_point()),
  title = NULL,
  theme = NULL,
  design = NULL,
  nrow = NULL,
  ncol = NULL,
  reverse = FALSE,
  position = NULL,
  direction = NULL,
  order = 0L
)
}
\arguments{
\item{labels}{Labels to display next to the keys. Can be a \verb{<character>} or \verb{<expression>}
vector to set labels, or \code{NULL} to draw no labels.}

\item{...}{Arguments interpreted as aesthetics. For example: \code{colour = "red"}.
The aesthetics must have the same size as the \code{labels} argument, or have
size 1. These aesthetics may be overruled by fixed aesthetics set in the
\code{layers} argument.}

\item{layers}{A \verb{<list>} of layers (\verb{<LayerInstance>} objects) created by the \verb{geom_*()} or
\verb{stat_*()} family of functions. These layers are used for their
\code{\link[ggplot2:draw_key]{key_glyph}} drawing functions, as well as to populate
default aesthetics. Any fixed aesthetics provided to these layers overrule
aesthetics passed to the \code{...} argument.}

\item{title}{One of the following to indicate the title of the guide:
\itemize{
\item A \verb{<character[1]>} or \verb{<expression[1]>} to set a custom title.
\item \code{NULL} to not display any title.
\item \code{\link[ggplot2:waiver]{waiver()}} (default) to take the name of the scale
object or the name specified in \code{\link[ggplot2:labs]{labs()}} as the title.
}}

\item{theme}{A \code{\link[ggplot2:theme]{<theme>}} object to style the guide individually or
differently from the plot's theme settings. The \code{theme} argument in the
guide overrides and is combined with the plot's theme.}

\item{design}{Specification of the legend layout. One of the following:
\itemize{
\item \code{NULL} (default) to use the layout algorithm of
\code{\link[ggplot2:guide_legend]{guide_legend()}}.
\item A \verb{<character[1]>} string representing a cell layout wherein \verb{#} defines
an empty cell. See examples.
\item A \verb{<matrix[n, m]>} representing a cell layout wherein \code{NA} defines an
empty cell. See examples. Non-string atomic vectors will be treated with
\code{as.matrix()}.
}}

\item{nrow, ncol}{A positive \verb{<integer[1]>} setting the desired dimensions of
the legend layout. When \code{NULL} (default), the dimensions will be derived
from the \code{design} argument or fit to match the number of keys.}

\item{reverse}{A \verb{<logical[1]>} whether the order of keys should be inverted.}

\item{position}{A \verb{<character[1]>} giving the location of the guide. Can be one of \code{"top"},
\code{"bottom"}, \code{"left"} or \code{"right"}.}

\item{direction}{A \verb{<character[1]>} indicating the direction of the guide. Can be on of
\code{"horizontal"} or \code{"vertical"}.}

\item{order}{A positive \verb{<integer[1]>} that specifies the order of this guide among
multiple guides. This controls in which order guides are merged if there
are multiple guides for the same position. If \code{0} (default), the order is
determined by a hashing indicative settings of a guide.}
}
\value{
A \verb{<GuideCustom>} object.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

This is a guide that displays user-defined keys independent of scales.
It should only be used as a last resort when struggling to format a
conventional legend.
}
\details{
Because this guide is not tied to a scale, it can be given an arbitrary name
in \code{guides()}; as long as it doesn't clash with other aesthetics.
\subsection{Styling options}{

Below are the \link[ggplot2:theme]{theme} options that determine the styling of
this guide.\tabular{lll}{
   \strong{Theme setting} \tab \strong{Type} \tab \strong{Description} \cr
   \code{legend.background} \tab \code{\link[ggplot2:element_rect]{element_rect()}} \tab Background of the legend. \cr
   \code{legend.margin} \tab \code{\link[ggplot2:margin]{margin()}} \tab Padding around the legend. \cr
   \code{legend.text} \tab \code{\link[ggplot2:element_text]{element_text()}} \tab Labels displayed next to keys. \cr
   \code{legend.text.position} \tab \verb{<character[1]>} \tab One of \code{"top"}, \code{"right"}, \code{"bottom"} or \code{"left"}. \cr
   \code{legend.title} \tab \code{\link[ggplot2:element_text]{element_text()}} \tab Title of the legend. \cr
   \code{legend.title.position} \tab \verb{<character[1]>} \tab One of \code{"top"}, \code{"right"}, \code{"bottom"} or \code{"left"}. \cr
   \code{legend.key} \tab \code{\link[ggplot2:element_rect]{element_rect()}} \tab Background of the key areas. \cr
   \code{legend.key.height} \tab \code{\link[=unit]{unit()}} \tab Height of keys. \cr
   \code{legend.key.width} \tab \code{\link[=unit]{unit()}} \tab Width of keys. \cr
   \code{legend.key.justification} \tab \verb{<numeric[2]>} \tab Justification for placing legend keys in excess space. \cr
   \code{legend.key.spacing.x} \tab \code{\link[=unit]{unit()}} \tab Horizontal spacing between keys. \cr
   \code{legend.key.spacing.y} \tab \code{\link[=unit]{unit()}} \tab Vertical spacing between keys. Taken literally. \cr
   \code{legend.byrow} \tab \verb{<logical[1]>} \tab Row-order key filling (\code{TRUE}) or column-order (\code{FALSE}) \cr
}


The context-agnostic alternative to using \code{theme()} is to use
\code{\link[=theme_guide]{theme_guide()}}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{guide_legend_manual(legend_args = list(theme = theme_guide(
  text = element_text(),
  text.position = "right",
  title = element_text(),
  title.position = "top",
  key = element_rect(),
  key.height = unit(5, "mm"),
  key.width = unit(5, "mm"),
  key.justification = c(0.5, 0.5),
  key.spacing.x = unit(5, "mm"),
  key.spacing.y = unit(5, "mm"),
  margin = margin(5),
  background = element_rect(),
)))
}\if{html}{\out{</div>}}
}
}
\examples{
# A standard plot
p <- ggplot(mtcars, aes(disp, mpg)) +
  geom_point()

# Typical usage: set `label` and some aesthetics
p + guides(
  some_name = guide_legend_manual(
    label  = c("foo", "bar"),
    colour = c(NA, "black"),
    fill   = c("grey40", NA),
    layers = geom_col()
  )
)

# Alternative: use `layers` to set aesthetics
p + guides(
  some_name = guide_legend_manual(
    label = c("foo", "bar"),
    layers = geom_col(
      # Must match length of `label`
      colour = c(NA, "black"),
      fill = c("grey40", NA)
    )
  )
)

# You can use >1 layer
p + guides(
  some_name = guide_legend_manual(
    label = c("foo", "bar"),
    colour = c("tomato", "dodgerblue"),
    fill = NA,
    layers = list(geom_col(), geom_point())
  )
)
}
\seealso{
Other standalone guides:
\code{\link[=guide_axis_annotation]{guide_axis_annotation()}},
\code{\link[=guide_axis_base]{guide_axis_base()}},
\code{\link[=guide_axis_dendro]{guide_axis_dendro()}},
\code{\link[=guide_axis_nested]{guide_axis_nested()}},
\code{\link[=guide_axis_plot]{guide_axis_plot()}},
\code{\link[=guide_axis_symbols]{guide_axis_symbols()}},
\code{\link[=guide_circles]{guide_circles()}},
\code{\link[=guide_colbar]{guide_colbar()}},
\code{\link[=guide_colring]{guide_colring()}},
\code{\link[=guide_colsteps]{guide_colsteps()}},
\code{\link[=guide_legend_base]{guide_legend_base()}},
\code{\link[=guide_legend_cross]{guide_legend_cross()}},
\code{\link[=guide_legend_group]{guide_legend_group()}}

Other legend guides:
\code{\link[=guide_legend_base]{guide_legend_base()}},
\code{\link[=guide_legend_cross]{guide_legend_cross()}},
\code{\link[=guide_legend_group]{guide_legend_group()}}
}
\concept{legend guides}
\concept{standalone guides}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale-dendro.R
\name{scale_x_dendro}
\alias{scale_x_dendro}
\alias{scale_y_dendro}
\title{Dendrogram scales}
\usage{
scale_x_dendro(
  clust,
  ...,
  expand = waiver(),
  guide = "axis_dendro",
  position = "bottom"
)

scale_y_dendro(
  clust,
  ...,
  expand = waiver(),
  guide = "axis_dendro",
  position = "left"
)
}
\arguments{
\item{clust}{A data structure that can be coerced to an
\code{\link[stats:hclust]{<hclust>}} object through
\code{\link[stats:as.hclust]{as.hclust()}}.}

\item{...}{
  Arguments passed on to \code{\link[ggplot2:discrete_scale]{ggplot2::discrete_scale}}
  \describe{
    \item{\code{aesthetics}}{The names of the aesthetics that this scale works with.}
    \item{\code{palette}}{A palette function that when called with a single integer
argument (the number of levels in the scale) returns the values that
they should take (e.g., \code{\link[scales:pal_hue]{scales::pal_hue()}}).}
    \item{\code{name}}{The name of the scale. Used as the axis or legend title. If
\code{waiver()}, the default, the name of the scale is taken from the first
mapping used for that aesthetic. If \code{NULL}, the legend title will be
omitted.}
    \item{\code{breaks}}{One of:
\itemize{
\item \code{NULL} for no breaks
\item \code{waiver()} for the default breaks (the scale limits)
\item A character vector of breaks
\item A function that takes the limits as input and returns breaks
as output. Also accepts rlang \link[rlang:as_function]{lambda} function
notation.
}}
    \item{\code{labels}}{One of the options below. Please note that when \code{labels} is a
vector, it is highly recommended to also set the \code{breaks} argument as a
vector to protect against unintended mismatches.
\itemize{
\item \code{NULL} for no labels
\item \code{waiver()} for the default labels computed by the
transformation object
\item A character vector giving labels (must be same length as \code{breaks})
\item An expression vector (must be the same length as breaks). See ?plotmath for details.
\item A function that takes the breaks as input and returns labels
as output. Also accepts rlang \link[rlang:as_function]{lambda} function
notation.
}}
    \item{\code{na.translate}}{Unlike continuous scales, discrete scales can easily show
missing values, and do so by default. If you want to remove missing values
from a discrete scale, specify \code{na.translate = FALSE}.}
    \item{\code{na.value}}{If \code{na.translate = TRUE}, what aesthetic value should the
missing values be displayed as? Does not apply to position scales
where \code{NA} is always placed at the far right.}
    \item{\code{drop}}{Should unused factor levels be omitted from the scale?
The default, \code{TRUE}, uses the levels that appear in the data;
\code{FALSE} includes the levels in the factor. Please note that to display
every level in a legend, the layer should use \code{show.legend = TRUE}.}
    \item{\code{fallback.palette}}{Function to use when \code{palette = NULL} and the
palette is not represented in the theme.}
    \item{\code{call}}{The \code{call} used to construct the scale for reporting messages.}
    \item{\code{super}}{The super class to use for the constructed scale}
  }}

\item{expand}{For position scales, a vector of range expansion constants used to add some
padding around the data to ensure that they are placed some distance
away from the axes. Use the convenience function \code{\link[ggplot2:expansion]{expansion()}}
to generate the values for the \code{expand} argument. The defaults are to
expand the scale by 5\% on each side for continuous variables, and by
0.6 units on each side for discrete variables.}

\item{guide}{A function used to create a guide or its name. See
\code{\link[ggplot2:guides]{guides()}} for more information.}

\item{position}{For position scales, The position of the axis.
\code{left} or \code{right} for y axes, \code{top} or \code{bottom} for x axes.}
}
\value{
A \verb{<ScaleDiscretePosition>} object that can be added to a plot.
}
\description{
These are speciality scales for use with hierarchically clustered data. The
scale automatically orders the limits according to the clustering result
and comes with a \link[=guide_axis_dendro]{dendrogram axis}.
}
\details{
The scale limits are determined by the order and labels in the \code{clust}
argument. While \code{limits} is not an argument in these scales, the \code{breaks}
argument can still be used to selectively omit some breaks and the \code{labels}
can be used for formatting purposes.
}
\examples{
# Hierarchically cluster data, separately for rows and columns
car_clust <- hclust(dist(scale(mtcars)), "ave")
var_clust <- hclust(dist(scale(t(mtcars))), "ave")

long_mtcars <- data.frame(
  car = rownames(mtcars)[row(mtcars)],
  var = colnames(mtcars)[col(mtcars)],
  value = as.vector(scale(mtcars))
)

# A standard heatmap adorned with dendrograms
p <- ggplot(long_mtcars, aes(var, car, fill = value)) +
  geom_tile() +
  scale_x_dendro(var_clust) +
  scale_y_dendro(car_clust)
p

# Styling the dendrograms
p +
  guides(
    y = guide_axis_dendro(key_dendro(type = "triangle")),
    x = guide_axis_dendro(space = rel(5))
  ) +
  theme(
    axis.text.y.left = element_text(margin = margin(r = 3, l = 3)),
    axis.ticks.y = element_line("red"),
    axis.ticks.x = element_line(linetype = "dotted")
  )

# In polar coordinates, plus some formatting
p +
  coord_radial(
    theta = "y", inner.radius = 0.5,
    start = 0.25 * pi, end = 1.75 * pi
  ) +
  guides(
    theta = primitive_labels(angle = 90),
    theta.sec = primitive_segments("dendro", vanish = TRUE),
    r = guide_axis_dendro(angle = 0)
  )
}
\seealso{
\code{\link[=guide_axis_dendro]{guide_axis_dendro()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compose-crux.R
\name{compose_crux}
\alias{compose_crux}
\title{Compose guides in a cross}
\usage{
compose_crux(
  key = NULL,
  centre = "none",
  left = "none",
  right = "none",
  top = "none",
  bottom = "none",
  args = list(),
  complete = FALSE,
  theme = NULL,
  theme_defaults = list(),
  reverse = FALSE,
  order = 0L,
  title = waiver(),
  position = waiver(),
  available_aes = NULL
)
}
\arguments{
\item{key}{A \link[=key_standard]{standard key} specification. The key is shared
among all guides that have \code{NULL} keys themselves. See more information
in the linked topic.}

\item{centre, left, right, top, bottom}{Guides to use in
\link[=guide-composition]{composition} per position. Each guide can be
specified as one of the following:
\itemize{
\item A \verb{<Guide>} class object.
\item A \verb{<function>} that returns a \verb{<Guide>} class object.
\item A \verb{<character>} naming such a function, without the \code{guide_} or
\code{primitive_} prefix.
}}

\item{args}{A \verb{<list>} of arguments to pass to guides that are given either
as a function or as a string.}

\item{complete}{A \verb{<logical[1]>} whether to treat the composition as a
complete guide. If \code{TRUE}, a title and margin are added to the result.
If \code{FALSE} (default), no titles and margins are added.}

\item{theme}{A \code{\link[ggplot2:theme]{<theme>}} object to style the guide
individually of differently from the plot's theme settings. The \code{theme}
arguments in the guide overrides, and is combined with, the plot's theme.}

\item{theme_defaults}{A \verb{<list>} of theme elements to override undeclared
theme arguments.}

\item{reverse}{A \verb{<logical[1]>} whether to reverse continuous guides.
If \code{TRUE}, guides like colour bars are flipped. If \code{FALSE} (default),
the original order is maintained.}

\item{order}{A positive \verb{<integer[1]>} that specifies the order of this guide among
multiple guides. This controls in which order guides are merged if there
are multiple guides for the same position. If \code{0} (default), the order is
determined by a hashing indicative settings of a guide.}

\item{title}{One of the following to indicate the title of the guide:
\itemize{
\item A \verb{<character[1]>} or \verb{<expression[1]>} to set a custom title.
\item \code{NULL} to not display any title.
\item \code{\link[ggplot2:waiver]{waiver()}} (default) to take the name of the scale
object or the name specified in \code{\link[ggplot2:labs]{labs()}} as the title.
}}

\item{position}{Where this guide should be drawn: one of \code{"top"}, \code{"bottom"},
\code{"left"}, or \code{"right"}.}

\item{available_aes}{A \verb{<character>} vector listing the aesthetics for which this guide can
be build.}
}
\value{
A \verb{<ComposeCrux>} guide object.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

This guide composition has a central guide optionally surrounded by other
guides on all four sides.
}
\details{
\subsection{Styling options}{

Below are the \link[ggplot2:theme]{theme} options that determine the styling of
this guide.\tabular{lll}{
   \strong{Theme setting} \tab \strong{Type} \tab \strong{Description} \cr
   \code{legend.title} \tab \code{\link[ggplot2:element_text]{element_text()}} \tab The title of the legend. \cr
   \code{legend.title.position} \tab \verb{<character[1]>} \tab One of \code{"top"}, \code{"right"}, \code{"bottom"} or \code{"left"}. \cr
   \code{legend.text.position} \tab \verb{<character[1]>} \tab One of \code{"top"}, \code{"right"}, \code{"bottom"} or \code{"left"}. \cr
   \code{legend.margin} \tab \code{\link[ggplot2:margin]{margin()}} \tab Padding around the legend. \cr
   \code{legend.background} \tab \code{\link[ggplot2:element_rect]{element_rect()}} \tab Background of the legend. \cr
}


There are no further styling options.

The context-agnostic alternative to using \code{theme()} is to use
\code{\link[=theme_guide]{theme_guide()}}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{compose_crux(theme = theme_guide(
  title = element_text(),
  title.position = "top",
  text.position = "right",
  margin = margin(5),
  background = element_rect()
))
}\if{html}{\out{</div>}}
}
}
\examples{
# Roughly recreating a colour bar with extra text on top and bottom
crux <- compose_crux(
  centre = gizmo_barcap(), left = "axis_base",
  right = "axis_base",
  top = primitive_title("A lot"),
  bottom = primitive_title("A little")
)

ggplot(mpg, aes(displ, hwy)) +
  geom_point(aes(colour = cty)) +
  guides(colour = crux)
}
\seealso{
Other composition:
\code{\link[=compose_ontop]{compose_ontop()}},
\code{\link[=compose_sandwich]{compose_sandwich()}},
\code{\link[=compose_stack]{compose_stack()}},
\code{\link{guide-composition}}
}
\concept{composition}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gizmo-grob.R
\name{gizmo_grob}
\alias{gizmo_grob}
\title{Guide gizmo: custom grob}
\usage{
gizmo_grob(
  grob,
  width = grobWidth(grob),
  height = grobHeight(grob),
  hjust = 0.5,
  vjust = 0.5,
  position = waiver()
)
}
\arguments{
\item{grob}{A \code{\link[grid:grob]{<grob>}} to display.}

\item{width, height}{A [\verb{<unit[1]>}][grid::unit] setting the allocated
width and height of the the grob respectively.}

\item{hjust, vjust}{A \verb{<numeric[1]>} between 0 and 1 setting the horizontal
and vertical justification of the grob when used as a guide for the \code{x}
and \code{y} aesthetics.}

\item{position}{Where this guide should be drawn: one of \code{"top"}, \code{"bottom"},
\code{"left"}, or \code{"right"}.}
}
\value{
A \verb{<GizmoGrob>} object.
}
\description{
This guide displays a user-provided grob.
}
\details{
\subsection{Styling options}{

There are no \code{theme()} styling options for \code{gizmo_grob()}.
}
}
\examples{
circle <- grid::circleGrob()

# A standard plot with grob gizmos
ggplot(mpg, aes(displ, hwy, colour = cty)) +
  geom_point() +
  guides(
    x.sec = gizmo_grob(
      circle, hjust = 0.75,
      width = unit(2, "cm"), height = unit(2, "cm")
    ),
    colour = gizmo_grob(
      circle, width = unit(1, "cm"), height = unit(1, "cm")
    )
  )
}
\seealso{
Other gizmos:
\code{\link[=gizmo_barcap]{gizmo_barcap()}},
\code{\link[=gizmo_density]{gizmo_density()}},
\code{\link[=gizmo_histogram]{gizmo_histogram()}},
\code{\link[=gizmo_stepcap]{gizmo_stepcap()}}
}
\concept{gizmos}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/key-.R
\name{key_standard}
\alias{key_standard}
\alias{key_auto}
\alias{key_manual}
\alias{key_map}
\alias{key_minor}
\alias{key_log}
\alias{key_none}
\title{Standard keys}
\usage{
key_auto(..., call = NULL)

key_manual(
  aesthetic,
  value = aesthetic,
  label = as.character(value),
  type = NULL,
  ...,
  call = NULL
)

key_map(data, ..., call = NULL)

key_minor(..., call = NULL)

key_log(
  prescale_base = NULL,
  negative_small = 0.1,
  expanded = TRUE,
  labeller = NULL,
  ...,
  call = NULL
)

key_none()
}
\arguments{
\item{...}{The \code{...} parameter has two purposes.
\enumerate{
\item In \code{key_map()} it is \code{\link[rlang:topic-data-mask]{<data-masking>}}. A set of
mappings similar to those provided to \code{\link[ggplot2:aes]{aes()}}, which will
be evaluated in the \code{data} argument. These must contain \code{aesthetic}
mapping.
\item In other keys, \code{...} can be used to transfer graphical properties to the
individual breaks of a guide. For example, using \code{colour = "blue"} will
draw parts of the guides associated with breaks in blue. There is a shallow
hierarchy in that \code{text_colour}, \code{line_colour}, \code{rect_colour} and
\code{point_colour} are the specific properties for elements, but all inherit
from the main \code{colour} setting. Likewise, \code{size}, \code{linewidth}, \code{linetype}
and \code{fill} have specific variants for elements.
}}

\item{call}{A \link[rlang:topic-error-call]{call} to display in messages.}

\item{aesthetic, value}{A vector of values for the guide to represent
equivalent to the \code{breaks} argument in scales. The \code{aesthetic} will be
mapped, whereas \code{value} will not. For most intents and purposes,
\code{aesthetic} and \code{value} should be identical.}

\item{label}{A \verb{<character>} or list of expressions to use as labels.}

\item{type}{A \verb{<character>} vector representing the one of the break types:
\code{"major"}, \code{"minor"} or \code{"mini"}. If \code{NULL} (default), all breaks are
treated as major breaks.}

\item{data}{A \verb{<data.frame>} or similar object coerced by
\code{\link[ggplot2:fortify]{fortify()}} to a \verb{<data.frame>}, in which the \code{mapping}
argument is evaluated.}

\item{prescale_base}{A \verb{<numeric[1]>} giving the base of logarithm to
transform data manually. The default, \code{NULL}, will use the scale
transformation to calculate positions. It is only advisable to set the
\code{prescale_base} argument when the data have already been log-transformed.
When using a log-transform in the scale or in
\code{\link[ggplot2:coord_transform]{coord_transform()}}, the default \code{NULL} is
recommended.}

\item{negative_small}{A \verb{<numeric[1]>} setting the smallest absolute value
that is marked with a tick in case the scale limits include 0 or negative
numbers.}

\item{expanded}{A \verb{<logical[1]>} determining whether the ticks should cover
the entire range after scale expansion (\code{TRUE}, default), or be restricted
to the scale limits (\code{FALSE}).}

\item{labeller}{A \verb{<function>} that receives major breaks and returns
formatted labels. For \code{key_log()}, \code{NULL} will default to
\code{\link[scales:label_log]{scales::label_log()}} for strictly positive numbers and a custom labeller
when negative numbers are included.}
}
\value{
For \code{key_auto()}, \code{key_minor()} and \code{key_log()} a function. For
\code{key_manual()} and \code{key_map()} a \verb{<data.frame>} with the \verb{<key_standard>}
class.
}
\description{
These functions are helper functions for working with tick marks as keys
in guides. They all share the goal of creating a guide key, but have
different outcomes:
\itemize{
\item \code{key_auto()} is a function factory whose functions extract a typical
key from major breaks in a scale.
\item \code{key_manual()} uses user-provided vectors to make a key.
\item \code{key_map()} makes mappings from a \verb{<data.frame>} to make a key.
\item \code{key_minor()} is a function factory whose functions also extract minor
break positions for minor tick marks.
\item \code{key_log()} is a function factory whose functions place ticks at intervals
in log10 space.
\item \code{key_none()} makes an empty key with no entries.
}
}
\examples{
# An example scale
template <- scale_x_continuous(limits = c(0, 10))

# The auto, minor and log keys operate on scales
key_auto()(template)
key_minor()(template)

# So does the log key
template <- scale_x_continuous(transform = "log10", limits = c(0.1, 10))
key_log()(template)

# Providing custom values
key_manual(
  aesthetic = 1:5,
  label = c("one", "two", "three", "four", "five")
)

# Values from a `<data.frame>`
key_map(ToothGrowth, aesthetic = unique(supp))

# Empty key
key_none()
}
\seealso{
Other keys:
\code{\link{key_group}},
\code{\link{key_range}},
\code{\link{key_segments}},
\code{\link{key_specialty}}
}
\concept{keys}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/primitive-spacer.R
\name{primitive_spacer}
\alias{primitive_spacer}
\title{Guide primitive: spacer}
\usage{
primitive_spacer(
  space = NULL,
  title = waiver(),
  theme = NULL,
  position = waiver()
)
}
\arguments{
\item{space}{A [\verb{<unit[1]>}][grid::unit()]}

\item{title}{One of the following to indicate the title of the guide:
\itemize{
\item A \verb{<character[1]>} or \verb{<expression[1]>} to set a custom title.
\item \code{NULL} to not display any title.
\item \code{\link[ggplot2:waiver]{waiver()}} (default) to take the name of the scale
object or the name specified in \code{\link[ggplot2:labs]{labs()}} as the title.
}}

\item{theme}{A \code{\link[ggplot2:theme]{<theme>}} object to style the guide individually or
differently from the plot's theme settings. The \code{theme} argument in the
guide overrides and is combined with the plot's theme.}

\item{position}{A \verb{<character[1]>} giving the location of the guide. Can be one of \code{"top"},
\code{"bottom"}, \code{"left"} or \code{"right"}.}
}
\value{
A \verb{<PrimitiveSpacer>} primitive guide that can be used inside
other guides.
}
\description{
This function constructs a spacer \link[=guide-primitives]{guide primitive}.
The spacer is intended for use in \link[=guide-composition]{guide composition}.
}
\details{
\subsection{Styling options}{

Below are the \link[ggplot2:theme]{theme} options that determine the styling of
this guide. In context to many primitive guides, whether it is used in an
axis or legend has no bearing on the style.\tabular{llll}{
   \strong{Theme setting} \tab \strong{Context} \tab \strong{Type} \tab \strong{Description} \cr
   \code{legendry.guide.spacing} \tab Any \tab \code{\link[=unit]{unit()}} \tab Fallback amount of spacing when the \code{space} argument is \code{NULL} \cr
}


There are no other styling options.
The context-agnostic alternative to using \code{theme()} is to use
\code{\link[=theme_guide]{theme_guide()}}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{primitive_spacer(theme = theme_guide(
  spacing = unit(5, "mm"),
))
}\if{html}{\out{</div>}}
}
}
\examples{
ggplot(mpg, aes(displ, hwy)) +
  geom_point() +
  guides(
    x = guide_axis_stack("axis", primitive_spacer(unit(1, "cm")), "axis")
  )
}
\seealso{
Other primitives:
\code{\link[=primitive_box]{primitive_box()}},
\code{\link[=primitive_bracket]{primitive_bracket()}},
\code{\link[=primitive_fence]{primitive_fence()}},
\code{\link[=primitive_labels]{primitive_labels()}},
\code{\link[=primitive_line]{primitive_line()}},
\code{\link[=primitive_segments]{primitive_segments()}},
\code{\link[=primitive_ticks]{primitive_ticks()}},
\code{\link[=primitive_title]{primitive_title()}}
}
\concept{primitives}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/guide-legend-cross.R
\name{guide_legend_cross}
\alias{guide_legend_cross}
\title{Cross legend guide}
\usage{
guide_legend_cross(
  key = NULL,
  title = waiver(),
  row_title = waiver(),
  col_title = waiver(),
  swap = FALSE,
  col_text = element_text(angle = 90, vjust = 0.5),
  subtitle_position = position_text(angle = c(0, -90, 0, 90), hjust = 0.5),
  override.aes = list(),
  reverse = FALSE,
  theme = NULL,
  position = NULL,
  direction = NULL,
  order = 0L
)
}
\arguments{
\item{key}{One of the following key specifications:
\itemize{
\item A \link[=key_group_split]{group split} specification when using the legend to
display a compound variable like \code{paste(var1, var2)}.
\item A \link[=key_standard]{standard key} specification, like \code{\link[=key_auto]{key_auto()}}, when
crossing two separate variables across two scales.
}}

\item{title}{One of the following to indicate the title of the guide:
\itemize{
\item A \verb{<character[1]>} or \verb{<expression[1]>} to set a custom title.
\item \code{NULL} to not display any title.
\item \code{\link[ggplot2:waiver]{waiver()}} (default) to take the name of the scale
object or the name specified in \code{\link[ggplot2:labs]{labs()}} as the title.
}}

\item{row_title, col_title}{One of the following to indicate subtitles
spanning the rows and columns of the guide:
\itemize{
\item A \verb{<character[1]>} or \verb{<expression[1]>} to set a custom title.
\item \code{NULL} to not display any title.
\item \code{\link[ggplot2:waiver]{waiver()}} to propagate subtitles from
merging guides (default).
}}

\item{swap}{A \verb{<logical[1]>} which when \code{TRUE} exchanges the column and row
variables in the displayed legend.}

\item{col_text}{An \verb{<element_text>} object giving adjustments to text for
the column labels. Can be \code{NULL} to display column labels in equal fashion
to the row labels.}

\item{subtitle_position}{A named list of 4
\link[ggplot2:element_text]{text elements}, having the names \code{"top"},
\code{"right"}, \code{"bottom"} and \verb{"left}. These govern the display of subtitles
when placed in any of these positions relative to the keys. See
\code{\link[=position_text]{position_text()}} for a convenient helper.}

\item{override.aes}{A named \verb{<list>} specifying aesthetic parameters of the
key glyphs. See details and examples in
\code{\link[ggplot2:guide_legend]{guide_legend()}}.}

\item{reverse}{A \verb{<logical[2]>} whether the order of the keys should be
inverted, where the first value controls the row order and second value
the column order. Input as \verb{<logical[1]>} will be recycled.}

\item{theme}{A \code{\link[ggplot2:theme]{<theme>}} object to style the guide individually or
differently from the plot's theme settings. The \code{theme} argument in the
guide overrides and is combined with the plot's theme.}

\item{position}{A \verb{<character[1]>} giving the location of the guide. Can be one of \code{"top"},
\code{"bottom"}, \code{"left"} or \code{"right"}.}

\item{direction}{A \verb{<character[1]>} indicating the direction of the guide. Can be on of
\code{"horizontal"} or \code{"vertical"}.}

\item{order}{A positive \verb{<integer[1]>} that specifies the order of this guide among
multiple guides. This controls in which order guides are merged if there
are multiple guides for the same position. If \code{0} (default), the order is
determined by a hashing indicative settings of a guide.}
}
\value{
A \verb{<GuideLegend>} object.
}
\description{
This is a legend type similar to \code{\link[ggplot2:guide_legend]{guide_legend()}}
that displays crosses, or: interactions, between two variables.
}
\details{
\subsection{Styling options}{

Below are the \link[ggplot2:theme]{theme} options that determine the styling of
this guide. Note that these are almost the same for
\code{\link[ggplot2:guide_legend]{ggplot2::guide_legend()}}.\tabular{lll}{
   \strong{Theme setting} \tab \strong{Type} \tab \strong{Description} \cr
   \code{legend.background} \tab \code{\link[ggplot2:element_rect]{element_rect()}} \tab Background of the legend. \cr
   \code{legend.margin} \tab \code{\link[ggplot2:margin]{margin()}} \tab Padding around the legend. \cr
   \code{legend.text} \tab \code{\link[ggplot2:element_text]{element_text()}} \tab Labels displayed next to keys. \cr
   \code{legend.text.position} \tab \verb{<character[1]>} \tab One of \code{"top"}, \code{"right"}, \code{"bottom"} or \code{"left"}. \cr
   \code{legend.title} \tab \code{\link[ggplot2:element_text]{element_text()}} \tab Title of the legend. \cr
   \code{legend.title.position} \tab \verb{<character[1]>} \tab One of \code{"top"}, \code{"right"}, \code{"bottom"} or \code{"left"}. \cr
   \code{legend.key} \tab \code{\link[ggplot2:element_rect]{element_rect()}} \tab Background of the key areas. \cr
   \code{legend.key.height} \tab \code{\link[=unit]{unit()}} \tab Height of keys. \cr
   \code{legend.key.width} \tab \code{\link[=unit]{unit()}} \tab Width of keys. \cr
   \code{legend.key.justification} \tab \verb{<numeric[2]>} \tab Justification for placing legend keys in excess space. \cr
   \code{legend.key.spacing.x} \tab \code{\link[=unit]{unit()}} \tab Horizontal spacing between keys. \cr
   \code{legend.key.spacing.y} \tab \code{\link[=unit]{unit()}} \tab Vertical spacing between keys. Taken literally. \cr
}


The context-agnostic alternative to using \code{theme()} is to use
\code{\link[=theme_guide]{theme_guide()}}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{guide_legend_cross(theme = theme_guide(
  text = element_text(),
  text.position = "right",
  title = element_text(),
  title.position = "top",
  key = element_rect(),
  key.height = unit(5, "mm"),
  key.width = unit(5, "mm"),
  key.justification = c(0.5, 0.5),
  key.spacing.x = unit(5, "mm"),
  key.spacing.y = unit(5, "mm"),
  margin = margin(5),
  background = element_rect(),
))
}\if{html}{\out{</div>}}
}
}
\examples{
# Standard use for single aesthetic. The default is to split labels to
# disentangle aesthetics that are already crossed (by e.g. `paste()`)
ggplot(mpg, aes(displ, hwy)) +
  geom_point(aes(colour = paste(year, drv))) +
  guides(colour = "legend_cross")

# If legends should be merged between identical aesthetics, both need the
# same legend type.
ggplot(mpg, aes(displ, hwy)) +
  geom_point(aes(colour = paste(year, drv), shape = paste(year, drv))) +
  guides(colour = "legend_cross", shape = "legend_cross")

# Crossing two aesthetics requires a shared title and `key = "auto"`. The
# easy way to achieve this is to predefine a shared guide.
my_guide <- guide_legend_cross(key = "auto", title = "My title")

ggplot(mpg, aes(displ, hwy)) +
  geom_point(aes(colour = drv, shape = factor(year))) +
  guides(colour = my_guide, shape  = my_guide)

# You can cross more than 2 aesthetics but not more than 2 unique aesthetics.
ggplot(mpg, aes(displ, hwy)) +
  geom_point(aes(colour = drv, shape = factor(year), size = factor(drv))) +
  scale_size_ordinal() +
  guides(colour = my_guide, shape = my_guide, size = my_guide)

# You can merge an aesthetic that is already crossed with an aesthetic that
# contributes to only one side of the cross.
ggplot(mpg, aes(displ, hwy)) +
  geom_point(aes(colour = paste(year, drv), shape  = drv)) +
  guides(
    colour = guide_legend_cross(title = "My Title"),
    shape  = guide_legend_cross(title = "My Title", key = "auto")
  )
}
\seealso{
Other standalone guides:
\code{\link[=guide_axis_annotation]{guide_axis_annotation()}},
\code{\link[=guide_axis_base]{guide_axis_base()}},
\code{\link[=guide_axis_dendro]{guide_axis_dendro()}},
\code{\link[=guide_axis_nested]{guide_axis_nested()}},
\code{\link[=guide_axis_plot]{guide_axis_plot()}},
\code{\link[=guide_axis_symbols]{guide_axis_symbols()}},
\code{\link[=guide_circles]{guide_circles()}},
\code{\link[=guide_colbar]{guide_colbar()}},
\code{\link[=guide_colring]{guide_colring()}},
\code{\link[=guide_colsteps]{guide_colsteps()}},
\code{\link[=guide_legend_base]{guide_legend_base()}},
\code{\link[=guide_legend_group]{guide_legend_group()}},
\code{\link[=guide_legend_manual]{guide_legend_manual()}}

Other legend guides:
\code{\link[=guide_legend_base]{guide_legend_base()}},
\code{\link[=guide_legend_group]{guide_legend_group()}},
\code{\link[=guide_legend_manual]{guide_legend_manual()}}
}
\concept{legend guides}
\concept{standalone guides}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/guide-circles.R
\name{guide_circles}
\alias{guide_circles}
\title{Circle size guide}
\usage{
guide_circles(
  key = NULL,
  title = waiver(),
  theme = NULL,
  hjust = 0.5,
  vjust = 0,
  text_position = NULL,
  clip_text = FALSE,
  override.aes = list(shape = 1L),
  position = waiver(),
  direction = NULL
)
}
\arguments{
\item{key}{A \link[=key_standard]{standard key} specification. Defaults to
\code{\link[=key_auto]{key_auto()}}. See more information in the linked topic.}

\item{title}{One of the following to indicate the title of the guide:
\itemize{
\item A \verb{<character[1]>} or \verb{<expression[1]>} to set a custom title.
\item \code{NULL} to not display any title.
\item \code{\link[ggplot2:waiver]{waiver()}} (default) to take the name of the scale
object or the name specified in \code{\link[ggplot2:labs]{labs()}} as the title.
}}

\item{theme}{A \code{\link[ggplot2:theme]{<theme>}} object to style the guide individually or
differently from the plot's theme settings. The \code{theme} argument in the
guide overrides and is combined with the plot's theme.}

\item{hjust, vjust}{A \verb{<numeric[1]>} between 0 and 1 giving the horizontal
and vertical justification, respectively, of the central shapes. It is
recommended \code{hjust = 0.5} when text is placed on the left or right and
\code{vjust = 0.5} is recommended when text is placed on top or in the bottom.}

\item{text_position}{A string, one of \code{"ontop"}, \code{"top"}, \code{"right"},
\code{"bottom"}, or \code{"left"} do describe the placement of labels. The
default (\code{NULL}), will take the \code{legend.text.position} theme setting.}

\item{clip_text}{A \verb{<logical[1]>} whether to give text in the \code{"ontop"}
position a small rectangle of background colour.}

\item{override.aes}{A named \verb{<list>} specifying aesthetic parameters of the
key glyphs. See details and examples in
\code{\link[ggplot2:guide_legend]{guide_legend()}}.}

\item{position}{A \verb{<character[1]>} giving the location of the guide. Can be one of \code{"top"},
\code{"bottom"}, \code{"left"} or \code{"right"}.}

\item{direction}{A \verb{<character[1]>} indicating the direction of the guide. Can be on of
\code{"horizontal"} or \code{"vertical"}.}
}
\value{
A \verb{<GuideCircles>} object.
}
\description{
This guide displays the sizes of points as a series of circles. It is
typically paired with \code{\link[ggplot2:geom_point]{geom_point()}} with
\code{\link[ggplot2:draw_key_point]{draw_key_point()}} glyphs.
}
\details{
Please note that the default size scales scale to area, not radius, so
equidistant breaks will appear at irregularly spaced positions due to
labelling the diameter of a circle.

This graph was designed with standard round \link[ggplot2:scale_shape]{shapes}
in mind, i.e. shapes 1, 16, 19 and 21. For that reason, \code{shape = 1} is the
default \code{override.aes} argument. Other shapes will probably be drawn but the
quality of their alignment and label placement may be unsatisfactory.
\subsection{Styling options}{

Below are the \link[ggplot2:theme]{theme} options that determine the styling of
this guide.\tabular{lll}{
   \strong{Theme setting} \tab \strong{Type} \tab \strong{Description} \cr
   \code{legend.background} \tab \code{\link[ggplot2:element_rect]{element_rect()}} \tab Background of the legend. \cr
   \code{legend.margin} \tab \code{\link[ggplot2:margin]{margin()}} \tab Padding around the legend. \cr
   \code{legend.key} \tab \code{\link[ggplot2:element_rect]{element_rect()}} \tab Background of the key area underneath circles. \cr
   \code{legend.text} \tab \code{\link[ggplot2:element_text]{element_text()}} \tab Labels displayed next to tick marks or on top of circles. \cr
   \code{legendry.legend.key.margin} \tab \code{\link[ggplot2:margin]{margin()}} \tab Padding between circles and edge of key area. \cr
   \code{legend.ticks} \tab \code{\link[ggplot2:element_line]{element_line()}} \tab Tick marks connecting circle to label. \cr
   \code{legend.title} \tab \code{\link[ggplot2:element_text]{element_text()}} \tab Title of the legend. \cr
   \code{legend.title.position} \tab \verb{<character[1]>} \tab One of \code{"top"}, \code{"right"}, \code{"bottom"} or \code{"left"}. \cr
}


Styling options \emph{per break} can be set in the \link[=key_standard]{standard key}.
These override theme settings.

The context-agnostic alternative to using \code{theme()} is to use
\code{\link[=theme_guide]{theme_guide()}}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{guide_circles(theme = theme_guide(
  text = element_text(),
  title = element_text(),
  title.position = "top",
  margin = margin(5),
  key = element_rect(),
  key.margin = margin(5),
  background = element_rect(),
  ticks = element_line(),
))
}\if{html}{\out{</div>}}
}
}
\examples{
# A standard plot
p <- ggplot(mtcars, aes(disp, mpg)) +
  geom_point(aes(size = hp), alpha = 0.3)

# By default, the sizes aren't large enough to make this guide clear
p + scale_size_area(guide = "circles")

# Update with a more approrpriate scale
p <- p +
  scale_size_area(
    max_size = 30,
    limits = c(0, NA),
    breaks = c(0, 25, 100, 250)
  )
p + guides(size = "circles")

# Horizontal orientation
p + guides(size = guide_circles(
  vjust = 0.5, hjust = 0, text_position = "bottom"
))

# Alternative text placement
p + guides(size = guide_circles(
  text_position = "ontop",
  clip_text = TRUE
))

# More styling options
p + guides(size = guide_circles(override.aes = aes(colour = "red")))+
  theme(
    # Key background
    legend.key = element_rect(colour = "black", fill = 'white'),
    # Padding around central shapes
    legendry.legend.key.margin = margin(1, 1, 1, 1, "cm"),
    legend.ticks = element_line(colour = "blue"),
    legend.text.position = "left"
  )
}
\seealso{
Other standalone guides:
\code{\link[=guide_axis_annotation]{guide_axis_annotation()}},
\code{\link[=guide_axis_base]{guide_axis_base()}},
\code{\link[=guide_axis_dendro]{guide_axis_dendro()}},
\code{\link[=guide_axis_nested]{guide_axis_nested()}},
\code{\link[=guide_axis_plot]{guide_axis_plot()}},
\code{\link[=guide_axis_symbols]{guide_axis_symbols()}},
\code{\link[=guide_colbar]{guide_colbar()}},
\code{\link[=guide_colring]{guide_colring()}},
\code{\link[=guide_colsteps]{guide_colsteps()}},
\code{\link[=guide_legend_base]{guide_legend_base()}},
\code{\link[=guide_legend_cross]{guide_legend_cross()}},
\code{\link[=guide_legend_group]{guide_legend_group()}},
\code{\link[=guide_legend_manual]{guide_legend_manual()}}
}
\concept{standalone guides}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/guide-colring.R
\name{guide_colring}
\alias{guide_colring}
\title{Colour rings and arcs}
\usage{
guide_colring(
  title = waiver(),
  key = "auto",
  start = 0,
  end = NULL,
  outer_guide = "axis_base",
  inner_guide = "axis_base",
  nbin = 300L,
  reverse = FALSE,
  show_labels = "outer",
  theme = NULL,
  vanilla = TRUE,
  position = waiver(),
  available_aes = c("colour", "fill"),
  ...
)
}
\arguments{
\item{title}{One of the following to indicate the title of the guide:
\itemize{
\item A \verb{<character[1]>} or \verb{<expression[1]>} to set a custom title.
\item \code{NULL} to not display any title.
\item \code{\link[ggplot2:waiver]{waiver()}} (default) to take the name of the scale
object or the name specified in \code{\link[ggplot2:labs]{labs()}} as the title.
}}

\item{key}{A \link[=key_standard]{standard key} specification. Defaults to
\code{\link[=key_auto]{key_auto()}}.}

\item{start, end}{A \verb{<numeric[1]>} in radians specifying the offset of the
starting and end points from 12 o'clock. The \code{NULL} default for \code{end},
internally defaults to \code{start + 2 * pi}.}

\item{outer_guide, inner_guide}{Guides to display on the outside and inside
of the colour ring. Each guide can be specified using one of the following:
\itemize{
\item A \verb{<Guide>} class object.
\item A \verb{<function>} that returns a \verb{<Guide>} class object.
\item A \verb{<character>} naming such function, without the \code{guide_} or
\code{primitive_} prefix.
}}

\item{nbin}{A positive \verb{<integer[1]>} determining how many colours to display.}

\item{reverse}{A \verb{<logical[1]>} whether to reverse continuous guides.
If \code{TRUE}, guides like colour bars are flipped. If \code{FALSE} (default),
the original order is maintained.}

\item{show_labels}{A \verb{<character[1]>} indicating for which guide labels
should be shown. Can be one of \code{"outer"} (default), \code{"inner"}, \code{"both"} or
\code{"none"}. Note that labels can only be omitted if the related guide
has a label suppression mechanism.}

\item{theme}{A \code{\link[ggplot2:theme]{<theme>}} object to style the guide individually or
differently from the plot's theme settings. The \code{theme} argument in the
guide overrides and is combined with the plot's theme.}

\item{vanilla}{A \verb{<logical[1]>} whether to have the default style match
the vanilla \code{guide_colourbar()} (\code{TRUE}) or take the theme
verbatim (\code{FALSE}).}

\item{position}{A \verb{<character[1]>} giving the location of the guide. Can be one of \code{"top"},
\code{"bottom"}, \code{"left"} or \code{"right"}.}

\item{available_aes}{A \verb{<character>} vector listing the aesthetics for which this guide can
be build.}

\item{...}{Arguments forwarded to the \code{outer_guide} and \code{inner_guide} if
provided as functions or strings.}
}
\value{
A \verb{<Guide>} object.
}
\description{
Similar to \code{\link[ggplot2:guide_colourbar]{guide_colourbar()}}, this guide
displays continuous \code{colour} or \code{fill} aesthetics. Instead of a bar, the
gradient in shown in a ring or arc, which can be convenient for cyclical
palettes such as some provided in the \pkg{scico} package.
}
\details{
\subsection{Styling options}{

This guide is a hybrid composition guide, where the \link[ggplot2:theme]{theme}
settings apply both this guide itself and the constituents it manages.
The constituents are linked below so you can find their 'Styling options'
sections. Note that \code{guide_axis_base()} is just a default that can be
swapped out.\tabular{ll}{
   \strong{Constituent} \tab \strong{Description} \cr
   \code{\link[=guide_axis_base]{guide_axis_base()}} \tab Makes up the tick marks and labels at inner and outer rings. \cr
}


In addition to the constituent's theme setting, it also has the following
settings:\tabular{lll}{
   \strong{Theme setting} \tab \strong{Type} \tab \strong{Description} \cr
   \code{legend.background} \tab \code{\link[ggplot2:element_rect]{element_rect()}} \tab Background of the legend. \cr
   \code{legend.margin} \tab \code{\link[ggplot2:margin]{margin()}} \tab Padding around the legend. \cr
   \code{legend.key.width} \tab \code{\link[=unit]{unit()}} \tab Radial thickness of the donut ring. \cr
   \code{legend.key.size} \tab \code{\link[=unit]{unit()}} \tab Size of the legend. Multiplied by 5 to roughly match dimensions of colour bar guides. \cr
   \code{legend.frame} \tab \code{\link[ggplot2:element_rect]{element_rect()}} \tab Outline of the donut. The \code{fill} setting is ignored. \cr
   \code{legend.title} \tab \code{\link[ggplot2:element_text]{element_text()}} \tab Title of the legend. \cr
   \code{legend.title.position} \tab \verb{<character[1]>} \tab One of \code{"top"}, \code{"right"}, \code{"bottom"} or \code{"left"}. \cr
}


Styling options \emph{per break} can be set in the \link[=key_standard]{standard key}.
These override theme settings.

The context-agnostic alternative to using \code{theme()} is to use
\code{\link[=theme_guide]{theme_guide()}}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{guide_colring(theme = theme_guide(
  # Ring settings
  title = element_text(),
  title.position = "top",
  margin = margin(5),
  background = element_rect(),
  frame = element_rect(),
  key.size = unit(1, "cm"),
  key.width = unit(5, "mm"),

  # Common options for `guide_axis_base()`
  line = element_line(),
  text = element_text(),
  ticks = element_line(),
  ticks.length = unit(5, "mm"),
))
}\if{html}{\out{</div>}}
}
}
\examples{
# Rings works best with a cyclical palette
my_pal <- c("black", "tomato", "white", "dodgerblue", "black")

p <- ggplot(mpg, aes(displ, hwy, colour = cty)) +
  geom_point() +
  scale_colour_gradientn(colours = my_pal)

# Standard colour ring
p + guides(colour = "colring")

# As an arc
p + guides(colour = guide_colring(
  start = 1.25 * pi, end = 2.75 * pi
))

# Removing the inner tick marks
p + guides(colour = guide_colring(inner_guide = "none"))

# Include labels on the inner axis
p + guides(colour = guide_colring(show_labels = "both"))

# Passing an argument to inner/outer guides
p + guides(colour = guide_colring(angle = 0))
}
\seealso{
Other standalone guides:
\code{\link[=guide_axis_annotation]{guide_axis_annotation()}},
\code{\link[=guide_axis_base]{guide_axis_base()}},
\code{\link[=guide_axis_dendro]{guide_axis_dendro()}},
\code{\link[=guide_axis_nested]{guide_axis_nested()}},
\code{\link[=guide_axis_plot]{guide_axis_plot()}},
\code{\link[=guide_axis_symbols]{guide_axis_symbols()}},
\code{\link[=guide_circles]{guide_circles()}},
\code{\link[=guide_colbar]{guide_colbar()}},
\code{\link[=guide_colsteps]{guide_colsteps()}},
\code{\link[=guide_legend_base]{guide_legend_base()}},
\code{\link[=guide_legend_cross]{guide_legend_cross()}},
\code{\link[=guide_legend_group]{guide_legend_group()}},
\code{\link[=guide_legend_manual]{guide_legend_manual()}}
}
\concept{standalone guides}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/primitive-.R
\name{guide-primitives}
\alias{guide-primitives}
\title{Guide primitives}
\description{
Guide primitives are the building blocks of more complex guides. On their
own, they are not very useful as a visual reflection of a scale.

Their purpose is to be combined with one another to form a more complex,
complete guides that \emph{do} reflect a scale in some way.
}
\details{
The guide primitives are simple, but flexible in that they are not tailored
for one particular aesthetic. That way they can be reused and combined
at will.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compose-.R
\name{guide-composition}
\alias{guide-composition}
\alias{new_compose}
\title{Guide composition}
\usage{
new_compose(
  guides,
  args = list(),
  ...,
  available_aes = c("any", "x", "y", "r", "theta"),
  call = caller_env(),
  super = Compose
)
}
\arguments{
\item{guides}{A \verb{<list>} of guides wherein each element is one of the
following:
\itemize{
\item A \verb{<Guide>} class object.
\item A \verb{<function>} that returns a \verb{<Guide>} class object.
\item A \verb{<character[1]>} naming such a function, without the \code{guide_} or
\code{primitive_} prefix.
}}

\item{args}{A \verb{<list>} of arguments to pass to guides that are given either
as a function or as a string.}

\item{...}{Additional parameters to pass on to
\code{\link[ggplot2:new_guide]{new_guide()}}.}

\item{available_aes}{A \verb{<character>} giving aesthetics that must match the
the guides.}

\item{call}{A \link[rlang:topic-error-call]{call} to display in messages.}

\item{super}{A \verb{<Compose>} class object giving a meta-guide for composition.}
}
\value{
A \verb{<Compose>} (sub-)class guide that composes other guides.
}
\description{
Guide composition is a meta-guide orchestrating an ensemble of other guides.
On their own, a 'composing' guide is not very useful as a visual reflection
of a scale.
}
\details{
\subsection{Styling options}{

Below are the \link[ggplot2:theme]{theme} options that determine the styling of
this guide.\tabular{lll}{
   \strong{Theme setting} \tab \strong{Type} \tab \strong{Description} \cr
   \code{legendry.guide.spacing} \tab \code{\link[=unit]{unit()}} \tab Spacing between guides. \cr
}


There are no further styling options.
}
}
\examples{
# The `new_compose()` function is not intended to be used directly
my_composition <- new_compose(list("axis", "axis"), super = ComposeStack)

# Is the same as
my_composition <- compose_stack("axis", "axis")
}
\seealso{
Other composition:
\code{\link[=compose_crux]{compose_crux()}},
\code{\link[=compose_ontop]{compose_ontop()}},
\code{\link[=compose_sandwich]{compose_sandwich()}},
\code{\link[=compose_stack]{compose_stack()}}
}
\concept{composition}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/guide-axis-dendro.R
\name{guide_axis_dendro}
\alias{guide_axis_dendro}
\title{Dendrogram guide}
\usage{
guide_axis_dendro(
  key = "dendro",
  title = waiver(),
  theme = NULL,
  labels = TRUE,
  space = rel(10),
  vanish = TRUE,
  n.dodge = 1L,
  angle = waiver(),
  check.overlap = FALSE,
  ticks = "none",
  axis_line = "none",
  order = 0L,
  position = waiver()
)
}
\arguments{
\item{key}{A \link[=key_segments]{segment key} specification. See more information
in the linked topic. Alternatively, an object of class
\code{\link[stats:hclust]{<hclust>}} that automatically invokes \code{key_dendro()}.}

\item{title}{One of the following to indicate the title of the guide:
\itemize{
\item A \verb{<character[1]>} or \verb{<expression[1]>} to set a custom title.
\item \code{NULL} to not display any title.
\item \code{\link[ggplot2:waiver]{waiver()}} (default) to take the name of the scale
object or the name specified in \code{\link[ggplot2:labs]{labs()}} as the title.
}}

\item{theme}{A \code{\link[ggplot2:theme]{<theme>}} object to style the guide individually or
differently from the plot's theme settings. The \code{theme} argument in the
guide overrides and is combined with the plot's theme.}

\item{labels, ticks, axis_line}{Guides to use as labels, ticks or axis lines. Can be specified as one of
the following:
\itemize{
\item A \verb{<logical[1]>} which when \code{FALSE} will set the guide to \code{guide_none()}
and if \code{TRUE}, will set guide to appropriate primitive.
\item A \verb{<Guide>} class object.
\item A \verb{<function>} that returns a \verb{<Guide>} class object.
\item A \verb{<character[1]>} naming such a function, without the \code{guide_} or
\code{primitive_} prefix.
}}

\item{space}{Either a \code{\link[grid:unit]{<unit>}} or \code{\link[ggplot2:rel]{<rel>}}
object of length 1 determining the space allocated in the orthogonal
direction. When the \code{space} argument is of class \verb{<rel>} (default) the
base size is taken from the tick length theme setting.}

\item{vanish}{Only relevant when the guide is used in the secondary theta
position: a \verb{<logical[1]>} on whether the continue to draw the segments
until they meed in the center (\code{TRUE}) or strictly observe the \code{space}
setting (\code{FALSE}).}

\item{n.dodge}{An positive \verb{<integer[1]>} setting the number of layers text
labels can occupy to avoid overlapping labels.}

\item{angle}{A specification for the text angle. Compared to setting the \code{angle} argument
in \code{\link[ggplot2:element_text]{element_text()}}, this argument uses some
heuristics to automatically pick the \code{hjust} and \code{vjust} that you
probably want. Can be one of the following:
\itemize{
\item \code{NULL} to take angles and justification settings directly from the theme.
\item \code{\link[ggplot2:waiver]{waiver()}} to allow reasonable defaults in special
cases.
\item A \verb{<numeric[1]>} between -360 and 360 for the text angle in degrees.
}}

\item{check.overlap}{A \verb{<logical[1]>} indicating whether to check for and
omit overlapping text. If \code{TRUE}, first, last and middle labels are
recursively prioritised in that order. If \code{FALSE}, all labels are drawn.}

\item{order}{A positive \verb{<integer[1]>} that specifies the order of this guide among
multiple guides. This controls in which order guides are merged if there
are multiple guides for the same position. If \code{0} (default), the order is
determined by a hashing indicative settings of a guide.}

\item{position}{A \verb{<character[1]>} giving the location of the guide. Can be one of \code{"top"},
\code{"bottom"}, \code{"left"} or \code{"right"}.}
}
\value{
A \verb{<Guide>} object.
}
\description{
This axis is a speciality axis for discrete data that has been
hierarchically clustered. Please be aware that the guide cannot affect the
scale limits, which should be set appropriately. This guide will give
misleading results when this step is skipped!
}
\details{
\subsection{Styling options}{

Because this guide is pure composite guide, the \link[ggplot2:theme]{theme}
options that govern the styling are determined by its constituents. They are
linked below so you can find their 'Styling options' sections.\tabular{ll}{
   \strong{Primitive} \tab \strong{Description} \cr
   \code{\link{compose_stack}} \tab Stacks the lines, tick marks and labels and dendrogram. \cr
   \code{\link[=primitive_segments]{primitive_segments()}} \tab The dendrogram. \cr
   \code{\link[=primitive_line]{primitive_line()}} \tab Makes up the axis line. \cr
   \code{\link[=primitive_ticks]{primitive_ticks()}} \tab Makes up the tick marks. \cr
   \code{\link[=primitive_labels]{primitive_labels()}} \tab Makes up the labels. \cr
}


The context-agnostic alternative to using \code{theme()} is to use
\code{\link[=theme_guide]{theme_guide()}}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{guide_axis_dendro(theme = theme_guide(
  line = element_line(),
  text = element_text(),
  ticks = element_line(),
  ticks.length = unit(5, "mm"),
))
}\if{html}{\out{</div>}}
}
}
\examples{
# Hierarchically cluster data
clust <- hclust(dist(scale(mtcars)), "ave")

# Using the guide along with appropriate limits
p <- ggplot(mtcars, aes(disp, rownames(mtcars))) +
  geom_col() +
  scale_y_discrete(limits = clust$labels[clust$order])

# Standard usage
p + guides(y = guide_axis_dendro(clust))

# Adding ticks and axis line
p +
  guides(y = guide_axis_dendro(clust, ticks = "ticks", axis_line = "line")) +
  theme(axis.line = element_line())

# Controlling space allocated to dendrogram
p + guides(y = guide_axis_dendro(clust, space = unit(4, "cm"))) +
  theme(axis.ticks.y.left = element_line("red"))

# If want just the dendrogram, use `labels = FALSE`
p + guides(y = guide_axis_dendro(clust, labels = FALSE), y.sec = "axis")
}
\seealso{
Other standalone guides:
\code{\link[=guide_axis_annotation]{guide_axis_annotation()}},
\code{\link[=guide_axis_base]{guide_axis_base()}},
\code{\link[=guide_axis_nested]{guide_axis_nested()}},
\code{\link[=guide_axis_plot]{guide_axis_plot()}},
\code{\link[=guide_axis_symbols]{guide_axis_symbols()}},
\code{\link[=guide_circles]{guide_circles()}},
\code{\link[=guide_colbar]{guide_colbar()}},
\code{\link[=guide_colring]{guide_colring()}},
\code{\link[=guide_colsteps]{guide_colsteps()}},
\code{\link[=guide_legend_base]{guide_legend_base()}},
\code{\link[=guide_legend_cross]{guide_legend_cross()}},
\code{\link[=guide_legend_group]{guide_legend_group()}},
\code{\link[=guide_legend_manual]{guide_legend_manual()}}
}
\concept{standalone guides}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/primitive-ticks.R
\name{primitive_ticks}
\alias{primitive_ticks}
\title{Guide primitive: line}
\usage{
primitive_ticks(key = NULL, bidi = FALSE, theme = NULL, position = waiver())
}
\arguments{
\item{key}{A \link[=key_standard]{standard key} specification. See more information
in the linked topic.}

\item{bidi}{A \verb{<logical[1]>}: whether ticks should be drawn bidirectionally
(\code{TRUE}) or in a single direction (\code{FALSE}, default).}

\item{theme}{A \code{\link[ggplot2:theme]{<theme>}} object to style the guide individually or
differently from the plot's theme settings. The \code{theme} argument in the
guide overrides and is combined with the plot's theme.}

\item{position}{A \verb{<character[1]>} giving the location of the guide. Can be one of \code{"top"},
\code{"bottom"}, \code{"left"} or \code{"right"}.}
}
\value{
A \code{PrimitiveTicks} primitive guide that can be used inside other
guides.
}
\description{
This function constructs a ticks \link[=guide-primitives]{guide primitive}.
}
\details{
\subsection{Styling options}{

Below are the \link[ggplot2:theme]{theme} options that determine the styling of
this guide, which may differ depending on whether the guide is used in
an axis or in a legend context.

The ticks can come in three variants: major, minor and minimal.
Which variants are drawn depends on the keys: \code{\link[=key_minor]{key_minor()}} draws major
and minor ticks, whereas \code{\link[=key_log]{key_log()}} also has minimal ticks.
Each variant has a corresponding length setting.

The possible \code{{position}} suffixes mentioned below are \code{x}, \code{x.top},
\code{x.bottom}, \code{y}, \code{y.left}, \code{y.right}. The \code{theta} and \code{r} position suffixes
in \pkg{ggplot2} are \emph{not} obeyed in \pkg{legendry}.\tabular{llll}{
   \strong{Theme setting} \tab \strong{Context} \tab \strong{Type} \tab \strong{Description} \cr
   \verb{axis.ticks.\{position\}} \tab Axis \tab \code{\link[ggplot2:element_line]{element_line()}} \tab Major tick lines \cr
   \verb{axis.ticks.length.\{position\}} \tab Axis \tab \code{\link[=unit]{unit()}} \tab Major tick length \cr
   \verb{axis.minor.ticks.\{position\}} \tab Axis \tab \code{\link[ggplot2:element_line]{element_line()}} \tab Minor tick lines \cr
   \verb{axis.minor.ticks.length.\{position\}} \tab Axis \tab \code{\link[=unit]{unit()}} \tab Minor tick length \cr
   \code{legendry.axis.mini.ticks} \tab Axis \tab \code{\link[ggplot2:element_line]{element_line()}} \tab Minimal tick lines \cr
   \code{legendry.axis.mini.ticks.length} \tab Axis \tab \code{\link[=unit]{unit()}} \tab Minimal tick length \cr
   \code{legend.ticks} \tab Legend \tab \code{\link[ggplot2:element_line]{element_line()}} \tab Major tick lines \cr
   \code{legend.ticks.length} \tab Legend \tab \code{\link[=unit]{unit()}} \tab Major ticks length \cr
   \code{legendry.legend.minor.ticks} \tab Legend \tab \code{\link[ggplot2:element_line]{element_line()}} \tab Minor tick lines \cr
   \code{legendry.legend.minor.ticks.length} \tab Legend \tab \code{\link[=unit]{unit()}} \tab Minor ticks length \cr
   \code{legendry.legend.mini.ticks} \tab Legend \tab \code{\link[ggplot2:element_line]{element_line()}} \tab Minimal tick lines \cr
   \code{legendry.legend.mini.ticks.length} \tab Legend \tab \code{\link[=unit]{unit()}} \tab Minimal tick length \cr
}


Styling options \emph{per break} can be set in the \link[=key_standard]{key}.
The \code{line} and prefixed properties are prioritised for the tick lines.
These override theme settings.

The context-agnostic alternative to using \code{theme()} is to use
\code{\link[=theme_guide]{theme_guide()}}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{primitive_ticks(theme = theme_guide(
  ticks = element_line(),
  ticks.length = unit(5, "mm"),
  minor.ticks = element_line(),
  minor.ticks.length = unit(4, "mm"),
  mini.ticks = element_line(),
  mini.ticks.length = unit(3, "mm")
))
}\if{html}{\out{</div>}}
}
}
\examples{
# A standard plot
p <- ggplot(mpg, aes(displ, hwy)) +
  geom_point()

# Adding as secondary guides
p + guides(x.sec = primitive_ticks(), y.sec = primitive_ticks())
}
\seealso{
Other primitives:
\code{\link[=primitive_box]{primitive_box()}},
\code{\link[=primitive_bracket]{primitive_bracket()}},
\code{\link[=primitive_fence]{primitive_fence()}},
\code{\link[=primitive_labels]{primitive_labels()}},
\code{\link[=primitive_line]{primitive_line()}},
\code{\link[=primitive_segments]{primitive_segments()}},
\code{\link[=primitive_spacer]{primitive_spacer()}},
\code{\link[=primitive_title]{primitive_title()}}
}
\concept{primitives}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/themes.R
\name{theme_guide}
\alias{theme_guide}
\title{Theme wrapper for guides}
\usage{
theme_guide(
  text = NULL,
  line = NULL,
  title = NULL,
  subtitle = NULL,
  text.position = NULL,
  title.position = NULL,
  subtitle.position = NULL,
  ticks = NULL,
  minor.ticks = NULL,
  mini.ticks = NULL,
  ticks.length = NULL,
  minor.ticks.length = NULL,
  mini.ticks.length = NULL,
  spacing = NULL,
  group.spacing = NULL,
  table.spacing = NULL,
  key = NULL,
  key.size = NULL,
  key.width = NULL,
  key.height = NULL,
  key.spacing = NULL,
  key.spacing.x = NULL,
  key.spacing.y = NULL,
  key.margin = NULL,
  key.justification = NULL,
  frame = NULL,
  byrow = NULL,
  background = NULL,
  margin = NULL,
  bracket = NULL,
  bracket.size = NULL,
  box = NULL,
  fence = NULL,
  fence.post = NULL,
  fence.rail = NULL,
  zebra.light = NULL,
  zebra.dark = NULL,
  point = NULL,
  connector = NULL
)
}
\arguments{
\item{text}{An \code{\link[ggplot2:element_text]{<element_text>}} setting both
\code{legend.text} and \code{axis.text} elements.}

\item{line}{An \code{\link[ggplot2:element_line]{<element_line>}} setting both
\code{legend.axis.line} and \code{axis.line} elements.}

\item{title}{An \code{\link[ggplot2:element_text]{<element_text>}} setting both
\code{legend.title} and \code{axis.title} elements.}

\item{subtitle}{An \code{\link[ggplot2:element_text]{<element_text>}} setting both
\code{legendry.legend.subtitle} and \code{legendry.axis.subtitle} elements.}

\item{text.position, title.position, subtitle.position}{One of \code{"top"},
\code{"right"}, \code{"bottom"} or \code{"right"} setting the following elements:
\itemize{
\item \code{text.position}: sets only \code{legend.text.position}.
\item \code{title.position}: sets only \code{legend.title.position}.
\item \code{subtitle.position} sets both \code{legendry.legend.subtitle.position} and
\code{legendry.axis.subtitle.position}
}}

\item{ticks}{An \code{\link[ggplot2:element_line]{<element_line>}} setting both
\code{axis.ticks} and \code{legend.ticks} elements.}

\item{minor.ticks}{An \code{\link[ggplot2:element_line]{<element_line>}} setting
\code{legendry.legend.minor.ticks} and all 6 of the
\verb{axis.ticks.minor.\{r/theta/x.top/x.bottom/y.left/y.right\}} elements.}

\item{mini.ticks}{An \code{\link[ggplot2:element_line]{<element_line>}} setting both
\code{legendry.legend.mini.ticks} and \code{legendry.axis.mini.ticks} elements.}

\item{ticks.length, minor.ticks.length, mini.ticks.length}{A
[\verb{<unit[1]>}][grid::unit()] setting the following elements:
\itemize{
\item \code{ticks.length}: sets both \code{legend.ticks.length} and \code{axis.ticks.length}.
\item \code{minor.ticks.length} sets both \code{axis.minor.ticks.length} and
\code{legendry.legend.minor.ticks.length}.
\item \code{mini.ticks.length} sets both \code{legendry.axis.mini.ticks.length} and
\code{legendry.legend.mini.ticks.length}.
}}

\item{spacing, group.spacing, table.spacing}{A [\verb{<unit[1]>}][grid::unit()]
settingthe \code{legendry.guide.spacing}, \code{legendry.group.spacing} and
\code{legendry.table.spacing} theme elements respectively.}

\item{key}{An \code{\link[ggplot2:element_rect]{<element_rect>}} setting the
\code{legend.key} element.}

\item{key.size, key.width, key.height}{A \code{\link[grid:unit]{<unit>}} setting the
\code{legend.key.size}, \code{legend.key.width} and \code{legend.key.height} elements
respectively.}

\item{key.spacing, key.spacing.x, key.spacing.y}{A [\verb{<unit[1]>}][grid::unit()]
setting the \code{legend.key.spacing}, \code{legend.key.spacing.x} and
\code{legend.key.spacing.y} elements respectively.}

\item{key.margin}{A \code{\link[ggplot2:margin]{<margin>}} setting the margin around
legend glyphs.}

\item{key.justification}{A [\verb{<numeric[2]>}] passed to the
\code{legend.key.justification} setting.}

\item{frame}{An \code{\link[ggplot2:element_rect]{<element_rect>}} setting the
\code{legend.frame} element.}

\item{byrow}{A \verb{<logical[1]>} setting the \code{legend.byrow} element.}

\item{background}{An \code{\link[ggplot2:element_rect]{<element_rect>}} setting the
\code{legend.background} element.}

\item{margin}{A \code{\link[ggplot2:margin]{<margin>}} setting the \code{legend.margin}
element.}

\item{bracket}{An \code{\link[ggplot2:element_line]{<element_line>}} setting the
\code{legendry.bracket} element.}

\item{bracket.size}{A [\verb{<unit[1]>}][grid::unit()] setting the
\code{legendry.bracket.size} element.}

\item{box}{An \code{\link[ggplot2:element_rect]{<element_rect>}} setting the
\code{legendry.box} element.}

\item{fence, fence.post, fence.rail}{An
\code{\link[ggplot2:element_line]{<element_line>}} setting the \code{legendry.fence},
\code{legendry.fence.post} and \code{legendry.fence.rail} respectively.}

\item{zebra.light, zebra.dark}{An \code{\link[ggplot2:element_rect]{<element_rect>}}
setting the \code{legendry.zebra.light} and \code{legendry.zebra.dark} settings
respectively.}

\item{point}{An \code{\link[ggplot2:element_point]{<element_point>}} setting the
\code{legendry.point} element.}

\item{connector}{An \code{\link[ggplot2:element_line]{<elemenet_line>}} setting the
\code{legendry.connector} element.}
}
\value{
A \verb{<theme>} object that can be provided to a guide.
}
\description{
This function does three things different than \code{\link[ggplot2:theme]{theme()}}.
}
\details{
\enumerate{
\item It has shorthand names for various guide settings, similar to
\code{\link[ggplot2:theme_sub_legend]{theme_sub_legend()}} and \code{\link[ggplot2:theme_sub_axis]{theme_sub_axis()}}
\item It simultaneously sets theme elements for axes and legends. This makes
it contextually agnostic. For example, setting \code{theme_guide(text)} will
populate \code{legend.text} and also \code{axis.text}.
\item It includes \pkg{legendry} specific settings.
}

The first two things make it very good for setting the \code{guide_*(theme)}
arguments. The second thing makes it very bad for setting plot-wide or
global themes.
}
\examples{
red_ticks <- theme_guide(
  ticks = element_line(colour = "red", linewidth = 0.5)
)

# Both axis and colourbar gain red ticks
ggplot(mpg, aes(displ, hwy, colour = cty)) +
  geom_point() +
  guides(
    colour = guide_colourbar(theme = red_ticks),
    x = guide_axis(theme = red_ticks)
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gizmo-stepcap.R
\name{gizmo_stepcap}
\alias{gizmo_stepcap}
\title{Guide gizmo: capped colour steps}
\usage{
gizmo_stepcap(
  key = "bins",
  shape = "triangle",
  size = NULL,
  show = NA,
  alpha = NA,
  oob = "keep",
  theme = NULL,
  position = waiver(),
  direction = NULL
)
}
\arguments{
\item{key}{A \link[=key_bins]{bins key} specificiation. Defaults to
\code{key_bins(even.steps = FALSE, show.limits = NULL)}. Changing the
arguments to \code{key_bins()} is fine, but changing the key type is not
advised.}

\item{shape}{A \link[=cap_options]{cap} specification by providing one of the
following:
\itemize{
\item A cap \verb{<function>}, such as \code{cap_triangle()}.
\item A \verb{<character[1]>} naming a cap function without the '\code{cap_}'-prefix,
e.g. \code{"round"}.
\item A two column \verb{<matrix[n, 2]>} giving coordinates for a cap, like those
created by cap functions such as \code{cap_arch()}.
}}

\item{size}{A \code{\link[grid:unit]{<unit>}} setting the size of the cap. When
\code{NULL} (default), cap size will be proportional to the \code{shape} coordinates
and the \code{legend.key.size} theme setting.}

\item{show}{A \verb{<logical>} to control how caps are displayed at the ends
of the bar. When \code{TRUE}, caps are always displayed. When \code{FALSE}, caps
are never displayed. When \code{NA} (default), caps are displayed when the
data range exceed the limits. When given as \verb{<logical[2]>}, \code{show[1]}
controls the display at the lower end and \code{show[2]} at the upper end.}

\item{alpha}{A \verb{<numeric[1]>} between 0 and 1 setting the colour transparency
of the bar. Use \code{NA} to preserve the alpha encoded in the colour itself.}

\item{oob}{An out-of-bounds handling function that affects the cap colour.
Can be one of the following:
\itemize{
\item A \verb{<function>} like \code{\link[scales:oob_squish]{oob_squish}}.
\item A \verb{<character[1]>} naming such a function without the '\code{oob}'-prefix,
such as \code{"keep"}.
}}

\item{theme}{A \code{\link[ggplot2:theme]{<theme>}} object to style the guide individually or
differently from the plot's theme settings. The \code{theme} argument in the
guide overrides and is combined with the plot's theme.}

\item{position}{A \verb{<character[1]>} giving the location of the guide. Can be one of \code{"top"},
\code{"bottom"}, \code{"left"} or \code{"right"}.}

\item{direction}{A \verb{<character[1]>} indicating the direction of the guide. Can be on of
\code{"horizontal"} or \code{"vertical"}.}
}
\value{
A \code{GizmoStepcap} object.
}
\description{
This guide displays a binned variant of the colour bar with optional caps at
either ends of the bar.
}
\details{
\subsection{Styling options}{

Below are the \link[ggplot2:theme]{theme} options that determine the styling of
this guide. Note that the width or height (depending on the \code{direction}
argument) \emph{includes} the cap.\tabular{lll}{
   \strong{Theme setting} \tab \strong{Type} \tab \strong{Description} \cr
   \code{legend.frame} \tab \code{\link[ggplot2:element_rect]{element_rect()}} \tab Frame drawn around the bar and caps. The \code{fill} setting is ignored. \cr
   \code{legend.key.width} \tab \code{\link[=unit]{unit()}} \tab Width of the bar \cr
   \code{legend.key.height} \tab \code{\link[=unit]{unit()}} \tab Height of the bar \cr
}


Please note that depending on the \code{direction} argument, the
\code{legend.key.width}/\code{legend.key.height} setting are expanded 5-fold if
originating from the global theme. To set these directly, you can use the
local \code{theme} argument in the guide.
These settings have shorthands in \code{\link[=theme_guide]{theme_guide()}}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{gizmo_stepcap(theme = theme_guide(
  frame = element_rect(),
  key.width = unit(5, "mm")
  key.height = unit(5, "cm")
))
}\if{html}{\out{</div>}}
}
}
\examples{
# A standard plot
p <- ggplot(mpg, aes(displ, hwy, colour = cty)) +
  geom_point()

# Just some recangles
p + scale_colour_viridis_c(guide = gizmo_stepcap())

# Caps show up when there is data outside the limits
p + scale_colour_viridis_c(
  limits = c(10, 30),
  guide = gizmo_stepcap()
)

# The scale's out-of-bounds handler determines cap colour
p + scale_colour_viridis_c(
  limits = c(10, 30), oob = scales::oob_squish,
  guide = gizmo_stepcap()
)

# Customising the display of the guide
p +
  scale_colour_viridis_c(
    oob = scales::oob_squish,
    guide = gizmo_stepcap(
      shape = "round", show = c(FALSE, TRUE),
      size = unit(1, "cm"),
      theme = theme(legend.key.height = unit(4, "cm"))
    )
  ) +
  theme(
    legend.frame = element_rect(colour = "black"),
    legend.key.width = unit(0.5, "cm")
  )
}
\seealso{
Other gizmos:
\code{\link[=gizmo_barcap]{gizmo_barcap()}},
\code{\link[=gizmo_density]{gizmo_density()}},
\code{\link[=gizmo_grob]{gizmo_grob()}},
\code{\link[=gizmo_histogram]{gizmo_histogram()}}
}
\concept{gizmos}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/guide-axis-symbols.R
\name{guide_axis_symbols}
\alias{guide_axis_symbols}
\alias{guide_axis_upset}
\title{Symbol and upset axis guide}
\usage{
guide_axis_symbols(
  key = NULL,
  connect = NULL,
  title = waiver(),
  theme = NULL,
  override.aes = list(),
  position = waiver(),
  direction = NULL,
  call = NULL
)

guide_axis_upset(
  key = "upset",
  connect = "perpendicular",
  title = waiver(),
  theme = NULL,
  override.aes = list(),
  position = waiver(),
  direction = NULL,
  call = NULL
)
}
\arguments{
\item{key}{One of the following:
\itemize{
\item An \link[=key_upset]{upset key} specification. For \code{guide_axis_upset}, specifying
a \verb{<character[n]>} is also passed to the \code{key_upset(order)} argument. An
exception is made when the string is a valid key specification.
\item A \link[=key_symbols]{symbol key} specification.
}}

\item{connect}{One of the following:
\itemize{
\item A \verb{<data.frame>} containing the following columns:
\itemize{
\item \code{value_start}, \code{value_end} Scale break values or \verb{<numeric[n]>} values
connecting along the axis.
\item \code{level_start}, \code{level_end} must be \verb{<integer[n]>} values connecting
perpendicular to the axis.
\item (Optional) columns for graphical parameters: \code{colour}, \code{linewidth}
and \code{linetype}.
}
\item A \verb{<character[1]>} keyword for upset guides or logical symbols:
\itemize{
\item \code{"perpendicular"}: connect \code{TRUE} symbols (set membership for upset)
perpendicular to the axis.
\item \code{"parallel"}: connect \code{TRUE} symbols parallel to the axis. This is
\emph{not} appropriate for upset guides.
}
}}

\item{title}{One of the following to indicate the title of the guide:
\itemize{
\item A \verb{<character[1]>} or \verb{<expression[1]>} to set a custom title.
\item \code{NULL} to not display any title.
\item \code{\link[ggplot2:waiver]{waiver()}} (default) to take the name of the scale
object or the name specified in \code{\link[ggplot2:labs]{labs()}} as the title.
}}

\item{theme}{A \code{\link[ggplot2:theme]{<theme>}} object to style the guide individually or
differently from the plot's theme settings. The \code{theme} argument in the
guide overrides and is combined with the plot's theme.}

\item{override.aes}{A named \verb{<list>} specifying graphical properties of points to apply to
symbols. Every element must either be length 1 or match the number of symbols
determined by the key. 3 symbols are used in upset guides or for logical
symbolism. Otherwise the number of unique values to the \code{key_symbols(symbol)}
argument determines the number of symbols.}

\item{position}{A \verb{<character[1]>} giving the location of the guide. Can be one of \code{"top"},
\code{"bottom"}, \code{"left"} or \code{"right"}.}

\item{direction}{A \verb{<character[1]>} indicating the direction of the guide. Can be on of
\code{"horizontal"} or \code{"vertical"}.}

\item{call}{A \link[rlang:topic-error-call]{call} to display in messages.}
}
\value{
A \verb{<Guide>} object.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

These axis guides can be used for set annotations of discrete categories.
The upset guide displays set intersections in matrix and is can be used
to replace Venn/Euler diagrams. The symbol guide also displays a matrix of
symbols, but requires manually specifying them.
}
\details{
The upset axis does not predetermine the order the categories. If any sorting
based on set size needs to occur, the scale is the correct tool to handle
this task.
\subsection{Styling options}{

Several styling options are provided in the theme, while individualised
styling is discussed below the table.\tabular{lll}{
   \strong{Theme setting} \tab \strong{Type} \tab \strong{Description} \cr
   \code{legendry.axis.subtitle} \tab \code{\link[ggplot2:element_text]{element_text()}} \tab Titles on the side labelling levels. \cr
   \code{legendry.axis.subtitle.position} \tab \verb{<character[1]>} \tab One of \code{"top"}, \code{"right"}, \code{"bottom"} or \code{"left"}. \cr
   \code{legendry.zebra.light} \tab \code{\link[ggplot2:element_rect]{element_rect()}} \tab Row shading. \cr
   \code{legendry.zebra.dark} \tab \code{\link[ggplot2:element_rect]{element_rect()}} \tab Alternate row shading. \cr
   \code{legendry.table.spacing} \tab \code{\link[ggplot2:rel]{rel()}}/\code{\link[=unit]{unit()}} \tab Padding between levels. \cr
   \code{legendry.point} \tab \code{\link[ggplot2:element_point]{element_point()}} \tab Styling of the symbols \cr
   \code{legendry.connector} \tab \code{\link[ggplot2:element_line]{element_line()}} Drawing the \code{connect} lines. \tab  \cr
}


Moreover, styling options \emph{per group} of symbols can be set via the
\code{override.aes} argument. These override theme settings.

Styling options \emph{per symbol} can be set in \code{\link[=key_symbols]{key_symbols()}} via the \code{...}
argument. These override theme settings and 'per group' settings.

Styling options \emph{per line} in line connectors can be set by including
graphical parameters as columns in the \verb{connect = <data.frame>} argument.
These override theme settings.

The context-agnostic alternative to using \code{theme()} is to use
\code{\link[=theme_guide]{theme_guide()}}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{guide_axis_symbols(theme = theme_guide(
  subtitle = element_line(),
  subtitle.position = "left",
  zebra.light = element_rect(),
  zebra.dark = element_rect(),
  table.spacing = unit(5, "mm"),
  point = element_point(),
  connector = element_line()
))
}\if{html}{\out{</div>}}
}
}
\examples{
# Example plot
p <- ggplot(mpg, aes(paste(drv, year))) +
  geom_bar()

# A standard upset axis might not have right order of levels
p + guides(x = "axis_upset")

# The levels can be manually adjusted to taste
p + guides(x = guide_axis_upset(c("1999", "2008", "4", "f", "r")))

# The connections can be turned off to just show symbols
p + guides(x = guide_axis_upset(connect = NULL))

# The style can be changed per group of symbols.
# We need to give 3 colours to also cover NA-breaks
p + guides(x = guide_axis_upset(
  override.aes = list(colour = c("purple", "orange", NA))
))

# For symbol guides you have to manually specify where you want symbols and
# connection lines appear.
p + guides(x = guide_axis_symbols(
  key_symbols(
    aesthetic = c("4 1999", "4 2008", "r 1999"),
    level = c("Lvl 1", "Lvl 2", "Lvl 3")
  ),
  connect = data.frame(
    value_start = "4 2008", value_end = "r 1999",
    level_start = 2, level_end = 3
  )
))
}
\seealso{
Other standalone guides:
\code{\link[=guide_axis_annotation]{guide_axis_annotation()}},
\code{\link[=guide_axis_base]{guide_axis_base()}},
\code{\link[=guide_axis_dendro]{guide_axis_dendro()}},
\code{\link[=guide_axis_nested]{guide_axis_nested()}},
\code{\link[=guide_axis_plot]{guide_axis_plot()}},
\code{\link[=guide_circles]{guide_circles()}},
\code{\link[=guide_colbar]{guide_colbar()}},
\code{\link[=guide_colring]{guide_colring()}},
\code{\link[=guide_colsteps]{guide_colsteps()}},
\code{\link[=guide_legend_base]{guide_legend_base()}},
\code{\link[=guide_legend_cross]{guide_legend_cross()}},
\code{\link[=guide_legend_group]{guide_legend_group()}},
\code{\link[=guide_legend_manual]{guide_legend_manual()}}
}
\concept{standalone guides}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/primitive-segments.R
\name{primitive_segments}
\alias{primitive_segments}
\title{Guide primitives: segments}
\usage{
primitive_segments(
  key = NULL,
  space = rel(10),
  vanish = FALSE,
  theme = NULL,
  position = waiver()
)
}
\arguments{
\item{key}{A \link[=key_segments]{segment key} specification. See more information
in the linked topic. Alternatively, an object of class
\code{\link[stats:hclust]{<hclust>}} that automatically invokes \code{key_dendro()}.}

\item{space}{Either a \code{\link[grid:unit]{<unit>}} or \code{\link[ggplot2:rel]{<rel>}}
object of length 1 determining the space allocated in the orthogonal
direction. When the \code{space} argument is of class \verb{<rel>} (default) the
base size is taken from the tick length theme setting.}

\item{vanish}{Only relevant when the guide is used in the secondary theta
position: a \verb{<logical[1]>} on whether the continue to draw the segments
until they meed in the center (\code{TRUE}) or strictly observe the \code{space}
setting (\code{FALSE}).}

\item{theme}{A \code{\link[ggplot2:theme]{<theme>}} object to style the guide individually or
differently from the plot's theme settings. The \code{theme} argument in the
guide overrides and is combined with the plot's theme.}

\item{position}{A \verb{<character[1]>} giving the location of the guide. Can be one of \code{"top"},
\code{"bottom"}, \code{"left"} or \code{"right"}.}
}
\value{
A \verb{<PrimitiveSegments>} primitive guide that can be used inside other
guides.
}
\description{
This function constructs a \link[=guide-primitives]{guide primitive}.
}
\details{
\subsection{Styling options}{

Below are the \link[ggplot2:theme]{theme} options that determine the styling of
this guide, which may differ depending on whether the guide is used in
an axis or in a legend context.

The possible \code{{position}} suffixes mentioned below are \code{x}, \code{x.top},
\code{x.bottom}, \code{y}, \code{y.left}, \code{y.right}. The \code{theta} and \code{r} position suffixes
in \pkg{ggplot2} are \emph{not} obeyed in \pkg{legendry}.\tabular{llll}{
   \strong{Theme setting} \tab \strong{Context} \tab \strong{Type} \tab \strong{Description} \cr
   \verb{axis.ticks.\{position\}} \tab Axis \tab \code{\link[ggplot2:element_line]{element_line()}} \tab The line segments. \cr
   \verb{axis.ticks.length.\{position\}} \tab Axis \tab \code{\link[=unit]{unit()}} \tab Basis for the \code{space} argument \cr
   \code{legend.ticks} \tab Legend \tab \code{\link[ggplot2:element_line]{element_line()}} \tab The line segments \cr
   \code{legend.ticks.length} \tab Legend \tab \code{\link[=unit]{unit()}} \tab Basis for the \code{space} argument \cr
}


Styling options \emph{per segment} can be set in the \link[=key_segments]{segment key}.
The \code{line} prefixed properties are prioritised for segments. These override
theme settings.

The context-agnostic alternative to using \code{theme()} is to use
\code{\link[=theme_guide]{theme_guide()}}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{primitive_segments(theme = theme_guide(
  ticks = element_line(),
  ticks.length = unit(5, "mm")
))
}\if{html}{\out{</div>}}
}
}
\examples{
# Building a key
key <- key_segment_manual(
  value     = c(1.6, 1.6, 3.4, 5.2),
  value_end = c(7.0, 7.0, 3.4, 5.2),
  oppo      = c(1.0, 2.0, 0.0, 0.0),
  oppo_end  = c(1.0, 2.0, 3.0, 3.0)
)

# Using the primitive in a plot
ggplot(mpg, aes(displ, hwy)) +
  geom_point() +
  scale_x_continuous(
    guide = primitive_segments(key = key)
  )
}
\seealso{
Other primitives:
\code{\link[=primitive_box]{primitive_box()}},
\code{\link[=primitive_bracket]{primitive_bracket()}},
\code{\link[=primitive_fence]{primitive_fence()}},
\code{\link[=primitive_labels]{primitive_labels()}},
\code{\link[=primitive_line]{primitive_line()}},
\code{\link[=primitive_spacer]{primitive_spacer()}},
\code{\link[=primitive_ticks]{primitive_ticks()}},
\code{\link[=primitive_title]{primitive_title()}}
}
\concept{primitives}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legendry-package.R
\docType{package}
\name{legendry-package}
\alias{legendry}
\alias{legendry-package}
\title{legendry: Extended Legends and Axes for 'ggplot2'}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

A 'ggplot2' extension that focusses on expanding the plotter's arsenal of guides. Guides in 'ggplot2' include axes and legends. 'legendry' offers new axes and annotation options, as well as new legends and colour displays.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://teunbrand.github.io/legendry/}
  \item \url{https://github.com/teunbrand/legendry}
  \item Report bugs at \url{https://github.com/teunbrand/legendry/issues}
}

}
\author{
\strong{Maintainer}: Teun van den Brand \email{tahvdbrand@gmail.com} (\href{https://orcid.org/0000-0002-9335-7468}{ORCID}) [copyright holder]

Authors:
\itemize{
  \item Teun van den Brand \email{tahvdbrand@gmail.com} (\href{https://orcid.org/0000-0002-9335-7468}{ORCID}) [copyright holder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/guide-axis-base.R
\name{guide_axis_base}
\alias{guide_axis_base}
\title{Custom axis guide}
\usage{
guide_axis_base(
  key = NULL,
  title = waiver(),
  subtitle = NULL,
  theme = NULL,
  n.dodge = 1L,
  check.overlap = FALSE,
  angle = waiver(),
  cap = "none",
  bidi = FALSE,
  order = 0L,
  position = waiver()
)
}
\arguments{
\item{key}{A \link[=key_standard]{standard key} specification. Defaults to
\code{\link[=key_auto]{key_auto()}}. See more information in the linked topic and the 'Details'
section.}

\item{title}{One of the following to indicate the title of the guide:
\itemize{
\item A \verb{<character[1]>} or \verb{<expression[1]>} to set a custom title.
\item \code{NULL} to not display any title.
\item \code{\link[ggplot2:waiver]{waiver()}} (default) to take the name of the scale
object or the name specified in \code{\link[ggplot2:labs]{labs()}} as the title.
}}

\item{subtitle}{Passed on to \code{\link[=primitive_title]{primitive_title(title)}}.
Follow the linked topic for more details.}

\item{theme}{A \code{\link[ggplot2:theme]{<theme>}} object to style the guide individually or
differently from the plot's theme settings. The \code{theme} argument in the
guide overrides and is combined with the plot's theme.}

\item{n.dodge}{An positive \verb{<integer[1]>} setting the number of layers text
labels can occupy to avoid overlapping labels.}

\item{check.overlap}{A \verb{<logical[1]>} indicating whether to check for and
omit overlapping text. If \code{TRUE}, first, last and middle labels are
recursively prioritised in that order. If \code{FALSE}, all labels are drawn.}

\item{angle}{A specification for the text angle. Compared to setting the \code{angle} argument
in \code{\link[ggplot2:element_text]{element_text()}}, this argument uses some
heuristics to automatically pick the \code{hjust} and \code{vjust} that you
probably want. Can be one of the following:
\itemize{
\item \code{NULL} to take angles and justification settings directly from the theme.
\item \code{\link[ggplot2:waiver]{waiver()}} to allow reasonable defaults in special
cases.
\item A \verb{<numeric[1]>} between -360 and 360 for the text angle in degrees.
}}

\item{cap}{A method to cap the axes. One of the following:
\itemize{
\item A \verb{<character[1]>} with one of the following:
\itemize{
\item \code{"none"} to perform no capping.
\item \code{"both"} to cap the line at both ends at the most extreme breaks.
\item \code{"upper"} to cap the line at the upper extreme break.
\item \code{"lower"} to cap the line at the lower extreme break.
}
\item A \verb{<logical>[1]}, where \code{TRUE} is equivalent to \code{"both"} and \code{FALSE}
is equivalent to \code{"none"} in the options above.
\item A sorted \verb{<numeric>[2n]} with an even number of members. The lines
will be drawn between every odd-even pair.
\item A \verb{<function>} that takes the scale's breaks as the first argument, the
scale's limits as the second argument and returns a \verb{<numeric>[2n]} as
described above.
}}

\item{bidi}{A \verb{<logical[1]>}: whether ticks should be drawn bidirectionally
(\code{TRUE}) or in a single direction (\code{FALSE}, default).}

\item{order}{A positive \verb{<integer[1]>} that specifies the order of this guide among
multiple guides. This controls in which order guides are merged if there
are multiple guides for the same position. If \code{0} (default), the order is
determined by a hashing indicative settings of a guide.}

\item{position}{A \verb{<character[1]>} giving the location of the guide. Can be one of \code{"top"},
\code{"bottom"}, \code{"left"} or \code{"right"}.}
}
\value{
A \verb{<Guide>} object.
}
\description{
This axis guide is a visual representation of position scales and can
represent the \code{x}, \code{y}, \code{theta} and \code{r} aesthetics. It differs from
\code{\link[ggplot2:guide_axis]{guide_axis()}} in that it can accept custom keys
and is can act as an axis for \code{\link[ggplot2:coord_radial]{coord_radial()}} like
\code{\link[ggplot2:guide_axis_theta]{guide_axis_theta()}}.
}
\details{
Under the hood, this guide is a \link[=compose_stack]{stack composition} of a
\link[=primitive_line]{line}, \link[=primitive_ticks]{ticks} and
\link[=primitive_labels]{labels} primitives.

To set minor ticks, use \code{key = "minor"}, or use the \code{type} argument in
\code{key_manual()} or \code{key_map()}.

To use this as a logarithmic axis, set \code{key = "log"}.
\subsection{Styling options}{

Because this guide is pure composite guide, the \link[ggplot2:theme]{theme}
options that govern the styling are determined by its constituents. They are
linked below so you can find their 'Styling options' sections.\tabular{ll}{
   \strong{Primitive} \tab \strong{Description} \cr
   \code{\link{compose_stack}} \tab Stacks the lines, tick marks and labels. \cr
   \code{\link[=primitive_line]{primitive_line()}} \tab Makes up the axis line. \cr
   \code{\link[=primitive_ticks]{primitive_ticks()}} \tab Makes up the tick marks. \cr
   \code{\link[=primitive_labels]{primitive_labels()}} \tab Makes up the labels. \cr
}


Styling options \emph{per break} can be set in the \link[=key_standard]{standard key}.
These override theme settings.

The context-agnostic alternative to using \code{theme()} is to use
\code{\link[=theme_guide]{theme_guide()}}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{guide_axis_base(theme = theme_guide(
  # Common options
  line = element_line(),
  text = element_text(),
  ticks = element_line(),
  ticks.length = unit(5, "mm"),

  # Niche options below
  minor.ticks = element_line(),
  minor.ticks.length = unit(5, "mm"),
  mini.ticks = element_line(),
  mini.ticks.length = unit(5, "mm"),
))
}\if{html}{\out{</div>}}
}
}
\examples{
# A standard plot with custom keys
p <- ggplot(mpg, aes(displ, hwy)) +
  geom_point() +
  scale_x_continuous(
    guide = guide_axis_base(key = key_minor())
  ) +
  scale_y_continuous(
    guide = guide_axis_base(key = key_manual(c(20, 25, 30, 40)))
  )
p

# Is translated to theta axis without fuss
p + coord_radial()

# To use as logarithmic axis:
ggplot(msleep, aes(bodywt, brainwt)) +
  geom_point(na.rm = TRUE) +
  scale_x_continuous(
    transform = "log10",
    guide = guide_axis_base("log")
  )
}
\seealso{
Other standalone guides:
\code{\link[=guide_axis_annotation]{guide_axis_annotation()}},
\code{\link[=guide_axis_dendro]{guide_axis_dendro()}},
\code{\link[=guide_axis_nested]{guide_axis_nested()}},
\code{\link[=guide_axis_plot]{guide_axis_plot()}},
\code{\link[=guide_axis_symbols]{guide_axis_symbols()}},
\code{\link[=guide_circles]{guide_circles()}},
\code{\link[=guide_colbar]{guide_colbar()}},
\code{\link[=guide_colring]{guide_colring()}},
\code{\link[=guide_colsteps]{guide_colsteps()}},
\code{\link[=guide_legend_base]{guide_legend_base()}},
\code{\link[=guide_legend_cross]{guide_legend_cross()}},
\code{\link[=guide_legend_group]{guide_legend_group()}},
\code{\link[=guide_legend_manual]{guide_legend_manual()}}
}
\concept{standalone guides}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/key-other.R
\name{key_specialty}
\alias{key_specialty}
\alias{key_sequence}
\alias{key_bins}
\alias{key_upset}
\alias{key_symbols}
\title{Speciality keys}
\usage{
key_sequence(n = 15L)

key_bins(even.steps = FALSE, show.limits = NULL)

key_upset(sep = "[^[:alnum:]]+", order = NULL, empty_label = "Other")

key_symbols(aesthetic, level, symbol = NULL, ...)
}
\arguments{
\item{n}{A positive \verb{<integer[1]>} giving the number of colours to use for a
gradient.}

\item{even.steps}{A \verb{<logical[1]>} indicating whether the size of bins
should be displayed as equal (\code{TRUE}) or proportional to their length in
data space (\code{FALSE}).}

\item{show.limits}{A \verb{<logical[1]>} stating whether the limits of the scale
should be shown with labels and ticks (\code{TRUE}) or remain hidden (\code{FALSE}).
Note that breaks coinciding with limits are shown regardless of this
setting. The default, \code{NULL}, consults the scale's \code{show.limits} setting
or defaults to \code{FALSE}.}

\item{sep}{A \verb{<character[1]>} giving a \link[base:regex]{regular expression} to
use for splitting labels provided by the scale using the
\code{\link[base:strsplit]{strsplit()}} function. By defaults, labels are splitted
on any non-alphanumeric character.}

\item{order}{Order to set the upset layers in. One of the following:
\itemize{
\item A \verb{<character[n]>} giving pieces of split labels.
\item An \verb{<integer[n]>} giving the numerical order in which pieces of split
labels should appear.
}}

\item{empty_label}{A \verb{<character[1]>} giving a level label to assign to the
breaks that match no values to the pieces of split labels. Can be \code{NULL}
to omit labels for empty levels.}

\item{aesthetic}{A vector of values for the guide to represent equivalent to
the \code{breaks} argument in scales. These will be mapped by the scale to
positions. Alternatively, a \verb{<numeric[n]>} vector to set positions
directly. Positions are used to place symbols.}

\item{level}{A \verb{<factor[n]>} or \verb{<character[n]>} parallel to the \code{aesthetic}
argument setting the label level of the symbol.}

\item{symbol}{(Optional) An \verb{<integer[n]>} indexing the guide's
\code{override.aes} parameter.}

\item{...}{Additional graphical properties to set for each symbol. Valid
properties are \code{colour}, \code{shape}, \code{size}, \code{fill} and \code{stroke}. These
graphical properties have priority over properties derived via \code{symbol}
or the theme.}
}
\value{
A function.
}
\description{
These functions are helper functions for working with keys in guides. The
functions described here are not widely applicable and may only apply
to a small subset of guides. As such, it is fine to adjust the arguments
of a speciality key, but swapping types is ill-advised.
\itemize{
\item \code{key_sequence()} is a function factory whose functions create a regularly
spaced sequence between the limits of a scale. It is used in colour bar
guides.
\item \code{key_bins()} is a function factory whose function create a binned key
given the breaks in the scale. It is used in colour steps guides.
\item \code{key_upset()} is a function factory whose function creates an upset key
from splitting the breaks in the scale. It is used in the upset guide.
\item \code{key_symbols()} is a function factory whose function creates a key
from the literal provided values. It is used in the symbols guide.
}
}
\examples{
# An example scale
template <- scale_fill_viridis_c(limits = c(0, 10), breaks = c(2, 4, 6, 8))

# Retrieving colourbar and colourstep keys
key_sequence()(template)
key_bins()(template)

# Upset key with example scale
template <- scale_x_discrete(limits = c("A", "A,B", ""))
key_upset()(template)
# Putting 'B' in 1st level
key_upset(order = c("B", "A"))(template)
# Omit level for the empty break
key_upset(empty_label = NULL)(template)

# Symbol key with example scale
template <- scale_x_discrete(limits = LETTERS[1:5])
key_symbols(aesthetic = LETTERS[1:3], level = 3:1)(template)
# Aesthetic can also be numeric
key_symbols(1:3, 3:1)(template)
# Setting level order via factors
ordered <- factor(c("X", "Y", "Z"), c("Y", "X", "Z"))
key_symbols(1:3, level = ordered)(template)
# Setting groups for symbols, for `guide_axis_symbols(override.aes)`
key_symbols(1:3, 1:3, symbol = c(1, 1, 2))(template)
# Passing individual graphical parameters
key_symbols(1:3, 3:1, colour = c("red", "green", "blue"))(template)
}
\seealso{
Other keys:
\code{\link{key_group}},
\code{\link{key_range}},
\code{\link{key_segments}},
\code{\link{key_standard}}
}
\concept{keys}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/guide-legend-cross.R
\name{position_text}
\alias{position_text}
\title{Helper to position text}
\usage{
position_text(angle = NA, hjust = NA, vjust = NA, ..., element = element_text)
}
\arguments{
\item{angle}{A \verb{<numeric[1-4]>} value setting the angle of the text.}

\item{hjust, vjust}{A \verb{<numeric[1-4]>} between 0 and 1 to set the horizontal
justification (\code{hjust}) or vertical justification (\code{vjust}) for the text.}

\item{...}{Other arguments passed to the \code{element} function.}

\item{element}{An \verb{<function>} creating an object that inherits from the
\code{\link[ggplot2:element_text]{<element_text>}} class.}
}
\value{
A named list of \code{element} objects of length 4. The list has the
names \code{"top"}, \code{"right"}, \code{"bottom"} and \code{"left"}.
}
\description{
This is a helper function for use in \code{\link[=guide_legend_cross]{guide_legend_cross()}}. It creates
a list of text elements, each corresponding the top, right, bottom or left
positions. Input is given for that order as well.
}
\details{
Input other than the \code{element} argument will be recycled to length 4. \code{NA}
and 0-length input will be dropped.
}
\examples{
# Red text turning along with position
position_text(angle = c(0, -90, 180, 90), colour = "red")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/key-group.R
\name{key_group}
\alias{key_group}
\alias{key_group_split}
\alias{key_group_lut}
\title{Group keys}
\usage{
key_group_split(sep = "[^[:alnum:]]+", reverse = FALSE)

key_group_lut(members, group, ungrouped = "Other")
}
\arguments{
\item{sep}{A \verb{<character[1]>} giving a \link[base:regex]{regular expression} to
use for splitting labels provided by the scale using the
\code{\link[base:strsplit]{strsplit()}} function. By defaults, labels are splitted
on any non-alphanumeric character.}

\item{reverse}{A \verb{<logical[1]>} which if \code{FALSE} (default) treats the first
part of the split string as groups and later parts as members. If \code{TRUE},
treats the last part as groups.}

\item{members}{A vector including the scale's \code{breaks} values.}

\item{group}{A vector parallel to \code{members} giving  the group of each member.}

\item{ungrouped}{A \verb{<character[1]>} giving a group label to assign to the
scale's \code{breaks} that match no values in the \code{members} argument.}
}
\value{
A function to use as the \code{key} argument in a guide.
}
\description{
These functions are helper functions for working with grouped data as keys in
guides. They all share the goal of creating a guide key, but have different
methods.
\itemize{
\item \code{key_group_split()} is a function factory whose functions make an attempt
to infer groups from the scale's labels.
\item \code{key_group_lut()} is a function factory whose functions use a look up table
to sort out group membership.
}
}
\examples{
# Example scale
values <- c("group A:value 1", "group A:value 2", "group B:value 1")
template <- scale_colour_hue(limits = values)

# Treat the 'group X' part as groups
key <- key_group_split(sep = ":")
key(template)

# Treat the 'value X' part as groups
key <- key_group_split(sep = ":", reverse = TRUE)
key(template)

# Example scale
template <- scale_colour_hue(limits = msleep$name[c(1, 7, 9, 23, 24)])

# A lookup table can have more entries than needed
key <- key_group_lut(msleep$name, msleep$order)
key(template)

# Or less entries than needed
key <- key_group_lut(
  msleep$name[23:24], msleep$order[23:24],
  ungrouped = "Other animals"
)
key(template)
}
\seealso{
Other keys:
\code{\link{key_range}},
\code{\link{key_segments}},
\code{\link{key_specialty}},
\code{\link{key_standard}}
}
\concept{keys}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shapes-caps.R
\name{cap_options}
\alias{cap_options}
\alias{cap_triangle}
\alias{cap_round}
\alias{cap_arch}
\alias{cap_ogee}
\alias{cap_none}
\title{Cap options}
\usage{
cap_triangle()

cap_round(n = 100L)

cap_arch(n = 100L)

cap_ogee(n = 100L)

cap_none()
}
\arguments{
\item{n}{An \verb{<integer[n]>} number of points to use for the cap.}
}
\value{
A \verb{<matrix[n, 2]>} with coordinates for points on the brackets.
}
\description{
These functions construct various sorts of caps. They construct a matrix
that can be supplied as the \code{shape} argument in \code{\link[=gizmo_barcap]{gizmo_barcap()}}.
}
\details{
When designing custom cap shapes, the expectation is that the first point
starts at the \verb{(0, 0)} coordinate and the last point ends at the \verb{(0, 1)}
coordinate. The first column follows the orthogonal direction of the bar
whereas the second column follows the direction of the bar.
}
\section{Functions}{
\itemize{
\item \code{cap_triangle()}: An equilateral triangle with \code{n = 3} points.

\item \code{cap_round()}: A semicircle.

\item \code{cap_arch()}: Two circular arcs forming an equilateral Gothic arch.

\item \code{cap_ogee()}: Four circular arcs forming an 'ogee' arch.

\item \code{cap_none()}: No cap.

}}
\examples{
plot(cap_arch(), type = 'l')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/guide-axis-annotation.R
\name{guide_axis_annotation}
\alias{guide_axis_annotation}
\alias{annotate_top}
\alias{annotate_right}
\alias{annotate_bottom}
\alias{annotate_left}
\title{Annotation axis guide}
\usage{
guide_axis_annotation(
  aesthetic,
  label = as.character(aesthetic),
  ...,
  key = NULL,
  arrow = NULL,
  inner = waiver(),
  title = waiver(),
  theme = NULL,
  order = 0L,
  position = waiver(),
  call = NULL
)

annotate_top(..., position = "top")

annotate_right(..., position = "right")

annotate_bottom(..., position = "bottom")

annotate_left(..., position = "left")
}
\arguments{
\item{aesthetic}{A vector of values for the guide to represent.}

\item{label}{A \verb{<character[n]>} or list of expressions to use as labels.}

\item{...}{Additional graphical properties parallel to \code{aesthetic}. Can be
\code{text_colour}, \code{size}, \code{face}, \code{hjust}, \code{vjust}, \code{angle} or \code{lineheight} for
the labels. Can be \code{line_colour}, \code{linewidth} and \code{linetype} for ticks.
Setting \code{colour} will also set \code{text_colour} and \code{line_colour}.
For \code{annotate_top()}, \code{annotate_right()}, \code{annotate_bottom()} and
\code{annotate_left()}, arguments are passed on to \code{guide_axis_annotation()}.}

\item{key}{A \link[=key_standard]{standard key} overriding the \code{aesthetic}, \code{label} and \code{...}
arguments.}

\item{arrow}{A \code{\link[grid:arrow]{grid::arrow()}} specification. Can be \code{NULL} (default) to draw no arrow.}

\item{inner}{A guide that supports the aesthetic to draw the annotation across.
When \code{waiver()} (default), populates a \code{guide_axis_base()} except in the
\code{"top"}, \code{"right"} and \code{"theta.sec"} positions.}

\item{title}{One of the following to indicate the title of the guide:
\itemize{
\item A \verb{<character[1]>} or \verb{<expression[1]>} to set a custom title.
\item \code{NULL} to not display any title.
\item \code{\link[ggplot2:waiver]{waiver()}} (default) to take the name of the scale
object or the name specified in \code{\link[ggplot2:labs]{labs()}} as the title.
}}

\item{theme}{A \code{\link[ggplot2:theme]{<theme>}} object to style the \emph{annotation} part of this
guide differently from the plot's theme settings. The \code{theme} argument
in this guide overrides and is combined with the plot's theme.}

\item{order}{A positive \verb{<integer[1]>} that specifies the order of this guide among
multiple guides. This controls in which order guides are merged if there
are multiple guides for the same position. If \code{0} (default), the order is
determined by a hashing indicative settings of a guide.}

\item{position}{A \verb{<character[1]>} giving the location of the guide. Can be one of \code{"top"},
\code{"bottom"}, \code{"left"} or \code{"right"}.}

\item{call}{A \link[rlang:topic-error-call]{call} to display in messages.}
}
\value{
A \verb{<Guide>} object
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

This axis guide acts as annotation: it draws labels at specified places.
It also wraps an inner guide, making the behaviour look like one is 'adding'
the annotation on top of the regular guide.
}
\details{
Under the hood, this guide is a hybrid composition guide. The \code{theme()}
options that govern the styling are partially determined by its consituents.
They are linked below so you can find their 'Styling options' sections.

| \strong{Constituent} | \strong{Description} |
| \code{compose_ontop()} | Composes the annotation on top of the \code{inner} guide |
| \code{guide_axis_base()} | The default \code{inner} guide |
| \code{primitive_ticks()} | Display of the annotation ticks |
| \code{primitive_labels()} | Display of the annotation labels |

Styling options \emph{per annotation} can be set via \code{...} as described above.

The context-agnostic alternative to using \code{theme()} is to use
\code{\link[=theme_guide]{theme_guide()}}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{# Note that `theme` is used *only* for the annotation
guide_axis_annotation(theme = theme_guide(
  text = element_text(),
  ticks = element_line(),
  ticks.length = unit(5, "mm")
))
}\if{html}{\out{</div>}}
}
\examples{
# Basic plot
p <- ggplot(mpg, aes(displ, hwy)) +
  geom_point()

# Typical use
p +
  annotate_top(5, face = "bold") +
  annotate_bottom(c(2.5, 4.5), c("Bottom annotation", "Second label"))

# If you want to combine them with secondary axes, you must
# set the `inner` argument manually.
p +
  scale_y_continuous(
    sec.axis = dup_axis(breaks = c(15, 25, 35))
  ) +
  annotate_right(30, inner = "axis")

# Use in theta axis
p + coord_radial() +
  guides(theta = guide_axis_annotation(4.5, "Theta annotation"))

# Specialised use as part of other guides
# Note that `guide_colbar` imposes white inward ticks
# We can overrule these impositions with a replacement theme
p + aes(colour = cty) +
  guides(colour = guide_colbar(
    second_guide = guide_axis_annotation(22, "This", theme = theme_gray())
  ))
}
\seealso{
Other standalone guides:
\code{\link[=guide_axis_base]{guide_axis_base()}},
\code{\link[=guide_axis_dendro]{guide_axis_dendro()}},
\code{\link[=guide_axis_nested]{guide_axis_nested()}},
\code{\link[=guide_axis_plot]{guide_axis_plot()}},
\code{\link[=guide_axis_symbols]{guide_axis_symbols()}},
\code{\link[=guide_circles]{guide_circles()}},
\code{\link[=guide_colbar]{guide_colbar()}},
\code{\link[=guide_colring]{guide_colring()}},
\code{\link[=guide_colsteps]{guide_colsteps()}},
\code{\link[=guide_legend_base]{guide_legend_base()}},
\code{\link[=guide_legend_cross]{guide_legend_cross()}},
\code{\link[=guide_legend_group]{guide_legend_group()}},
\code{\link[=guide_legend_manual]{guide_legend_manual()}}
}
\concept{standalone guides}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gizmo-.R
\name{guide-gizmos}
\alias{guide-gizmos}
\title{Guide gizmos}
\description{
Guide gizmos are a speciality guide components that are very specific to
one or a few aesthetics to display.

Typically they can be \link[=guide-composition]{composed} with other guides or
guide \link[=guide-primitives]{primitives} to form a complete guide.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/primitive-fence.R
\name{primitive_fence}
\alias{primitive_fence}
\title{Guide primitive: fence}
\usage{
primitive_fence(
  key = "range_auto",
  rail = "none",
  angle = waiver(),
  oob = "squish",
  drop_zero = TRUE,
  pad_discrete = 0.5,
  levels_text = NULL,
  levels_post = NULL,
  levels_rail = NULL,
  theme = NULL,
  position = waiver()
)
}
\arguments{
\item{key}{A \link[=key_range]{range key} specification. See more information
in the linked topic.}

\item{rail}{A \verb{<character[1]>} giving an option for how to display fence
railing. Can be either \code{"none"} (default) to display no railings, \code{"inner"}
to draw one rail closer to the plot panel, \code{"outer"} to display one rail
farther from the plot panel, or \code{"both"} to sandwich the labels between
rails.}

\item{angle}{A specification for the text angle. Compared to setting the \code{angle} argument
in \code{\link[ggplot2:element_text]{element_text()}}, this argument uses some
heuristics to automatically pick the \code{hjust} and \code{vjust} that you
probably want. Can be one of the following:
\itemize{
\item \code{NULL} to take angles and justification settings directly from the theme.
\item \code{\link[ggplot2:waiver]{waiver()}} to allow reasonable defaults in special
cases.
\item A \verb{<numeric[1]>} between -360 and 360 for the text angle in degrees.
}}

\item{oob}{A method for dealing with out-of-bounds (oob) ranges. Can be one
of \code{"squish"}, \code{"censor"} or \code{"none"}.}

\item{drop_zero}{A \verb{<logical[1]>} whether to drop near-zero width ranges
(\code{TRUE}, default) or preserve them (\code{FALSE}).}

\item{pad_discrete}{A \verb{<numeric[1]>} giving the amount ranges should be
extended when given as a discrete variable. This is applied after
the \code{drop_zero} setting.}

\item{levels_text}{A list of \verb{<element_text>} objects to customise how
text appears at every level.}

\item{levels_post, levels_rail}{A list of \verb{<element_line>} objects to
customise how fence posts and rails are displayed at every level.}

\item{theme}{A \code{\link[ggplot2:theme]{<theme>}} object to style the guide individually or
differently from the plot's theme settings. The \code{theme} argument in the
guide overrides and is combined with the plot's theme.}

\item{position}{A \verb{<character[1]>} giving the location of the guide. Can be one of \code{"top"},
\code{"bottom"}, \code{"left"} or \code{"right"}.}
}
\value{
A \verb{<PrimitiveFence>} primitive guie that can be used inside other
guides.
}
\description{
This function constructs a fence \link[=guide-primitives]{guide primitive}. The
customisation options are easier to understand if we view fence 'post' as the
vertical pieces of a real world fence, and the 'rail' as the horizontal
pieces.
}
\details{
\subsection{Styling options}{

Below are the \link[ggplot2:theme]{theme} options that determine the styling of
this guide, which may differ depending on whether the guide is used in
an axis or in a legend context.

The possible \code{{position}} suffixes mentioned below are \code{x}, \code{x.top},
\code{x.bottom}, \code{y}, \code{y.left}, \code{y.right}. The \code{theta} and \code{r} position suffixes
in \pkg{ggplot2} are \emph{not} obeyed in \pkg{legendry}.\tabular{llll}{
   \strong{Theme setting} \tab \strong{Context} \tab \strong{Type} \tab \strong{Description} \cr
   \code{legendry.fence} \tab Both \tab \code{\link[ggplot2:element_line]{element_line()}} \tab Line segments for both 'post' and 'rail' segments \cr
   \code{legendry.fence.post} \tab Both \tab \code{\link[ggplot2:element_line]{element_line()}} \tab Line segments orthogonal to the scale \cr
   \code{legendry.fence.rail} \tab Both \tab \code{\link[ggplot2:element_line]{element_line()}} \tab Line segments parallel to the scale \cr
   \verb{axis.text.\{position\}} \tab Axis \tab \code{\link[ggplot2:element_text]{element_text()}} \tab The text labels at the fence. \cr
   \code{legend.text} \tab Legend \tab \code{\link[ggplot2:element_text]{element_text()}} \tab The text labels at the fence. \cr
}


Styling options \emph{per level} can be set in the \code{levels_post}, \code{levels_rail}
and \code{levels_text} arguments. These override theme settings.

Styling options \emph{per range} can be set in the \link[=key_range]{range key}.
The \code{line} and \code{text} prefixed properties are prioritised for the fence
and text respectively. The 'post' and 'rail' distinction does not apply
at the 'per range' settings. These override theme settings and the
'per level' settings.

The context-agnostic alternative to using \code{theme()} is to use
\code{\link[=theme_guide]{theme_guide()}}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{primitive_fence(theme = theme_guide(
  fence = element_line(),
  fence.post = element_line(),
  fence.rail = element_line(),
  text = element_text()
))
}\if{html}{\out{</div>}}
}
}
\examples{
# A standard plot
p <- ggplot(mpg, aes(interaction(drv, year), displ)) +
  geom_point()

key <- key_range_manual(c(2, 4), c(5, 6), c("A", "B"))

# Adding as secondary guides
p + guides(
  x.sec = primitive_fence(rail = "inner"),
  y.sec = primitive_fence(key = key, rail = "outer")
)
}
\seealso{
Other primitives:
\code{\link[=primitive_box]{primitive_box()}},
\code{\link[=primitive_bracket]{primitive_bracket()}},
\code{\link[=primitive_labels]{primitive_labels()}},
\code{\link[=primitive_line]{primitive_line()}},
\code{\link[=primitive_segments]{primitive_segments()}},
\code{\link[=primitive_spacer]{primitive_spacer()}},
\code{\link[=primitive_ticks]{primitive_ticks()}},
\code{\link[=primitive_title]{primitive_title()}}
}
\concept{primitives}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/guide-axis-plot.R
\name{guide_axis_plot}
\alias{guide_axis_plot}
\title{Side-plot axis}
\usage{
guide_axis_plot(
  plot,
  title = NULL,
  size = unit(2, "cm"),
  reposition = TRUE,
  theme = theme_sub_legend(position = "none"),
  position = waiver()
)
}
\arguments{
\item{plot}{A \verb{<ggplot>} object, subject to limitations listed in the 'Details' section.
Alternatively, a \verb{<function>} that takes the scale as argument and returns
a \verb{<ggplot>} object.}

\item{title}{One of the following to indicate the title of the guide:
\itemize{
\item A \verb{<character[1]>} or \verb{<expression[1]>} to set a custom title.
\item \code{NULL} (default) to not display any title.
\item \code{\link[ggplot2:waiver]{waiver()}} to take the name of the scale
object or the name specified in \code{\link[ggplot2:labs]{labs()}} as the title.
Please note that \code{plot} will still display a title unless instructed
otherwise. To avoid duplicated titles, the default is to have no title for
the guide.
}}

\item{size}{An absolute \code{\link[grid:unit]{<unit>}} to set the size of the \code{plot} panel in
the orthogonal direction.}

\item{reposition}{A \verb{<logical[1]>}. If \code{TRUE} (default) the \code{position} argument of this guide
will be propagated to the position scale bestowed upon the \code{plot} argument.
If \code{FALSE}, that position scale will retain its original \code{position} field.
Setting \code{reposition = TRUE} will generally tend to point axes outwards.}

\item{theme}{A \code{\link[ggplot2:theme]{<theme>}} object to style the guide individually or
differently from the plot's theme settings. The order in which themes are
applies is as follows: (1) the main plot's theme (2) the \code{plot} argument's
theme and (3) this \code{theme} argument. The default \code{theme} argument suppresses
legends.}

\item{position}{A \verb{<character[1]>} giving the location of the guide. Can be one of \code{"top"},
\code{"bottom"}, \code{"left"} or \code{"right"}.}
}
\value{
A \verb{<Guide>} object.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Displays an axis-sharing plot to the side of the panel.
}
\details{
This guide is subject to the following limitations:
\itemize{
\item The x- or y-scale of the main plot override the corresponding scale in the
\code{plot} argument. This ensures that the scales line up. The \code{plot} argument
should not have the relevant scale.
\item The \code{plot} argument cannot have custom facets. It must use the default
\code{facet_null()}.
\item This guide cannot be used in non-linear coordinate systems of the main plot
and does not support non-linear coordinate systems in the \code{plot} argument.
\item The \code{theme(panel.widths, panel.heights)} setting in the \code{plot} argument
will be ignored in favour of the \code{size} argument.
\item There is no mechanism to accommodate extra space needed by plot components
outside the panel. This applies in the horizontal direction for x-axes and
the vertical direction for y-axes. You may need to manually tweak the
\code{theme(plot.margin)} setting of the main plot to accommodate these
components.
}
\subsection{Styling options}{

This guide has no style options in \code{theme()} on itself. However, the
\code{plot} herein is subject to styling as described under the \code{theme} argument.
}
}
\examples{
# A standard plot
main_plot <- ggplot(mpg, aes(displ, hwy, colour = drv)) +
  geom_point()

# Simple plot sharing the x-variable
x_plot <- ggplot(mpg, aes(displ, fill = drv)) +
  geom_density(alpha = 0.7)

# Simple plot sharing the y-variable
y_plot <- ggplot(mpg, aes(drv, hwy, colour = drv)) +
  geom_boxplot()

# Typical use
main_plot + guides(
  x = guide_axis_plot(x_plot),
  y = guide_axis_plot(y_plot)
)

main_plot + guides(
  # Include `fill` legend by overriding theme
  x = guide_axis_plot(x_plot, theme = NULL),
  # Change the size of the side-plot
  y = guide_axis_plot(y_plot, size = unit(4, "cm"))
)

# Components outside panels may need to be manually acommodated
main_plot +
  guides(y = guide_axis_plot(y_plot + labs(title = "Boxplot"))) +
  theme(plot.margin = margin(25, 5.5, 5.5, 5.5))

# Recursive use of this guide
main_plot + guides(x = guide_axis_plot(
  main_plot + guides(x = guide_axis_plot(x_plot))
))
}
\seealso{
Other standalone guides:
\code{\link[=guide_axis_annotation]{guide_axis_annotation()}},
\code{\link[=guide_axis_base]{guide_axis_base()}},
\code{\link[=guide_axis_dendro]{guide_axis_dendro()}},
\code{\link[=guide_axis_nested]{guide_axis_nested()}},
\code{\link[=guide_axis_symbols]{guide_axis_symbols()}},
\code{\link[=guide_circles]{guide_circles()}},
\code{\link[=guide_colbar]{guide_colbar()}},
\code{\link[=guide_colring]{guide_colring()}},
\code{\link[=guide_colsteps]{guide_colsteps()}},
\code{\link[=guide_legend_base]{guide_legend_base()}},
\code{\link[=guide_legend_cross]{guide_legend_cross()}},
\code{\link[=guide_legend_group]{guide_legend_group()}},
\code{\link[=guide_legend_manual]{guide_legend_manual()}}
}
\concept{standalone guides}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gizmo-histogram.R
\name{gizmo_histogram}
\alias{gizmo_histogram}
\title{Guide gizmo: histogram}
\usage{
gizmo_histogram(
  key = waiver(),
  hist = NULL,
  hist.args = list(),
  hist.fun = graphics::hist,
  just = 1,
  oob = oob_keep,
  metric = "counts",
  alpha = NA,
  theme = NULL,
  position = waiver(),
  direction = NULL
)
}
\arguments{
\item{key}{A \link[=key_sequence]{sequence key} or \link[=key_bins]{binned key}
specification. Internally defaults to a sequence key when the scale is
continuous and a binned key when the scale is binned.}

\item{hist}{One of the following:
\itemize{
\item \code{NULL} for computing histograms on the data values (default).
\item an atomic \verb{<vector>} to feed to the \code{hist.fun} function.
\item A named \verb{<list>} with \code{breaks} and \code{counts} numeric items, where the
\code{breaks} item is exactly one element longer than the \code{counts} item. A
typical way to construct such list is using the \code{\link[graphics:hist]{hist()}}
function. Please note that \verb{<list>} input is expected in scale-transformed
space, not original data space.
}}

\item{hist.args}{A \verb{<list>} with additional arguments to the \code{hist.fun}
argument. Only applies when \code{hist} is not provided as a \verb{<list>} already.
Please note that these arguments are only used for binning and counting:
graphical arguments are ignored.}

\item{hist.fun}{A \verb{<function>} to use for computing histograms when the
\code{hist} argument is not provided as a list already.}

\item{just}{A \verb{<numeric[1]>} between 0 and 1. Use 0 for bottom- or
left-aligned histograms, use 1 for top- or right-aligned histograms and 0.5
for centred histograms.}

\item{oob}{An out-of-bounds handling function that affects the cap colour.
Can be one of the following:
\itemize{
\item A \verb{<function>} like \code{\link[scales:oob_squish]{oob_squish}}.
\item A \verb{<character[1]>} naming such a function without the '\code{oob}'-prefix,
such as \code{"keep"}.
}}

\item{metric}{A \verb{<character[1]>} either \code{"counts"} or \code{"density"} stating
which field of the \verb{<histogram>} class to display. The \code{"density"} metric
might be more appropriate to display when the histogram breaks have
non-constant intervals.}

\item{alpha}{A \verb{<numeric[1]>} between 0 and 1 setting the colour transparency
of the bar. Use \code{NA} to preserve the alpha encoded in the colour itself.}

\item{theme}{A \code{\link[ggplot2:theme]{<theme>}} object to style the guide individually or
differently from the plot's theme settings. The \code{theme} argument in the
guide overrides and is combined with the plot's theme.}

\item{position}{A \verb{<character[1]>} giving the location of the guide. Can be one of \code{"top"},
\code{"bottom"}, \code{"left"} or \code{"right"}.}

\item{direction}{A \verb{<character[1]>} indicating the direction of the guide. Can be on of
\code{"horizontal"} or \code{"vertical"}.}
}
\value{
A \verb{<GizmoHistogram>} object.
}
\description{
This guide displays a histogram of the aesthetic. If the aesthetic is
\code{colour} or \code{fill}, the shape will reflect this.
}
\details{
Non-finite values such as \code{NA} and \code{NaN} are ignored while infinite values
such as \code{-Inf} and \code{Inf} are \link[scales:oob_squish]{squished} to the limits.
\subsection{Styling options}{

Below are the \link[ggplot2:theme]{theme} options that determine the styling of
this guide.\tabular{lll}{
   \strong{Theme setting} \tab \strong{Type} \tab \strong{Description} \cr
   \code{legend.frame} \tab \code{\link[ggplot2:element_rect]{element_rect()}} \tab Outline drawn around the histogram itself. The \code{fill} setting is ignored. \cr
   \code{legend.key} \tab \code{\link[ggplot2:element_rect]{element_rect()}} \tab Background underneath the histogram area. \cr
   \code{legend.key.width} \tab \code{\link[=unit]{unit()}} \tab Width of the histogram area. \cr
   \code{legend.key.height} \tab \code{\link[=unit]{unit()}} \tab Height of the histogram area. \cr
}


Please note that depending on the \code{direction} argument, the
\code{legend.key.width}/\code{legend.key.height} setting are expanded 5-fold if
originating from the global theme. To set these directly, you can use the
local \code{theme} argument in the guide.
These settings have shorthands in \code{\link[=theme_guide]{theme_guide()}}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{gizmo_histogram(theme = theme_guide(
  frame = element_rect(),
  key = element_rect(),
  key.width = unit(5, "mm")
  key.height = unit(5, "cm")
))
}\if{html}{\out{</div>}}
}
}
\examples{
# A standard plot
p <- ggplot(mpg, aes(displ, hwy, colour = cty)) +
  geom_point() +
  scale_colour_viridis_c()

# Histogram from plot data
p + guides(colour = gizmo_histogram())

# Using bins instead of gradient
p + guides(colour = gizmo_histogram("bins"))

# Providing custom values to compute histogram
p + guides(colour = gizmo_histogram(hist = runif(1000, min = 5, max = 35)))

# Providing precomputed histogram
p + guides(colour = gizmo_histogram(hist = hist(mpg$cty, breaks = 10)))

# Alternatively, parameters may be passed through hist.args
p + guides(colour = gizmo_histogram(hist.arg = list(breaks = 10)))
}
\seealso{
Other gizmos:
\code{\link[=gizmo_barcap]{gizmo_barcap()}},
\code{\link[=gizmo_density]{gizmo_density()}},
\code{\link[=gizmo_grob]{gizmo_grob()}},
\code{\link[=gizmo_stepcap]{gizmo_stepcap()}}
}
\concept{gizmos}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/primitive-bracket.R
\name{primitive_bracket}
\alias{primitive_bracket}
\title{Guide primitive: brackets}
\usage{
primitive_bracket(
  key = "range_auto",
  bracket = "line",
  angle = waiver(),
  oob = "squish",
  drop_zero = TRUE,
  pad_discrete = 0.4,
  levels_brackets = NULL,
  levels_text = NULL,
  theme = NULL,
  position = waiver()
)
}
\arguments{
\item{key}{A \link[=key_range]{range key} specification. See more information
in the linked topic.}

\item{bracket}{A \link[=bracket_options]{bracket} by providing one of the
following:
\itemize{
\item A bracket \verb{<function>}, such as \code{bracket_square}.
\item A \verb{<character[1]>} naming a bracket function without the
'\code{bracket_}'-prefix, e.g. \code{"square"}.
\item A two-column \verb{<matrix[n, 2]>} giving line coordinates for a bracket,
like those created by bracket functions, such as \code{bracket_round()}.
}}

\item{angle}{A specification for the text angle. Compared to setting the \code{angle} argument
in \code{\link[ggplot2:element_text]{element_text()}}, this argument uses some
heuristics to automatically pick the \code{hjust} and \code{vjust} that you
probably want. Can be one of the following:
\itemize{
\item \code{NULL} to take angles and justification settings directly from the theme.
\item \code{\link[ggplot2:waiver]{waiver()}} to allow reasonable defaults in special
cases.
\item A \verb{<numeric[1]>} between -360 and 360 for the text angle in degrees.
}}

\item{oob}{A method for dealing with out-of-bounds (oob) ranges. Can be one
of \code{"squish"}, \code{"censor"} or \code{"none"}.}

\item{drop_zero}{A \verb{<logical[1]>} whether to drop near-zero width ranges
(\code{TRUE}, default) or preserve them (\code{FALSE}).}

\item{pad_discrete}{A \verb{<numeric[1]>} giving the amount ranges should be
extended when given as a discrete variable. This is applied after
the \code{drop_zero} setting.}

\item{levels_brackets}{A list of \verb{<element_line>} objects to customise how
brackets appear at every level.}

\item{levels_text}{A list of \verb{<element_text>} objects to customise how
text appears at every level.}

\item{theme}{A \code{\link[ggplot2:theme]{<theme>}} object to style the guide individually or
differently from the plot's theme settings. The \code{theme} argument in the
guide overrides and is combined with the plot's theme.}

\item{position}{A \verb{<character[1]>} giving the location of the guide. Can be one of \code{"top"},
\code{"bottom"}, \code{"left"} or \code{"right"}.}
}
\value{
A \verb{<PrimitiveBracket>} primitive guide that can be used inside other
guides.
}
\description{
This function constructs a brackets \link[=guide-primitives]{guide primitive}.
}
\details{
\subsection{Styling options}{

Below are the \link[ggplot2:theme]{theme} options that determine the styling of
this guide, which may differ depending on whether the guide is used in
an axis or in a legend context.

The possible \code{{position}} suffixes mentioned below are \code{x}, \code{x.top},
\code{x.bottom}, \code{y}, \code{y.left}, \code{y.right}. The \code{theta} and \code{r} position suffixes
in \pkg{ggplot2} are \emph{not} obeyed in \pkg{legendry}.\tabular{llll}{
   \strong{Theme setting} \tab \strong{Context} \tab \strong{Type} \tab \strong{Description} \cr
   \code{legendry.bracket} \tab Both \tab \code{\link[ggplot2:element_line]{element_line()}} \tab The bracket lines themselves. \cr
   \code{legendry.bracket.size} \tab Both \tab \code{\link[=unit]{unit()}} \tab The space (in the orthogonal direction) afforded to a bracket. \cr
   \verb{axis.text.\{position\}} \tab Axis \tab \code{\link[ggplot2:element_text]{element_text()}} \tab The text over brackets. \cr
   \code{legend.text} \tab Legend \tab \code{\link[ggplot2:element_text]{element_text()}} \tab The text over brackets. \cr
}


Styling options \emph{per level} can be set in the \code{levels_brackets} and
\code{levels_text} arguments. These override theme settings.

Styling options \emph{per range} can be set in the \link[=key_range]{range key}.
The \code{line} and \code{text} prefixed properties are prioritised for the brackets
and text respectively. These override theme settings and 'per level'
settings.

The context-agnostic alternative to using \code{theme()} is to use
\code{\link[=theme_guide]{theme_guide()}}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{primitive_bracket(theme = theme_guide(
  bracket = element_line(),
  bracket.size = unit(5, "mm")
))
}\if{html}{\out{</div>}}
}
}
\examples{
# A standard plot
p <- ggplot(mpg, aes(interaction(drv, year), displ)) +
 geom_point()

key <- key_range_manual(c(2, 4), c(5, 6), c("A", "B"))

# Adding as secondary guides
p + guides(
  x.sec = primitive_bracket(),
  y.sec = primitive_bracket(key = key)
)
}
\seealso{
Other primitives:
\code{\link[=primitive_box]{primitive_box()}},
\code{\link[=primitive_fence]{primitive_fence()}},
\code{\link[=primitive_labels]{primitive_labels()}},
\code{\link[=primitive_line]{primitive_line()}},
\code{\link[=primitive_segments]{primitive_segments()}},
\code{\link[=primitive_spacer]{primitive_spacer()}},
\code{\link[=primitive_ticks]{primitive_ticks()}},
\code{\link[=primitive_title]{primitive_title()}}
}
\concept{primitives}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compose-.R, R/compose-crux.R,
%   R/compose-ontop.R, R/compose-sandwich.R, R/compose-stack.R,
%   R/gizmo-barcap.R, R/gizmo-density.R, R/gizmo-grob.R, R/gizmo-histogram.R,
%   R/gizmo-stepcap.R, R/guide-axis-annotation.R, R/guide-axis-plot.R,
%   R/guide-axis-symbols.R, R/guide-circles.R, R/guide-colring.R,
%   R/guide-legend-base.R, R/guide-legend-group.R, R/guide-legend-manual.R,
%   R/legendry-package.R, R/primitive-box.R, R/primitive-bracket.R,
%   R/primitive-fence.R, R/primitive-labels.R, R/primitive-line.R,
%   R/primitive-spacer.R, R/primitive-ticks.R, R/primitive-title.R
\name{Compose}
\alias{Compose}
\alias{ComposeCrux}
\alias{ComposeOntop}
\alias{ComposeSandwich}
\alias{ComposeStack}
\alias{GizmoBarcap}
\alias{GizmoDensity}
\alias{GizmoGrob}
\alias{GizmoHistogram}
\alias{GizmoStepcap}
\alias{GuideAxisAnnotation}
\alias{GuideAxisPlot}
\alias{GuideAxisSymbols}
\alias{GuideCircles}
\alias{GuideColring}
\alias{GuideLegendBase}
\alias{GuideLegendGroup}
\alias{GuideLegendManual}
\alias{legendry_extensions}
\alias{PrimitiveBox}
\alias{PrimitiveBracket}
\alias{PrimitiveFence}
\alias{PrimitiveLabels}
\alias{PrimitiveLine}
\alias{PrimitiveSpacer}
\alias{PrimitiveTicks}
\alias{PrimitiveTitle}
\title{\code{ggproto} objects in \pkg{legendry}}
\description{
The \pkg{legendry} package relies on an extension system of
\pkg{ggplot2} through \code{\link[ggplot2:ggproto]{ggproto}} class objects, which
allow cross-package inheritance of objects such as geoms, stats, facets,
scales and coordinate systems. For the purpose of making plots, users are
invited to wholly ignore these objects, since interacting with these
objects is preferred through various constructor functions. The
\pkg{legendry} package introduces a new \verb{<Guide>} ggproto class to support
variations on axes, legends and colourbars.
}
\seealso{
The documentation over at \code{\link[ggplot2:ggproto]{ggproto}}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/primitive-line.R
\name{primitive_line}
\alias{primitive_line}
\title{Guide primitive: line}
\usage{
primitive_line(key = NULL, cap = "none", theme = NULL, position = waiver())
}
\arguments{
\item{key}{A \link[=key_standard]{standard key} specification. See more information
in the linked topic.}

\item{cap}{A method to cap the axes. One of the following:
\itemize{
\item A \verb{<character[1]>} with one of the following:
\itemize{
\item \code{"none"} to perform no capping.
\item \code{"both"} to cap the line at both ends at the most extreme breaks.
\item \code{"upper"} to cap the line at the upper extreme break.
\item \code{"lower"} to cap the line at the lower extreme break.
}
\item A \verb{<logical>[1]}, where \code{TRUE} is equivalent to \code{"both"} and \code{FALSE}
is equivalent to \code{"none"} in the options above.
\item A sorted \verb{<numeric>[2n]} with an even number of members. The lines
will be drawn between every odd-even pair.
\item A \verb{<function>} that takes the scale's breaks as the first argument, the
scale's limits as the second argument and returns a \verb{<numeric>[2n]} as
described above.
}}

\item{theme}{A \code{\link[ggplot2:theme]{<theme>}} object to style the guide individually or
differently from the plot's theme settings. The \code{theme} argument in the
guide overrides and is combined with the plot's theme.}

\item{position}{A \verb{<character[1]>} giving the location of the guide. Can be one of \code{"top"},
\code{"bottom"}, \code{"left"} or \code{"right"}.}
}
\value{
A \code{PrimitiveLine} primitive guide that can be used inside other
guides.
}
\description{
This function constructs a line \link[=guide-primitives]{guide primitive}.
}
\details{
\subsection{Styling options}{

Below are the \link[ggplot2:theme]{theme} options that determine the styling of
this guide, which may differ depending on whether the guide is used in
an axis or in a legend context.\tabular{llll}{
   \strong{Theme setting} \tab \strong{Context} \tab \strong{Type} \tab \strong{Description} \cr
   \verb{axis.line.\{x/y\}.\{position\}} \tab Axis \tab \code{\link[ggplot2:element_line]{element_line()}} \tab The axis line. \cr
   \code{legend.axis.line} \tab Legend \tab \code{\link[ggplot2:element_line]{element_line()}} \tab The axis line. \cr
}


There are no other styling options.
The context-agnostic alternative to using \code{theme()} is to use
\code{\link[=theme_guide]{theme_guide()}}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{primitive_line(theme = theme_guide(
  line = element_line()
))
}\if{html}{\out{</div>}}
}
}
\examples{
# A standard plot
p <- ggplot(mpg, aes(displ, hwy)) +
  geom_point() +
  theme(axis.line = element_line())

# Adding as secondary guides
p + guides(
  x.sec = primitive_line(),
  y.sec = primitive_line(cap = "both")
)
}
\seealso{
Other primitives:
\code{\link[=primitive_box]{primitive_box()}},
\code{\link[=primitive_bracket]{primitive_bracket()}},
\code{\link[=primitive_fence]{primitive_fence()}},
\code{\link[=primitive_labels]{primitive_labels()}},
\code{\link[=primitive_segments]{primitive_segments()}},
\code{\link[=primitive_spacer]{primitive_spacer()}},
\code{\link[=primitive_ticks]{primitive_ticks()}},
\code{\link[=primitive_title]{primitive_title()}}
}
\concept{primitives}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/primitive-labels.R
\name{primitive_labels}
\alias{primitive_labels}
\title{Guide primitive: labels}
\usage{
primitive_labels(
  key = NULL,
  angle = waiver(),
  n.dodge = 1L,
  check.overlap = FALSE,
  theme = NULL,
  position = waiver()
)
}
\arguments{
\item{key}{A \link[=key_standard]{standard key} specification. See more information
in the linked topic.}

\item{angle}{A specification for the text angle. Compared to setting the \code{angle} argument
in \code{\link[ggplot2:element_text]{element_text()}}, this argument uses some
heuristics to automatically pick the \code{hjust} and \code{vjust} that you
probably want. Can be one of the following:
\itemize{
\item \code{NULL} to take angles and justification settings directly from the theme.
\item \code{\link[ggplot2:waiver]{waiver()}} to allow reasonable defaults in special
cases.
\item A \verb{<numeric[1]>} between -360 and 360 for the text angle in degrees.
}}

\item{n.dodge}{An positive \verb{<integer[1]>} setting the number of layers text
labels can occupy to avoid overlapping labels.}

\item{check.overlap}{A \verb{<logical[1]>} indicating whether to check for and
omit overlapping text. If \code{TRUE}, first, last and middle labels are
recursively prioritised in that order. If \code{FALSE}, all labels are drawn.}

\item{theme}{A \code{\link[ggplot2:theme]{<theme>}} object to style the guide individually or
differently from the plot's theme settings. The \code{theme} argument in the
guide overrides and is combined with the plot's theme.}

\item{position}{A \verb{<character[1]>} giving the location of the guide. Can be one of \code{"top"},
\code{"bottom"}, \code{"left"} or \code{"right"}.}
}
\value{
A \verb{<PrimitiveLabels>} primitive guide that can be used inside other
guides.
}
\description{
This function constructs a labels \link[=guide-primitives]{guide primitive}.
}
\details{
\subsection{Styling options}{

Below are the \link[ggplot2:theme]{theme} options that determine the styling of
this guide, which may differ depending on whether the guide is used in
an axis or in a legend context.

The possible \code{{position}} suffixes mentioned below are \code{x}, \code{x.top},
\code{x.bottom}, \code{y}, \code{y.left}, \code{y.right}. The \code{theta} and \code{r} position suffixes
in \pkg{ggplot2} are \emph{not} obeyed in \pkg{legendry}.\tabular{llll}{
   \strong{Theme setting} \tab \strong{Context} \tab \strong{Type} \tab \strong{Description} \cr
   \verb{axis.text.\{position\}} \tab Axis \tab \code{\link[ggplot2:element_text]{element_text()}} \tab The text labels. \cr
   \code{legend.text} \tab Legend \tab \code{\link[ggplot2:element_text]{element_text()}} \tab The text labels. \cr
}


Styling options \emph{per break} can be set in the \link[=key_standard]{standard key}.
The \code{text} prefixed properties are prioritised. These override theme
settings.

The context-agnostic alternative to using \code{theme()} is to use
\code{\link[=theme_guide]{theme_guide()}}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{primitive_labels(theme = theme_guide(
  text = element_line()
))
}\if{html}{\out{</div>}}
}
}
\examples{
# A standard plot
p <- ggplot(mpg, aes(displ, hwy)) +
 geom_point()

# Adding as secondary guides
p + guides(
  x.sec = primitive_labels(),
  y.sec = primitive_labels(n.dodge = 2)
)
}
\seealso{
Other primitives:
\code{\link[=primitive_box]{primitive_box()}},
\code{\link[=primitive_bracket]{primitive_bracket()}},
\code{\link[=primitive_fence]{primitive_fence()}},
\code{\link[=primitive_line]{primitive_line()}},
\code{\link[=primitive_segments]{primitive_segments()}},
\code{\link[=primitive_spacer]{primitive_spacer()}},
\code{\link[=primitive_ticks]{primitive_ticks()}},
\code{\link[=primitive_title]{primitive_title()}}
}
\concept{primitives}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/primitive-title.R
\name{primitive_title}
\alias{primitive_title}
\title{Guide primitive: title}
\usage{
primitive_title(
  title = waiver(),
  angle = waiver(),
  theme = NULL,
  position = waiver()
)
}
\arguments{
\item{title}{One of the following to indicate the title of the guide:
\itemize{
\item A \verb{<character[1]>} or \verb{<expression[1]>} to set a custom title.
\item \code{NULL} to not display any title.
\item \code{\link[ggplot2:waiver]{waiver()}} (default) to take the name of the scale
object or the name specified in \code{\link[ggplot2:labs]{labs()}} as the title.
}}

\item{angle}{A specification for the text angle. Compared to setting the \code{angle} argument
in \code{\link[ggplot2:element_text]{element_text()}}, this argument uses some
heuristics to automatically pick the \code{hjust} and \code{vjust} that you
probably want. Can be one of the following:
\itemize{
\item \code{NULL} to take angles and justification settings directly from the theme.
\item \code{\link[ggplot2:waiver]{waiver()}} to allow reasonable defaults in special
cases.
\item A \verb{<numeric[1]>} between -360 and 360 for the text angle in degrees.
}}

\item{theme}{A \code{\link[ggplot2:theme]{<theme>}} object to style the guide individually or
differently from the plot's theme settings. The \code{theme} argument in the
guide overrides and is combined with the plot's theme.}

\item{position}{A \verb{<character[1]>} giving the location of the guide. Can be one of \code{"top"},
\code{"bottom"}, \code{"left"} or \code{"right"}.}
}
\value{
A \verb{<PrimitiveTitle>} primitive guide that can be used inside other
guides.
}
\description{
This function constructs a title \link[=guide-primitives]{guide primitive}.
}
\details{
\subsection{Styling options}{

Below are the \link[ggplot2:theme]{theme} options that determine the styling of
this guide, which may differ depending on whether the guide is used in
an axis or in a legend context.\tabular{llll}{
   \strong{Theme setting} \tab \strong{Context} \tab \strong{Type} \tab \strong{Description} \cr
   \verb{axis.title.\{x/y\}.\{position\}} \tab Axis \tab \code{\link[ggplot2:element_text]{element_text()}} \tab The title itself. \cr
   \code{legend.title} \tab Legend \tab \code{\link[ggplot2:element_text]{element_text()}} \tab The title itself. \cr
}


There are no further styling options.

The context-agnostic alternative to using \code{theme()} is to use
\code{\link[=theme_guide]{theme_guide()}}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{primitive_title(theme = theme_guide(
  title = element_text(),
))
}\if{html}{\out{</div>}}
}
}
\examples{
# A standard plot
p <- ggplot(mpg, aes(displ, hwy)) +
 geom_point()

# Adding as secondary guides
p + guides(
  x.sec = primitive_title("Horizontal Title"),
  y.sec = primitive_title(c("along vertical", "Multiple tiles"))
)

# 'Real' titles occur once per plot.
# Primitive titles repeat over facets and hide the 'real' title.
p + facet_wrap(~ drv) +
  guides(x = primitive_title("I am a repeated subtitle")) +
  labs(x = "I am the hidden real title")
}
\seealso{
Other primitives:
\code{\link[=primitive_box]{primitive_box()}},
\code{\link[=primitive_bracket]{primitive_bracket()}},
\code{\link[=primitive_fence]{primitive_fence()}},
\code{\link[=primitive_labels]{primitive_labels()}},
\code{\link[=primitive_line]{primitive_line()}},
\code{\link[=primitive_segments]{primitive_segments()}},
\code{\link[=primitive_spacer]{primitive_spacer()}},
\code{\link[=primitive_ticks]{primitive_ticks()}}
}
\concept{primitives}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/key-segment.R
\name{key_segments}
\alias{key_segments}
\alias{key_segment_manual}
\alias{key_segment_map}
\alias{key_dendro}
\title{Segment keys}
\usage{
key_segment_manual(value, oppo, value_end = value, oppo_end = oppo, ...)

key_segment_map(data, ..., .call = caller_env())

key_dendro(dendro = NULL, type = "rectangle", ..., .call = NULL)
}
\arguments{
\item{value, value_end}{A vector that is interpreted to be along the scale
that the guide codifies.}

\item{oppo, oppo_end}{A vector that is interpreted to be orthogonal to the
\code{value} and \code{value_end} variables.}

\item{...}{The \code{...} parameter has two purposes.
\enumerate{
\item In \code{key_segments_map()} it is \code{\link[rlang:topic-data-mask]{<data-masking>}}.
A set of mappings similar to those provided to \code{\link[ggplot2:aes]{aes()}},
which will be evaluated in the \code{data} argument. These \emph{must} contain
\code{value} and \code{oppo} mappings.
\item In other keys, \code{...} can be used to transfer graphical properties to the
individual breaks of a guide. For example, using \code{colour = "blue"} will
draw parts of the guides associated with breaks in blue. There is a shallow
hierarchy in that \code{line_colour} is the specific property for segment
elements, but others inherit from the main \code{colour} setting. Likewise,
\code{linewidth} and \code{linetype} have specific variants for line elements.
}}

\item{data}{A \verb{<data.frame>} or similar object coerced by
\code{\link[ggplot2:fortify]{fortify()}} to a \verb{<data.frame>}, in which the \code{mapping}
argument is evaluated.}

\item{.call}{A \link[rlang:topic-error-call]{call} to display in messages.}

\item{dendro}{A data structure that can be coerced to a dendrogram through
the \code{\link[stats:as.dendrogram]{as.dendrogram()}} function. When \code{NULL}
(default) an attempt is made to search for such data in the scale.}

\item{type}{A string, either \code{"rectangle"} or \code{"triangle"}, indicating the
shape of edges between nodes of the dendrogram.}
}
\value{
For \code{key_segments_manual()} and \code{key_segments_map()}, a \verb{<data.frame>} with
the \verb{<key_range>} class.
}
\description{
These functions are helper functions for working with segment data as keys
in guides. They all share the goal of creating a guide key, but have
different methods:
\itemize{
\item \code{key_segment_manual()} directly uses user-provided vectors to set segments.
\item \code{key_segment_map()} makes mappings from a \verb{<data.frame>} to set segments.
\item \code{key_dendro()} is a specialty case for coercing dendrogram data to
segments. Be aware that setting the key alone cannot affect the scale
limits, and will give misleading results when used incorrectly!
}
}
\examples{
# Giving vectors directly
key_segment_manual(
  value = 0:1, value_end = 2:3,
  oppo  = 1:0, oppo_end  = 3:2
)

# Taking columns of a data frame
data <- data.frame(x = 0:1, y = 1:0, xend = 2:3, yend = 3:2)
key_segment_map(data, value = x, oppo = y, value_end = xend, oppo_end = yend)

# Using dendrogram data
clust <- hclust(dist(USArrests), "ave")
key_dendro(clust)(scale_x_discrete())
}
\seealso{
Other keys:
\code{\link{key_group}},
\code{\link{key_range}},
\code{\link{key_specialty}},
\code{\link{key_standard}}
}
\concept{keys}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/key-range.R
\name{key_range}
\alias{key_range}
\alias{key_range_auto}
\alias{key_range_manual}
\alias{key_range_map}
\alias{key_range_rle}
\title{Range keys}
\usage{
key_range_auto(sep = "[^[:alnum:]]+", reverse = FALSE, ...)

key_range_manual(start, end, name = NULL, level = NULL, ..., call = NULL)

key_range_map(data, ..., call = NULL)

key_range_rle(x, ..., call = NULL)
}
\arguments{
\item{sep}{A \verb{<character[1]>} giving a \link[base:regex]{regular expression} to
use for splitting labels provided by the scale using
\code{\link[base:strsplit]{strsplit()}}. Defaults to splitting on any
non-alphanumeric character.}

\item{reverse}{A \verb{<logical[1]>} which if \code{FALSE} (default) treats the first
labels as the inner labels and the last labels as the outer labels.
If \code{TRUE}, thee first labels are treated as the outer labels and the last
labels are treated as the inner labels.}

\item{...}{The \code{...} parameter has two purposes.
\enumerate{
\item In \code{key_range_map()} it is \code{\link[rlang:topic-data-mask]{<data-masking>}}.
A set of mappings similar to those provided to \code{\link[ggplot2:aes]{aes()}},
which will be evaluated in the \code{data} argument. These \emph{must} contain
\code{start} and \code{end} mappings.
\item In other keys, \code{...} can be used to transfer graphical properties to the
individual ranges of a guide. For example, using \code{colour = "blue"} will
draw parts of the guides associated with ranges in blue. There is a shallow
hierarchy in that \code{text_colour}, \code{line_colour}, \code{rect_colour} and
\code{point_colour} are the specific properties for elements, but all inherit
from the main \code{colour} setting. Likewise, \code{size}, \code{linewidth}, \code{linetype}
and \code{fill} have specific variants for elements.
}}

\item{start, end}{A vector that can be interpreted by the scale, giving the
start and end positions of each range respectively.}

\item{name}{A \verb{<character>} or list of expressions}

\item{level}{An \verb{<integer>} giving the depth of each range to avoid overlaps
between different ranges. When \code{level} is smaller than 1, no brackets are
drawn.}

\item{call}{A \link[rlang:topic-error-call]{call} to display in messages.}

\item{data}{A \verb{<data.frame>} or similar object coerced by
\code{\link[ggplot2:fortify]{fortify()}} to a \verb{<data.frame>}, in which the \code{mapping}
argument is evaluated.}

\item{x}{A \verb{<vector[n]>} for which to determine run-starts and run-ends.}
}
\value{
For \code{key_range_auto()} a function. For \code{key_range_manual()} and
\code{key_range_map()} a \verb{<data.frame>} with the \verb{<key_range>} class.
}
\description{
These functions are helper functions for working with ranged data as keys
in guides. They all share the goal creating of a guide key, but have
different methods:
\itemize{
\item \code{key_range_auto()} is a function factory whose functions make an attempt
to infer ranges from the scale's labels.
\item \code{key_range_manual()} uses user-provided vectors to set ranges.
\item \code{key_range_map()} makes mappings from a \verb{<data.frame>} to set ranges.
\item \code{key_range_rle()} uses run-length encoding to determine the start and
end of runs (blocks of repeated data values).
}
}
\details{
The \code{level} variable is optional and when missing, the guides use an
algorithm similar to \code{IRanges::disjointBins()} to avoid overlaps.

The \code{key_range_auto()} does \emph{not} work with expression labels.
}
\examples{
# Example scale
template <- scale_x_discrete(limits = c("A 1", "B 1", "C&1", "D&2", "E&2"))

# By default, splits on all non-alphanumeric characters
auto <- key_range_auto()
auto(template)

# Only split on a specific character
auto <- key_range_auto(sep = "&")
auto(template)

# Treating the letters as outer labels and numbers as inner labels
auto <- key_range_auto(reverse = TRUE)
auto(template)

# Providing custom values
key_range_manual(
  start = c(1, 5,  10),
  end   = c(4, 15, 11),
  level = c(0, 2, 1),
  name  = c("A", "B", "C")
)

# Values from a <data.frame>
key_range_map(presidential, start = start, end = end, name = name)

# Values from run length encoding
key_range_rle(c("AB", "AB", "C", "DEF", "DEF", "DEF"))
}
\seealso{
Other keys:
\code{\link{key_group}},
\code{\link{key_segments}},
\code{\link{key_specialty}},
\code{\link{key_standard}}
}
\concept{keys}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/guide-legend-base.R
\name{guide_legend_base}
\alias{guide_legend_base}
\title{Custom legend guide}
\usage{
guide_legend_base(
  key = NULL,
  title = waiver(),
  theme = NULL,
  design = NULL,
  nrow = NULL,
  ncol = NULL,
  reverse = FALSE,
  override.aes = list(),
  position = NULL,
  direction = NULL,
  order = 0L
)
}
\arguments{
\item{key}{A \link[=key_standard]{standard key} specification. Defaults to
\code{\link[=key_auto]{key_auto()}}. See more information in the linked topic.}

\item{title}{One of the following to indicate the title of the guide:
\itemize{
\item A \verb{<character[1]>} or \verb{<expression[1]>} to set a custom title.
\item \code{NULL} to not display any title.
\item \code{\link[ggplot2:waiver]{waiver()}} (default) to take the name of the scale
object or the name specified in \code{\link[ggplot2:labs]{labs()}} as the title.
}}

\item{theme}{A \code{\link[ggplot2:theme]{<theme>}} object to style the guide individually or
differently from the plot's theme settings. The \code{theme} argument in the
guide overrides and is combined with the plot's theme.}

\item{design}{Specification of the legend layout. One of the following:
\itemize{
\item \code{NULL} (default) to use the layout algorithm of
\code{\link[ggplot2:guide_legend]{guide_legend()}}.
\item A \verb{<character[1]>} string representing a cell layout wherein \verb{#} defines
an empty cell. See examples.
\item A \verb{<matrix[n, m]>} representing a cell layout wherein \code{NA} defines an
empty cell. See examples. Non-string atomic vectors will be treated with
\code{as.matrix()}.
}}

\item{nrow, ncol}{A positive \verb{<integer[1]>} setting the desired dimensions of
the legend layout. When \code{NULL} (default), the dimensions will be derived
from the \code{design} argument or fit to match the number of keys.}

\item{reverse}{A \verb{<logical[1]>} whether the order of keys should be inverted.}

\item{override.aes}{A named \verb{<list>} specifying aesthetic parameters of the
key glyphs. See details and examples in
\code{\link[ggplot2:guide_legend]{guide_legend()}}.}

\item{position}{A \verb{<character[1]>} giving the location of the guide. Can be one of \code{"top"},
\code{"bottom"}, \code{"left"} or \code{"right"}.}

\item{direction}{A \verb{<character[1]>} indicating the direction of the guide. Can be on of
\code{"horizontal"} or \code{"vertical"}.}

\item{order}{A positive \verb{<integer[1]>} that specifies the order of this guide among
multiple guides. This controls in which order guides are merged if there
are multiple guides for the same position. If \code{0} (default), the order is
determined by a hashing indicative settings of a guide.}
}
\value{
A \verb{<GuideLegend>} object.
}
\description{
This legend closely mirrors \code{ggplot2::guide_legend()}, but has two
adjustments. First, \code{guide_legend_base()} supports a \code{design} argument
for a more flexible layout. Secondly, the \code{legend.spacing.y} theme element
is observed verbatim instead of overruled.
}
\details{
\subsection{Styling options}{

Below are the \link[ggplot2:theme]{theme} options that determine the styling of
this guide. Note that these are the same for \code{\link[ggplot2:guide_legend]{ggplot2::guide_legend()}}.\tabular{lll}{
   \strong{Theme setting} \tab \strong{Type} \tab \strong{Description} \cr
   \code{legend.background} \tab \code{\link[ggplot2:element_rect]{element_rect()}} \tab Background of the legend. \cr
   \code{legend.margin} \tab \code{\link[ggplot2:margin]{margin()}} \tab Padding around the legend. \cr
   \code{legend.text} \tab \code{\link[ggplot2:element_text]{element_text()}} \tab Labels displayed next to keys. \cr
   \code{legend.text.position} \tab \verb{<character[1]>} \tab One of \code{"top"}, \code{"right"}, \code{"bottom"} or \code{"left"}. \cr
   \code{legend.title} \tab \code{\link[ggplot2:element_text]{element_text()}} \tab Title of the legend. \cr
   \code{legend.title.position} \tab \verb{<character[1]>} \tab One of \code{"top"}, \code{"right"}, \code{"bottom"} or \code{"left"}. \cr
   \code{legend.key} \tab \code{\link[ggplot2:element_rect]{element_rect()}} \tab Background of the key areas. \cr
   \code{legend.key.height} \tab \code{\link[=unit]{unit()}} \tab Height of keys. \cr
   \code{legend.key.width} \tab \code{\link[=unit]{unit()}} \tab Width of keys. \cr
   \code{legend.key.justification} \tab \verb{<numeric[2]>} \tab Justification for placing legend keys in excess space. \cr
   \code{legend.key.spacing.x} \tab \code{\link[=unit]{unit()}} \tab Horizontal spacing between keys. \cr
   \code{legend.key.spacing.y} \tab \code{\link[=unit]{unit()}} \tab Vertical spacing between keys. Taken literally. \cr
   \code{legend.byrow} \tab \verb{<logical[1]>} \tab Row-order key filling (\code{TRUE}) or column-order (\code{FALSE}) \cr
}


The context-agnostic alternative to using \code{theme()} is to use
\code{\link[=theme_guide]{theme_guide()}}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{guide_legend_base(theme = theme_guide(
  text = element_text(),
  text.position = "right",
  title = element_text(),
  title.position = "top",
  key = element_rect(),
  key.height = unit(5, "mm"),
  key.width = unit(5, "mm"),
  key.justification = c(0.5, 0.5),
  key.spacing.x = unit(5, "mm"),
  key.spacing.y = unit(5, "mm"),
  byrow = TRUE,
  margin = margin(5),
  background = element_rect(),
))
}\if{html}{\out{</div>}}
}
}
\examples{
# A dummy plot
p <- ggplot(data.frame(x = 1:3, type = c("tic", "tac", "toe"))) +
  aes(x, x, shape = type) +
  geom_point(na.rm = TRUE) +
  scale_shape_manual(values = c(1, 4, NA))

# A design string, each character giving a cell value.
# Newlines separate rows, white space is ignored.
design <- "
  123
  213
  321
"

# Alternatively, the same can be specified using a matrix directly
# design <- matrix(c(1, 2, 3, 2, 1, 3, 3, 2, 1), 3, 3, byrow = TRUE)

p + guides(shape = guide_legend_base(design = design))

# Empty cells can be created using `#`
design <- "
  #2#
  1#3
"

# Alternatively:
# design <- matrix(c(NA, 1, 2, NA, NA, 3), nrow = 2)

p + guides(shape = guide_legend_base(design = design))
}
\seealso{
Other standalone guides:
\code{\link[=guide_axis_annotation]{guide_axis_annotation()}},
\code{\link[=guide_axis_base]{guide_axis_base()}},
\code{\link[=guide_axis_dendro]{guide_axis_dendro()}},
\code{\link[=guide_axis_nested]{guide_axis_nested()}},
\code{\link[=guide_axis_plot]{guide_axis_plot()}},
\code{\link[=guide_axis_symbols]{guide_axis_symbols()}},
\code{\link[=guide_circles]{guide_circles()}},
\code{\link[=guide_colbar]{guide_colbar()}},
\code{\link[=guide_colring]{guide_colring()}},
\code{\link[=guide_colsteps]{guide_colsteps()}},
\code{\link[=guide_legend_cross]{guide_legend_cross()}},
\code{\link[=guide_legend_group]{guide_legend_group()}},
\code{\link[=guide_legend_manual]{guide_legend_manual()}}

Other legend guides:
\code{\link[=guide_legend_cross]{guide_legend_cross()}},
\code{\link[=guide_legend_group]{guide_legend_group()}},
\code{\link[=guide_legend_manual]{guide_legend_manual()}}
}
\concept{legend guides}
\concept{standalone guides}
